@@ -289,7 +289,6 @@ int insert(CSOUND *csound, int insno, EVTBLK *newevtp)
     ip->init_done = 0;
 #endif
 
-
     if (csound->realtime_audio_flag == 0) {
      csound->curip    = ip;
      csound->ids      = (OPDS *)ip;
@@ -771,7 +770,7 @@ static void deact(CSOUND *csound, INSDS *ip)
 
 int kill_instance(CSOUND *csound, KILLOP *p) {
     if (LIKELY(*p->inst)) xturnoff(csound, (INSDS *) ((long)*p->inst));
-  else csound->Warning(csound, "instance not valid \n");
+    else csound->Warning(csound, Str("instance not valid \n"));
   return OK;
 }
 
@@ -1058,7 +1057,10 @@ int subinstrset_(CSOUND *csound, SUBINST *p, int instno)
       csound->engineState.instrtxtp[instno]->active++;
       csound->engineState.instrtxtp[instno]->instcnt++;
       p->ip->p1.value = (MYFLT) instno;
-      p->ip->opcod_iobufs = (void*) &p->buf;
+      /* VL 21-10-16: iobufs are not used here and
+         are causing trouble elsewhere. Commenting
+         it out */
+      /* p->ip->opcod_iobufs = (void*) &p->buf; */
       /* link into deact chain */
       p->ip->subins_deact = saved_curip->subins_deact;
       p->ip->opcod_deact = NULL;
@@ -1112,9 +1114,29 @@ int subinstrset_(CSOUND *csound, SUBINST *p, int instno)
     if (UNLIKELY(p->INOCOUNT >
                  (unsigned int)(csound->engineState.instrtxtp[instno]->pmax + 1)))
       return csoundInitError(csound, Str("subinstr: too many p-fields"));
-    for (n = 1; (unsigned int) n < p->INOCOUNT; n++)
-      (pfield + n)->value = *p->ar[inarg_ofs + n];
-
+    union {
+      MYFLT d;
+      int32 i;
+    } ch;
+    int str_cnt = 0, len = 0;
+    char *argstr;
+    for (n = 1; (unsigned int) n < p->INOCOUNT; n++){
+      if(IS_STR_ARG(p->ar[inarg_ofs + n])){
+        ch.d = SSTRCOD;
+        ch.i = str_cnt & 0xffff;
+        (pfield + n)->value = ch.d;
+        argstr = ((STRINGDAT *)p->ar[inarg_ofs + n])->data;
+        if(str_cnt == 0)
+          p->ip->strarg = csound->Calloc(csound, strlen(argstr)+1);
+        else
+          p->ip->strarg = csound->ReAlloc(csound, p->ip->strarg,
+                                          len+strlen(argstr)+1);
+        strcpy(p->ip->strarg + len, argstr);
+        len += strlen(argstr)+1;
+        str_cnt++;
+      }
+      else (pfield + n)->value = *p->ar[inarg_ofs + n];
+    }
     /* allocate memory for a temporary store of spout buffers */
     if (!init_op && !(pip->reinitflag | pip->tieflag))
       csoundAuxAlloc(csound,
@@ -1224,7 +1246,6 @@ int useropcdset(CSOUND *csound, UOPCODE *p)
     if (!p->ip) {
       /* search for already allocated, but not active instance */
       /* if none was found, allocate a new instance */
-
       if (!tp->act_instance)
         instance(csound, instno);
     /* **** COVERITY: note that call to instance fills in structure to
@@ -1584,6 +1605,8 @@ int subinstr(CSOUND *csound, SUBINST *p)
     unsigned int nsmps = CS_KSMPS;
     INSDS *ip = p->ip;
 
+    //printf("%s \n", p->ip->strarg);
+
     if (UNLIKELY(p->ip == NULL)) {                /* IV - Oct 26 2002 */
       return csoundPerfError(csound, p->h.insdshead,
                              Str("subinstr: not initialised"));
@@ -2103,10 +2126,10 @@ static void instance(CSOUND *csound, int insno)
     /* alloc new space,  */
     pextent = sizeof(INSDS) + pextrab + pextra*sizeof(CS_VAR_MEM);
     ip = (INSDS*) csound->Calloc(csound,
-                          (size_t) pextent + tp->varPool->poolSize +
-                                 (tp->varPool->varCount * CS_VAR_TYPE_OFFSET) +
-                                 (tp->varPool->varCount * sizeof(CS_VARIABLE*)) +
-                                 tp->opdstot);
+                     (size_t) pextent + tp->varPool->poolSize +
+                     (tp->varPool->varCount * CS_FLOAT_ALIGN(CS_VAR_TYPE_OFFSET)) +
+                     (tp->varPool->varCount * sizeof(CS_VARIABLE*)) +
+                     tp->opdstot);
     ip->csound = csound;
     ip->m_chnbp = (MCHNBLK*) NULL;
     ip->instr = tp;
@@ -2122,7 +2145,8 @@ static void instance(CSOUND *csound, int insno)
     tp->act_instance = ip;
     ip->insno = insno;
     if (UNLIKELY(csound->oparms->odebug))
-      csoundMessage(csound,"instance(): tp->act_instance = %p \n", tp->act_instance);
+      csoundMessage(csound,"instance(): tp->act_instance = %p \n",
+                    tp->act_instance);
 
 
     if (insno > csound->engineState.maxinsno) {
@@ -2140,7 +2164,7 @@ static void instance(CSOUND *csound, int insno)
     initializeVarPool((void *)csound, lclbas, tp->varPool);
 
     opMemStart = nxtopds = (char*) lclbas + tp->varPool->poolSize +
-                (tp->varPool->varCount * CS_VAR_TYPE_OFFSET);
+                (tp->varPool->varCount * CS_FLOAT_ALIGN(CS_VAR_TYPE_OFFSET));
     opdslim = nxtopds + tp->opdstot;
     if (UNLIKELY(odebug))
       csound->Message(csound,
@@ -2230,7 +2254,7 @@ static void instance(CSOUND *csound, int insno)
           fltp = &(pfield->value);
         }
         else {
-          csound->Message(csound, "FIXME: Unhandled out-arg type: %d\n",
+          csound->Message(csound, Str("FIXME: Unhandled out-arg type: %d\n"),
                           arg->type);
           fltp = NULL;
         }
@@ -2270,7 +2294,7 @@ static void instance(CSOUND *csound, int insno)
                               findLabelMemOffset(csound, tp, (char*)arg->argPtr));
         }
         else {
-          csound->Message(csound, "FIXME: instance unexpected arg: %d\n",
+          csound->Message(csound, Str("FIXME: instance unexpected arg: %d\n"),
                           arg->type);
         }
       }
