@@ -25,35 +25,264 @@
 #include "csoundCore.h"     /*                              MEMFILES.C      */
 #include "soundio.h"
 #include "pvfileio.h"
+#include "convolve.h"
+#include "lpc.h"
 #include "pstream.h"
 #include "namedins.h"
+#include <sndfile.h>
+#include <string.h>
 
-static int Load_File_(CSOUND *csound, const char *filnam,
-                       char **allocp, long *len, int csFileType)
+static int Load_Het_File_(CSOUND *csound, const char *filnam,
+                          char **allocp, int32 *len)
 {
     FILE *f;
+    int length = 1024;
+    int i = 0;
+    int cc;
+    int16 x;
+    char *all;
+    char buffer[16];
+    void *dummy = 0;
+    f = fopen(filnam, "r");
+    csoundNotifyFileOpened(csound, filnam, CSFTYPE_HETRO, 0, 0);
+    all = (char *)csound->Malloc(csound, (size_t) length);
+    for (i=0; i<6; i++) fgetc(f); /* Skip HETRO */
+    dummy = fgets(buffer, 10, f);         /* number of partials */
+    x = atoi(buffer);
+    memcpy(&all[0], &x, sizeof(int16));
+    /* Read data until end, pack as int16 */
+    for (i=sizeof(int16);;i+=sizeof(int16)) {
+      int p = 0;
+      while ((cc=getc(f))!=',' && cc!='\n' && p<15) {
+        if (cc == EOF) {
+          goto out;
+        }
+        buffer[p++] = cc;
+      }
+      buffer[p]='\0';
+      /* Expand as necessary */
+      if (i>=length-4) all = csound->ReAlloc(csound, all, length+=1024);
+      x = atoi(buffer);
+      memcpy(&all[i], &x, sizeof(int16));
+    }
+ out:
+    fclose(f);                                  /*   and close it      */
+    *len = i;
+    all = csound->ReAlloc(csound, all, i);
+    *allocp = all;
+    return 0;                                   /*   return 0 for OK   */
+}
 
+static MYFLT read_ieee(FILE* f, int *end)
+{
+    char buff[120];
+    double x;
+    char *p = fgets(buff, 120, f);
+
+    if (p==NULL || feof(f)) {
+      *end = 1;
+      return FL(0.0);
+    }
+    x = cs_strtod(buff, NULL);
+    return (MYFLT)x;
+    /* union { */
+    /*   double d; */
+    /*   struct {int  j,k;}  n; */
+    /*   int64_t  i; */
+    /* } x; */
+    /* int sign=1, ex; */
+    /* int64_t man; */
+    /* int64_t bit = 1; */
+    /* char buff[32]; */
+    /* char *p; */
+    /* bit <<= 62; */
+    /* p = fgets(buff, 32, f); */
+    /* printf("... %s", buff); */
+    /* if (p==NULL || feof(f)) { */
+    /*   printf("ending\n"); */
+    /*   *end = 1; */
+    /*   return FL(0.0); */
+    /* } */
+    /* if (strstr(p, "0x0p+0")) { */
+    /*   return FL(0.0); */
+    /* } */
+    /* if (buff[0]=='-') sign=-1; */
+    /* p = strchr(buff, '.')+1; */
+    /* sscanf(p, "%lxp%d", &man, &ex); */
+    /* x.i = man; */
+    /* if (man!=(int64_t)0) x.i |= bit; */
+    /* x.d = ldexp(x.d, ex-1); */
+    /* if (sign<0) x.d =-x.d; */
+    /* return (MYFLT)x.d; */
+}
+
+static int Load_CV_File_(CSOUND *csound, const char *filnam,
+                          char **allocp, int32 *len)
+{
+    FILE *f;
+    int length = 4096;
+    unsigned int i = 0;
+    int          j = 0;
+    MYFLT x;
+    char *all;
+    CVSTRUCT cvh = {0,0,0,0,0.0,0,0,0,0,{0}};
+    char buff[120];
+    char *p;
+    void *dummy = 0;
+
+    f = fopen(filnam, "r");
+    csoundNotifyFileOpened(csound, filnam, CSFTYPE_CVANAL, 0, 0);
+    all = (char *)csound->Malloc(csound, (size_t) length);
+    dummy = fgets(buff, 120, f); /* Skip CVANAL */
+    cvh.magic = CVMAGIC;
+    p = fgets(buff, 120, f);
+    if (p==NULL) {
+      fclose(f);
+      return csoundInitError(csound, Str("Ill-formed CV file\n"));
+    }
+    cvh.headBsize = strtol(p, &p, 10);
+    cvh.dataBsize = strtol(p, &p, 10);
+    cvh.dataFormat = strtol(p, &p, 10);
+    cvh.samplingRate = (MYFLT)cs_strtod(p, &p);
+    cvh.src_chnls = strtol(p, &p, 10);
+    cvh.channel = strtol(p, &p, 10);
+    cvh.Hlen = strtol(p, &p, 10);
+    cvh.Format = strtol(p, &p, 10);
+    /* fscanf(f, "%d %d %d %g %d %d %d %d\n",  */
+    /*        &cvh.headBsize, &cvh.dataBsize, &cvh.dataFormat, */
+    /*        &cvh.samplingRate, &cvh.src_chnls, &cvh.channel, */
+    /*        &cvh.Hlen, &cvh.Format); */
+    cvh.headBsize = sizeof(int32)*8 + sizeof(MYFLT);
+    memcpy(&all[0], &cvh, sizeof(CVSTRUCT));
+
+    /* Read data until end, pack as MYFLTs */
+    for (i=sizeof(CVSTRUCT);;i+=sizeof(MYFLT)) {
+      /* Expand as necessary */
+      if (i>=length-sizeof(MYFLT)-4) {
+        //printf("expanding from %p[%d] to\n", all, length);
+        all = csound->ReAlloc(csound, all, length+=4096);
+        //printf("i=%d                     %p[%d]\n", i, all, length);
+      }      x = read_ieee(f, &j);
+      if (j) break;
+      memcpy(&all[i], &x, sizeof(MYFLT));
+    }
+    fclose(f);                                  /*   and close it      */
+    //printf("length=%d i=%d\n", length, i);
+    *len = i;
+    all = csound->ReAlloc(csound, all, i);
+    *allocp = all;
+    return 0;                                   /*   return 0 for OK   */
+}
+
+static int Load_LP_File_(CSOUND *csound, const char *filnam,
+                          char **allocp, int32 *len)
+{
+    FILE *f;
+    int length = 4096;
+    unsigned int i = 0;
+    int          j = 0;
+    MYFLT x;
+    char *all, *p;
+    LPHEADER lph = {0,0,0,0,0.0,0.0,0.0,{0}};
+    char buff[120];
+    void *dummy = 0;
+
+    f = fopen(filnam, "r");
+    csoundNotifyFileOpened(csound, filnam, CSFTYPE_LPC, 0, 0);
+    all = (char *)csound->Malloc(csound, (size_t) length);
+    for (i=0; i<6; i++) fgetc(f); /* Skip LPANAL */
+    if (4!=fscanf(f, "%d %d %d %d\n",
+                  &lph.headersize, &lph.lpmagic, &lph.npoles, &lph.nvals)) {
+      fclose(f);
+      return csound->InitError(csound, Str("Ill-formed LPC file\n"));
+    }
+    dummy = fgets(buff, 120, f);
+    lph.framrate = (MYFLT)cs_strtod(buff, &p);
+    lph.srate = (MYFLT)cs_strtod(p, &p);
+    lph.duration = (MYFLT)cs_strtod(p, &p);
+    /* lph.text[0] = (char)strtol(p, &p, 0); */
+    /* lph.text[1] = (char)strtol(p, &p, 0); */
+    /* lph.text[2] = (char)strtol(p, &p, 0); */
+    /* lph.text[3] = (char)strtol(p, &p, 0); */
+    /* printf("LPHeader %d %d %d %d\n%f %f %f\n", */
+    /*        lph.headersize, lph.lpmagic, lph.npoles, lph.nvals, */
+    /*        lph.framrate, lph.srate, lph.duration); */
+    /* fscanf(f, "%f %f %f %.2x %.2x %.2x %.2x\n", */
+    /*        &lph.framrate, &lph.srate, &lph.duration, */
+    /*        &lph.text[0], &lph.text[1], &lph.text[2], &lph.text[3]); */
+    // This needs surgery if in/out different MYFLT sizes *** FIX ME ***
+    lph.headersize = sizeof(int32)*4+sizeof(MYFLT)*3;
+    memcpy(&all[0], &lph, lph.headersize);
+
+    /* Read data until end, pack as MYFLTs */
+    for (i=lph.headersize;;i+=sizeof(MYFLT)) {
+      /* Expand as necessary */
+      if (i>=length-sizeof(MYFLT)-8) {
+        //printf("expanding from %p[%d] to\n", all, length);
+        all = csound->ReAlloc(csound, all, length+=4096);
+        //printf("i=%d                     %p[%d]\n", i, all, length);
+      }
+      x = read_ieee(f, &j);
+      if (j) break;
+      memcpy(&all[i], &x, sizeof(MYFLT));
+    }
+    fclose(f);                                  /*   and close it      */
+    printf("length=%d i=%u\n", length, i);
+    *len = i;
+    all = csound->ReAlloc(csound, all, i);
+    *allocp = all;
+    return 0;                                   /*   return 0 for OK   */
+}
+
+static int Load_File_(CSOUND *csound, const char *filnam,
+                       char **allocp, int32 *len, int csFileType)
+{
+    FILE *f;
+    void *dummy = 0;
     *allocp = NULL;
     f = fopen(filnam, "rb");
-    if (f == NULL)                              /* if cannot open the file */
+    if (UNLIKELY(f == NULL))                    /* if cannot open the file */
       return 1;                                 /*    return 1             */
+    if (csFileType==CSFTYPE_HETRO) {
+      char buff[8];
+      dummy = fgets(buff, 6, f);
+      if (strcmp(buff, "HETRO")==0) {
+        fclose(f);
+        return Load_Het_File_(csound, filnam, allocp, len);
+      }
+    }
+    else if (csFileType==CSFTYPE_CVANAL) {
+      char buff[8];
+      dummy = fgets(buff, 7, f);
+      if (strcmp(buff, "CVANAL")==0) {
+        fclose(f);
+        return Load_CV_File_(csound, filnam, allocp, len);
+      }
+    }
+    else if (csFileType==CSFTYPE_LPC) {
+      char buff[8];
+      dummy = fgets(buff, 7, f);
+      if (strcmp(buff, "LPANAL")==0) {
+        fclose(f);
+        return Load_LP_File_(csound, filnam, allocp, len);
+      }
+    }
     /* notify the host if it asked */
     csoundNotifyFileOpened(csound, filnam, csFileType, 0, 0);
     fseek(f, 0L, SEEK_END);                     /* then get its length     */
-    *len = (long) ftell(f);
+    *len = (int32) ftell(f);
     fseek(f, 0L, SEEK_SET);
-    if (*len < 1L)
+    if (UNLIKELY(*len < 1L))
       goto err_return;
-    *allocp = mmalloc(csound, (size_t) (*len)); /*   alloc as reqd     */
-    if (fread(*allocp, (size_t) 1,              /*   read file in      */
-              (size_t) (*len), f) != (size_t) (*len))
+    *allocp = csound->Malloc(csound, (size_t) (*len)); /*   alloc as reqd     */
+    if (UNLIKELY(fread(*allocp, (size_t) 1,     /*   read file in      */
+                       (size_t) (*len), f) != (size_t) (*len)))
       goto err_return;
     fclose(f);                                  /*   and close it      */
     return 0;                                   /*   return 0 for OK   */
-
  err_return:
     if (*allocp != NULL) {
-      mfree(csound, *allocp);
+      csound->Free(csound, *allocp);
       *allocp = NULL;
     }
     fclose(f);
@@ -61,19 +290,35 @@ static int Load_File_(CSOUND *csound, const char *filnam,
 }
 
 /* Backwards-compatible wrapper for ldmemfile2().
-   Please use ldmemfile2() in all new code instead. */
+   Please use ldmemfile2() or ldmemfile2withCB() in all new code instead.
 MEMFIL *ldmemfile(CSOUND *csound, const char *filnam)
 {
-        return ldmemfile2(csound, filnam, CSFTYPE_UNKNOWN);
+    return ldmemfile2withCB(csound, filnam, CSFTYPE_UNKNOWN, NULL);
 }
-
+*/
 /* Takes an additional parameter specifying the type of the file being opened.
-   The type constants are defined in the enumeration CSOUND_FILETYPES. */
+   The type constants are defined in the enumeration CSOUND_FILETYPES.
+   Use ldmemfile2() to load file without additional processing.
 MEMFIL *ldmemfile2(CSOUND *csound, const char *filnam, int csFileType)
+{
+    return ldmemfile2withCB(csound, filnam, csFileType, NULL);
+}
+*/
+
+/* This version of ldmemfile2 allows you to specify a callback procedure
+   to process the file's data after it is loaded.  This method ensures that
+   your procedure is only called once even if the file is "loaded" multiple
+   times by several opcodes.  callback can be NULL.
+
+   Callback signature:     int myfunc(CSOUND* csound, MEMFIL* mfp)
+   Callback return value:  OK (0) or NOTOK (-1)
+ */
+MEMFIL *ldmemfile2withCB(CSOUND *csound, const char *filnam, int csFileType,
+                         int (*callback)(CSOUND*, MEMFIL*))
 {                               /* read an entire file into memory and log it */
     MEMFIL  *mfp, *last = NULL; /* share the file with all subsequent requests*/
     char    *allocp;            /* if not fullpath, look in current directory,*/
-    long    len;                /*   then SADIR (if defined).                 */
+    int32    len;                /*   then SADIR (if defined).                 */
     char    *pathnam;           /* Used by adsyn, pvoc, and lpread            */
 
     mfp = csound->memfiles;
@@ -84,25 +329,25 @@ MEMFIL *ldmemfile2(CSOUND *csound, const char *filnam, int csFileType)
       mfp = mfp->next;
     }
     /* Add new file description */
-    mfp = (MEMFIL*) mcalloc(csound, sizeof(MEMFIL));
+    mfp = (MEMFIL*) csound->Calloc(csound, sizeof(MEMFIL));
     if (last != NULL)
       last->next = mfp;
     else
       csound->memfiles = mfp;
     mfp->next = NULL;
-    strcpy(mfp->filename, filnam);
+    strncpy(mfp->filename, filnam, 255);
 
     pathnam = csoundFindInputFile(csound, filnam, "SADIR");
-    if (pathnam == NULL) {
+    if (UNLIKELY(pathnam == NULL)) {
       csoundMessage(csound, Str("cannot load %s\n"), filnam);
       delete_memfile(csound, filnam);
       return NULL;
     }
-    if (Load_File_(csound, pathnam, &allocp, &len, csFileType) != 0) {
+    if (UNLIKELY(Load_File_(csound, pathnam, &allocp, &len, csFileType) != 0)) {
       /* loadfile */
       csoundMessage(csound, Str("cannot load %s, or SADIR undefined\n"),
                             pathnam);
-      mfree(csound, pathnam);
+      csound->Free(csound, pathnam);
       delete_memfile(csound, filnam);
       return NULL;
     }
@@ -110,9 +355,17 @@ MEMFIL *ldmemfile2(CSOUND *csound, const char *filnam, int csFileType)
     mfp->beginp = allocp;
     mfp->endp = allocp + len;
     mfp->length = len;
+    if (callback != NULL) {
+      if (callback(csound, mfp) != OK) {
+        csoundMessage(csound, Str("error processing file %s\n"), filnam);
+        csound->Free(csound, pathnam);
+        delete_memfile(csound, filnam);
+        return NULL;
+      }
+    }
     csoundMessage(csound, Str("file %s (%ld bytes) loaded into memory\n"),
-                          pathnam, len);
-    mfree(csound, pathnam);
+                  pathnam, (long) len);
+    csound->Free(csound, pathnam);
     return mfp;                                          /* rtn new slotadr */
 }
 
@@ -124,8 +377,8 @@ void rlsmemfiles(CSOUND *csound)
 
     while (mfp != NULL) {
       nxt = mfp->next;
-      mfree(csound, mfp->beginp);       /*   free the space */
-      mfree(csound, mfp);
+      csound->Free(csound, mfp->beginp);       /*   free the space */
+      csound->Free(csound, mfp);
       mfp = nxt;
     }
     csound->memfiles = NULL;
@@ -149,8 +402,8 @@ int delete_memfile(CSOUND *csound, const char *filnam)
       csound->memfiles = mfp->next;
     else
       prv->next = mfp->next;
-    mfree(csound, mfp->beginp);
-    mfree(csound, mfp);
+    csound->Free(csound, mfp->beginp);
+    csound->Free(csound, mfp);
     return 0;
 }
 
@@ -184,11 +437,11 @@ int PVOCEX_LoadFile(CSOUND *csound, const char *fname, PVOCEX_MEMFILE *p)
     WAVEFORMATEX  fmt;
     PVOCEX_MEMFILE  *pp;
     int           i, j, rc = 0, pvx_id, hdr_size, name_size;
-    long          mem_wanted;
-    long          totalframes, framelen;
+    int32          mem_wanted;
+    int32          totalframes, framelen;
     float         *pFrame;
 
-    if (fname == NULL || fname[0] == '\0') {
+    if (UNLIKELY(fname == NULL || fname[0] == '\0')) {
       memset(p, 0, sizeof(PVOCEX_MEMFILE));
       return pvx_err_msg(csound, Str("Empty or NULL file name"));
     }
@@ -207,30 +460,30 @@ int PVOCEX_LoadFile(CSOUND *csound, const char *fname, PVOCEX_MEMFILE *p)
     memset(&pvdata, 0, sizeof(PVOCDATA));
     memset(&fmt, 0, sizeof(WAVEFORMATEX));
     pvx_id = csound->PVOC_OpenFile(csound, fname, &pvdata, &fmt);
-    if (pvx_id < 0) {
+    if (UNLIKELY(pvx_id < 0)) {
       return pvx_err_msg(csound, Str("unable to open pvocex file %s: %s"),
                                  fname, csound->PVOC_ErrorString(csound));
     }
     framelen = 2 * pvdata.nAnalysisBins;
     /* also, accept only 32bit floats for now */
-    if (pvdata.wWordFormat != PVOC_IEEE_FLOAT) {
+    if (UNLIKELY(pvdata.wWordFormat != PVOC_IEEE_FLOAT)) {
       return pvx_err_msg(csound, Str("pvoc-ex file %s is not 32bit floats"),
                                  fname);
     }
     /* FOR NOW, accept only PVOC_AMP_FREQ: later, we can convert */
     /* NB Csound knows no other: frameFormat is not read anywhere! */
-    if (pvdata.wAnalFormat != PVOC_AMP_FREQ) {
+    if (UNLIKELY(pvdata.wAnalFormat != PVOC_AMP_FREQ)) {
       return pvx_err_msg(csound, Str("pvoc-ex file %s not in AMP_FREQ format"),
                                  fname);
     }
     /* ignore the window spec until we can use it! */
     totalframes = csound->PVOC_FrameCount(csound, pvx_id);
-    if (totalframes <= 0) {
+    if (UNLIKELY(totalframes <= 0)) {
       return pvx_err_msg(csound, Str("pvoc-ex file %s is empty!"), fname);
     }
     mem_wanted = totalframes * 2 * pvdata.nAnalysisBins * sizeof(float);
     /* try for the big block first! */
-    pp = (PVOCEX_MEMFILE*) mmalloc(csound, (size_t) (hdr_size + name_size)
+    pp = (PVOCEX_MEMFILE*) csound->Malloc(csound, (size_t) (hdr_size + name_size)
                                            + (size_t) mem_wanted);
     memset((void*) pp, 0, (size_t) (hdr_size + name_size));
     pp->filename = (char*) ((uintptr_t) pp + (uintptr_t) hdr_size);
@@ -245,7 +498,7 @@ int PVOCEX_LoadFile(CSOUND *csound, const char *fname, PVOCEX_MEMFILE *p)
      */
     for (pFrame = pp->data, i = 0; i < totalframes; i++) {
       rc = csound->PVOC_GetFrames(csound, pvx_id, pFrame, 1);
-      if (rc != 1)
+      if (UNLIKELY(rc != 1))
         break;          /* read error, but may still have something to use */
       /* scale amps to Csound range, to fit fsig */
       for (j = 0; j < framelen; j += 2) {
@@ -254,12 +507,12 @@ int PVOCEX_LoadFile(CSOUND *csound, const char *fname, PVOCEX_MEMFILE *p)
       pFrame += framelen;
     }
     csound->PVOC_CloseFile(csound, pvx_id);
-    if (rc < 0) {
-      mfree(csound, pp);
+    if (UNLIKELY(rc < 0)) {
+      csound->Free(csound, pp);
       return pvx_err_msg(csound, Str("error reading pvoc-ex file %s"), fname);
     }
-    if (i < totalframes) {
-      mfree(csound, pp);
+    if (UNLIKELY(i < totalframes)) {
+      csound->Free(csound, pp);
       return pvx_err_msg(csound, Str("error reading pvoc-ex file %s "
                                      "after %d frames"), fname, i);
     }
@@ -268,14 +521,13 @@ int PVOCEX_LoadFile(CSOUND *csound, const char *fname, PVOCEX_MEMFILE *p)
       csound->Warning(csound, Str("%s's srate = %8.0f, orch's srate = %8.0f"),
                               fname, pp->srate, csound->esr);
     }
-    pp->nframes = (unsigned long) totalframes;
+    pp->nframes = (uint32) totalframes;
     pp->format  = PVS_AMP_FREQ;
     pp->fftsize = 2 * (pvdata.nAnalysisBins - 1);
     pp->overlap = pvdata.dwOverlap;
     pp->winsize = pvdata.dwWinlen;
     pp->chans   = fmt.nChannels;
     switch ((pv_wtype) pvdata.wWindowType) {
-      case PVOC_DEFAULT:
       case PVOC_HAMMING:
         pp->wintype = PVS_WIN_HAMMING;
         break;
@@ -294,7 +546,7 @@ int PVOCEX_LoadFile(CSOUND *csound, const char *fname, PVOCEX_MEMFILE *p)
     /* link into PVOC-EX memfile chain */
     csound->pvx_memfiles = pp;
     csound->Message(csound, Str("file %s (%ld bytes) loaded into memory\n"),
-                            fname, (long) mem_wanted);
+                            fname, (int32) mem_wanted);
 
     memcpy(p, pp, sizeof(PVOCEX_MEMFILE));
     return 0;
@@ -317,31 +569,26 @@ int PVOCEX_LoadFile(CSOUND *csound, const char *fname, PVOCEX_MEMFILE *p)
  * be undefined in this case).
  */
 
-SNDMEMFILE *csoundLoadSoundFile(CSOUND *csound, const char *fileName,
-                                                 SF_INFO *sfinfo)
+SNDMEMFILE *csoundLoadSoundFile(CSOUND *csound, const char *fileName, void *sfi)
 {
+    SF_INFO       *sfinfo = sfi;
     SNDFILE       *sf;
     void          *fd;
     SNDMEMFILE    *p = NULL;
     SF_INFO       tmp;
-    unsigned char h;
 
-    if (fileName == NULL || fileName[0] == '\0')
+
+    if (UNLIKELY(fileName == NULL || fileName[0] == '\0'))
       return NULL;
+
     /* check if file is already loaded */
-    h = name_hash_2(csound, fileName);
     if (csound->sndmemfiles != NULL) {
-      p = ((SNDMEMFILE**) csound->sndmemfiles)[(int) h];
-      while (p != NULL && sCmp(p->name, fileName) != 0)
-        p = p->nxt;
+      p = cs_hash_table_get(csound, csound->sndmemfiles, (char*)fileName);
     }
     else {
-      int i;
-      /* if no files loaded yet, allocate table */
-      csound->sndmemfiles = csound->Malloc(csound, sizeof(SNDMEMFILE*) * 256);
-      for (i = 0; i < 256; i++)
-        ((SNDMEMFILE**) csound->sndmemfiles)[i] = NULL;
+      csound->sndmemfiles = cs_hash_table_create(csound);
     }
+
     if (p != NULL) {
       /* if file was loaded earlier: */
       if (sfinfo != NULL) {
@@ -360,7 +607,7 @@ SNDMEMFILE *csoundLoadSoundFile(CSOUND *csound, const char *fileName,
     }
     fd = csound->FileOpen2(csound, &sf, CSFILE_SND_R, fileName, sfinfo,
                             "SFDIR;SSDIR", CSFTYPE_UNKNOWN_AUDIO, 0);
-    if (fd == NULL) {
+    if (UNLIKELY(fd == NULL)) {
       csound->ErrorMsg(csound,
                        Str("csoundLoadSoundFile(): failed to open '%s'"),
                        fileName);
@@ -368,7 +615,7 @@ SNDMEMFILE *csoundLoadSoundFile(CSOUND *csound, const char *fileName,
     }
     p = (SNDMEMFILE*)
             csound->Malloc(csound, sizeof(SNDMEMFILE)
-                                   + (size_t) sfinfo->frames * sizeof(float));
+                           + (size_t)  sfinfo->frames * sizeof(float));
     /* set parameters */
     p->name = (char*) csound->Malloc(csound, strlen(fileName) + 1);
     strcpy(p->name, fileName);
@@ -387,8 +634,6 @@ SNDMEMFILE *csoundLoadSoundFile(CSOUND *csound, const char *fileName,
     p->loopEnd = 0.0;
     p->baseFreq = 1.0;
     p->scaleFac = 1.0;
-#if defined(HAVE_LIBSNDFILE) && HAVE_LIBSNDFILE >= 1013
-    /* sampler information requires libsndfile version 1.0.13 or later */
     {
       SF_INSTRUMENT lpd;
       if (sf_command(sf, SFC_GET_INSTRUMENT, &lpd, sizeof(SF_INSTRUMENT))
@@ -405,12 +650,10 @@ SNDMEMFILE *csoundLoadSoundFile(CSOUND *csound, const char *fileName,
           p->loopMode = 1;
         }
         p->baseFreq = pow(2.0, (double) (((int) lpd.basenote - 69) * 100
-                                         + (int) lpd.detune) / 1200.0) * 440.0;
+                                         + (int) lpd.detune) / 1200.0) * csound->A4;
         p->scaleFac = pow(10.0, (double) lpd.gain * 0.05);
       }
     }
-#endif      /* HAVE_LIBSNDFILE >= 1013 */
-    p->nxt = ((SNDMEMFILE**) csound->sndmemfiles)[(int) h];
     if ((size_t) sf_readf_float(sf, &(p->data[0]), (sf_count_t) p->nFrames)
         != p->nFrames) {
       csound->FileClose(csound, fd);
@@ -427,10 +670,11 @@ SNDMEMFILE *csoundLoadSoundFile(CSOUND *csound, const char *fileName,
                                 "sample frames) loaded into memory\n"),
                             p->fullName, (int) sfinfo->samplerate,
                             (int) sfinfo->channels,
-                            (unsigned long) sfinfo->frames);
+                            (uint32) sfinfo->frames);
+
     /* link into database */
-    ((SNDMEMFILE**) csound->sndmemfiles)[(int) h] = p;
+    cs_hash_table_put(csound, csound->sndmemfiles, (char*)fileName, p);
+
     /* return with pointer to file structure */
     return p;
 }
-
