@@ -30,10 +30,23 @@ int gen21_rand(FGDATA *ff, FUNC *ftp);
 typedef struct  {
         OPDS    h;
         MYFLT   *sr, *in, *powerOf, *norm;
-} POW;
+ } POW;
 
 typedef struct  {
         OPDS    h;
         MYFLT   *out, *arg1, *arg2, *arg3;
 } PRAND;
 
+typedef struct  {
+        OPDS    h;
+        MYFLT   *ar, *arg1, *xamp, *xcps;
+        MYFLT   *iseed;
+        MYFLT   dfdmax, num1, num2;
+        int32   phs;
+        int     ampcod, cpscod;
+} PRANDI;
+
+typedef struct {
+        OPDS   h;
+        MYFLT  *ans;
+} GETSEED;
