@@ -25,7 +25,9 @@
 /*                                              RTPA.C for PortAudio    */
 
 #include "csdl.h"
+#if !defined(WIN32)
 #include "soundio.h"
+#endif
 #include <portaudio.h>
 
 #define NO_FULLDUPLEX_PA_LOCK   0
@@ -62,8 +64,9 @@ typedef struct PA_BLOCKING_STREAM_ {
     PaStreamParameters outputPaParameters;
 #ifdef WIN32
     int         paused;                 /* VL: to allow for smooth pausing  */
-    int         paLockTimeout;
 #endif
+    int         paLockTimeout;
+    int         complete;
 } PA_BLOCKING_STREAM;
 
 static int pa_PrintErrMsg(CSOUND *csound, const char *fmt, ...)
@@ -97,40 +100,85 @@ static int initPortAudio(CSOUND *csound)
 }
 
 /* list available input or output devices; returns the number of devices */
-
-static int listPortAudioDevices_blocking(CSOUND *csound,
-                                         int print_list, int play)
-{
-    PaDeviceInfo  *dev_info;
+int listDevices(CSOUND *csound, CS_AUDIODEVICE *list, int isOutput){
+     PaDeviceInfo  *dev_info;
     int           i, j, ndev;
+    char          tmp[256], *s;
+
+    if (initPortAudio(csound) != 0)
+      return 0;
+
+     if ((s = (char*) csound->QueryGlobalVariable(csound, "_RTAUDIO")) == NULL)
+      return 0;
 
     ndev = (int) Pa_GetDeviceCount();
     for (i = j = 0; i < ndev; i++) {
       dev_info = (PaDeviceInfo*) Pa_GetDeviceInfo((PaDeviceIndex) i);
-      if ((play && dev_info->maxOutputChannels > 0) ||
-          (!play && dev_info->maxInputChannels > 0))
+      if ((isOutput && dev_info->maxOutputChannels > 0) ||
+          (!isOutput && dev_info->maxInputChannels > 0))
         j++;
     }
-    if (!j) {
-      pa_PrintErrMsg(csound, Str("No %s devices are available\n"),
-                             (play ? Str("output") : Str("input")));
-      return 0;
-    }
-    if (!print_list)
-      return j;
-    csound->Message(csound, Str("PortAudio: available %s devices:\n"),
-                            (play ? Str("output") : Str("input")));
+    if (!j) return 0;
+    if(list!=NULL) {
     for (i = j = 0; i < ndev; i++) {
       dev_info = (PaDeviceInfo*) Pa_GetDeviceInfo((PaDeviceIndex) i);
-      if ((play && dev_info->maxOutputChannels > 0) ||
-          (!play && dev_info->maxInputChannels > 0)) {
-        csound->Message(csound, " %3d: %s\n", j, dev_info->name);
+      if ((isOutput && dev_info->maxOutputChannels > 0) ||
+          (!isOutput && dev_info->maxInputChannels > 0)) {
+        strncpy(list[j].device_name, dev_info->name, 63);
+        if (isOutput) {
+          snprintf(tmp, 256, "dac%d", j);
+        } else {
+          snprintf(tmp, 256, "adc%d", j);
+        }
+        strncpy(list[j].device_id, tmp, 63);
+        strncpy(list[j].rt_module, s, 63);
+        list[j].max_nchnls =
+          isOutput ?  dev_info->maxOutputChannels : dev_info->maxInputChannels;
+        list[j].isOutput = isOutput;
         j++;
       }
     }
+    }
     return j;
 }
 
+
+static int listPortAudioDevices_blocking(CSOUND *csound,
+                                         int print_list, int play)
+{
+    int i,n = listDevices(csound, NULL, play);
+    CS_AUDIODEVICE *devs =
+      (CS_AUDIODEVICE *) csound->Malloc(csound, n*sizeof(CS_AUDIODEVICE));
+    listDevices(csound, devs, play);
+    for(i=0; i < n; i++)
+      csound->Message(csound, " %3d: %s (%s)\n",
+                      i, devs[i].device_id, devs[i].device_name);
+    csound->Free(csound, devs);
+    return n;
+}
+
+static void DAC_channels(CSOUND *csound, int chans){
+    int *dachans = (int *) csound->QueryGlobalVariable(csound, "_DAC_CHANNELS_");
+    if (dachans == NULL) {
+      if (csound->CreateGlobalVariable(csound, "_DAC_CHANNELS_",
+                                       sizeof(int)) != 0)
+        return;
+      dachans = (int *) csound->QueryGlobalVariable(csound, "_DAC_CHANNELS_");
+      *dachans = chans;
+    }
+}
+
+static void ADC_channels(CSOUND *csound, int chans){
+    int *dachans = (int *) csound->QueryGlobalVariable(csound, "_ADC_CHANNELS_");
+    if (dachans == NULL) {
+      if (csound->CreateGlobalVariable(csound, "_ADC_CHANNELS_",
+                                       sizeof(int)) != 0)
+        return;
+      dachans = (int *) csound->QueryGlobalVariable(csound, "_ADC_CHANNELS_");
+      *dachans = chans;
+    }
+}
+
 /* select PortAudio device; returns the actual device number */
 
 static int selectPortAudioDevice(CSOUND *csound, int devNum, int play)
@@ -166,9 +214,17 @@ static int selectPortAudioDevice(CSOUND *csound, int devNum, int play)
       devNum = i;
     }
     dev_info = (PaDeviceInfo*) Pa_GetDeviceInfo((PaDeviceIndex) devNum);
-    csound->Message(csound, Str("PortAudio: selected %s device '%s'\n"),
-                            (play ? Str("output") : Str("input")),
-                            dev_info->name);
+    if (dev_info) {
+      csound->Message(csound, Str("PortAudio: selected %s device '%s'\n"),
+                      (play ? Str("output") : Str("input")),
+                      dev_info->name);
+      if(play) {
+      csound->system_sr(csound, (MYFLT) dev_info->defaultSampleRate);
+      DAC_channels(csound, dev_info->maxOutputChannels);
+      } else ADC_channels(csound, dev_info->maxInputChannels);
+    }
+    else
+      csound->Message(csound, Str("PortAudio: failed to obtain device info.\n"));
     return devNum;
 }
 
@@ -212,7 +268,7 @@ static int paBlockingReadWriteOpen(CSOUND *csound)
                                                              "_rtpaGlobals");
     if (pabs == NULL)
       return -1;
-    if (initPortAudio(csound) != 0)
+   if (initPortAudio(csound) != 0)
       goto err_return;
 
     if ((int) Pa_GetDeviceCount() <= 0) {
@@ -255,8 +311,8 @@ static int paBlockingReadWriteOpen(CSOUND *csound)
         pa_PrintErrMsg(csound, Str("Inconsistent full-duplex sample rates"));
         goto err_return;
       }
-      if (((pabs->inParm.bufSamp_SW / csound->ksmps) * csound->ksmps)
-          != pabs->inParm.bufSamp_SW)
+      if (UNLIKELY(((pabs->inParm.bufSamp_SW / csound->GetKsmps(csound)) *
+                    csound->GetKsmps(csound)) != pabs->inParm.bufSamp_SW))
         csound->MessageS(csound,
                          CSOUNDMSG_WARNING,
                          Str("WARNING: buffer size should be an integer "
@@ -277,7 +333,7 @@ static int paBlockingReadWriteOpen(CSOUND *csound)
 #endif
       csound->WaitThreadLock(pabs->paLock, (size_t) 500);
     csound->WaitThreadLock(pabs->clientLock, (size_t) 500);
-#ifdef WIN32
+
     pabs->paLockTimeout =
         (int) (1.33 * (pabs->outputPaParameters.suggestedLatency
                          >= pabs->inputPaParameters.suggestedLatency ?
@@ -287,7 +343,7 @@ static int paBlockingReadWriteOpen(CSOUND *csound)
       pabs->paLockTimeout = 25;
     else if (pabs->paLockTimeout > 1000)
       pabs->paLockTimeout = 1000;
-#endif
+
 
     err = Pa_OpenStream(&stream,
                         (pabs->mode & 1 ? &(pabs->inputPaParameters)
@@ -299,7 +355,8 @@ static int paBlockingReadWriteOpen(CSOUND *csound)
                         (unsigned long) (pabs->mode & 2 ?
                                          pabs->outParm.bufSamp_SW
                                          : pabs->inParm.bufSamp_SW),
-                        (PaStreamFlags) paNoFlag,
+                        (csound->GetDitherMode(csound) ?
+                         (PaStreamFlags) paNoFlag : (PaStreamFlags) paDitherOff),
                         paBlockingReadWriteStreamCallback,
                         (void*) pabs);
     if (err != paNoError) {
@@ -327,10 +384,11 @@ static CS_NOINLINE void paClearOutputBuffer(PA_BLOCKING_STREAM *pabs,
                                             float *buf)
 {
     int   nsmps = pabs->outBufSamples;
-    int   i = 0;
-    do {
-      buf[i] = 0.0f;
-    } while (++i < nsmps);
+    memset(buf,0, nsmps*sizeof(float));
+    /* int   i = 0; */
+    /* do { */
+    /*   buf[i] = 0.0f; */
+    /* } while (++i < nsmps); */
 }
 
 static int paBlockingReadWriteStreamCallback(const void *input,
@@ -347,30 +405,35 @@ static int paBlockingReadWriteStreamCallback(const void *input,
     float   *paInput = (float*) input;
     float   *paOutput = (float*) output;
 
-    if (pabs->paStream == NULL
-#ifdef WIN32
-        || pabs->paused
-#endif
-        ) {
-      if (pabs->mode & 2)
-        paClearOutputBuffer(pabs, paOutput);
-      return paContinue;
+
+    if (pabs->complete == 1) {
+        return paComplete;
     }
 
+#ifdef WIN32
+if (pabs->paStream == NULL
+      || pabs->paused
+) {
+if (pabs->mode & 2)
+        paClearOutputBuffer(pabs, paOutput);
+return paContinue;
+}
+#endif
+
 #if NO_FULLDUPLEX_PA_LOCK
     err = 0;
     if (!pabs->noPaLock)
 #endif
-#ifndef __MACH__
-#  ifdef WIN32
+      /*#ifndef __MACH__*/
+      /*#  ifdef WIN32 */
       err = csound->WaitThreadLock(pabs->paLock, (size_t) pabs->paLockTimeout);
-#  else
-      err = csound->WaitThreadLock(pabs->paLock, (size_t) 500);
+    /*#  else
+    err = csound->WaitThreadLock(pabs->paLock, (size_t) 500);
 #  endif
-#else
+    #else
       csound->WaitThreadLock(pabs->paLock, (size_t) 500);
-    err = 0;
-#endif
+      err = 0;
+     #endif*/
 
     if (pabs->mode & 1) {
       int n = pabs->inBufSamples;
@@ -395,7 +458,8 @@ static int paBlockingReadWriteStreamCallback(const void *input,
       }
     }
 
-    csound->NotifyThreadLock(pabs->clientLock);
+   paClearOutputBuffer(pabs, pabs->outputBuffer);
+   csound->NotifyThreadLock(pabs->clientLock);
 
     return paContinue;
 }
@@ -484,6 +548,8 @@ static int recopen_(CSOUND *csound, const csRtAudioParams *parm)
     memcpy(&(pabs->inParm), parm, sizeof(csRtAudioParams));
     *(p->GetRtRecordUserData(p)) = (void*) pabs;
 
+    pabs->complete = 0;
+
     return 0;
 }
 
@@ -506,6 +572,8 @@ static int playopen_(CSOUND *csound, const csRtAudioParams *parm)
     memcpy(&(pabs->outParm), parm, sizeof(csRtAudioParams));
     *(p->GetRtPlayUserData(p)) = (void*) pabs;
 
+    pabs->complete = 0;
+
     return (paBlockingReadWriteOpen(p));
 }
 
@@ -514,27 +582,31 @@ static int playopen_(CSOUND *csound, const csRtAudioParams *parm)
 static void rtclose_(CSOUND *csound)
 {
     PA_BLOCKING_STREAM *pabs;
-
     pabs = (PA_BLOCKING_STREAM*) csound->QueryGlobalVariable(csound,
                                                              "_rtpaGlobals");
+
+    csound->Message(csound, Str("closing device\n"));
     if (pabs == NULL)
       return;
 
+    pabs->complete = 1;
+
     if (pabs->paStream != NULL) {
       PaStream  *stream = pabs->paStream;
-      int       i;
-      pabs->paStream = NULL;
-      for (i = 0; i < 4; i++) {
+      unsigned int i;
+
+      for (i = 0; i < 4u; i++) {
 #if NO_FULLDUPLEX_PA_LOCK
         if (!pabs->noPaLock)
 #endif
           csound->NotifyThreadLock(pabs->paLock);
         csound->NotifyThreadLock(pabs->clientLock);
-        Pa_Sleep(80);
+        //Pa_Sleep(80);
       }
-      Pa_AbortStream(stream);
+
+      Pa_StopStream(stream);
       Pa_CloseStream(stream);
-      Pa_Sleep(80);
+      //Pa_Sleep(80);
     }
 
     if (pabs->clientLock != NULL) {
@@ -556,7 +628,7 @@ static void rtclose_(CSOUND *csound)
       free(pabs->inputBuffer);
       pabs->inputBuffer = NULL;
     }
-
+    pabs->paStream = NULL;
     *(csound->GetRtRecordUserData(csound)) = NULL;
     *(csound->GetRtPlayUserData(csound)) = NULL;
     csound->DestroyGlobalVariable(csound, "_rtpaGlobals");
@@ -607,7 +679,9 @@ static int set_device_params(CSOUND *csound, DEVPARAMS *dev,
       err = (int) Pa_OpenStream(&(dev->handle), NULL, &streamParams,
                                 (double) parm->sampleRate,
                                 (unsigned long) parm->bufSamp_SW,
-                                paNoFlag, NULL, NULL);
+                                (csound->GetDitherMode(csound) ?
+                                 paNoFlag:paDitherOff),
+                                NULL, NULL);
     }
     else {
       err = (int) Pa_OpenStream(&(dev->handle), &streamParams, NULL,
@@ -724,7 +798,7 @@ static void rtplay_blocking(CSOUND *csound, const MYFLT *outbuf, int nbytes)
 static void rtclose_blocking(CSOUND *csound)
 {
     DEVPARAMS *dev;
-
+    csound->Message(csound, Str("closing device\n"));
     dev = (DEVPARAMS*) (*(csound->GetRtRecordUserData(csound)));
     if (dev != NULL) {
       *(csound->GetRtRecordUserData(csound)) = NULL;
@@ -754,8 +828,8 @@ static void rtclose_blocking(CSOUND *csound)
 PUBLIC int csoundModuleCreate(CSOUND *csound)
 {
     /* nothing to do, report success */
-    csound->Message(csound,
-                    Str("PortAudio real-time audio module for Csound\n"));
+  //csound->Message(csound,
+  // Str("PortAudio real-time audio module for Csound\n"));
     return 0;
 }
 
@@ -763,7 +837,8 @@ PUBLIC int csoundModuleInit(CSOUND *csound)
 {
     char    *s, drv[12];
     int     i;
-
+    csound->module_list_add(csound, "pa_bl", "audio");
+    csound->module_list_add(csound, "pa_cb", "audio");
     if ((s = (char*) csound->QueryGlobalVariable(csound, "_RTAUDIO")) == NULL)
       return 0;
     for (i = 0; s[i] != '\0' && i < 11; i++)
@@ -772,7 +847,7 @@ PUBLIC int csoundModuleInit(CSOUND *csound)
     if (!(strcmp(drv, "PORTAUDIO") == 0 || strcmp(drv, "PA") == 0 ||
           strcmp(drv, "PA_BL") == 0 || strcmp(drv, "PA_CB") == 0))
       return 0;
-    csound->Message(csound, "rtaudio: PortAudio module enabled ... ");
+    csound->Message(csound, Str("rtaudio: PortAudio module enabled ... \n"));
     /* set function pointers */
 #ifdef LINUX
     if (strcmp(drv, "PA_CB") != 0)
@@ -780,21 +855,25 @@ PUBLIC int csoundModuleInit(CSOUND *csound)
     if (strcmp(drv, "PA_BL") == 0)
 #endif
     {
-      csound->Message(csound, "using blocking interface\n");
+      csound->Message(csound, Str("using blocking interface\n"));
       csound->SetPlayopenCallback(csound, playopen_blocking);
       csound->SetRecopenCallback(csound, recopen_blocking);
       csound->SetRtplayCallback(csound, rtplay_blocking);
       csound->SetRtrecordCallback(csound, rtrecord_blocking);
       csound->SetRtcloseCallback(csound, rtclose_blocking);
+      csound->SetAudioDeviceListCallback(csound, listDevices);
     }
     else {
-      csound->Message(csound, "using callback interface\n");
+      csound->Message(csound, Str("using callback interface\n"));
       csound->SetPlayopenCallback(csound, playopen_);
       csound->SetRecopenCallback(csound, recopen_);
       csound->SetRtplayCallback(csound, rtplay_);
       csound->SetRtrecordCallback(csound, rtrecord_);
       csound->SetRtcloseCallback(csound, rtclose_);
+      csound->SetAudioDeviceListCallback(csound, listDevices);
     }
+
+    csound->module_list_add(csound, s, "audio");
     return 0;
 }
 
@@ -811,4 +890,3 @@ PUBLIC int csoundModuleInfo(void)
 {
     return ((CS_APIVERSION << 16) + (CS_APISUBVER << 8) + (int) sizeof(MYFLT));
 }
-
