@@ -89,7 +89,7 @@ int assign(CSOUND *csound, ASSIGN *p)
     return OK;
 }
 
-int aassign(CSOUND *csound, ASSIGN *p)
+int aassign(CSOUND *csound, ASSIGN *p, int islocal)
 {
     uint32_t nsmps = CS_KSMPS;
     if (LIKELY(nsmps!=1)) {
@@ -98,10 +98,10 @@ int aassign(CSOUND *csound, ASSIGN *p)
       uint32_t nsmps = CS_KSMPS;
       /* the orchestra parser converts '=' to 'upsamp' if input arg is k-rate, */
       /* and skips the opcode if outarg == inarg */
-      if (UNLIKELY(offset)) memset(p->r, '\0', offset*sizeof(MYFLT));
+      if (UNLIKELY(islocal &&offset)) memset(p->r, '\0', offset*sizeof(MYFLT));
       if (UNLIKELY(early)) {
         nsmps -= early;
-        memset(&p->r[nsmps], '\0', early*sizeof(MYFLT));
+        if (islocal) memset(&p->r[nsmps], '\0', early*sizeof(MYFLT));
       }
       memcpy(&p->r[offset], &p->a[offset], (nsmps-offset) * sizeof(MYFLT));
     }
@@ -110,6 +110,12 @@ int aassign(CSOUND *csound, ASSIGN *p)
     return OK;
 }
 
+int gaassign(CSOUND *csound, ASSIGN *p)
+{  return aassign(csound, p, 0); }
+
+int laassign(CSOUND *csound, ASSIGN *p)
+{  return aassign(csound, p, 1); }
+
 int ainit(CSOUND *csound, ASSIGN *p)
 {
     uint32_t offset = p->h.insdshead->ksmps_offset;
@@ -1056,7 +1062,8 @@ int cpspch(CSOUND *csound, EVAL *p)
 int cpsmidinn(CSOUND *csound, EVAL *p)
 {
     IGN(csound);
-    *p->r = pow(FL(2.0), (*p->a - FL(69.0)) / FL(12.0)) * FL(440.0);
+    *p->r = pow(FL(2.0),
+                (*p->a - FL(69.0)) / FL(12.0)) * (MYFLT)(csound->A4);
     return OK;
 }
 
