@@ -26,10 +26,13 @@
  *  Conversion by Trevor Wishart and Keith Henderson, York Univ.
  */
 /*
-static char *rcsid = "$Id: mxfft.c,v 1.14 2005/08/13 14:44:37 istvanv Exp $";
+static char *rcsid = "$Id$";
  */
 /*
- *      $Log: mxfft.c,v $
+ *      $Log$
+ *      Revision 1.15  2008-12-04 14:55:04  jpff
+ *      Branch prediction
+ *
  *      Revision 1.14  2005/08/13 14:44:37  istvanv
  *      Minor code changes
  *
@@ -55,7 +58,7 @@ static char *rcsid = "$Id: mxfft.c,v 1.14 2005/08/13 14:44:37 istvanv Exp $";
  *      Warnings
  *
  *      Revision 1.6  2005/02/18 16:21:14  istvanv
- *      added csound pointer to mmalloc, auxalloc, and other functions
+ *      added csound pointer to csound->Malloc, auxalloc, and other functions
  *
  *      Revision 1.5  2005/01/27 19:22:50  istvanv
  *      Merged changes from 4.24.1, including new localization system,
@@ -82,6 +85,7 @@ static char *rcsid = "$Id: mxfft.c,v 1.14 2005/08/13 14:44:37 istvanv Exp $";
  */
 #include "csoundCore.h"
 #include <math.h>
+#include <assert.h>
 
 static void fft_(CSOUND *,MYFLT *, MYFLT *, int, int, int, int);
 static void fftmx(MYFLT *, MYFLT *, int, int, int, int, int,
@@ -106,17 +110,17 @@ static void fft_(CSOUND *csound, MYFLT *a, MYFLT *b,
   /*    *a,       pointer to array 'anal'  */
   /*    *b;       pointer to array 'banal' */
 {
-    int nfac[16];               /*  These are one bigger than needed   */
+    int32 nfac[32];             /*  These are one bigger than needed   */
                                 /*  because wish to use Fortran array  */
                                 /* index which runs 1 to n, not 0 to n */
 
-    int         m = 0,
-                nf,
+    int32       m = 0,
                 k,
                 kt,
-                ntot,
-                j,
                 jj,
+                j,
+                nf,
+                ntot,
                 maxf, maxp=-1;
 
     /* work space pointers */
@@ -131,14 +135,15 @@ static void fft_(CSOUND *csound, MYFLT *a, MYFLT *b,
     /*
      * determine the factors of n
      */
-    k=nf=abs(n);
+    k = nf = abs(n);
     if (nf==1)
       return;
 
-    nspn=abs(nf*nspn);
-    ntot=abs(nspn*nseg);
+    nspn = abs(nf*nspn);
+    ntot = abs(nspn*nseg);
 
     for (m=0; !(k%16); nfac[++m]=4,k/=16);
+    assert(m<16);
     for (j=3,jj=9; jj<=k; j+=2,jj=j*j)
       for (; !(k%jj); nfac[++m]=j,k/=jj);
 
@@ -164,9 +169,10 @@ static void fft_(CSOUND *csound, MYFLT *a, MYFLT *b,
     }
     if (m <= kt+1)
       maxp = m + kt + 1;
-    if (m+kt > 15) {
-      csound->Die(csound, Str("\nerror - fft parameter n has "
+    if (UNLIKELY(m+kt > 15)) {
+      csound->Warning(csound, Str("\nerror - fft parameter n has "
                               "more than 15 factors : %d"), n);
+      return;
     }
     if (kt!=0) {
       j = kt;
@@ -208,12 +214,12 @@ static void fftmx(MYFLT *a, MYFLT *b,
                   int *np, int nfac[])
 {
     int i,inc,
-      j,jc,jf, jj,
-      k, k1, k2, k3=0, k4,
-      kk,klim,ks,kspan, kspnn,
-      lim,
-      maxf,mm,
-      nn,nt;
+        j,jc,jf, jj,
+        k, k1, k2, k3=0, k4,
+        kk,klim,ks,kspan, kspnn,
+        lim,
+        maxf,mm,
+        nn,nt;
     double  aa, aj, ajm, ajp, ak, akm, akp,
       bb, bj, bjm, bjp, bk, bkm, bkp,
       c1, c2=0, c3=0, c72, cd,
@@ -226,7 +232,7 @@ static void fftmx(MYFLT *a, MYFLT *b,
     nt = inc*ntot;
     ks = inc*nspan;
 /******************* REPLACED MARCH 29: ***********************
-                                        rad = atan((double)1.0);
+                                        rad = atan(1.0);
 **************************************************************/
     rad = 0.785398163397448278900;
 /******************* REPLACED MARCH 29: ***********************
@@ -237,7 +243,7 @@ static void fftmx(MYFLT *a, MYFLT *b,
     c72 = 0.309016994374947451270;
     s72 = 0.951056516295153531190;
 /******************* REPLACED MARCH 29: ***********************
-                                        s120 = sqrt((double)0.75);
+                                        s120 = sqrt(0.75);
 **************************************************************/
     s120 = 0.866025403784438707600;
 
@@ -270,13 +276,13 @@ static void fftmx(MYFLT *a, MYFLT *b,
       maxf = nfac[*kt];
 
 /*
- * compute fourier transform
+ * compute Fourier transform
  */
 
  lbl40:
     dr = (8.0 * (double)jc)/((double)kspan);
 /*************************** APRIL 1991 POW & POW2 not WORKING.. REPLACE *******
-                    cd = 2.0 * (pow2 ( sin((double)0.5 * dr * rad)) );
+                    cd = 2.0 * (pow2 ( sin(0.5 * dr * rad)) );
 *******************************************************************************/
     xx =  sin(0.5 * dr * rad);
     cd = 2.0 * xx * xx;
@@ -311,7 +317,7 @@ static void fftmx(MYFLT *a, MYFLT *b,
     ak = c1 - ((cd*c1)+(sd*s1));
     s1 = ((sd*c1)-(cd*s1)) + s1;
     c1 = ak;
-lbl80:
+ lbl80:
     do {
       do {
         k2 = kk + kspan;
@@ -437,7 +443,7 @@ lbl80:
     if (kk <= jc)   goto lbl120;
     if (kspan==jc)  goto lbl350;
     goto lbl40;
-lbl180:
+ lbl180:
     akp = akm + bjm;
     akm = akm - bjm;
     bkp = bkm - ajm;
@@ -929,8 +935,11 @@ void csoundRealFFTnp2(CSOUND *csound, MYFLT *buf, int FFTsize)
       buf[FFTsize] = buf[1];
     }
     else {
-      if (FFTsize < 2 || (FFTsize & 1))
-        csoundDie(csound, Str("csoundRealFFTnp2(): invalid FFT size"));
+      if (UNLIKELY(FFTsize < 2 || (FFTsize & 1))) {
+        csound->Warning(csound,
+                        Str("csoundRealFFTnp2(): invalid FFT size, %d"), FFTsize);
+        return;
+      }
       buf[FFTsize] = buf[FFTsize + 1] = FL(0.0);
       fft_(csound, buf, &(buf[1]), 1, (FFTsize >> 1), 1, -2);
       reals_(csound, buf, &(buf[1]), (FFTsize >> 1), -2);
@@ -950,11 +959,30 @@ void csoundRealFFTnp2(CSOUND *csound, MYFLT *buf, int FFTsize)
  */
 void csoundInverseRealFFTnp2(CSOUND *csound, MYFLT *buf, int FFTsize)
 {
-    if (FFTsize < 2 || (FFTsize & 1))
-      csoundDie(csound, Str("csoundInverseRealFFTnp2(): invalid FFT size"));
+  if (UNLIKELY(FFTsize < 2 || (FFTsize & 1))){
+      csound->Warning(csound, Str("csoundInverseRealFFTnp2(): invalid FFT size"));
+      return;
+  }
     buf[1] = buf[FFTsize + 1] = FL(0.0);
     reals_(csound, buf, &(buf[1]), (FFTsize >> 1), 2);
     fft_(csound, buf, &(buf[1]), 1, (FFTsize >> 1), 1, 2);
     buf[FFTsize] = buf[FFTsize + 1] = FL(0.0);
 }
 
+void csoundInverseComplexFFTnp2(CSOUND *csound, MYFLT *buf, int FFTsize)
+{
+  if (UNLIKELY(FFTsize < 2 || (FFTsize & 1))){
+      csound->Warning(csound, Str("csoundInverseRealFFTnp2(): invalid FFT size"));
+      return;
+  }
+    fft_(csound, buf, buf, 1, FFTsize, 1, 2);
+}
+
+void csoundComplexFFTnp2(CSOUND *csound, MYFLT *buf, int FFTsize)
+{
+      if (UNLIKELY(FFTsize < 2 || (FFTsize & 1))) {
+        csound->Warning(csound, Str("csoundRealFFTnp2(): invalid FFT size"));
+        return;
+      }
+      fft_(csound, buf, buf, 1, FFTsize, 1, -2);
+}
