@@ -58,7 +58,9 @@
 #include "csoundCore.h"
 #include "pvfileio.h"
 
+#if !defined(WAVE_FORMAT_EXTENSIBLE)
 #define WAVE_FORMAT_EXTENSIBLE  (0xFFFE)
+#endif
 #ifndef WAVE_FORMAT_PCM
 #define WAVE_FORMAT_PCM         (0x0001)
 #endif
@@ -74,12 +76,12 @@ const GUID KSDATAFORMAT_SUBTYPE_PVOC = {
 typedef struct pvoc_file {
     WAVEFORMATEX fmtdata;
     PVOCDATA    pvdata;
-    long        datachunkoffset;
-    long        nFrames;        /* no of frames in file */
-    long        FramePos;       /* where we are in file */
+    int32       datachunkoffset;
+    int32       nFrames;        /* no of frames in file */
+    int32       FramePos;       /* where we are in file */
     FILE        *fp;
     void        *fd;
-    long        curpos;
+    int32       curpos;
     int         to_delete;
     int         readonly;
     char        *name;
@@ -87,49 +89,49 @@ typedef struct pvoc_file {
 } PVOCFILE;
 
 static const char *pvErrorStrings[] = {
-    "\npvsys: (no error)",                                          /*   0 */
-    "\npvsys: unknown error",                                       /*  -1 */
-    "\npvsys: already initialised",                                 /*  -2 */
-    "\npvsys: bad arguments",                                       /*  -3 */
-    "\npvsys: bad format parameter",                                /*  -4 */
-    "\npvsys: bad window type",                                     /*  -5 */
-    "\npvsys: too many files open",                                 /*  -6 */
-    "\npvsys: unable to create file",                               /*  -7 */
-    "\npvsys: Internal error: NULL data arrays",                    /*  -8 */
-    "\npvsys: unable to open file",                                 /*  -9 */
-    "\npvsys: error reading Source format data",                    /* -10 */
-    "\npvsys: not a WAVE_EX file",                                  /* -11 */
-    "\npvsys: bad size for fmt chunk",                              /* -12 */
-    "\npvsys: error reading Extended format data",                  /* -13 */
-    "\npvsys: not a PVOC-EX file",                                  /* -14 */
-    "\npvsys: error reading Extended pvoc format data",             /* -15 */
-    "\npvsys: unknown pvocex Version",                              /* -16 */
-    "\npvsys: error reading header",                                /* -17 */
-    "\npvsys: not a RIFF file",                                     /* -18 */
-    "\npvsys: file too small",                                      /* -19 */
-    "\npvsys: not a WAVE file",                                     /* -20 */
-    "\npvsys: error reading format chunk",                          /* -21 */
-    "\npvsys: PVXW chunk found before fmt chunk.",                  /* -22 */
-    "\npvsys: PVXW chunk found but custom window not specified",    /* -23 */
-    "\npvsys: error reading window data.",                          /* -24 */
-    "\npvsys: bad RIFF file",                                       /* -25 */
-    "\npvsys: bad format, data chunk before fmt chunk",             /* -26 */
-    "\npvsys: custom window chunk PVXW not found",                  /* -27 */
-    "\npvsys: error skipping unknown WAVE chunk",                   /* -28 */
-    "\npvsys: bad format in RIFF file",                             /* -29 */
-    "\npvsys: error writing header",                                /* -30 */
-    "\npvsys: error writing fmt chunk",                             /* -31 */
-    "\npvsys: error writing window data.",                          /* -32 */
-    "\npvsys: error updating data chunk",                           /* -33 */
-    "\npvsys: error updating riff chunk",                           /* -34 */
-    "\npvsys: error seeking to end of file",                        /* -35 */
-    "\npvsys: file does not exist",                                 /* -36 */
-    "\npvsys: file not open",                                       /* -37 */
-    "\npvsys: bad file descriptor",                                 /* -38 */
-    "\npvsys: error writing data",                                  /* -39 */
-    "\npvsys: error reading data",                                  /* -40 */
-    "\npvsys: error rewinding file",                                /* -41 */
-    "\npvsys: unable to close file on termination",                 /* -42 */
+    Str_noop("\npvsys: (no error)"),                                /*   0 */
+    Str_noop("\npvsys: unknown error"),                             /*  -1 */
+    Str_noop("\npvsys: already initialised"),                       /*  -2 */
+    Str_noop("\npvsys: bad arguments"),                             /*  -3 */
+    Str_noop("\npvsys: bad format parameter"),                      /*  -4 */
+    Str_noop("\npvsys: bad window type"),                           /*  -5 */
+    Str_noop("\npvsys: too many files open"),                       /*  -6 */
+    Str_noop("\npvsys: unable to create file"),                     /*  -7 */
+    Str_noop("\npvsys: Internal error: NULL data arrays"),          /*  -8 */
+    Str_noop("\npvsys: unable to open file"),                       /*  -9 */
+    Str_noop("\npvsys: error reading Source format data"),          /* -10 */
+    Str_noop("\npvsys: not a WAVE_EX file"),                        /* -11 */
+    Str_noop("\npvsys: bad size for fmt chunk"),                    /* -12 */
+    Str_noop("\npvsys: error reading Extended format data"),        /* -13 */
+    Str_noop("\npvsys: not a PVOC-EX file"),                        /* -14 */
+    Str_noop("\npvsys: error reading Extended pvoc format data"),   /* -15 */
+    Str_noop("\npvsys: unknown pvocex Version"),                    /* -16 */
+    Str_noop("\npvsys: error reading header"),                      /* -17 */
+    Str_noop("\npvsys: not a RIFF file"),                           /* -18 */
+    Str_noop("\npvsys: file too small"),                            /* -19 */
+    Str_noop("\npvsys: not a WAVE file"),                           /* -20 */
+    Str_noop("\npvsys: error reading format chunk"),                /* -21 */
+    Str_noop("\npvsys: PVXW chunk found before fmt chunk."),        /* -22 */
+    Str_noop("\npvsys: PVXW chunk found but custom window not specified"),/* -23 */
+    Str_noop("\npvsys: error reading window data."),                /* -24 */
+    Str_noop("\npvsys: bad RIFF file"),                             /* -25 */
+    Str_noop("\npvsys: bad format, data chunk before fmt chunk"),   /* -26 */
+    Str_noop("\npvsys: custom window chunk PVXW not found"),        /* -27 */
+    Str_noop("\npvsys: error skipping unknown WAVE chunk"),         /* -28 */
+    Str_noop("\npvsys: bad format in RIFF file"),                   /* -29 */
+    Str_noop("\npvsys: error writing header"),                      /* -30 */
+    Str_noop("\npvsys: error writing fmt chunk"),                   /* -31 */
+    Str_noop("\npvsys: error writing window data."),                /* -32 */
+    Str_noop("\npvsys: error updating data chunk"),                 /* -33 */
+    Str_noop("\npvsys: error updating riff chunk"),                 /* -34 */
+    Str_noop("\npvsys: error seeking to end of file"),              /* -35 */
+    Str_noop("\npvsys: file does not exist"),                       /* -36 */
+    Str_noop("\npvsys: file not open"),                             /* -37 */
+    Str_noop("\npvsys: bad file descriptor"),                       /* -38 */
+    Str_noop("\npvsys: error writing data"),                        /* -39 */
+    Str_noop("\npvsys: error reading data"),                        /* -40 */
+    Str_noop("\npvsys: error rewinding file"),                      /* -41 */
+    Str_noop("\npvsys: unable to close file on termination"),       /* -42 */
     NULL                                                            /* -43 */
 };
 
@@ -150,7 +152,7 @@ static inline int byte_order(void)
 
 static inline int pvfile_read_tag(PVOCFILE *p, char *s)
 {
-    if ((int) fread(s, 1, 4, p->fp) != 4) {
+    if (UNLIKELY((int) fread(s, 1, 4, p->fp) != 4)) {
       s[0] = '\0';
       return -1;
     }
@@ -160,18 +162,18 @@ static inline int pvfile_read_tag(PVOCFILE *p, char *s)
 
 static inline int pvfile_write_tag(PVOCFILE *p, const char *s)
 {
-    if ((int) fwrite((void*) s, 1, 4, p->fp) != 4)
+    if (UNLIKELY((int) fwrite((void*) s, 1, 4, p->fp) != 4))
       return -1;
     return 0;
 }
 
-static inline long pvfile_read_16(PVOCFILE *p, void *data, long cnt)
+static inline int32 pvfile_read_16(PVOCFILE *p, void *data, int32 cnt)
 {
-    long n = (long) fread(data, sizeof(uint16_t), (size_t) cnt, p->fp);
+    int32 n = (int32) fread(data, sizeof(uint16_t), (size_t) cnt, p->fp);
     if (byte_order()) {
-      long      i;
+      int32     i;
       uint16_t  tmp;
-      for (i = 0L; i < n; i++) {
+      for (i = 0; i < n; i++) {
         tmp = ((uint16_t*) data)[i];
         tmp = ((tmp & (uint16_t) 0xFF) << 8) | ((tmp & (uint16_t) 0xFF00) >> 8);
         ((uint16_t*) data)[i] = tmp;
@@ -180,13 +182,13 @@ static inline long pvfile_read_16(PVOCFILE *p, void *data, long cnt)
     return n;
 }
 
-static inline int pvfile_write_16(PVOCFILE *p, void *data, long cnt)
+static inline int pvfile_write_16(PVOCFILE *p, void *data, int32 cnt)
 {
-    long  n;
+    int32  n;
 
     if (byte_order()) {
       uint16_t  tmp;
-      for (n = 0L; n < cnt; n++) {
+      for (n = 0; n < cnt; n++) {
         tmp = ((uint16_t*) data)[n];
         tmp = ((tmp & (uint16_t) 0xFF) << 8) | ((tmp & (uint16_t) 0xFF00) >> 8);
         if (fwrite(&tmp, sizeof(uint16_t), 1, p->fp) != (size_t) 1)
@@ -198,13 +200,13 @@ static inline int pvfile_write_16(PVOCFILE *p, void *data, long cnt)
     return (n != cnt);
 }
 
-static inline long pvfile_read_32(PVOCFILE *p, void *data, long cnt)
+static inline int32 pvfile_read_32(PVOCFILE *p, void *data, int32 cnt)
 {
-    long  n = (long) fread(data, sizeof(uint32_t), (size_t) cnt, p->fp);
+    int32  n = (int32) fread(data, sizeof(uint32_t), (size_t) cnt, p->fp);
     if (byte_order()) {
-      long      i;
+      int32     i;
       uint32_t  tmp;
-      for (i = 0L; i < n; i++) {
+      for (i = 0; i < n; i++) {
         tmp = ((uint32_t*) data)[i];
         tmp =   ((tmp & (uint32_t) 0x000000FFU) << 24)
               | ((tmp & (uint32_t) 0x0000FF00U) << 8)
@@ -216,13 +218,13 @@ static inline long pvfile_read_32(PVOCFILE *p, void *data, long cnt)
     return n;
 }
 
-static inline int pvfile_write_32(PVOCFILE *p, void *data, long cnt)
+static inline int pvfile_write_32(PVOCFILE *p, void *data, int32 cnt)
 {
-    long  n;
+    int32  n;
 
     if (byte_order()) {
       uint32_t  tmp;
-      for (n = 0L; n < cnt; n++) {
+      for (n = 0; n < cnt; n++) {
         tmp = ((uint32_t*) data)[n];
         tmp =   ((tmp & (uint32_t) 0x000000FFU) << 24)
               | ((tmp & (uint32_t) 0x0000FF00U) << 8)
@@ -284,19 +286,19 @@ static int write_fmt(PVOCFILE *p)
 
 static int pvoc_writeWindow(PVOCFILE *p, float *window, uint32_t length)
 {
-    return (pvfile_write_32(p, window, (long) length));
+    return (pvfile_write_32(p, window, (int32) length));
 }
 
 static int pvoc_readWindow(PVOCFILE *p, float *window, uint32_t length)
 {
-    return (pvfile_read_32(p, window, (long) length) != (long) length);
+    return (pvfile_read_32(p, window, (int32) length) != (int32) length);
 }
 
 const char *pvoc_errorstr(CSOUND *csound)
 {
     int i = -(csound->pvErrorCode);
 
-    if (i < 0 || i > 42) i = 1;
+    if (UNLIKELY(i < 0 || i > 42)) i = 1;
     return (const char *) Str(pvErrorStrings[i]);
 }
 
@@ -311,7 +313,7 @@ const char *pvoc_errorstr(CSOUND *csound)
 
 int init_pvsys(CSOUND *csound)
 {
-    if (csound->pvNumFiles) {
+    if (UNLIKELY(csound->pvNumFiles)) {
       csound->pvErrorCode = -2;
       return 0;
     }
@@ -321,7 +323,7 @@ int init_pvsys(CSOUND *csound)
 
 static inline PVOCFILE *pvsys_getFileHandle(CSOUND *csound, int fd)
 {
-    if (fd < 0 || fd >= csound->pvNumFiles)
+    if (UNLIKELY(fd < 0 || fd >= csound->pvNumFiles))
       return (PVOCFILE*) NULL;
     return (PVFILETABLE[fd]);
 }
@@ -360,8 +362,8 @@ static int pvsys_createFileHandle(CSOUND *csound)
     return i;
 }
 
-static void prepare_pvfmt(WAVEFORMATEX *pfmt, unsigned long chans,
-                          unsigned long srate, pv_stype stype)
+static void prepare_pvfmt(WAVEFORMATEX *pfmt, uint32 chans,
+                          uint32 srate, pv_stype stype)
 {
     pfmt->wFormatTag       = WAVE_FORMAT_EXTENSIBLE;
     pfmt->nChannels        = (uint16_t) chans;
@@ -401,13 +403,13 @@ static void prepare_pvfmt(WAVEFORMATEX *pfmt, unsigned long chans,
 /* probably it should... */
 
 int  pvoc_createfile(CSOUND *csound, const char *filename,
-                     unsigned long fftlen, unsigned long overlap,
-                     unsigned long chans, unsigned long format,
-                     long srate, int stype, int wtype,
-                     float wparam, float *fWindow, unsigned long dwWinlen)
+                     uint32 fftlen, uint32 overlap,
+                     uint32 chans, uint32 format,
+                     int32 srate, int stype, int wtype,
+                     float wparam, float *fWindow, uint32 dwWinlen)
 {
     int       fd;
-    long      N, D;
+    int32     N, D;
     char      *pname;
     PVOCFILE  *p = NULL;
     float     winparam = 0.0f;
@@ -416,15 +418,15 @@ int  pvoc_createfile(CSOUND *csound, const char *filename,
     D = overlap;
     csound->pvErrorCode = -1;
 
-    if (N == 0 || (int) chans <= 0 || filename == NULL || D > N) {
+    if (UNLIKELY(N == 0 || (int) chans <= 0 || filename == NULL || D > N)) {
       csound->pvErrorCode = -3;
       return -1;
     }
-    if (format < PVOC_AMP_FREQ || format > PVOC_COMPLEX) {
+    if (UNLIKELY(/*format < PVOC_AMP_FREQ ||*/ format > PVOC_COMPLEX)) {
       csound->pvErrorCode = -4;
       return -1;
     }
-    if (!(wtype >= PVOC_DEFAULT && wtype <= PVOC_CUSTOM)) {
+    if (UNLIKELY(!(wtype >= PVOC_DEFAULT && wtype <= PVOC_CUSTOM))) {
       csound->pvErrorCode = -5;
       return -1;
     }
@@ -434,21 +436,21 @@ int  pvoc_createfile(CSOUND *csound, const char *filename,
 
     }
 
-    if (wtype == PVOC_DEFAULT)
+    else if (wtype == PVOC_DEFAULT)
       wtype = PVOC_HAMMING;
 
-    if (wtype == PVOC_KAISER)
+    else if (wtype == PVOC_KAISER)
       if (wparam != 0.0f)
         winparam = wparam;
     /* will need an internal default for window parameters... */
 
     fd = pvsys_createFileHandle(csound);
-    if (fd < 0) {
+    if (UNLIKELY(fd < 0)) {
       csound->pvErrorCode = -6;
       return -1;
     }
     p = pvsys_getFileHandle(csound, fd);
-    pname = (char *) mmalloc(csound, strlen(filename) + 1);
+    pname = (char *) csound->Malloc(csound, strlen(filename) + 1);
     strcpy(pname, filename);
     p->customWindow = NULL;
 
@@ -474,16 +476,16 @@ int  pvoc_createfile(CSOUND *csound, const char *filename,
     p->pvdata.fAnalysisRate   = (float) srate / (float) p->pvdata.dwOverlap;
     p->pvdata.fWindowParam    = winparam;
     if (fWindow != NULL) {
-      p->customWindow = mmalloc(csound, dwWinlen * sizeof(float));
+      p->customWindow = csound->Malloc(csound, dwWinlen * sizeof(float));
       memcpy(p->customWindow, fWindow, dwWinlen * sizeof(float));
     }
 
     p->fd = csound->FileOpen2(csound, &(p->fp), CSFILE_STD, filename, "wb",
                                "", CSFTYPE_PVCEX, 0);
-    if (p->fd == NULL) {
-      mfree(csound, pname);
+    if (UNLIKELY(p->fd == NULL)) {
+      csound->Free(csound, pname);
       if (p->customWindow)
-        mfree(csound, p->customWindow);
+        csound->Free(csound, p->customWindow);
       free(p);
       PVFILETABLE[fd] = NULL;
       csound->pvErrorCode = -7;
@@ -493,10 +495,10 @@ int  pvoc_createfile(CSOUND *csound, const char *filename,
 
     if (pvoc_writeheader(csound, p) != 0) {
       csound->FileClose(csound, p->fd);
-      remove(p->name);
-      mfree(csound, p->name);
+      (void)remove(p->name);
+      csound->Free(csound, p->name);
       if (p->customWindow)
-        mfree(csound, p->customWindow);
+        csound->Free(csound, p->customWindow);
       free(p);
       PVFILETABLE[fd] = NULL;
       return -1;
@@ -507,20 +509,22 @@ int  pvoc_createfile(CSOUND *csound, const char *filename,
 }
 
 int pvoc_openfile(CSOUND *csound,
-                  const char *filename, PVOCDATA *data, WAVEFORMATEX *fmt)
+                  const char *filename, void *data_, void *fmt_)
 {
     WAVEFORMATPVOCEX  wfpx;
     char              *pname;
     PVOCFILE          *p = NULL;
     int               fd;
+    PVOCDATA         *data = (PVOCDATA *) data_;
+    WAVEFORMATEX      *fmt = (WAVEFORMATEX *) fmt_;
 
     csound->pvErrorCode = -1;
-    if (data == NULL || fmt == NULL) {
+    if (UNLIKELY(data == NULL || fmt == NULL)) {
       csound->pvErrorCode = -8;
       return -1;
     }
     fd = pvsys_createFileHandle(csound);
-    if (fd < 0) {
+    if (UNLIKELY(fd < 0)) {
       csound->pvErrorCode = -6;
       return -1;
     }
@@ -529,22 +533,22 @@ int pvoc_openfile(CSOUND *csound,
     p->customWindow = NULL;
     p->fd = csound->FileOpen2(csound, &(p->fp), CSFILE_STD, filename,
                                    "rb", "SADIR", CSFTYPE_PVCEX, 0);
-    if (p->fd == NULL) {
+    if (UNLIKELY(p->fd == NULL)) {
       csound->pvErrorCode = -9;
       free(p);
       PVFILETABLE[fd] = NULL;
       return -1;
     }
-    pname = (char*) mmalloc(csound, strlen(filename) + 1);
+    pname = (char*) csound->Malloc(csound, strlen(filename) + 1);
     strcpy(pname, filename);
     p->name = pname;
     p->readonly = 1;
 
-    if (pvoc_readheader(csound, p, &wfpx) != 0) {
+    if (UNLIKELY(pvoc_readheader(csound, p, &wfpx) != 0)) {
       csound->FileClose(csound, p->fd);
-      mfree(csound, p->name);
+      csound->Free(csound, p->name);
       if (p->customWindow)
-        mfree(csound, p->customWindow);
+        csound->Free(csound, p->customWindow);
       free(p);
       PVFILETABLE[fd] = NULL;
       return -1;
@@ -570,16 +574,16 @@ static int pvoc_readfmt(CSOUND *csound, PVOCFILE *p, WAVEFORMATPVOCEX *pWfpx)
     err |= (pvfile_read_16(p, &(fmt->nBlockAlign), 1L) != 1L);
     err |= (pvfile_read_16(p, &(fmt->wBitsPerSample), 1L) != 1L);
     err |= (pvfile_read_16(p, &(fmt->cbSize), 1L) != 1L);
-    if (err) {
+    if (UNLIKELY(err)) {
       csound->pvErrorCode = -10;
       return err;
     }
     /* the first clues this is pvx format...*/
-    if (fmt->wFormatTag != WAVE_FORMAT_EXTENSIBLE) {
+    if (UNLIKELY(fmt->wFormatTag != WAVE_FORMAT_EXTENSIBLE)) {
       csound->pvErrorCode = -11;
       return -1;
     }
-    if (fmt->cbSize != 62) {
+    if (UNLIKELY(fmt->cbSize != 62)) {
       csound->pvErrorCode = -12;
       return -1;
     }
@@ -589,13 +593,13 @@ static int pvoc_readfmt(CSOUND *csound, PVOCFILE *p, WAVEFORMATPVOCEX *pWfpx)
     err |= (pvfile_read_16(p, &(wxfmt->SubFormat.Data2), 1L) != 1L);
     err |= (pvfile_read_16(p, &(wxfmt->SubFormat.Data3), 1L) != 1L);
     err |= ((int) fread(&(wxfmt->SubFormat.Data4[0]), 1, 8, p->fp) != 8);
-    if (err) {
+    if (UNLIKELY(err)) {
       csound->pvErrorCode = -13;
       return -1;
     }
     /* ... but this is the clincher */
-    if (!compare_guids(&(pWfpx->wxFormat.SubFormat),
-                       &KSDATAFORMAT_SUBTYPE_PVOC)) {
+    if (UNLIKELY(!compare_guids(&(pWfpx->wxFormat.SubFormat),
+                                &KSDATAFORMAT_SUBTYPE_PVOC))) {
       csound->pvErrorCode = -14;
       return -1;
     }
@@ -611,15 +615,16 @@ static int pvoc_readfmt(CSOUND *csound, PVOCFILE *p, WAVEFORMATPVOCEX *pWfpx)
     err |= (pvfile_read_32(p, &(pWfpx->data.dwFrameAlign), 1L) != 1L);
     err |= (pvfile_read_32(p, &(pWfpx->data.fAnalysisRate), 1L) != 1L);
     err |= (pvfile_read_32(p, &(pWfpx->data.fWindowParam), 1L) != 1L);
-    if (err) {
+    if (UNLIKELY(err)) {
       csound->pvErrorCode = -15;
       return -1;
     }
-    if (pWfpx->dwVersion != PVX_VERSION) {
+    if (UNLIKELY(pWfpx->dwVersion != PVX_VERSION)) {
       csound->pvErrorCode = -16;
       return -1;
     }
 
+
     return 0;
 }
 
@@ -631,37 +636,37 @@ static int pvoc_readheader(CSOUND *csound, PVOCFILE *p,
     uint32_t  riffsize;
     int       fmtseen = 0, windowseen = 0;
 
-    if (pvfile_read_tag(p, &(tag[0])) != 0 ||
+    if (UNLIKELY(pvfile_read_tag(p, &(tag[0])) != 0 ||
         strcmp(tag, "RIFF") != 0 ||
-        pvfile_read_32(p, &size, 1L) != 1L) {
+                 pvfile_read_32(p, &size, 1L) != 1L)) {
       csound->pvErrorCode = -17;
       return -1;
     }
-    if (size < 24 * sizeof(uint32_t) + SIZEOF_FMTPVOCEX) {
+    if (UNLIKELY(size < 24 * sizeof(uint32_t) + SIZEOF_FMTPVOCEX)) {
       csound->pvErrorCode = -19;
       return -1;
     }
     riffsize = size;
-    if (pvfile_read_tag(p, &(tag[0])) != 0 || strcmp(tag, "WAVE") != 0) {
+    if (UNLIKELY(pvfile_read_tag(p, &(tag[0])) != 0 || strcmp(tag, "WAVE") != 0)) {
       csound->pvErrorCode = -20;
       return -1;
     }
     riffsize -= sizeof(uint32_t);
     /* loop for chunks */
     while (riffsize > (uint32_t) 0) {
-      if (pvfile_read_tag(p, &(tag[0])) != 0 ||
-          pvfile_read_32(p, &size, 1L) != 1L) {
+      if (UNLIKELY(pvfile_read_tag(p, &(tag[0])) != 0 ||
+                   pvfile_read_32(p, &size, 1L) != 1L)) {
         csound->pvErrorCode = -17;
         return -1;
       }
       riffsize -= 2 * sizeof(uint32_t);
       if (strcmp(tag, "fmt ") == 0) {
         /* bail out if not a pvoc file: not trying to read all WAVE formats!*/
-        if ((int) size < (int) SIZEOF_FMTPVOCEX) {
+        if (UNLIKELY((int) size < (int) SIZEOF_FMTPVOCEX)) {
           csound->pvErrorCode = -14;
           return -1;
         }
-        if (pvoc_readfmt(csound, p, pWfpx) != 0) {
+        if (UNLIKELY(pvoc_readfmt(csound, p, pWfpx) != 0)) {
           csound->pvErrorCode = -21;
           return -1;
         }
@@ -671,38 +676,39 @@ static int pvoc_readheader(CSOUND *csound, PVOCFILE *p,
         memcpy(&(p->pvdata), &(pWfpx->data), sizeof(PVOCDATA));
       }
       else if (strcmp(tag, "PVXW") == 0) {
-        if (!fmtseen) {
+        if (UNLIKELY(!fmtseen)) {
           csound->pvErrorCode = -22;
           return -1;
         }
-        if (p->pvdata.wWindowType != PVOC_CUSTOM) {
+        if (UNLIKELY(p->pvdata.wWindowType != PVOC_CUSTOM)) {
           /* whaddayado? can you warn the user and continue? */
           csound->pvErrorCode = -23;
           return -1;
         }
-        p->customWindow = mmalloc(csound, p->pvdata.dwWinlen * sizeof(float));
-        if (pvoc_readWindow(p, p->customWindow, p->pvdata.dwWinlen) != 0) {
+        p->customWindow = csound->Malloc(csound, p->pvdata.dwWinlen * sizeof(float));
+        if (UNLIKELY(pvoc_readWindow(p,
+                                     p->customWindow, p->pvdata.dwWinlen) != 0)) {
           csound->pvErrorCode = -24;
           return -1;
         }
         windowseen = 1;
       }
       else if (strcmp(tag, "data") == 0) {
-        if ((uint32_t) riffsize != size) {
+        if (UNLIKELY((uint32_t) riffsize != size)) {
           csound->pvErrorCode = -25;
           return -1;
         }
-        if (!fmtseen) {
+        if (UNLIKELY(!fmtseen)) {
           csound->pvErrorCode = -26;
           return -1;
         }
         if (p->pvdata.wWindowType == PVOC_CUSTOM) {
-          if (!windowseen) {
+          if (UNLIKELY(!windowseen)) {
             csound->pvErrorCode = -27;
             return -1;
           }
         }
-        p->datachunkoffset = (long) ftell(p->fp);
+        p->datachunkoffset = (int32) ftell(p->fp);
         p->curpos = p->datachunkoffset;
         /* not m/c frames, for now */
         p->nFrames = size / p->pvdata.dwFrameAlign;
@@ -711,7 +717,7 @@ static int pvoc_readheader(CSOUND *csound, PVOCFILE *p,
       else {
         /* skip any unknown chunks */
         riffsize -= 2 * sizeof(uint32_t);
-        if (fseek(p->fp, (long) size, SEEK_CUR) != 0) {
+        if (UNLIKELY(fseek(p->fp, (int32) size, SEEK_CUR) != 0)) {
           csound->pvErrorCode = -28;
           return -1;
         }
@@ -730,7 +736,7 @@ static int pvoc_writeheader(CSOUND *csound, PVOCFILE *p)
 
     err |= pvfile_write_tag(p, "RIFF");
     err |= pvfile_write_32(p, &size, 1L);
-    if (err) {
+    if (UNLIKELY(err)) {
       csound->pvErrorCode = -30;
       return -1;
     }
@@ -742,67 +748,67 @@ static int pvoc_writeheader(CSOUND *csound, PVOCFILE *p)
     err |= pvfile_write_tag(p, "WAVE");
     err |= pvfile_write_tag(p, "fmt ");
     err |= pvfile_write_32(p, &size, 1L);
-    if (err) {
+    if (UNLIKELY(err)) {
       csound->pvErrorCode = -30;
       return -1;
     }
-    if (write_fmt(p) != 0) {
+    if (UNLIKELY(write_fmt(p) != 0)) {
       csound->pvErrorCode = -31;
       return -1;
     }
-    if (pvfile_write_16(p, &(p->fmtdata.wBitsPerSample), 1L) != 0) {
+    if (UNLIKELY(pvfile_write_16(p, &(p->fmtdata.wBitsPerSample), 1L) != 0)) {
       csound->pvErrorCode = -31;
       return -1;
     }
     /* we will take this from a WAVE_EX file, in due course */
     size = 0;   /* dwChannelMask */
-    if (pvfile_write_32(p, &size, 1L) != 0) {
+    if (UNLIKELY(pvfile_write_32(p, &size, 1L) != 0)) {
       csound->pvErrorCode = -31;
       return -1;
     }
-    if (write_guid(p, &KSDATAFORMAT_SUBTYPE_PVOC) != 0) {
+    if (UNLIKELY(write_guid(p, &KSDATAFORMAT_SUBTYPE_PVOC) != 0)) {
       csound->pvErrorCode = -31;
       return -1;
     }
     version = (uint32_t) 1;
     size = sizeof(PVOCDATA);
-    if (pvfile_write_32(p, &version, 1L) != 0 ||
-        pvfile_write_32(p, &size, 1L) != 0) {
+    if (UNLIKELY(pvfile_write_32(p, &version, 1L) != 0 ||
+                 pvfile_write_32(p, &size, 1L) != 0)) {
       csound->pvErrorCode = -31;
       return -1;
     }
-    if (write_pvocdata(p) != 0) {
+    if (UNLIKELY(write_pvocdata(p) != 0)) {
       csound->pvErrorCode = -31;
       return -1;
     }
     /* VERY experimental; may not even be a good idea...*/
     if (p->customWindow) {
-      if (pvfile_write_tag(p, "PVXW") != 0) {
+      if (UNLIKELY(pvfile_write_tag(p, "PVXW") != 0)) {
         csound->pvErrorCode = -30;
         return -1;
       }
       size = p->pvdata.dwWinlen * sizeof(float);
-      if (pvfile_write_32(p, &size, 1L) != 0) {
+      if (UNLIKELY(pvfile_write_32(p, &size, 1L) != 0)) {
         csound->pvErrorCode = -30;
         return -1;
       }
-      if (pvoc_writeWindow(p, p->customWindow, p->pvdata.dwWinlen) != 0) {
+      if (UNLIKELY(pvoc_writeWindow(p, p->customWindow, p->pvdata.dwWinlen) != 0)) {
         csound->pvErrorCode = -32;
         return -1;
       }
     }
     /* no other chunks to write yet! */
-    if (pvfile_write_tag(p, "data") != 0) {
+    if (UNLIKELY(pvfile_write_tag(p, "data") != 0)) {
       csound->pvErrorCode = -30;
       return -1;
     }
     /* we need to update size later on... */
     size = (uint32_t) 0;
-    if (pvfile_write_32(p, &size, 1L) != 0) {
+    if (UNLIKELY(pvfile_write_32(p, &size, 1L) != 0)) {
       csound->pvErrorCode = -30;
       return -1;
     }
-    p->datachunkoffset = (long) ftell(p->fp);
+    p->datachunkoffset = (int32) ftell(p->fp);
     p->curpos = p->datachunkoffset;
 
     return 0;
@@ -813,30 +819,31 @@ static int pvoc_updateheader(CSOUND *csound, int ofd)
     PVOCFILE  *p = pvsys_getFileHandle(csound, ofd);
     uint32_t  riffsize, datasize;
 
-    if (p == NULL) {
+    if (UNLIKELY(p == NULL)) {
       csound->pvErrorCode = -38;
       return 0;
     }
-    if (fseek(p->fp, (long) (p->datachunkoffset - sizeof(uint32_t)), SEEK_SET)
-        != 0) {
+    if (UNLIKELY(fseek(p->fp,
+                       (int32) (p->datachunkoffset - sizeof(uint32_t)), SEEK_SET)
+                 != 0)) {
       csound->pvErrorCode = -33;
       return 0;
     }
     datasize = p->curpos - p->datachunkoffset;
-    if (pvfile_write_32(p, &datasize, 1L) != 0) {
+    if (UNLIKELY(pvfile_write_32(p, &datasize, 1L) != 0)) {
       csound->pvErrorCode = -33;
       return 0;
     }
-    if (fseek(p->fp, (long) sizeof(uint32_t), SEEK_SET) != 0) {
+    if (UNLIKELY(fseek(p->fp, (int32) sizeof(uint32_t), SEEK_SET) != 0)) {
       csound->pvErrorCode = -33;
       return 0;
     }
     riffsize = p->curpos - 2 * sizeof(uint32_t);
-    if (pvfile_write_32(p, &riffsize, 1L) != 0) {
+    if (UNLIKELY(pvfile_write_32(p, &riffsize, 1L) != 0)) {
       csound->pvErrorCode = -34;
       return 0;
     }
-    if (fseek(p->fp, 0L, SEEK_END) != 0) {
+    if (UNLIKELY(fseek(p->fp, 0L, SEEK_END) != 0)) {
       csound->pvErrorCode = -35;
       return 0;
     }
@@ -850,11 +857,11 @@ int pvoc_closefile(CSOUND *csound, int ofd)
     int       rc = 1;
 
     csound->pvErrorCode = 0;
-    if (p == NULL) {
+    if (UNLIKELY(p == NULL)) {
       csound->pvErrorCode = -36;
       return 0;
     }
-    if (p->fd == NULL) {
+    if (UNLIKELY(p->fd == NULL)) {
       csound->pvErrorCode = -37;
       free(p);
       PVFILETABLE[ofd] = NULL;
@@ -866,9 +873,9 @@ int pvoc_closefile(CSOUND *csound, int ofd)
 
     csound->FileClose(csound, p->fd);
     if (p->to_delete && !p->readonly)
-      remove(p->name);
-    mfree(csound, p->name);
-    mfree(csound, p->customWindow);
+      (void)remove(p->name);
+    csound->Free(csound, p->name);
+    csound->Free(csound, p->customWindow);
     free(p);
     PVFILETABLE[ofd] = NULL;
 
@@ -891,22 +898,22 @@ int pvoc_closefile(CSOUND *csound, int ofd)
  *
  * return 0 for error, 1 for success. This could change....
  */
-int pvoc_putframes(CSOUND *csound, int ofd, const float *frame, long numframes)
+int pvoc_putframes(CSOUND *csound, int ofd, const float *frame, int32 numframes)
 {
     PVOCFILE  *p = pvsys_getFileHandle(csound, ofd);
-    long      towrite;  /* count in 'words' */
+    int32     towrite;  /* count in 'words' */
 
-    if (p == NULL) {
+    if (UNLIKELY(p == NULL)) {
       csound->pvErrorCode = -38;
       return 0;
     }
-    if (p->fd == NULL) {
+    if (UNLIKELY(p->fd == NULL)) {
       csound->pvErrorCode = -37;
       return 0;
     }
     /* NB doubles not supported yet */
-    towrite = (long) p->pvdata.nAnalysisBins * 2L * numframes;
-    if (pvfile_write_32(p, (void*) frame, towrite) != 0) {
+    towrite = (int32) p->pvdata.nAnalysisBins * 2L * numframes;
+    if (UNLIKELY(pvfile_write_32(p, (void*) frame, towrite) != 0)) {
       csound->pvErrorCode = -39;
       return 0;
     }
@@ -920,33 +927,33 @@ int pvoc_putframes(CSOUND *csound, int ofd, const float *frame, long numframes)
  * return -1 for error, 0 for EOF, else numframes read
  */
 int pvoc_getframes(CSOUND *csound, int ifd, float *frames,
-                                    unsigned long nframes)
+                                    uint32 nframes)
 {
     PVOCFILE  *p = pvsys_getFileHandle(csound, ifd);
-    long      toread, got;
+    int32     toread, got;
 
-    if (p == NULL) {
+    if (UNLIKELY(p == NULL)) {
       csound->pvErrorCode = -38;
       return -1;
     }
-    if (p->fd == NULL) {
+    if (UNLIKELY(p->fd == NULL)) {
       csound->pvErrorCode = -37;
       return -1;
     }
-    toread = (long) p->pvdata.nAnalysisBins * 2L * (long) nframes;
+    toread = (int32) p->pvdata.nAnalysisBins * 2L * (int32) nframes;
     got = pvfile_read_32(p, frames, toread);
     if (got != toread) {
-      if (ferror(p->fp)) {
+      if (UNLIKELY(ferror(p->fp))) {
         csound->pvErrorCode = -40;
         return -1;
       }
-      p->curpos += (long) (got * sizeof(float));
-      got = got / (long) (p->pvdata.nAnalysisBins * 2);
+      p->curpos += (int32) (got * sizeof(float));
+      got = got / (int32) (p->pvdata.nAnalysisBins * 2);
       p->FramePos += got;
       return (int) got;
     }
     p->curpos += (toread * sizeof(float));
-    p->FramePos += (long) nframes;
+    p->FramePos += (int32) nframes;
 
     return (int) nframes;
 }
@@ -954,23 +961,23 @@ int pvoc_getframes(CSOUND *csound, int ifd, float *frames,
 int pvoc_fseek(CSOUND *csound, int ifd, int offset)
 {
     PVOCFILE  *p = pvsys_getFileHandle(csound, ifd);
-    long      pos, skipframes, skipsize;
+    int32     pos, skipframes, skipsize;
 
-    if (p == NULL) {
+    if (UNLIKELY(p == NULL)) {
       csound->pvErrorCode = -38;
       return -1;
     }
-    if (p->fd == NULL) {
+    if (UNLIKELY(p->fd == NULL)) {
       csound->pvErrorCode = -37;
       return -1;
     }
-    if(offset == 1)
-    skipframes = (long) p->fmtdata.nChannels;
+    if (offset == 1)
+    skipframes = (int32) p->fmtdata.nChannels;
     else skipframes = offset;
     skipsize = p->pvdata.dwFrameAlign * skipframes;
 
     pos = p->datachunkoffset + skipsize;
-    if (fseek(p->fp, (long) pos, SEEK_SET) != 0) {
+    if (UNLIKELY(fseek(p->fp, (int32) pos, SEEK_SET) != 0)) {
       csound->pvErrorCode = -41;
       return -1;
     }
@@ -989,7 +996,7 @@ int pvsys_release(CSOUND *csound)
     csound->pvErrorCode = 0;
     for (i = 0; i < csound->pvNumFiles; i++) {
       if (pvsys_getFileHandle(csound, i) != NULL) {
-        if (!pvoc_closefile(csound, i)) {
+        if (UNLIKELY(!pvoc_closefile(csound, i))) {
           csound->pvErrorCode = -42;
         }
       }
@@ -1007,10 +1014,9 @@ int pvsys_release(CSOUND *csound)
 int pvoc_framecount(CSOUND *csound, int ifd)
 {
     PVOCFILE  *p = pvsys_getFileHandle(csound, ifd);
-    if (p == NULL) {
+    if (UNLIKELY(p == NULL)) {
       csound->pvErrorCode = -38;
       return -1;
     }
     return p->nFrames;
 }
-
