@@ -29,7 +29,6 @@
 #include <math.h>
 #include "csoundCore.h"
 #include "pstream.h"
-#include "oload.h"
 
         double  besseli(double x);
 static  void    hamming(MYFLT *win, int winLen, int even);
@@ -70,12 +69,12 @@ static CS_NOINLINE int PVS_CreateWindow(CSOUND *csound, MYFLT *buf,
         }
         return OK;
       default:
-        if (type >= 0)
+        if (UNLIKELY(type >= 0))
           return csound->InitError(csound, Str("invalid window type"));
     }
     /* use table created with GEN20 */
-    flen = csound->GetTable(csound, &ftable, -(type));
-    if (flen < 0)
+    flen = csoundGetTable(csound, &ftable, -(type));
+    if (UNLIKELY(flen < 0))
       return csound->InitError(csound, Str("ftable for window not found"));
     inc = (double)flen / (double)(winLen & (~1));
     fpos = ((double)flen + (double)even * inc) * 0.5;
@@ -94,7 +93,6 @@ static CS_NOINLINE int PVS_CreateWindow(CSOUND *csound, MYFLT *buf,
     return OK;
 }
 
-#ifdef SDFT
 
 int pvssanalset(CSOUND *csound, PVSANAL *p)
 {
@@ -111,9 +109,9 @@ int pvssanalset(CSOUND *csound, PVSANAL *p)
 
     /* Need space for NB complex numbers for each of ksmps */
     if (p->fsig->frame.auxp==NULL ||
-        csound->ksmps*(N+2)*sizeof(MYFLT) > (unsigned int)p->fsig->frame.size)
-      csound->AuxAlloc(csound, csound->ksmps*(N+2)*sizeof(MYFLT),&p->fsig->frame);
-    else memset(p->fsig->frame.auxp, 0, csound->ksmps*(N+2)*sizeof(MYFLT));
+        CS_KSMPS*(N+2)*sizeof(MYFLT) > (unsigned int)p->fsig->frame.size)
+      csound->AuxAlloc(csound, CS_KSMPS*(N+2)*sizeof(MYFLT),&p->fsig->frame);
+    else memset(p->fsig->frame.auxp, 0, CS_KSMPS*(N+2)*sizeof(MYFLT));
     /* Space for remembering samples */
     if (p->input.auxp==NULL ||
         N*sizeof(MYFLT) > (unsigned int)p->input.size)
@@ -141,7 +139,7 @@ int pvssanalset(CSOUND *csound, PVSANAL *p)
       double *s = c+NB;
       p->cosine = c;
       p->sine = s;
-      c[0] = 1.0; s[0] = 0.0;
+      c[0] = 1.0; s[0] = 0.0; // assignment to s unnecessary as auxalloc zeros
         /*
           direct computation of c and s may be better for large n
           c[i] = cos(2*M_PI*i/n);
@@ -159,37 +157,40 @@ int pvssanalset(CSOUND *csound, PVSANAL *p)
     }
     return OK;
 }
-#endif
 
 int pvsanalset(CSOUND *csound, PVSANAL *p)
 {
     MYFLT *analwinhalf,*analwinbase;
     MYFLT sum;
-    long halfwinsize,buflen;
-    int i,nBins,Mf,Lf;
+    int32 halfwinsize,buflen;
+    int i,nBins,Mf/*,Lf*/;
 
     /* opcode params */
-    long N =(long) *(p->fftsize);
-    long overlap = (long) *(p->overlap);
-    long M = (long) *(p->winsize);
+    uint32_t N =(int32) *(p->fftsize);
+    uint32_t overlap = (uint32_t) *(p->overlap);
+    uint32_t M = (uint32_t) *(p->winsize);
     int wintype = (int) *p->wintype;
     /* deal with iinit and iformat later on! */
 
-#ifdef SDFT
-    if (overlap<csound->ksmps || overlap<=10) /* 10 is a guess.... */
+    if (overlap<CS_KSMPS || overlap<=10) /* 10 is a guess.... */
       return pvssanalset(csound, p);
-#endif
-    if (N <= 32)
-      csound->Die(csound, Str("pvsanal: fftsize of 32 is too small!\n"));
+    if (UNLIKELY(N <= 32))
+      return csound->InitError(csound,
+                               Str("pvsanal: fftsize of 32 is too small!\n"));
     /* check N for powof2? CARL fft routines and FFTW are not limited to that */
     N = N  + N%2;       /* Make N even */
-    if (M < N)
-      csound->Die(csound, Str("pvsanal: window size too small for fftsize\n"));
-    if (overlap > N / 2)
-      csound->Die(csound, Str("pvsanal: overlap too big for fft size\n"));
-#ifndef SDFT
-    if (overlap < csound->ksmps)
-      csound->Die(csound, Str("pvsanal: overlap must be >= ksmps\n"));
+    if (UNLIKELY(M < N)) {
+       csound->Warning(csound,
+                               Str("pvsanal: window size too small for fftsize"));
+       M = N;
+    }
+    if (UNLIKELY(overlap > N / 2))
+      return csound->InitError(csound,
+                               Str("pvsanal: overlap too big for fft size\n"));
+#ifdef OLPC
+    if (UNLIKELY(overlap < CS_KSMPS))
+      return csound->InitError(csound,
+                               Str("pvsanal: overlap must be >= ksmps\n"));
 #endif
     halfwinsize = M/2;
     buflen = M*4;
@@ -200,7 +201,7 @@ int pvsanalset(CSOUND *csound, PVSANAL *p)
     /* we can exclude/simplify all sorts of stuff in CARL
      * as we will never do time-scaling with this opcode
      */
-    Lf = Mf = 1 - M%2;
+    /*Lf =*/ Mf = 1 - M%2;
 
     csound->AuxAlloc(csound, overlap * sizeof(MYFLT), &p->overlapbuf);
     csound->AuxAlloc(csound, (N+2) * sizeof(MYFLT), &p->analbuf);
@@ -214,7 +215,7 @@ int pvsanalset(CSOUND *csound, PVSANAL *p)
     analwinbase = (MYFLT *) (p->analwinbuf.auxp);
     analwinhalf = analwinbase + halfwinsize;
 
-    if (PVS_CreateWindow(csound, analwinhalf, wintype, M) != OK)
+    if (UNLIKELY(PVS_CreateWindow(csound, analwinhalf, wintype, M) != OK))
       return NOTOK;
 
     for (i = 1; i <= halfwinsize; i++)
@@ -232,12 +233,14 @@ int pvsanalset(CSOUND *csound, PVSANAL *p)
     }
     /* get net amp */
     sum = FL(0.0);
+
     for (i = -halfwinsize; i <= halfwinsize; i++)
       sum += *(analwinhalf + i);
-    sum = FL(2.0) / sum;  /*factor of 2 comes in later in trig identity*/
+    sum = FL(2.0) / sum;  /* factor of 2 comes in later in trig identity */
     for (i = -halfwinsize; i <= halfwinsize; i++)
       *(analwinhalf + i) *= sum;
 
+
   /*    p->invR = (float)(FL(1.0) / csound->esr); */
     p->RoverTwoPi = (float)(p->arate / TWOPI_F);
     p->TwoPioverR = (float)(TWOPI_F / p->arate);
@@ -245,7 +248,7 @@ int pvsanalset(CSOUND *csound, PVSANAL *p)
     p->nI = -(halfwinsize / overlap) * overlap; /* input time (in samples) */
     /*Dd = halfwinsize + p->nI + 1;                     */
     /* in streaming mode, Dd = ovelap all the time */
-    p->Ii = 0; 
+    p->Ii = 0;
     p->IOi = 0;
     p->buflen = buflen;
     p->nextIn = (MYFLT *) p->input.auxp;
@@ -257,22 +260,23 @@ int pvsanalset(CSOUND *csound, PVSANAL *p)
     p->fsig->wintype = wintype;
     p->fsig->framecount = 1;
     p->fsig->format = PVS_AMP_FREQ;      /* only this, for now */
-#ifdef SDFT
     p->fsig->sliding = 0;
-#endif
+
+    if (!(N & (N - 1))) /* if pow of two use this */
+     p->setup = csound->RealFFT2Setup(csound,N,FFT_FWD);
     return OK;
 }
 
 static void generate_frame(CSOUND *csound, PVSANAL *p)
 {
-    int got, tocp,i,j,k;
+  int got, tocp,i,j,k,ii;
     int N = p->fsig->N;
     int N2 = N/2;
-    long buflen = p->buflen;
-    long analWinLen = p->fsig->winsize/2;
-    long synWinLen = analWinLen;
+    int32 buflen = p->buflen;
+    int32 analWinLen = p->fsig->winsize/2;
+    int32 synWinLen = analWinLen;
     float *ofp;                 /* RWD MUST be 32bit */
-    MYFLT *fp,*oi,*i0,*i1;
+    MYFLT *fp;
     MYFLT *anal = (MYFLT *) (p->analbuf.auxp);
     MYFLT *input = (MYFLT *) (p->input.auxp);
     MYFLT *analWindow = (MYFLT *) (p->analwinbuf.auxp) + analWinLen;
@@ -307,8 +311,9 @@ static void generate_frame(CSOUND *csound, PVSANAL *p)
        channels.   The subroutines fft and reals together implement
        one efficient FFT call for a real input sequence.  */
 
-    for (i = 0; i < N+2; i++)
-      *(anal + i) = FL(0.0);       /*initialize*/
+    /* for (i = 0; i < N+2; i++)
+     *(anal + i) = FL(0.0);  */     /*initialize*/
+    memset(anal, 0, sizeof(MYFLT)*(N+2));
 
     j = (p->nI - analWinLen - 1 + buflen) % buflen;     /*input pntr*/
 
@@ -317,14 +322,16 @@ static void generate_frame(CSOUND *csound, PVSANAL *p)
       k += N;
     k = k % N;
     for (i = -analWinLen; i <= analWinLen; i++) {
-      if (++j >= buflen)
+      if (UNLIKELY(++j >= buflen))
         j -= buflen;
-      if (++k >= N)
+      if (UNLIKELY(++k >= N))
         k -= N;
-      *(anal + k) += *(analWindow + i) * *(input + j);
+      /* *(anal + k) += *(analWindow + i) * *(input + j); */
+      anal[k] += analWindow[i] * input[j];
     }
     if (!(N & (N - 1))) {
-      csound->RealFFT(csound, anal, N);
+      /* csound->RealFFT(csound, anal, N);*/
+      csound->RealFFT2(csound,p->setup,anal);
       anal[N] = anal[1];
       anal[1] = anal[N + 1] = FL(0.0);
     }
@@ -343,21 +350,21 @@ static void generate_frame(CSOUND *csound, PVSANAL *p)
            i++,i0+=2,i1+=2, oi++) {
         real = *i0;
         imag = *i1;
-        *i0 =(MYFLT) hypot((double)real, *(double)imag);
+        *i0 = HYPOT(real, imag);
         /* phase unwrapping */
         /*if (*i0 == 0.)*/
-        if (*i0 < FL(1.0E-10))
+        if (UNLIKELY(*i0 < FL(1.0E-10)))
           /* angleDif = 0.0f; */
           phase = FL(0.0);
 
         else {
 
-          rratio = atan2((double)imag,(double)real);
+          phase = ATAN2(imag,real);
 
           /*angleDif  = (phase = (float)rratio) - *oi;
            *oi = phase;
+           phase = (MYFLT)rratio;
            */
-          phase = (MYFLT)rratio;
         }
 
         *i1 = phase;
@@ -365,18 +372,20 @@ static void generate_frame(CSOUND *csound, PVSANAL *p)
     }
 #endif
     /*if (format==PVS_AMP_FREQ) {*/
-    for (i=0,i0=anal,i1=anal+1,oi=oldInPhase; i <= N2; i++,i0+=2,i1+=2, oi++) {
-      real = *i0;
-      imag = *i1;
-      *i0 = (MYFLT)hypot((double)real, (double)imag);
+    for (i=ii=0    /*,i0=anal,i1=anal+1,oi=oldInPhase*/;
+         i <= N2;
+         i++,ii+=2 /*i0+=2,i1+=2, oi++*/) {
+      real = anal[ii] /* *i0 */;
+      imag = anal[ii+1] /* *i1 */;
+      /**i0*/ anal[ii] = HYPOT(real, imag);
       /* phase unwrapping */
       /*if (*i0 == 0.)*/
-      if (*i0 < FL(1.0E-10))
+      if (UNLIKELY(/* *i0 */ anal[ii] < FL(1.0E-10)))
         angleDif = FL(0.0);
       else {
         rratio =  atan2((double)imag,(double)real);
-        angleDif  = (phase = (MYFLT)rratio) - *oi;
-        *oi = phase;
+        angleDif  = (phase = (MYFLT)rratio) - /**oi*/ oldInPhase[i];
+        /* *oi */ oldInPhase[i] = phase;
       }
 
       if (angleDif > PI_F)
@@ -385,14 +394,16 @@ static void generate_frame(CSOUND *csound, PVSANAL *p)
         angleDif = angleDif + TWOPI_F;
 
       /* add in filter center freq.*/
-      *i1 = angleDif * p->RoverTwoPi + ((MYFLT) i * p->Fexact);
+      /* *i1 */ anal[ii+1]  = angleDif * p->RoverTwoPi + ((MYFLT) i * p->Fexact);
+
     }
     /* } */
     /* else must be PVOC_COMPLEX */
     fp = anal;
     ofp = (float *) (p->fsig->frame.auxp);      /* RWD MUST be 32bit */
     for (i=0;i < N+2;i++)
-      *ofp++ = (float)(*fp++);
+      /* *ofp++ = (float)(*fp++); */
+      ofp[i] = (float) fp[i];
 
     p->nI += p->fsig->overlap;                          /* increment time */
     if (p->nI > (synWinLen + p->fsig->overlap))
@@ -422,41 +433,46 @@ static void anal_tick(CSOUND *csound, PVSANAL *p,MYFLT samp)
       p->inptr = 0;
 
     }
+    //printf("inptr = %d fsig->overlap=%d\n", p->inptr, p->fsig->overlap);
     inbuf[p->inptr++] = samp;
 
 }
 
-#ifdef SDFT
 static inline double mod2Pi(double x)
 {
     x = fmod(x,TWOPI);
     if (x <= -PI) {
         return x + TWOPI;
     }
-    if (x > PI) {
+    else if (x > PI) {
         return x - TWOPI;
     }
-    return x;
+    else
+      return x;
 }
 
 int pvssanal(CSOUND *csound, PVSANAL *p)
 {
     MYFLT *ain;
-    int NB = p->Ii, i, loc;
+    int NB = p->Ii, loc;
     int N = p->fsig->N;
     MYFLT *data = (MYFLT*)(p->input.auxp);
     CMPLX *fw = (CMPLX*)(p->analwinbuf.auxp);
     double *c = p->cosine;
     double *s = p->sine;
     double *h = (double*)p->oldInPhase.auxp;
-    int nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t i, nsmps = CS_KSMPS;
     int wintype = p->fsig->wintype;
-    if (data==NULL) {
-      csound->Die(csound, Str("pvsanal: Not Initialised.\n"));
+    if (UNLIKELY(data==NULL)) {
+      return csound->PerfError(csound,p->h.insdshead,
+                               Str("pvsanal: Not Initialised.\n"));
     }
     ain = p->ain;               /* The input samples */
     loc = p->inptr;             /* Circular buffer */
-    for (i=0; i < nsmps; i++) {
+    nsmps -= early;
+    for (i=offset; i < nsmps; i++) {
       MYFLT re, im, dx;
       CMPLX* ff;
       int j;
@@ -474,20 +490,20 @@ int pvssanal(CSOUND *csound, PVSANAL *p)
         fw[j].re = ci*re - si*im;
         fw[j].im = ci*im + si*re;
       }
-      loc++; if (loc==p->nI) loc = 0; /* Circular buffer */
+      loc++; if (UNLIKELY(loc==p->nI)) loc = 0; /* Circular buffer */
       /* apply window and transfer to ff buffer*/
       /* Rectang :Fw_t =     F_t                          */
       /* Hamming :Fw_t = 0.54F_t - 0.23[ F_{t-1}+F_{t+1}] */
       /* Hamming :Fw_t = 0.5 F_t - 0.25[ F_{t-1}+F_{t+1}] */
-      /* Blackman:Fw_t = 0.42F_t - 0.25[ F_{t-1}+F_{t+1}] + 0.04 [F_{t-2}+F_{t+2}] */
-      /* Blackman_exact:Fw_t = 0.42659071367153912296F_t 
-         - 0.24828030954428202923 [F_{t-1}+F_{t+1}] 
+      /* Blackman:Fw_t = 0.42F_t - 0.25[ F_{t-1}+F_{t+1}]+0.04[F_{t-2}+F_{t+2}] */
+      /* Blackman_exact:Fw_t = 0.42659071367153912296F_t
+         - 0.24828030954428202923 [F_{t-1}+F_{t+1}]
          + 0.038424333619948409286 [F_{t-2}+F_{t+2}]      */
-      /* Nuttall_C3:Fw_t = 0.375  F_t - 0.25[ F_{t-1}+F_{t+1}] + 
+      /* Nuttall_C3:Fw_t = 0.375  F_t - 0.25[ F_{t-1}+F_{t+1}] +
                                       0.0625 [F_{t-2}+F_{t+2}] */
-      /* BHarris_3:Fw_t = 0.44959 F_t - 0.24682[ F_{t-1}+F_{t+1}] + 
+      /* BHarris_3:Fw_t = 0.44959 F_t - 0.24682[ F_{t-1}+F_{t+1}] +
                                       0.02838 [F_{t-2}+F_{t+2}] */
-      /* BHarris_min:Fw_t = 0.42323 F_t - 0.2486703 [ F_{t-1}+F_{t+1}] + 
+      /* BHarris_min:Fw_t = 0.42323 F_t - 0.2486703 [ F_{t-1}+F_{t+1}] +
                                       0.0391396 [F_{t-2}+F_{t+2}] */
       switch (wintype) {
       case PVS_WIN_HAMMING:
@@ -515,13 +531,14 @@ int pvssanal(CSOUND *csound, PVSANAL *p)
         ff[NB-1].re -= FL(0.5)*fw[NB-2].re;
         break;
       default:
-        csound->Message(csound,
+        csound->Warning(csound,
                         Str("Unknown window type; replaced by rectangular\n"));
       case PVS_WIN_RECT:
-        for (j=0; j<NB; j++) {
-          ff[j].re = fw[j].re;
-          ff[j].im = fw[j].im;
-        }
+        memcpy(ff, fw, NB*sizeof(CMPLX));
+        /* for (j=0; j<NB; j++) { */
+        /*   ff[j].re = fw[j].re; */
+        /*   ff[j].im = fw[j].im; */
+        /* } */
         break;
       case PVS_WIN_BLACKMAN:
         for (j=0; j<NB; j++) {
@@ -651,26 +668,27 @@ int pvssanal(CSOUND *csound, PVSANAL *p)
     p->inptr = loc;
     return OK;
 }
-#endif
 
 int pvsanal(CSOUND *csound, PVSANAL *p)
 {
     MYFLT *ain;
-    int i;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t i, nsmps = CS_KSMPS;
 
     ain = p->ain;
 
-    if (p->input.auxp==NULL) {
-      csound->Die(csound, Str("pvsanal: Not Initialised.\n"));
+    if (UNLIKELY(p->input.auxp==NULL)) {
+      return csound->PerfError(csound,p->h.insdshead,
+                               Str("pvsanal: Not Initialised.\n"));
     }
-#ifdef SDFT
     {
       int overlap = (int)*p->overlap;
-      if (overlap<csound->ksmps || overlap<10) /* 10 is a guess.... */
+      if (overlap<(int)nsmps || overlap<10) /* 10 is a guess.... */
         return pvssanal(csound, p);
     }
-#endif
-    for (i=0; i < csound->ksmps; i++)
+    nsmps -= early;
+    for (i=offset; i < nsmps; i++)
       anal_tick(csound,p,ain[i]);
     return OK;
 }
@@ -680,15 +698,15 @@ int pvsynthset(CSOUND *csound, PVSYNTH *p)
     MYFLT *analwinhalf;
     MYFLT *synwinhalf;
     MYFLT sum;
-    long halfwinsize,buflen;
+    int32 halfwinsize,buflen;
     int i,nBins,Mf,Lf;
     double IO;
 
     /* get params from input fsig */
     /* we TRUST they are legal */
-    long N = p->fsig->N;
-    long overlap = p->fsig->overlap;
-    long M = p->fsig->winsize;
+    int32 N = p->fsig->N;
+    int32 overlap = p->fsig->overlap;
+    int32 M = p->fsig->winsize;
     int wintype = p->fsig->wintype;
 
     p->fftsize = N;
@@ -696,7 +714,6 @@ int pvsynthset(CSOUND *csound, PVSYNTH *p)
     p->overlap = overlap;
     p->wintype = wintype;
     p->format = p->fsig->format;
-#ifdef SDFT
     if (p->fsig->sliding) {
       /* get params from input fsig */
       /* we TRUST they are legal */
@@ -708,7 +725,6 @@ int pvsynthset(CSOUND *csound, PVSYNTH *p)
       csound->AuxAlloc(csound, p->fsig->NB * sizeof(double), &p->output);
       return OK;
     }
-#endif
     /* and put into locals */
     halfwinsize = M/2;
     buflen = M*4;
@@ -726,20 +742,47 @@ int pvsynthset(CSOUND *csound, PVSYNTH *p)
     csound->AuxAlloc(csound, nBins * sizeof(MYFLT), &p->oldOutPhase);
     csound->AuxAlloc(csound, buflen * sizeof(MYFLT), &p->output);
 
-    /* have to make analysis window to get amp scaling */
-    /* so this ~could~ be a local alloc and free...*/
 
-    analwinhalf = (MYFLT *) (p->analwinbuf.auxp) + halfwinsize;
+
+
     synwinhalf = (MYFLT *) (p->synwinbuf.auxp) + halfwinsize;
 
-    if (PVS_CreateWindow(csound, analwinhalf, wintype, M) != OK)
+
+
+    /* synthesis windows */
+    if (M <= N) {
+      if (UNLIKELY(PVS_CreateWindow(csound, synwinhalf, wintype, M) != OK))
+        return NOTOK;
+
+      for (i = 1; i <= halfwinsize; i++)
+        *(synwinhalf - i) = *(synwinhalf + i - Lf);
+
+
+       sum = FL(0.0);
+        for (i = -halfwinsize; i <= halfwinsize; i++)
+           sum += *(synwinhalf + i);
+         sum = FL(2.0) / sum;
+
+       for (i = -halfwinsize; i <= halfwinsize; i++)
+        *(synwinhalf + i) *= sum;
+
+         sum = FL(0.0);
+   /* no timescaling, so I(nterpolation) will always = D(ecimation) = overlap */
+        for (i = -halfwinsize; i <= halfwinsize; i+=overlap)
+          sum += *(synwinhalf + i) * *(synwinhalf + i);
+    }
+    else {
+     /* have to make analysis window to get amp scaling */
+    /* so this ~could~ be a local alloc and free...*/
+      double dN = (double)N;
+      analwinhalf = (MYFLT *) (p->analwinbuf.auxp) + halfwinsize;
+    if (UNLIKELY(PVS_CreateWindow(csound, analwinhalf, wintype, M) != OK))
       return NOTOK;
 
     for (i = 1; i <= halfwinsize; i++)
-      *(analwinhalf - i) = *(analwinhalf + i - Mf);
-    if (M > N) {
-      double dN = (double)N;
-      /* sinc function */
+      analwinhalf[-i] = analwinhalf[i - Mf];
+
+      // sinc function
       if (Mf)
         *analwinhalf *= (MYFLT)(dN * sin(PI*0.5/dN) / ( PI*0.5));
       for (i = 1; i <= halfwinsize; i++)
@@ -747,33 +790,14 @@ int pvsynthset(CSOUND *csound, PVSYNTH *p)
           (dN * sin((double)(PI*(i+0.5*Mf)/dN)) / (PI*(i+0.5*Mf)));
       for (i = 1; i <= halfwinsize; i++)
         *(analwinhalf - i) = *(analwinhalf + i - Mf);
-    }
-    /* get net amp */
+
+     /* get net amp */
     sum = FL(0.0);
     for (i = -halfwinsize; i <= halfwinsize; i++)
-      sum += *(analwinhalf + i);
+    sum += *(analwinhalf + i);
     sum = FL(2.0) / sum;  /* factor of 2 comes in later in trig identity */
-    for (i = -halfwinsize; i <= halfwinsize; i++)
-      *(analwinhalf + i) *= sum;
 
-    /* synthesis windows */
-    if (M <= N) {
-      if (PVS_CreateWindow(csound, synwinhalf, wintype, M) != OK)
-        return NOTOK;
-
-      for (i = 1; i <= halfwinsize; i++)
-        *(synwinhalf - i) = *(synwinhalf + i - Lf);
-
-      for (i = -halfwinsize; i <= halfwinsize; i++)
-        *(synwinhalf + i) *= sum;
-
-      sum = FL(0.0);
-   /* no timescaling, so I(nterpolation) will always = D(ecimation) = overlap */
-      for (i = -halfwinsize; i <= halfwinsize; i+=overlap)
-        sum += *(synwinhalf + i) * *(synwinhalf + i);
-    }
-    else {
-      if (PVS_CreateWindow(csound, synwinhalf, wintype, M) != OK)
+      if (UNLIKELY(PVS_CreateWindow(csound, synwinhalf, wintype, M) != OK))
         return NOTOK;
 
       for (i = 1; i <= halfwinsize; i++)
@@ -788,11 +812,14 @@ int pvsynthset(CSOUND *csound, PVSYNTH *p)
       for (i = 1; i <= halfwinsize; i++)
         *(synwinhalf - i) = *(synwinhalf + i - Lf);
     }
-    if (!(N & (N - 1L)))
-      sum = csound->GetInverseRealFFTScale(csound, (int) N) / sum;
+
+
+   if (!(N & (N - 1L)))
+     sum = csound->GetInverseRealFFTScale(csound, (int) N)/ sum;
     else
       sum = FL(1.0) / sum;
-    for (i = -halfwinsize; i <= halfwinsize; i++)
+
+  for (i = -halfwinsize; i <= halfwinsize; i++)
       *(synwinhalf + i) *= sum;
 
 /*  p->invR = FL(1.0) / csound->esr; */
@@ -805,6 +832,9 @@ int pvsynthset(CSOUND *csound, PVSYNTH *p)
     p->outptr = 0;
     p->nextOut = (MYFLT *) (p->output.auxp);
     p->buflen = buflen;
+
+    if (!(N & (N - 1))) /* if pow of two use this */
+      p->setup = csound->RealFFT2Setup(csound,N,FFT_INV);
     return OK;
 }
 
@@ -821,16 +851,16 @@ static MYFLT synth_tick(CSOUND *csound, PVSYNTH *p)
 
 static void process_frame(CSOUND *csound, PVSYNTH *p)
 {
-    int n,i,j,k,NO,NO2;
+    int i,j,k,ii,NO,NO2;
     float *anal;                                        /* RWD MUST be 32bit */
-    MYFLT *syn,*bsyn,*i0,*i1,*output;
+    MYFLT *syn, *output;
     MYFLT *oldOutPhase = (MYFLT *) (p->oldOutPhase.auxp);
-    long N = p->fsig->N;
+    int32 N = p->fsig->N;
     MYFLT *obufptr,*outbuf,*synWindow;
     MYFLT mag,phase,angledif, the_phase;
-    long synWinLen = p->fsig->winsize / 2;
-    long overlap = p->fsig->overlap;
-    /*long format = p->fsig->format; */
+    int32 synWinLen = p->fsig->winsize / 2;
+    int32 overlap = p->fsig->overlap;
+    /*int32 format = p->fsig->format; */
 
     /* fsigs MUST be corect format, as we offer no mechanism for
        assignment to a different one*/
@@ -843,48 +873,48 @@ static void process_frame(CSOUND *csound, PVSYNTH *p)
     outbuf = (MYFLT *) (p->overlapbuf.auxp);
     synWindow = (MYFLT *) (p->synwinbuf.auxp) + synWinLen;
 
-    bsyn = syn+1;
     /* reconversion: The magnitude and angle-difference-per-second in syn
        (assuming an intermediate sampling rate of rOut) are
        converted to real and imaginary values and are returned in syn.
        This automatically incorporates the proper phase scaling for
        time modifications. */
 
-    if (NO <= N) {
+    if (LIKELY(NO <= N)) {
       for (i = 0; i < NO+2; i++)
-        *(syn+i) = (MYFLT) *(anal+i);
+        syn[i] = (MYFLT) anal[i];
     }
     else {
       for (i = 0; i <= N+1; i++)
-        *(syn+i) = (MYFLT) *(anal+i);
+        syn[i] = (MYFLT) anal[i];
       for (i = N+2; i < NO+2; i++)
-        *(syn+i) = FL(0.0);
+        syn[i] = FL(0.0);
     }
 #ifdef NOTDEF
     if (format==PVS_AMP_PHASE) {
-      for (i=0, i0=syn, i1=syn+1; i<= NO2; i++, i0+=2,  i1+=2) {
-        mag = *i0;
-        phase = *i1;
-        *i0 = (MYFLT)((double)mag * cos((double)phase));
-        *i1 = (MYFLT)((double)mag * sin((double)phase));
+      for (ii=0 /*, i0=syn, i1=syn+1*/; ii<= NO2; ii+=2 /* i++, i0+=2,  i1+=2*/) {
+        mag = syn[ii];    /* *i0; */
+        phase = syn[ii+1]; /* *i1; */
+        /* *i0 */ syn[ii] = (MYFLT)((double)mag * cos((double)phase));
+        /* *i1 */ syn[ii+1] = (MYFLT)((double)mag * sin((double)phase));
       }
     }
     else if (format == PVS_AMP_FREQ) {
 #endif
-      for (i=0, i0=syn, i1=syn+1; i<= NO2; i++, i0+=2,  i1+=2) {
-        mag = *i0;
+      for (i=ii=0 /*, i0=syn, i1=syn+1*/; i<= NO2; i++, ii+=2 /*i0+=2,  i1+=2*/) {
+        mag = syn[ii]; /* *i0; */
         /* RWD variation to keep phase wrapped within +- TWOPI */
         /* this is spread across several frame cycles, as the problem does not
            develop for a while */
 
-        angledif = p->TwoPioverR * (*i1  - ((MYFLT) i * p->Fexact));
-        the_phase = *(oldOutPhase + i) +angledif;
+        angledif = p->TwoPioverR * ( /* *i1 */ syn[ii+1] - ((MYFLT)i * p->Fexact));
+        the_phase = /* *(oldOutPhase + i) */ oldOutPhase[i] + angledif;
         if (i== p->bin_index)
           the_phase = (MYFLT) fmod(the_phase,TWOPI);
-        *(oldOutPhase + i) = the_phase;
+        /* *(oldOutPhase + i) = the_phase; */
+        oldOutPhase[i] = the_phase;
         phase = the_phase;
-        *i0 = (MYFLT)((double)mag * cos((double)phase));
-        *i1 = (MYFLT)((double)mag * sin((double)phase));
+        /* *i0 */ syn[ii]  = (MYFLT)((double)mag * cos((double)phase));
+        /* *i1 */ syn[ii+1] = (MYFLT)((double)mag * sin((double)phase));
       }
 #ifdef NOTDEF
     }
@@ -906,8 +936,10 @@ static void process_frame(CSOUND *csound, PVSYNTH *p)
        out (to standard output). The subroutines reals and fft
        together perform an efficient inverse FFT.  */
     if (!(NO & (NO - 1))) {
+      /*printf("N %d %d \n", NO, NO & (NO-1));*/
       syn[1] = syn[NO];
-      csound->InverseRealFFT(csound, syn, NO);
+      /* csound->InverseRealFFT(csound, syn, NO);*/
+      csound->RealFFT2(csound,p->setup,syn);
       syn[NO] = syn[NO + 1] = FL(0.0);
     }
     else
@@ -927,24 +959,29 @@ static void process_frame(CSOUND *csound, PVSYNTH *p)
         j -= p->buflen;
       if (++k >= NO)
         k -= NO;
-      *(output + j) += *(syn + k) * *(synWindow + i);
+      /* *(output + j) += *(syn + k) * *(synWindow + i); */
+      output[j] += syn[k] * synWindow[i];
     }
 
     obufptr = outbuf;
 
     for (i = 0; i < p->IOi;) {  /* shift out next IOi values */
-      int j;
       int todo = (p->IOi-i <= output+p->buflen - p->nextOut ?
                   p->IOi-i : output+p->buflen - p->nextOut);
       /*outfloats(nextOut, todo, ofd);*/
       /*copy data to external buffer */
-      for (n=0;n < todo;n++)
-        *obufptr++ = p->nextOut[n];
+      /*for (n=0;n < todo;n++)
+       *obufptr++ = p->nextOut[n]; */
+      memcpy(obufptr, p->nextOut, sizeof(MYFLT)*todo);
+      obufptr += todo;
 
       i += todo;
 
-      for (j = 0; j < todo; j++)
-        *p->nextOut++ = FL(0.0);
+      /* for (j = 0; j < todo; j++)
+       *p->nextOut++ = FL(0.0); */
+      memset(p->nextOut, 0, sizeof(MYFLT)*todo);
+      p->nextOut += todo;
+
       if (p->nextOut >= (output + p->buflen))
         p->nextOut -= p->buflen;
     }
@@ -961,16 +998,15 @@ static void process_frame(CSOUND *csound, PVSYNTH *p)
         p->Ii = 0;
         for (i=p->nO+synWinLen; i<p->buflen; i++)
           if (i > 0)
-            *(output+i) = FL(0.0);
+            output[i] = FL(0.0);
       }
     p->IOi =  p->Ii;
 }
 
-#ifdef SDFT
 int pvssynth(CSOUND *csound, PVSYNTH *p)
 {
     int i, k;
-    int ksmps = csound->ksmps;
+    int ksmps = CS_KSMPS;
     int N = p->fsig->N;
     int NB = p->fsig->NB;
     MYFLT *aout = p->aout;
@@ -983,7 +1019,7 @@ int pvssynth(CSOUND *csound, PVSYNTH *p)
       MYFLT a;
       ff = (CMPLX*)(p->fsig->frame.auxp) + i*NB;
       for (k=0; k<NB; k++) {
-        double tmp,phase;
+        double tmp, phase;
 
         tmp = ff[k].im; /* Actually frequency */
         /* subtract bin mid frequency */
@@ -1004,20 +1040,25 @@ int pvssynth(CSOUND *csound, PVSYNTH *p)
     }
     return OK;
 }
-#endif
 
 int pvsynth(CSOUND *csound, PVSYNTH *p)
 {
-    int i;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t i, nsmps = CS_KSMPS;
     MYFLT *aout = p->aout;
 
-    if (p->output.auxp==NULL) {
-      csound->Die(csound, Str("pvsynth: Not Initialised.\n"));
+    if (UNLIKELY(p->output.auxp==NULL)) {
+      return csound->PerfError(csound,p->h.insdshead,
+                               Str("pvsynth: Not Initialised.\n"));
     }
-#ifdef SDFT
     if (p->fsig->sliding) return pvssynth(csound, p);
-#endif
-    for (i=0;i < csound->ksmps;i++)
+    if (UNLIKELY(offset)) memset(aout, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&aout[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (i=offset; i<nsmps; i++)
       aout[i] = synth_tick(csound, p);
     return OK;
 }
@@ -1031,13 +1072,13 @@ static void hamming(MYFLT *win, int winLen, int even)
 
     if (even) {
       for (i=0; i<winLen; i++)
-        *(win+i) = (MYFLT)(0.54 + 0.46*cos(ftmp*((double)i+0.5)));
-      *(win+winLen) = FL(0.0);
+        win[i] = (MYFLT)(0.54 + 0.46*cos(ftmp*((double)i+0.5)));
+      win[winLen] = FL(0.0);
     }
     else {
-      *(win) = FL(1.0);
+      win[0] = FL(1.0);
       for (i=1; i<=winLen; i++)
-        *(win+i) = (MYFLT)(0.54 + 0.46*cos(ftmp*(double)i));
+        win[i] = (MYFLT)(0.54 + 0.46*cos(ftmp*(double)i));
     }
 
 }
@@ -1091,4 +1132,3 @@ static void vonhann(MYFLT *win, int winLen, int even)
         win[i] = (MYFLT)(0.5 + 0.5 * cos(ftmp*(double)i));
     }
 }
-
