@@ -1,3 +1,4 @@
+
 /*
     remote.c:
 
@@ -24,16 +25,29 @@
 #include "csoundCore.h"
 #include "remote.h"
 
-#ifdef HAVE_SOCKETS
-#ifndef WIN32
-#include <sys/ioctl.h>
-#ifdef LINUX
-#include <linux/if.h>
-#include <arpa/inet.h>
-#endif
-#include <net/if.h>
-#endif
-#endif /* HAVE_SOCKETS */
+/* Somewhat revised from the original.  Pete G. Nov 2012
+   More correct, I think, but I could be wrong... (:-/)
+*/
+
+/* #ifdef HAVE_SOCKETS */
+/*   #ifndef WIN32 */
+/*     #include <sys/ioctl.h> */
+/*     #ifdef LINUX */
+/*       #include <linux/if.h> */
+/*     #endif */
+/*     #ifdef __HAIKU__ */
+/*       #include <sys/sockio.h> */
+/*     #endif */
+/*     #include <sys/socket.h> */
+/*     #include <netinet/in.h> */
+/*     #include <arpa/inet.h> */
+/*     extern int inet_aton (const char *, struct in_addr *); */
+/*     #include <net/if.h> */
+/*   #else */
+/*     #include <winsock2.h> */
+/*   #endif /\* not WIN32 *\/ */
+/* #endif /\* HAVE_SOCKETS *\/ */
+
 
 #define MAXREMOTES 10
 
@@ -43,47 +57,91 @@ void remote_Cleanup(CSOUND *csound);
 
 void remoteRESET(CSOUND *csound)
 {
+    /* Recover space */
+    if (csound->remoteGlobals) csound->Free(csound, csound->remoteGlobals);
     csound->remoteGlobals = NULL;
 }
 
-#ifdef HAVE_SOCKETS
-
- /* get the IPaddress of this machine */
-static int getIpAddress(char *ipaddr, char *ifname)
+#if defined(HAVE_SOCKETS)
+#if !defined(WIN32) || defined(__CYGWIN__)
+#include <netdb.h>
+#endif
+#if 0
+static int foo(char *ipaddr)
 {
-    int ret = 1;
-#ifdef WIN32
     /* VL 12/10/06: something needs to go here */
-    /* gethostbyname is the real answer; code below id unsafe */
+    /* gethostbyname is the real answer; code below is unsafe */
     char hostname[1024];
     struct hostent *he;
     struct sockaddr_in sin;
     gethostname(hostname, sizeof(hostname));
+    printf("hostname=%s\n", hostname);
     he = gethostbyname(hostname);
 
     memset(&sin, 0, sizeof (struct sockaddr_in));
     memmove(&sin.sin_addr, he->h_addr_list[0], he->h_length);
     strcpy(ipaddr, inet_ntoa (sin.sin_addr));
-    ret = 0;
+    printf("IP: %s\n", ipaddr);
+    return 0;
+}
+#endif
 
+ /* get the IPaddress of this machine */
+static int getIpAddress(char *ipaddr)
+{
+#if defined(WIN32) && !defined(__CYGWIN__)
+    /* VL 12/10/06: something needs to go here */
+    /* gethostbyname is the real answer; code below is unsafe */
+    char hostname[1024];
+    struct hostent *he;
+    struct sockaddr_in sin;
+    if (gethostname(hostname, sizeof(hostname))<0) return 1;
+    if ((he = gethostbyname(hostname))==NULL) return 1;
+
+    memset(&sin, 0, sizeof (struct sockaddr_in));
+    memmove(&sin.sin_addr, he->h_addr_list[0], he->h_length);
+    strcpy(ipaddr, inet_ntoa (sin.sin_addr));
+    return 0;
 #else
+    int ret = 1;
     struct ifreq ifr;
     int fd;
 
     fd = socket(AF_INET,SOCK_DGRAM, 0);
     if (fd >= 0) {
-      strcpy(ifr.ifr_name, ifname);
-
+      char *dev = getenv("CS_ETHER");
+      if (dev)
+        strncpy(ifr.ifr_name, dev, IFNAMSIZ-1);
+      else {
+#ifdef MACOSX
+        strncpy(ifr.ifr_name, "en0", IFNAMSIZ-1);
+#else
+        strncpy(ifr.ifr_name, "eth0", IFNAMSIZ-1);
+#endif
+      }
+      ifr.ifr_name[IFNAMSIZ-1] = '\0';
       if (ioctl(fd, SIOCGIFADDR, &ifr) == 0) {
         char *local;
         local = inet_ntoa(((struct sockaddr_in *)(&ifr.ifr_addr))->sin_addr);
         strcpy(ipaddr, local);
+        printf("IP for remote: %s: %s\n", ifr.ifr_name, ipaddr);
         ret = 0;
       }
+      else {
+        strncpy(ifr.ifr_name, "wlan0", IFNAMSIZ-1);
+        ifr.ifr_name[IFNAMSIZ-1] = '\0';
+        if (ioctl(fd, SIOCGIFADDR, &ifr) == 0) {
+          char *local;
+          local = inet_ntoa(((struct sockaddr_in *)(&ifr.ifr_addr))->sin_addr);
+          strcpy(ipaddr, local);
+          printf("IP for remote: %s: %s\n", ifr.ifr_name, ipaddr);
+          ret = 0;
+        }
+      }
     }
-    close(fd);
-#endif
+    if (fd>=0) close(fd);
     return ret;
+#endif
 }
 
 char remoteID(CSOUND *csound)
@@ -96,68 +154,68 @@ static int callox(CSOUND *csound)
 {
     if (csound->remoteGlobals == NULL) {
       csound->remoteGlobals = csound->Calloc(csound, sizeof(REMOTE_GLOBALS));
-      if (csound->remoteGlobals == NULL) {
-        csound->Message(csound, Str("insufficient memory to initialize remote"
-                        " globals."));
+      if (UNLIKELY(csound->remoteGlobals == NULL)) {
+        csound->Message(csound, Str("insufficient memory to initialise remote"
+                                    " globals."));
         goto error;
       }
       ST(remote_port) = REMOT_PORT;
     }
 
     ST(socksout) = (SOCK*)csound->Calloc(csound,(size_t)MAXREMOTES * sizeof(SOCK));
-    if (ST(socksout) == NULL) {
-      csound->Message(csound, Str("insufficient memory to initialize outgoing "
-                      "socket table."));
+    if (UNLIKELY(ST(socksout) == NULL)) {
+      csound->Message(csound, Str("insufficient memory to initialise outgoing "
+                                  "socket table."));
       goto error;
     }
 
     ST(socksin) = (int*) csound->Calloc(csound,(size_t)MAXREMOTES * sizeof(int));
-    if (ST(socksin) == NULL) {
-      csound->Message(csound, Str("insufficient memory to initialize incoming "
-                      "socket table."));
+    if (UNLIKELY(ST(socksin) == NULL)) {
+      csound->Message(csound, Str("insufficient memory to initialise incoming "
+                                  "socket table."));
       goto error;
     }
 
     ST(insrfd_list) =
       (int*) csound->Calloc(csound,(size_t)MAXREMOTES * sizeof(int));
-    if (ST(insrfd_list) == NULL) {
-      csound->Message(csound, Str("insufficient memory to initialize "
-                      "insrfd_list."));
+    if (UNLIKELY(ST(insrfd_list) == NULL)) {
+      csound->Message(csound, Str("insufficient memory to initialise "
+                                  "insrfd_list."));
       goto error;
     }
 
     ST(chnrfd_list) =
       (int*) csound->Calloc(csound,(size_t)MAXREMOTES * sizeof(int));
-    if (ST(chnrfd_list) == NULL) {
-      csound->Message(csound, Str("insufficient memory to initialize "
-                      "chnrfd_list."));
+    if (UNLIKELY(ST(chnrfd_list) == NULL)) {
+      csound->Message(csound, Str("insufficient memory to initialise "
+                                  "chnrfd_list."));
       goto error;
     }
 
     ST(insrfd) = (int*) csound->Calloc(csound,(size_t)129 * sizeof(int));
-    if (ST(insrfd) == NULL) {
-      csound->Message(csound, Str("insufficient memory to initialize "
-                      "insrfd table."));
+    if (UNLIKELY(ST(insrfd) == NULL)) {
+      csound->Message(csound, Str("insufficient memory to initialise "
+                                  "insrfd table."));
       goto error;
     }
 
     ST(chnrfd) = (int*) csound->Calloc(csound,(size_t)17 * sizeof(int));
-    if (ST(chnrfd) == NULL) {
-      csound->Message(csound, Str("insufficient memory to initialize "
-                      "chnrfd table."));
+    if (UNLIKELY(ST(chnrfd) == NULL)) {
+      csound->Message(csound, Str("insufficient memory to initialise "
+                                  "chnrfd table."));
       goto error;
     }
 
     ST(ipadrs) = (char*) csound->Calloc(csound,(size_t)15 * sizeof(char));
-    if (ST(ipadrs) == NULL) {
-      csound->Message(csound, Str("insufficient memory to initialize "
-                      "local ip address."));
+    if (UNLIKELY(ST(ipadrs) == NULL)) {
+      csound->Message(csound, Str("insufficient memory to initialise "
+                                  "local ip address."));
       goto error;
     }
 
     /* get IP adrs of this machine */
     /* FIXME: don't hardcode eth0 */
-    if (getIpAddress(ST(ipadrs), "eth0") < 0) {
+    if (UNLIKELY(getIpAddress(ST(ipadrs)) != 0)) {
       csound->Message(csound, Str("unable to get local ip address."));
       goto error;
     }
@@ -210,7 +268,6 @@ void remote_Cleanup(CSOUND *csound)
 static int CLopen(CSOUND *csound, char *ipadrs)     /* Client -- open to send */
 {
     int rfd, i;
-
     SOCK *sop = ST(socksout), *sop_end = sop + MAXREMOTES;
     do {
       if (ipadrs == sop->adr)                      /* if socket already exists */
@@ -218,25 +275,26 @@ static int CLopen(CSOUND *csound, char *ipadrs)     /* Client -- open to send */
     } while (++sop < sop_end);
 
     /* create a STREAM (TCP) socket in the INET (IP) protocol */
-    if (( rfd = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
+    if (UNLIKELY(( rfd = socket(AF_INET, SOCK_STREAM, 0)) < 0)) {
       return csound->InitError(csound, Str("could not open remote port"));
     }
     memset(&(ST(to_addr)), 0, sizeof(ST(to_addr)));    /* clear sock mem */
     ST(to_addr).sin_family = AF_INET;                  /* set as INET address */
     /* server IP adr, netwk byt order */
-#ifdef WIN32
+#if defined(WIN32) && !defined(__CYGWIN__)
     ST(to_addr).sin_addr.S_un.S_addr = inet_addr((const char *)ipadrs);
 #else
     inet_aton((const char *)ipadrs, &(ST(to_addr).sin_addr));
 #endif
-    /*    ST(to_addr).sin_port = htons((int) REMOT_PORT); */
     ST(to_addr).sin_port = htons((int) ST(remote_port)); /* port we will listen on,
                                                             network byte order */
     for (i=0; i<10; i++){
-      if (connect(rfd, (struct sockaddr *) &ST(to_addr), sizeof(ST(to_addr))) < 0)
+      if (UNLIKELY(connect(rfd, (struct sockaddr *) &ST(to_addr),
+                           sizeof(ST(to_addr))) < 0))
         csound->Message(csound, Str("---> Could not connect \n"));
       else goto conok;
     }
+    close(rfd);
     return csound->InitError(csound,
                              Str("---> Failed all attempts to connect. \n"));
 
@@ -254,67 +312,70 @@ static int CLopen(CSOUND *csound, char *ipadrs)     /* Client -- open to send */
 int CLsend(CSOUND *csound, int conn, void *data, int length)
 {
     int nbytes;
-    if ((nbytes = write(conn, data, length)) <= 0) {
-      return csound->PerfError(csound, Str("write to socket failed"));
+    if (UNLIKELY((nbytes = write(conn, data, length)) <= 0)) {
+      csound->ErrorMsg(csound, Str("write to socket failed"));
+      return NOTOK;
     }
     /*    csound->Message(csound, "nbytes sent: %d \n", nbytes); */
     return OK;
 }
 
-static int SVopen(CSOUND *csound, char *ipadrs_local)
+static int SVopen(CSOUND *csound)
            /* Server -- open to receive */
 {
     int conn, socklisten,opt;
     char ipadrs[15];
     int *sop = ST(socksin), *sop_end = sop + MAXREMOTES;
-#ifdef WIN32
+#if defined(WIN32) && !defined(__CYGWIN__)
     int clilen;
 #else
     socklen_t clilen;
 #endif
     opt = 1;
 
-    if ((socklisten = socket(PF_INET, SOCK_STREAM, 0)) < 0) {
+    if (UNLIKELY((socklisten = socket(PF_INET, SOCK_STREAM, 0)) < 0)) {
       return csound->InitError(csound, Str("creating socket\n"));
     }
     else csound->Message(csound, Str("created socket \n"));
     /* set the addresse to be reusable */
-    if ( setsockopt(socklisten, SOL_SOCKET, SO_REUSEADDR,
-#ifdef WIN32
-                    (const char *)&opt,
+    if (UNLIKELY( setsockopt(socklisten, SOL_SOCKET, SO_REUSEADDR,
+#if defined(WIN32) && !defined(__CYGWIN__)
+                             (const char *)&opt,
 #else
-                    &opt,
+                             &opt,
 #endif
-                    sizeof(opt)) < 0 )
-
+                             sizeof(opt)) < 0 ))
       return
         csound->InitError(csound,
-                          Str("setting socket option to reuse the addresse \n"));
+                          Str("setting socket option to reuse the address\n"));
 
     memset(&(ST(to_addr)), 0, sizeof(ST(to_addr)));    /* clear sock mem */
     ST(local_addr).sin_family = AF_INET;               /* set as INET address */
     /* our adrs, netwrk byt order */
-#ifdef WIN32
+#if defined(WIN32) && !defined(__CYGWIN__)
     ST(to_addr).sin_addr.S_un.S_addr = inet_addr((const char *)ipadrs);
 #else
     inet_aton((const char *)ipadrs, &(ST(local_addr).sin_addr));
 #endif
 /*     ST(local_addr).sin_port = htons((int)REMOT_PORT); */
-    ST(local_addr).sin_port = htons((int) ST(remote_port)); /* port we will listen
-                                                             on, netwrk byt order */
+    ST(local_addr).sin_port =
+      htons((int) ST(remote_port)); /* port we will listen on, netwrk byt order */
     /* associate the socket with the address and port */
-    if (bind (socklisten,
+    if (UNLIKELY(bind (socklisten,
               (struct sockaddr *) &ST(local_addr),
-              sizeof(ST(local_addr))) < 0) {
+                       sizeof(ST(local_addr))) < 0)) {
+      shutdown(socklisten, SHUT_RD);
       return csound->InitError(csound, Str("bind failed"));
     }
-    if (listen(socklisten, 5) < 0) {    /* start the socket listening
-                                           for new connections -- may wait */
+    if (UNLIKELY(listen(socklisten, 5) < 0)) {    /* start the socket listening
+                                                for new connections -- may wait */
+      shutdown(socklisten, SHUT_RD);
       return csound->InitError(csound, Str("listen failed"));
     }
     clilen = sizeof(ST(local_addr));  /* FIX THIS FOR MULTIPLE CLIENTS !!!!!!!*/
     conn = accept(socklisten, (struct sockaddr *) &ST(local_addr), &clilen);
-    if (conn < 0) {
+    if (UNLIKELY(conn < 0)) {
+      shutdown(socklisten, SHUT_RD);
       return csound->InitError(csound, Str("accept failed"));
     }
     else {
@@ -331,13 +392,15 @@ static int SVopen(CSOUND *csound, char *ipadrs_local)
 int SVrecv(CSOUND *csound, int conn, void *data, int length)
 {
     struct sockaddr from;
-#ifdef WIN32 /* VL, 12/10/06: I'm guessing here. If someone knows better, fix it */
+    /* VL, 12/10/06: I'm guessing here. If someone knows better, fix it */
+#if defined(WIN32) && !defined(__CYGWIN__)
 #define MSG_DONTWAIT  0
-   int clilen = sizeof(from);
+    int clilen = sizeof(from);
 #else
     socklen_t clilen = sizeof(from);
 #endif
     size_t n;
+    IGN(csound);
     n = recvfrom(conn, data, length, MSG_DONTWAIT, &from, &clilen);
     /*  if (n>0) csound->Message(csound, "nbytes received: %d \n", (int)n); */
     return (int)n;
@@ -348,9 +411,9 @@ int SVrecv(CSOUND *csound, int conn, void *data, int length)
 int remoteport(CSOUND *csound, REMOTEPORT *p)
 {
     if (csound->remoteGlobals==NULL) {
-      if (callox(csound) < 0) {
+      if (UNLIKELY(callox(csound) < 0)) {
         return
-          csound->InitError(csound, Str("failed to initialize remote globals."));
+          csound->InitError(csound, Str("failed to initialise remote globals."));
       }
     }
     if (ST(socksin) == NULL) {
@@ -366,40 +429,46 @@ int remoteport(CSOUND *csound, REMOTEPORT *p)
 int insremot(CSOUND *csound, INSREMOT *p)
 /* declare certain instrs for remote Csounds */
 {   /*      INSTR 0 opcode  */
-    short nargs = p->INOCOUNT;
+    int16 nargs = p->INOCOUNT;
 
     if (csound->remoteGlobals==NULL || ST(socksin) == NULL) {
-      if (callox(csound) < 0) {
+      if (UNLIKELY(callox(csound) < 0)) {
         return
-          csound->InitError(csound, Str("failed to initialize remote globals."));
+          csound->InitError(csound, Str("failed to initialise remote globals."));
       }
     }
-    if (nargs < 3) {
+    if (UNLIKELY(nargs < 3)) {
       return csound->InitError(csound, Str("missing instr nos"));
     }
 /*     csound->Message(csound, Str("*** str1: %s own:%s\n"), */
 /*                     (char *)p->str1 , ST(ipadrs)); */
-    if (strcmp(ST(ipadrs), (char *)p->str1) == 0) {  /* if client is this adrs */
+    if (strcmp(ST(ipadrs), (char *)p->str1->data) == 0) {
+      /* if client is this adrs */
       MYFLT   **argp = p->insno;
       int rfd = 0;
-      if ((rfd = CLopen(csound, (char *)p->str2)) <= 0) /* open port to remote */
+      if ((rfd = CLopen(csound, (char *)p->str2->data)) < 0)
+        /* open port to remote */
         return NOTOK;
       for (nargs -= 2; nargs--; ) {
-        short insno = (short)**argp++;     /* & for each insno */
-        if (insno <= 0) {
+        int16 insno = (int16)**argp++;     /* & for each insno */
+        if (UNLIKELY(insno <= 0)) {
+          close(rfd);
           return csound->InitError(csound, Str("illegal instr no"));
         }
-        if (ST(insrfd)[insno]) {
+        if (UNLIKELY(ST(insrfd)[insno])) {
+          close(rfd);
           return csound->InitError(csound, Str("insno already remote"));
         }
         ST(insrfd)[insno] = rfd;   /*  record file descriptor   */
       }
       ST(insrfd_list)[ST(insrfd_count)++] = rfd;   /*  and make a list    */
     }
-    else if (!strcmp(ST(ipadrs),(char *)p->str2)) { /* if server is this adrs*/
+    else if (!strcmp(ST(ipadrs),(char *)p->str2->data)) {
+      /* if server is this adrs*/
 /*       csound->Message(csound, Str("*** str2: %s own:%s\n"), */
 /*                       (char *)p->str2 , ST(ipadrs)); */
-      if (SVopen(csound, (char *)p->str2) == NOTOK){ /* open port to listen */
+      /* open port to listen */
+      if (UNLIKELY(SVopen(csound) == NOTOK)){
         return csound->InitError(csound, Str("Failed to open port to listen"));
       }
     }
@@ -409,27 +478,28 @@ int insremot(CSOUND *csound, INSREMOT *p)
 int insglobal(CSOUND *csound, INSGLOBAL *p)
 /* declare certain instrs global remote Csounds */
 {   /*      INSTR 0 opcode  */
-    short nargs = p->INOCOUNT;
+    int16 nargs = p->INOCOUNT;
 
     if (csound->remoteGlobals==NULL || ST(socksin) == NULL) {
-      if (callox(csound) < 0) {
+      if (UNLIKELY(callox(csound) < 0)) {
         return
-          csound->InitError(csound, Str("failed to initialize remote globals."));
+          csound->InitError(csound, Str("failed to initialise remote globals."));
       }
     }
-    if (nargs < 2) {
+    if (UNLIKELY(nargs < 2)) {
       return csound->InitError(csound, Str("missing instr nos"));
     }
     csound->Message(csound, Str("*** str1: %s own:%s\n"),
-                    (char *)p->str1 , ST(ipadrs));
-    if (strcmp(ST(ipadrs), (char *)p->str1) == 0) { /* if client is this adrs */
+                    (char *)p->str1->data , ST(ipadrs));
+    if (strcmp(ST(ipadrs), (char *)p->str1->data) == 0) {
+      /* if client is this adrs */
       MYFLT   **argp = p->insno;
       for (nargs -= 1; nargs--; ) {
-        short insno = (short)**argp++;             /* for each insno */
-        if (insno <= 0 || insno > 128) {
+        int16 insno = (int16)**argp++;             /* for each insno */
+        if (UNLIKELY(insno <= 0 || insno > 128)) {
           return csound->InitError(csound, Str("illegal instr no"));
         }
-        if (ST(insrfd)[insno]) {
+        if (UNLIKELY(ST(insrfd)[insno])) {
           return csound->InitError(csound, Str("insno already specific remote"));
         }
         ST(insrfd)[insno] = GLOBAL_REMOT;             /*  mark as GLOBAL   */
@@ -441,36 +511,42 @@ int insglobal(CSOUND *csound, INSGLOBAL *p)
 int midremot(CSOUND *csound, MIDREMOT *p)    /* declare certain channels for
                                                 remote Csounds */
 {                                            /* INSTR 0 opcode  */
-    short nargs = p->INOCOUNT;
+    int16 nargs = p->INOCOUNT;
 
     if (csound->remoteGlobals==NULL || ST(socksin) == NULL) {
-      if (callox(csound) < 0) {
+      if (UNLIKELY(callox(csound) < 0)) {
         return
-          csound->InitError(csound, Str("failed to initialize remote globals."));
+          csound->InitError(csound, Str("failed to initialise remote globals."));
       }
     }
-    if (nargs < 3) {
+    if (UNLIKELY(nargs < 3)) {
       return csound->InitError(csound, Str("missing channel nos"));
     }
-    if (strcmp(ST(ipadrs), (char *)p->str1) == 0) {  /* if client is this adrs */
+    if (strcmp(ST(ipadrs), (char *)p->str1->data) == 0) {
+      /* if client is this adrs */
       MYFLT   **argp = p->chnum;
       int  rfd;
-      if ((rfd = CLopen(csound, (char *)p->str2)) <= 0) /* open port to remot */
+        /* open port to remote */
+      if (UNLIKELY((rfd = CLopen(csound, (char *)p->str2->data)) < 0))
         return NOTOK;
       for (nargs -= 2; nargs--; ) {
-        short chnum = (short)**argp++;               /* & for each channel   */
-        if (chnum <= 0 || chnum > 16) {              /* THESE ARE MIDCHANS+1 */
+        int16 chnum = (int16)**argp++;               /* & for each channel   */
+        if (UNLIKELY(chnum <= 0 || chnum > 16)) {    /* THESE ARE MIDCHANS+1 */
+          close(rfd);
           return csound->InitError(csound, Str("illegal channel no"));
         }
-        if (ST(chnrfd)[chnum]) {
+        if (UNLIKELY(ST(chnrfd)[chnum])) {
+          close(rfd);
           return csound->InitError(csound, Str("channel already remote"));
         }
         ST(chnrfd)[chnum] = rfd;                      /* record file descriptor */
-                }
-                ST(chnrfd_list)[ST(chnrfd_count)++] = rfd;   /* and make a list */
+      }
+      ST(chnrfd_list)[ST(chnrfd_count)++] = rfd;   /* and make a list */
     }
-    else if (!strcmp(ST(ipadrs), (char *)p->str2)) { /* if server is this adrs */
-      if(SVopen(csound, (char *)p->str2) == NOTOK){  /* open port to listen */
+    else if (!strcmp(ST(ipadrs), (char *)p->str2->data)) {
+      /* if server is this adrs */
+      /* open port to listen */
+      if (UNLIKELY(SVopen(csound) == NOTOK)){
         return csound->InitError(csound, Str("Failed to open port to listen"));
       }
       csound->oparms->RMidiin = 1;            /* & enable rtevents in */
@@ -481,27 +557,28 @@ int midremot(CSOUND *csound, MIDREMOT *p)    /* declare certain channels for
 int midglobal(CSOUND *csound, MIDGLOBAL *p)
 /* declare certain chnls global remote Csounds */
 {                                         /*       INSTR 0 opcode  */
-    short nargs = p->INOCOUNT;
+    int16 nargs = p->INOCOUNT;
 
     if (csound->remoteGlobals==NULL || ST(socksin) == NULL) {
-      if (callox(csound) < 0) {
+      if (UNLIKELY(callox(csound) < 0)) {
         return
-          csound->InitError(csound, Str("failed to initialize remote globals."));
+          csound->InitError(csound, Str("failed to initialise remote globals."));
       }
     }
-    if (nargs < 2) {
+    if (UNLIKELY(nargs < 2)) {
       return csound->InitError(csound, Str("missing channel nos"));
     }
 /*     csound->Message(csound, Str("*** str1: %s own:%s\n"), */
 /*                     (char *)p->str1 , ST(ipadrs)); */
-    if (strcmp(ST(ipadrs), (char *)p->str1) == 0) { /* if client is this adrs */
+    if (strcmp(ST(ipadrs), (char *)p->str1->data) == 0) {
+      /* if client is this adrs */
       MYFLT   **argp = p->chnum;
       for (nargs -= 1; nargs--; ) {
-        short chnum = (short)**argp++;             /* for each channel */
-        if (chnum <= 0 || chnum > 16) {
+        int16 chnum = (int16)**argp++;             /* for each channel */
+        if (UNLIKELY(chnum <= 0 || chnum > 16)) {
           return csound->InitError(csound, Str("illegal channel no"));
         }
-        if (ST(chnrfd)[chnum]) {
+        if (UNLIKELY(ST(chnrfd)[chnum])) {
           return csound->InitError(csound, Str("channel already specific remote"));
         }
         ST(chnrfd)[chnum] = GLOBAL_REMOT;              /*  mark as GLOBAL   */
@@ -518,7 +595,9 @@ int insSendevt(CSOUND *csound, EVTBLK *evt, int rfd)
     EVTBLK *cpp = (EVTBLK *)bp->data;       /* align an EVTBLK struct */
     int nn;
     MYFLT *f, *g;
+    cpp->pinstance = NULL;
     cpp->strarg = NULL;                     /* copy the initial header */
+    cpp->scnt = 0;
     cpp->opcod = evt->opcod;
     cpp->pcnt = evt->pcnt;
     f = &evt->p2orig;
@@ -527,8 +606,9 @@ int insSendevt(CSOUND *csound, EVTBLK *evt, int rfd)
       *g++ = *f++;
     bp->type = SCOR_EVT;                    /* insert type and len */
     bp->len = (char *)g - (char *)bp;
-    if (CLsend(csound, rfd, (void *)bp, (int)bp->len) < 0) {
-      return csound->PerfError(csound, Str("CLsend failed"));
+    if (UNLIKELY(CLsend(csound, rfd, (void *)bp, (int)bp->len) < 0)) {
+      csound->ErrorMsg(csound, Str("CLsend failed"));
+      return NOTOK;
     }
     else return OK;
 }
@@ -537,7 +617,7 @@ int insGlobevt(CSOUND *csound, EVTBLK *evt)  /* send an event to all remote fd's
 {
     int nn;
     for (nn = 0; nn < ST(insrfd_count); nn++) {
-      if (insSendevt(csound, evt, ST(insrfd_list)[nn]) == NOTOK)
+      if (UNLIKELY(insSendevt(csound, evt, ST(insrfd_list)[nn]) == NOTOK))
         return NOTOK;
     }
     return OK;
@@ -551,8 +631,9 @@ int MIDIsendevt(CSOUND *csound, MEVENT *evt, int rfd)
     bp->type = MIDI_EVT;                    /* insert type and len    */
     bp->len = sizeof(int) * 2 + sizeof(MEVENT);
 
-    if (CLsend(csound, rfd, (void *)bp, (size_t)bp->len) < 0) {
-      return csound->PerfError(csound, Str("CLsend failed"));
+    if (UNLIKELY(CLsend(csound, rfd, (void *)bp, (size_t)bp->len) < 0)) {
+      csound->ErrorMsg(csound, Str("CLsend failed"));
+      return NOTOK;
     }
     else return OK;
 }
@@ -561,7 +642,7 @@ int MIDIGlobevt(CSOUND *csound, MEVENT *evt) /* send an Mevent to all remote fd'
 {
     int nn;
     for (nn = 0; nn < ST(chnrfd_count); nn++) {
-      if (MIDIsendevt(csound, evt, ST(chnrfd_list)[nn]) == NOTOK)
+      if (UNLIKELY(MIDIsendevt(csound, evt, ST(chnrfd_list)[nn]) == NOTOK))
         return NOTOK;
     }
     return OK;
@@ -575,8 +656,9 @@ int MIDIsend_msg(CSOUND *csound, MEVENT *evt, int rfd)
     bp->type = MIDI_MSG;                    /* insert type and len    */
     bp->len = sizeof(int) * 2 + sizeof(MEVENT);
 
-    if (CLsend(csound, rfd, (void *)bp, (size_t)bp->len) < 0) {
-      return csound->PerfError(csound, Str("CLsend failed"));
+    if (UNLIKELY(CLsend(csound, rfd, (void *)bp, (size_t)bp->len) < 0)) {
+      csound->ErrorMsg(csound, Str("CLsend failed"));
+      return NOTOK;
     }
     else return OK;
 }
@@ -586,7 +668,7 @@ int MIDIGlob_msg(CSOUND *csound, MEVENT *evt)
 {
     int nn;
     for (nn = 0; nn < ST(chnrfd_count); nn++) {
-      if (MIDIsend_msg(csound, evt, ST(chnrfd_list)[nn]) == NOTOK)
+      if (UNLIKELY(MIDIsend_msg(csound, evt, ST(chnrfd_list)[nn]) == NOTOK))
         return NOTOK;
     }
     return OK;
@@ -623,7 +705,7 @@ int* getRemoteSocksIn(CSOUND *csound)
 
 char remoteID(CSOUND *csound)
 {
-    return (char)0;
+    return '\0';
 }
 
 /* Cleanup the above; called from musmon csoundCleanup */
