@@ -26,216 +26,116 @@
 #include "schedule.h"
 #include <math.h>
 #include "namedins.h"
+#include "linevent.h"
 
-extern INSDS  *insert_event(CSOUND*, MYFLT, MYFLT, MYFLT, int, MYFLT **, int);
 
-typedef struct rsched {
-  void          *parent;
-  INSDS         *kicked;
-  struct rsched *next;
-} RSCHED;
+int eventOpcodeI_(CSOUND *csound, LINEVENT *p, int s, char p1);
+int eventOpcode_(CSOUND *csound, LINEVENT *p, int s, char p1);
 
-/******************************************************************************/
-/* triginstr - Ignite instrument events at k-rate from orchestra.             */
-/* August 1999 by rasmus ekman.                                               */
-/******************************************************************************/
-
-#define FZERO (FL(0.0))    /* (Shouldn't there be global decl's for these?) */
-
-static void unquote(char *dst, char *src)
-{
-    if (src[0] == '"') {
-      int len = (int) strlen(src) - 2;
-      strcpy(dst, src + 1);
-      if (len >= 0 && dst[len] == '"')
-        dst[len] = '\0';
-    }
-    else
-      strcpy(dst, src);
-}
-
-static void queue_event(CSOUND *csound,
-                        MYFLT instr, double when, MYFLT dur,
-                        int narg, MYFLT **args)
-{
-    EVTBLK        evt;
-    int           i;
-
-    evt.strarg = NULL;
-    evt.opcod = 'i';
-    evt.pcnt = narg + 3;
-    evt.p[1] = instr;
-    evt.p[2] = FL(0.0);
-    evt.p[3] = dur;
-    for (i = 0; i < narg; i++)
-      evt.p[i + 4] = *(args[i]);
-    insert_score_event(csound, &evt, when);
-}
-
-/* ********** Need to add turnoff stuff *************** */
 int schedule(CSOUND *csound, SCHED *p)
 {
-    RSCHED *rr = (RSCHED*) csound->schedule_kicked;
-    RSCHED *ss = NULL;
-    int which;
-    /* First ensure any stragglers die really in case of reinit */
-    while (rr!=NULL) {
-      if (rr->parent==p) {
-        xturnoff(csound, rr->kicked);
-        {
-          RSCHED *tt = rr->next;
-          free(rr);
-          rr = tt;
-          if (ss == NULL)
-            csound->schedule_kicked = (void*) rr;
-        }
-      }
-      else {
-        ss = rr; rr = rr->next;
-      }
+    LINEVENT pp;
+    int i;
+    pp.h = p->h;
+    char c[2] = "i";
+    pp.args[0] = (MYFLT *) c;
+    pp.args[1] = p->which;
+    pp.args[2] = p->when;
+    pp.args[3] = p->dur;
+    pp.argno = p->INOCOUNT+1;
+    for (i=4; i < pp.argno ; i++) {
+      pp.args[i] = p->argums[i-4];
     }
-    if (p->XSTRCODE)
-      which = (int) named_instr_find(csound, (char*) p->which);
-    else if (*p->which == SSTRCOD)
-      which = (int) named_instr_find(csound, csound->currevent->strarg);
-    else
-      which = (int) (FL(0.5) + *p->which);
-    if (which < 1 || which > csound->maxinsno ||
-        csound->instrtxtp[which] == NULL) {
-      return csound->InitError(csound, Str("Instrument not defined"));
-    }
-    {
-      RSCHED *rr;
-      /* if duration is zero assume MIDI schedule */
-      MYFLT dur = *p->dur;
-/*       csound->Message(csound,"SCH: when = %f dur = %f\n", *p->when, dur); */
-      p->midi = (dur <= FL(0.0));
-      if (p->midi) {
-/*         csound->Message(csound,"SCH: MIDI case\n"); */
-        csound->Warning(csound,Str("schedule in MIDI mode is not "
-                                   "implemented correctly, do not use it\n"));
-        /* set 1 k-cycle of extratime in order to allow mtrnoff to
-           recognize whether the note is turned off */
-        if (p->h.insdshead->xtratim < 1)
-          p->h.insdshead->xtratim = 1;
-      }
-      if (*p->when <= FL(0.0)) {
-        p->kicked = insert_event(csound, (MYFLT) which,
-                                 (MYFLT) (csound->curTime - csound->timeOffs),
-                                 dur, p->INOCOUNT - 3, p->argums, p->midi);
-        if (p->midi) {
-          rr = (RSCHED*) malloc(sizeof(RSCHED));
-          rr->parent = p; rr->kicked = p->kicked;
-          rr->next = (RSCHED*) csound->schedule_kicked;
-          csound->schedule_kicked = (void*) rr;
-        }
-      }
-      else
-        queue_event(csound, (MYFLT) which, (double)*p->when + csound->curTime,
-                            dur, p->INOCOUNT - 3, p->argums);
+    pp.flag = 1;
+    return eventOpcodeI_(csound, &pp, 0, 'i');
+}
+
+int schedule_N(CSOUND *csound, SCHED *p)
+{
+    int i;
+    int argno = p->INOCOUNT+1;
+    char s[16384];
+    sprintf(s, "i %f %f %f", *p->which, *p->when, *p->dur);
+    for (i=4; i < argno ; i++) {
+       MYFLT *arg = p->argums[i-4];
+         if (csoundGetTypeForArg(arg) == &CS_VAR_TYPE_S)
+           sprintf(s, "%s \"%s\" ", s, ((STRINGDAT *)arg)->data);
+         else sprintf(s, "%s %f", s,  *arg);
     }
+    csoundInputMessage(csound, s);
     return OK;
 }
 
-int schedwatch(CSOUND *csound, SCHED *p)
-{                               /* If MIDI case watch for release */
-    if (p->midi && p->h.insdshead->relesing) {
-      p->midi = 0;
-      if (p->kicked==NULL) return OK;
-      xturnoff(csound, p->kicked);
-      {
-        RSCHED *rr = (RSCHED*) csound->schedule_kicked;
-        RSCHED *ss = NULL;
-        while (rr!=NULL) {
-          if (rr->parent==p) {
-            RSCHED *tt = rr->next;
-            free(rr);
-            rr = tt;
-            if (ss == NULL)
-              csound->schedule_kicked = (void*) rr;
-          }
-          else {
-            ss = rr; rr = rr->next;
-          }
-        }
-      }
-      p->kicked = NULL;
+int schedule_SN(CSOUND *csound, SCHED *p)
+{
+    int i;
+    int argno = p->INOCOUNT+1;
+    char s[16384];
+    sprintf(s, "i \"%s\" %f %f", ((STRINGDAT *)p->which)->data, *p->when, *p->dur);
+    for (i=4; i < argno ; i++) {
+       MYFLT *arg = p->argums[i-4];
+         if (csoundGetTypeForArg(arg) == &CS_VAR_TYPE_S)
+           sprintf(s, "%s \"%s\" ", s, ((STRINGDAT *)arg)->data);
+         else sprintf(s, "%s %f", s,  *arg);
     }
+    csoundInputMessage(csound, s);
     return OK;
 }
 
+
+int schedule_S(CSOUND *csound, SCHED *p)
+{
+    LINEVENT pp;
+    int i;
+    pp.h = p->h;
+    char c[2] = "i";
+    pp.args[0] = (MYFLT *) c;
+    pp.args[1] = p->which;
+    pp.args[2] = p->when;
+    pp.args[3] = p->dur;
+    pp.argno = p->INOCOUNT+1;
+    for (i=4; i < pp.argno ; i++) {
+      pp.args[i] = p->argums[i-4];
+    }
+    pp.flag = 1;
+    return eventOpcodeI_(csound, &pp, 1, 'i');
+}
+
+
+
 int ifschedule(CSOUND *csound, WSCHED *p)
 {                       /* All we need to do is ensure the trigger is set */
+    IGN(csound);
     p->todo = 1;
-    p->abs_when = p->h.insdshead->p2;
-    p->midi = 0;
     return OK;
 }
 
 int kschedule(CSOUND *csound, WSCHED *p)
 {
-    if (p->todo && *p->trigger != FL(0.0)) { /* If not done and trigger */
-      double starttime;
-      RSCHED *rr;
-      MYFLT dur = *p->dur;
-      int which;
-      if (p->XSTRCODE)
-        which = (int) named_instr_find(csound, (char*) p->which);
-      else if (*p->which == SSTRCOD)
-        which = (int) named_instr_find(csound, csound->currevent->strarg);
-      else
-        which = (int) (FL(0.5) + *p->which);
-      if (which < 1 || which > csound->maxinsno ||
-          csound->instrtxtp[which] == NULL) {
-        return csound->PerfError(csound, Str("Instrument not defined"));
+    if (p->todo && *p->trigger != FL(0.0)) {
+      LINEVENT pp;
+      int i;
+      pp.h = p->h;
+      char c[2] = "i";
+      pp.args[0] = (MYFLT *) c;
+      pp.args[1] = p->which;
+      pp.args[2] = p->when;
+      pp.args[3] = p->dur;
+      pp.argno = p->INOCOUNT+1;
+      for(i=4; i < pp.argno ; i++) {
+        pp.args[i] = p->argums[i-4];
       }
-      p->midi = (dur <= FL(0.0));
-      if (p->midi)
-        csound->Warning(csound,
-                        Str("schedule in MIDI mode is not "
-                            "implemented correctly, do not use it\n"));
-      p->todo = 0;
-                                /* Insert event */
-      starttime = (double)p->abs_when + (double)*(p->when) + csound->timeOffs;
-      if (starttime <= csound->curTime) {
-        p->kicked = insert_event(csound, (MYFLT) which,
-                                 (MYFLT) (csound->curTime - csound->timeOffs),
-                                 dur, p->INOCOUNT - 4, p->argums, p->midi);
-        if (p->midi) {
-          rr = (RSCHED*) malloc(sizeof(RSCHED));
-          rr->parent = p; rr->kicked = p->kicked;
-          rr->next = (RSCHED*) csound->schedule_kicked;
-          csound->schedule_kicked = (void*) rr;
-        }
-      }
-      else
-        queue_event(csound, (MYFLT) which,
-                    starttime, dur, p->INOCOUNT - 4, p->argums);
+      p->todo =0;
+      pp.flag = 1;
+       if (IS_STR_ARG(p->which)){
+         return eventOpcode_(csound, &pp, 1, 'i');
+       }
+       else {
+         pp.flag = 0;
+         return eventOpcode_(csound, &pp, 0, 'i');
+       }
     }
-    else if (p->midi && p->h.insdshead->relesing) {
-                                /* If MIDI case watch for release */
-      p->midi = 0;
-      if (p->kicked==NULL) return OK;
-      xturnoff(csound, p->kicked);
-      {
-        RSCHED *rr = (RSCHED*) csound->schedule_kicked;
-        RSCHED *ss = NULL;
-        while (rr!=NULL) {
-          if (rr->parent==p) {
-            RSCHED *tt = rr->next;
-            free(rr);
-            rr = tt;
-            if (ss == NULL)
-              csound->schedule_kicked = (void*) rr;
-          }
-          else {
-            ss = rr; rr = rr->next;
-          }
-        }
-      }
-      p->kicked = NULL;
-    }
-    return OK;
+    else return OK;
 }
 
 /* tables are 4096 entries always */
@@ -259,10 +159,10 @@ int lfoset(CSOUND *csound, LFO *p)
         p->sine = (MYFLT*)p->auxd.auxp;
       }
       for (i=0; i<4096; i++)
-        p->sine[i] = (MYFLT)sin(TWOPI*(double)i/4096.0);
+        p->sine[i] = SIN(TWOPI_F*(MYFLT)i/FL(4096.0));
 /*        csound->Message(csound,"Table set up (max is %d)\n", MAXPHASE>>10); */
     }
-    else if (type>5 || type<0) {
+    else if (UNLIKELY(type>5 || type<0)) {
       return csound->InitError(csound, Str("LFO: unknown oscilator type %d"),
                                        type);
     }
@@ -273,16 +173,17 @@ int lfoset(CSOUND *csound, LFO *p)
 
 int lfok(CSOUND *csound, LFO *p)
 {
-    long        phs;
+    int32        phs;
     MYFLT       fract;
     MYFLT       res;
-    long        iphs;
+    int32        iphs;
 
     phs = p->phs;
     switch (p->lasttype) {
     default:
-      return csound->PerfError(csound, Str("LFO: unknown oscilator type %d"),
-                                       p->lasttype);
+      return csound->PerfError(csound, p->h.insdshead,
+                               Str("LFO: unknown oscilator type %d"),
+                               p->lasttype);
     case 0:
       iphs = phs >> 12;
       fract = (MYFLT)(phs & 0xfff)/FL(4096.0);
@@ -314,7 +215,7 @@ int lfok(CSOUND *csound, LFO *p)
       res = FL(1.0) - (MYFLT)phs/(MYFLT)MAXPHASE;
       break;
     }
-    phs += (long)(*p->xcps * MAXPHASE * csound->onedkr);
+    phs += (int32)(*p->xcps * MAXPHASE * CS_ONEDKR);
     phs &= MAXMASK;
     p->phs = phs;
     *p->res = *p->kamp * res;
@@ -323,22 +224,30 @@ int lfok(CSOUND *csound, LFO *p)
 
 int lfoa(CSOUND *csound, LFO *p)
 {
-    int         n;
-    long        phs;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
+    int32       phs;
     MYFLT       fract;
     MYFLT       res;
-    long        iphs, inc;
+    int32       iphs, inc;
     MYFLT       *ar, amp;
 
     phs = p->phs;
-    inc = (long)((*p->xcps * (MYFLT)MAXPHASE) * csound->onedsr);
+    inc = (int32)((*p->xcps * (MYFLT)MAXPHASE) * csound->onedsr);
     amp = *p->kamp;
     ar = p->res;
-    for (n=0; n<csound->ksmps; n++) {
+    if (UNLIKELY(offset)) memset(ar, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&ar[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n=offset; n<nsmps; n++) {
       switch (p->lasttype) {
       default:
-        return csound->PerfError(csound, Str("LFO: unknown oscilator type %d"),
-                                         p->lasttype);
+        return csound->PerfError(csound, p->h.insdshead,
+                                 Str("LFO: unknown oscilator type %d"),
+                                 p->lasttype);
       case 0:
         iphs = phs >> 12;
         fract = (MYFLT)(phs & 0xfff)/FL(4096.0);
@@ -384,7 +293,24 @@ int lfoa(CSOUND *csound, LFO *p)
 /* Changes made also to Cs.h, Musmon.c and Insert.c; look for "(re Aug 1999)" */
 /******************************************************************************/
 
-int ktriginstr(CSOUND *csound, TRIGINSTR *p);
+/******************************************************************************/
+/* triginstr - Ignite instrument events at k-rate from orchestra.             */
+/* August 1999 by rasmus ekman.                                               */
+/******************************************************************************/
+
+static void unquote(char *dst, char *src, int maxsize)
+{
+    if (src[0] == '"') {
+      int len = (int) strlen(src) - 2;
+      strncpy(dst, src + 1, maxsize-1);
+      if (len >= 0 && dst[len] == '"')
+        dst[len] = '\0';
+    }
+    else
+      strncpy(dst, src, maxsize);
+}
+
+static int ktriginstr_(CSOUND *csound, TRIGINSTR *p, int stringname);
 
 int triginset(CSOUND *csound, TRIGINSTR *p)
 {
@@ -397,34 +323,60 @@ int triginset(CSOUND *csound, TRIGINSTR *p)
        But in ktriginstr() we'll need to use kcounter-1 to set the start time
        of new events. So add a separate variable for the kcounter offset (-1) */
     if (csound->global_kcounter == 0 &&
-        *p->trigger != FZERO /*&& *p->args[1] <= FZERO*/) {
+        *p->trigger != FL(0.0) /*&& *p->args[1] <= FL(0.0)*/) {
       p->kadjust = 0;   /* No kcounter offset at this time */
-      ktriginstr(csound, p);
+      ktriginstr_(csound, p, 0);
     }
     p->kadjust = -1;      /* Set kcounter offset for perf-time */
     /* Catch p3==0 (i-time only) event triggerings. */
     if (csound->global_kcounter > 0 &&
-        *p->trigger != FZERO && p->h.insdshead->p3 == 0)
-      ktriginstr(csound, p);
+        *p->trigger != FL(0.0) && p->h.insdshead->p3.value == 0)
+      ktriginstr_(csound, p,0);
     return OK;
 }
 
-static int get_absinsno(CSOUND *csound, TRIGINSTR *p)
+int triginset_S(CSOUND *csound, TRIGINSTR *p)
+{
+    p->prvmintim = *p->mintime;
+    p->timrem = 0;
+    /* An instrument is initialised before kcounter is incremented for
+       this k-cycle, and begins playing after kcounter++.
+       Therefore, if we should start something at the very first k-cycle of
+       performance, we must thus do it now, lest it be one k-cycle late.
+       But in ktriginstr() we'll need to use kcounter-1 to set the start time
+       of new events. So add a separate variable for the kcounter offset (-1) */
+    if (csound->global_kcounter == 0 &&
+        *p->trigger != FL(0.0) /*&& *p->args[1] <= FL(0.0)*/) {
+      p->kadjust = 0;   /* No kcounter offset at this time */
+      ktriginstr_(csound, p, 1);
+    }
+    p->kadjust = -1;      /* Set kcounter offset for perf-time */
+    /* Catch p3==0 (i-time only) event triggerings. */
+    if (csound->global_kcounter > 0 &&
+        *p->trigger != FL(0.0) && p->h.insdshead->p3.value == 0)
+      ktriginstr_(csound, p, 1);
+    return OK;
+}
+
+
+static int get_absinsno(CSOUND *csound, TRIGINSTR *p, int stringname)
 {
     int insno;
 
     /* Get absolute instr num */
     /* IV - Oct 31 2002: allow string argument for named instruments */
-    if (p->XSTRCODE)
-      insno = (int)strarg2insno_p(csound, (char*)p->args[0]);
-    else if (*p->args[0] == SSTRCOD)
-      insno = (int)strarg2insno_p(csound, csound->currevent->strarg);
+    if (stringname)
+      insno = (int)strarg2insno_p(csound, ((STRINGDAT*)p->args[0])->data);
+    else if (csound->ISSTRCOD(*p->args[0])) {
+      char *ss = get_arg_string(csound, *p->args[0]);
+      insno = (int)strarg2insno_p(csound, ss);
+    }
     else
-      insno = (int)fabs((double)*p->args[0]);
+      insno = (int)FABS(*p->args[0]);
     /* Check that instrument is defined */
-    if (insno < 1 || insno > csound->maxinsno ||
-        csound->instrtxtp[insno] == NULL) {
-      csound->Message(csound, Str("schedkwhen ignored. "
+    if (UNLIKELY(insno < 1 || insno > csound->engineState.maxinsno ||
+                 csound->engineState.instrtxtp[insno] == NULL)) {
+      csound->Warning(csound, Str("schedkwhen ignored. "
                                   "Instrument %d undefined\n"), insno);
       csound->perferrcnt++;
       return -1;
@@ -432,21 +384,22 @@ static int get_absinsno(CSOUND *csound, TRIGINSTR *p)
     return insno;
 }
 
-int ktriginstr(CSOUND *csound, TRIGINSTR *p)
+static int ktriginstr_(CSOUND *csound, TRIGINSTR *p, int stringname)
 {         /* k-rate event generator */
-    double  starttime;
+    long  starttime;
     int     i, argnum;
     EVTBLK  evt;
     char    name[512];
+    memset(&evt, 0, sizeof(EVTBLK));
 
     if (p->timrem > 0)
       p->timrem--;
-    if (*p->trigger == FZERO) /* Only do something if triggered */
+    if (*p->trigger == FL(0.0)) /* Only do something if triggered */
       return OK;
 
     /* Check if mintime has changed */
     if (p->prvmintim != *p->mintime) {
-      long timrem = (long) (*p->mintime * csound->global_ekr + FL(0.5));
+      int32 timrem = (int32) (*p->mintime * CS_EKR + FL(0.5));
       if (timrem > 0) {
         /* Adjust countdown for new mintime */
         p->timrem += timrem - p->prvktim;
@@ -457,17 +410,17 @@ int ktriginstr(CSOUND *csound, TRIGINSTR *p)
       p->prvmintim = *p->mintime;
     }
 
-    if (*p->args[0] >= FL(0.0) || *p->args[0] == SSTRCOD) {
+    if (*p->args[0] >= FL(0.0) || csound->ISSTRCOD(*p->args[0])) {
       /* Check for rate limit on event generation */
-      if (*p->mintime > FZERO && p->timrem > 0)
+      if (*p->mintime > FL(0.0) && p->timrem > 0)
         return OK;
       /* See if there are too many instances already */
       if (*p->maxinst >= FL(1.0)) {
         INSDS *ip;
         int absinsno, numinst = 0;
         /* Count active instr instances */
-        absinsno = get_absinsno(csound, p);
-        if (absinsno < 1)
+        absinsno = get_absinsno(csound, p, stringname);
+        if (UNLIKELY(absinsno < 1))
           return NOTOK;
         ip = &(csound->actanchor);
         while ((ip = ip->nxtact) != NULL)
@@ -478,39 +431,52 @@ int ktriginstr(CSOUND *csound, TRIGINSTR *p)
     }
 
     /* Create the new event */
-    if (p->XSTRCODE) {
-      evt.strarg = (char*) p->args[0];
-      evt.p[1] = SSTRCOD;
+    if (stringname) {
+      evt.p[1] = csound->strarg2insno(csound,((STRINGDAT *)p->args[0])->data, 1);
+      evt.strarg = NULL; evt.scnt = 0;
+      /*evt.strarg = ((STRINGDAT*)p->args[0])->data;
+        evt.p[1] = SSTRCOD;*/
     }
-    else if (*p->args[0] == SSTRCOD) {
-      unquote(name, csound->currevent->strarg);
-      evt.strarg = name;
-      evt.p[1] = SSTRCOD;
+    else if (csound->ISSTRCOD(*p->args[0])) {
+      unquote(name, get_arg_string(csound, *p->args[0]), 512);
+      evt.p[1] = csound->strarg2insno(csound,name, 1);
+      evt.strarg = NULL;
+      /* evt.strarg = name; */
+      evt.scnt = 0;
+      /* evt.p[1] = SSTRCOD; */
     }
     else {
-      evt.strarg = NULL;
+      evt.strarg = NULL; evt.scnt = 0;
       evt.p[1] = *p->args[0];
     }
     evt.opcod = 'i';
     evt.pcnt = argnum = p->INOCOUNT - 3;
     /* Add current time (see note about kadjust in triginset() above) */
-    starttime = (double)(csound->global_kcounter + p->kadjust)
-                / (double)csound->global_ekr;
+    starttime = CS_KSMPS*(csound->global_kcounter + p->kadjust);
     /* Copy all arguments to the new event */
     for (i = 1; i < argnum; i++)
       evt.p[i + 1] = *p->args[i];
     /* Set start time from kwhen */
-    if (evt.p[2] < FZERO) {
-      evt.p[2] = FZERO;
+    if (UNLIKELY(evt.p[2] < FL(0.0))) {
+      evt.p[2] = FL(0.0);
       csound->Warning(csound,
                       Str("schedkwhen warning: negative kwhen reset to zero"));
     }
     /* Reset min pause counter */
-    if (*p->mintime > FZERO)
-      p->timrem = (long) (*p->mintime * csound->global_ekr + FL(0.5));
+    if (*p->mintime > FL(0.0))
+      p->timrem = (int32) (*p->mintime * CS_EKR + FL(0.5));
     else
       p->timrem = 0;
-    return (insert_score_event(csound, &evt, starttime) == 0 ? OK : NOTOK);
+    return
+      (insert_score_event_at_sample(csound, &evt, starttime) == 0 ? OK : NOTOK);
+}
+
+int ktriginstr_S(CSOUND *csound, TRIGINSTR *p){
+  return ktriginstr_(csound,p,1);
+}
+
+int ktriginstr(CSOUND *csound, TRIGINSTR *p){
+  return ktriginstr_(csound,p,0);
 }
 
 /* Maldonado triggering of events */
@@ -518,13 +484,13 @@ int ktriginstr(CSOUND *csound, TRIGINSTR *p)
 int trigseq_set(CSOUND *csound, TRIGSEQ *p)      /* by G.Maldonado */
 {
     FUNC *ftp;
-    if ((ftp = csound->FTFind(csound, p->kfn)) == NULL) {
+    if (UNLIKELY((ftp = csound->FTnp2Find(csound, p->kfn)) == NULL)) {
       return csound->InitError(csound, Str("trigseq: incorrect table number"));
     }
     p->done  = 0;
     p->table = ftp->ftable;
-    p->pfn   = (long)*p->kfn;
-    p->ndx   = (long)*p->initndx;
+    p->pfn   = (int32)*p->kfn;
+    p->ndx   = (int32)*p->initndx;
     p->nargs = p->INOCOUNT-5;
     return OK;
 }
@@ -534,23 +500,23 @@ int trigseq(CSOUND *csound, TRIGSEQ *p)
     if (p->done) return OK;
     else {
       int j, nargs = p->nargs;
-      long start = (long) *p->kstart, loop = (long) *p->kloop;
-      long *ndx = &p->ndx;
-      MYFLT *out = *p->outargs;
+      int32 start = (int32) *p->kstart, loop = (int32) *p->kloop;
+      int32 *ndx = &p->ndx;
+      MYFLT **out = p->outargs;
 
-      if (p->pfn != (long)*p->kfn) {
+      if (p->pfn != (int32)*p->kfn) {
         FUNC *ftp;
-        if ((ftp = csound->FTFindP(csound, p->kfn)) == NULL) {
-          return csound->PerfError(csound,
+        if (UNLIKELY((ftp = csound->FTFindP(csound, p->kfn)) == NULL)) {
+          return csound->PerfError(csound, p->h.insdshead,
                                    Str("trigseq: incorrect table number"));
         }
-        p->pfn = (long)*p->kfn;
+        p->pfn = (int32)*p->kfn;
         p->table = ftp->ftable;
       }
       if (*p->ktrig) {
         int nn = nargs * (int)*ndx;
         for (j=0; j < nargs; j++) {
-          *out++ = p->table[nn+j];
+          *out[j] = p->table[nn+j];
         }
         if (loop > 0) {
           (*ndx)++;
@@ -577,4 +543,3 @@ int trigseq(CSOUND *csound, TRIGSEQ *p)
     }
     return OK;
 }
-
