@@ -285,6 +285,8 @@ int strcat_opcode(CSOUND *csound, STRCAT_OP *p)
     char *str1 = strdup(p->str1->data), *str2 = strdup(p->str2->data);
 
     if (str1 == NULL || str2 == NULL){
+      free(str1);
+      free(str2);
       if (UNLIKELY(((OPDS*) p)->insdshead->pds != NULL))
       return csoundPerfError(csound, ((OPDS*)p)->insdshead, "NULL string \n");
       else return csoundInitError(csound, "NULL string \n");
@@ -392,8 +394,10 @@ sprintf_opcode_(CSOUND *csound,
 
         continue;
       }
+
       /* if already a segment waiting, then lets print it */
       if (segwaiting != NULL) {
+
         maxChars = str->size - len;
         strseg[i] = '\0';
         if (UNLIKELY(numVals <= 0)) {
@@ -407,6 +411,7 @@ sprintf_opcode_(CSOUND *csound,
         /* } */
         strCode >>= 1;
         parm = kvals[j++];
+
         switch (*segwaiting) {
         case 'd':
         case 'i':
@@ -424,6 +429,7 @@ sprintf_opcode_(CSOUND *csound,
             maxChars += 24;
             outstring = str->data + offs;
             //printf("maxchars = %d  %s\n", maxChars, strseg);
+
           }
           n = snprintf(outstring, maxChars, strseg, (int) MYFLT2LRND(*parm));
 #else
@@ -484,12 +490,14 @@ sprintf_opcode_(CSOUND *csound,
         len += n;
         i = 0;
       }
+
       if (*fmt == '\0')
         break;
       /* copy the '%' */
       strseg[i++] = *fmt++;
       /* find the format code */
       segwaiting = fmt;
+
       while (!isalpha(*segwaiting) && *segwaiting != '\0')
         segwaiting++;
     }
@@ -504,7 +512,7 @@ sprintf_opcode_(CSOUND *csound,
 int sprintf_opcode(CSOUND *csound, SPRINTF_OP *p)
 {
     int size = p->sfmt->size+ 10*((int) p->INOCOUNT);
-    if (p->r->data == NULL || p->r->size < size) {
+    if (p->r->data == NULL && p->r->size < size) {
       /* this 10 is 1n incorrect guess which is OK with numbers*/
       p->r->data = csound->Calloc(csound, size);
       p->r->size = size;
