@@ -25,16 +25,18 @@
 #include "ugens4.h"
 #include <math.h>
 
+/* The branch prediction slows it down!! */
+
 int bzzset(CSOUND *csound, BUZZ *p)
 {
     FUNC        *ftp;
 
-    if ((ftp = csound->FTFind(csound, p->ifn)) != NULL) {
+    if (LIKELY((ftp = csound->FTFind(csound, p->ifn)) != NULL)) {
       p->ftp = ftp;
       if (*p->iphs >= 0)
-        p->lphs = (long)(*p->iphs * FL(0.5) * FMAXLEN);
-      p->ampcod = (XINARG1) ? 1 : 0;
-      p->cpscod = (XINARG2) ? 1 : 0;
+        p->lphs = (int32)(*p->iphs * FL(0.5) * FMAXLEN);
+      p->ampcod = IS_ASIG_ARG(p->xamp) ? 1 : 0;
+      p->cpscod = IS_ASIG_ARG(p->xcps) ? 1 : 0;
       p->reported = 0;          /* No errors yet */
       return OK;
     }
@@ -45,62 +47,70 @@ int buzz(CSOUND *csound, BUZZ *p)
 {
     FUNC        *ftp;
     MYFLT       *ar, *ampp, *cpsp, *ftbl;
-    long        phs, inc, lobits, dwnphs, tnp1, lenmask, nn;
+    int32       phs, inc, lobits, dwnphs, tnp1, lenmask;
     MYFLT       sicvt2, over2n, scal, num, denom;
-    int n;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
+    int32    nn;
 
     ftp = p->ftp;
-    if (ftp==NULL) {        /* RWD fix */
-      return csound->PerfError(csound, Str("buzz: not initialised"));
-    }
+    if (UNLIKELY(ftp==NULL)) goto err1; /* RWD fix */
     ftbl = ftp->ftable;
-    sicvt2 = csound->sicvt * FL(0.5);           /* for theta/2  */
+    sicvt2 = csound->sicvt * FL(0.5); /* for theta/2  */
     lobits = ftp->lobits;
     lenmask = ftp->lenmask;
     ampp = p->xamp;
     cpsp = p->xcps;
-    if ((n = (int)*p->knh) < 0) n = -n;
-    if (n == 0) {              /* fix n = knh */
-      n = 1;
+    if ((nn = (int)*p->knh) < 0) nn = -nn;
+    if (UNLIKELY(nn == 0)) {     /* fix nn = knh */
+      nn = 1;
     }
-    tnp1 = (n <<1) + 1;                 /* calc 2n + 1 */
-    over2n = FL(0.5) / (MYFLT)n;
+    tnp1 = (nn<<1) + 1;          /* calc 2n + 1 */
+    over2n = FL(0.5) / (MYFLT)nn;
     scal = *ampp * over2n;
-    inc = (long)(*cpsp * sicvt2);
+    inc = (int32)(*cpsp * sicvt2);
     ar = p->ar;
     phs = p->lphs;
-    nn = csound->ksmps;
-    do {
+    if (UNLIKELY(offset)) memset(ar, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      //printf("early=%d\n", early);
+      nsmps -= early;
+      memset(&ar[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n=offset; n<nsmps; n++) {
+      if (p->ampcod)
+        scal = ampp[n] * over2n;
+      if (p->cpscod)
+        inc = (int32)(cpsp[n] * sicvt2);
       dwnphs = phs >> lobits;
-      denom = *(ftbl+dwnphs);
+      denom = ftbl[dwnphs];
       if (denom > FL(0.0002) || denom < -FL(0.0002)) {
-        num = *(ftbl +(dwnphs * tnp1 & lenmask));
-        *ar++ = (num / denom - FL(1.0)) * scal;
+        num = ftbl[dwnphs * tnp1 & lenmask];
+        ar[n] = (num / denom - FL(1.0)) * scal;
       }
-      else *ar++ = *ampp;
+      else ar[n] = *ampp;
       phs += inc;
       phs &= PHMASK;
-      if (p->ampcod)
-        scal = *(++ampp) * over2n;
-      if (p->cpscod)
-        inc = (long)(*(++cpsp)* sicvt2);
-    } while (--nn);
+    }
     p->lphs = phs;
     return OK;
+ err1:
+    return csound->PerfError(csound, p->h.insdshead, Str("buzz: not initialised"));
 }
 
 int gbzset(CSOUND *csound, GBUZZ *p)
 {
     FUNC        *ftp;
 
-    if ((ftp = csound->FTFind(csound, p->ifn)) != NULL) {
+    if (LIKELY((ftp = csound->FTFind(csound, p->ifn)) != NULL)) {
       p->ftp = ftp;
       if (*p->iphs >= 0) {
-        p->lphs = (long)(*p->iphs * FMAXLEN);
+        p->lphs = (int32)(*p->iphs * FMAXLEN);
         p->prvr = FL(0.0);
       }
-      p->ampcod = (XINARG1) ? 1 : 0;
-      p->cpscod = (XINARG2) ? 1 : 0;
+      p->ampcod = IS_ASIG_ARG(p->xamp) ? 1 : 0;
+      p->cpscod = IS_ASIG_ARG(p->xcps) ? 1 : 0;
       p->reported = 0;
       p->last = FL(1.0);
       return OK;
@@ -108,13 +118,9 @@ int gbzset(CSOUND *csound, GBUZZ *p)
     return NOTOK;
 }
 
-MYFLT intpow(MYFLT x, long n)   /* Binary power function */
+static inline MYFLT intpow1(MYFLT x, int32 n)   /* Binary positive power function */
 {
     MYFLT ans = FL(1.0);
-    if (n<0) {
-      n = -n;
-      x = FL(1.0)/x;
-    }
     while (n!=0) {
       if (n&1) ans = ans * x;
       n >>= 1;
@@ -123,88 +129,103 @@ MYFLT intpow(MYFLT x, long n)   /* Binary power function */
     return ans;
 }
 
+MYFLT intpow(MYFLT x, int32 n)   /* Binary power function */
+{
+    if (n<0) {
+      n = -n;
+      x = FL(1.0)/x;
+    }
+    return intpow1(x, n);
+}
+
 int gbuzz(CSOUND *csound, GBUZZ *p)
 {
     FUNC        *ftp;
     MYFLT       *ar, *ampp, *cpsp, *ftbl;
-    long        phs, inc, lobits, lenmask, k, km1, kpn, kpnm1, nn;
-    long        n;
+    int32       phs, inc, lobits, lenmask, k, km1, kpn, kpnm1;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     MYFLT       r, absr, num, denom, scal, last = p->last;
-    long        lphs = p->lphs;
+    int32       nn, lphs = p->lphs;
 
     ftp = p->ftp;
-    if (ftp==NULL) {
-      return csound->PerfError(csound, Str("gbuzz: not initialised"));
-    }
+    if (UNLIKELY(ftp==NULL)) goto err1;
     ftbl = ftp->ftable;
     lobits = ftp->lobits;
     lenmask = ftp->lenmask;
     ampp = p->xamp;
     cpsp = p->xcps;
-    k = (long)*p->kk;                   /* fix k and n  */
-    if ((n = (long)*p->kn)<0) n = -n;
-    if (n == 0) {               /* n must be > 0 */
-      n = 1;
+    k = (int32)*p->kk;                   /* fix k and n  */
+    if ((nn = (int32)*p->kn)<0) nn = -nn;
+    if (UNLIKELY(nn == 0)) {              /* n must be > 0 */
+      nn = 1;
     }
     km1 = k - 1;
-    kpn = k + n;
+    kpn = k + nn;
     kpnm1 = kpn - 1;
-    if ((r = *p->kr) != p->prvr || n != p->prvn) {
-      p->twor = r * FL(2.0);
+    if ((r = *p->kr) != p->prvr || nn != p->prvn) {
+      p->twor = r + r;
       p->rsqp1 = r * r + FL(1.0);
-      p->rtn = intpow(r, n);
+      p->rtn = intpow1(r, nn);
       p->rtnp1 = p->rtn * r;
-      if ((absr = (MYFLT)fabs(r)) > FL(0.999) && absr < FL(1.001))
-        p->rsumr = FL(1.0) / n;
-      else p->rsumr = (FL(1.0) - absr) / (FL(1.0) - (MYFLT)fabs(p->rtn));
+      if ((absr = FABS(r)) > FL(0.999) && absr < FL(1.001))
+        p->rsumr = FL(1.0) / nn;
+      else p->rsumr = (FL(1.0) - absr) / (FL(1.0) - FABS(p->rtn));
       p->prvr = r;
-      p->prvn = (short)n;
+      p->prvn = (int16)nn;
     }
     scal =  *ampp * p->rsumr;
-    inc = (long)(*cpsp * csound->sicvt);
+    inc = (int32)(*cpsp * csound->sicvt);
     ar = p->ar;
-    nn = csound->ksmps;
-    do {
+    if (UNLIKELY(offset)) memset(ar, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&ar[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n=offset; n<nsmps; n++) {
+      if (p->ampcod)
+        scal =  p->rsumr * ampp[n];
+      if (p->cpscod)
+        inc = (int32)(cpsp[n] * csound->sicvt);
       phs = lphs >>lobits;
-      denom = p->rsqp1 - p->twor * *(ftbl + phs);
-      num = *(ftbl + (phs * k & lenmask))
-        - r * *(ftbl + (phs * km1 & lenmask))
-        - p->rtn * *(ftbl + (phs * kpn & lenmask))
-        + p->rtnp1 * *(ftbl + (phs * kpnm1 & lenmask));
-      if (denom > FL(0.0002) || denom < -FL(0.0002)) {
-        *ar++ = last = num / denom * scal;
+      denom = p->rsqp1 - p->twor * ftbl[phs];
+      num = ftbl[phs * k & lenmask]
+        - r * ftbl[phs * km1 & lenmask]
+        - p->rtn * ftbl[phs * kpn & lenmask]
+        + p->rtnp1 * ftbl[phs * kpnm1 & lenmask];
+      if (LIKELY(denom > FL(0.0002) || denom < -FL(0.0002))) {
+        ar[n] = last = num / denom * scal;
       }
       else if (last<0)
-        *ar++ = last = - *ampp;
+        ar[n] = last = - (p->ampcod ? ampp[n] : *ampp);
       else
-        *ar++ = last = *ampp;
-      if (p->ampcod)
-        scal =  p->rsumr * *(++ampp);
+        ar[n] = last = (p->ampcod ? ampp[n] : *ampp);
       lphs += inc;
       lphs &= PHMASK;
-      if (p->cpscod)
-        inc = (long)(*(++cpsp) * csound->sicvt);
-    } while (--nn);
+    }
     p->last = last;
     p->lphs = lphs;
     return OK;
+ err1:
+    return csound->PerfError(csound, p->h.insdshead, Str("gbuzz: not initialised"));
 }
 
 #define PLUKMIN 64
 
-static  short   rand15(CSOUND *);
-static  short   rand16(CSOUND *);
+static  int16   rand15(CSOUND *);
+static  int16   rand16(CSOUND *);
 
 int plukset(CSOUND *csound, PLUCK *p)
 {
-    int n;
-    long        npts, iphs;
+    int         n;
+    int32       npts, iphs;
     char        *auxp;
     FUNC        *ftp;
     MYFLT       *ap, *fp;
     MYFLT       phs, phsinc;
 
-    if ((npts = (long)(csound->esr / *p->icps)) < PLUKMIN) {
+    if (UNLIKELY((npts = (int32)(csound->esr / *p->icps)) < PLUKMIN)) {
                                         /* npts is wavelen in sampls */
       npts = PLUKMIN;                   /*  (but at least min size)  */
     }
@@ -218,13 +239,13 @@ int plukset(CSOUND *csound, PLUCK *p)
     if (*p->ifn == 0.0)
       for (n=npts; n--; )                       /* f0: fill w. rands */
         *ap++ = (MYFLT) rand16(csound) * DV32768;
-    else if ((ftp = csound->FTFind(csound, p->ifn)) != NULL) {
+    else if ((ftp = csound->FTnp2Find(csound, p->ifn)) != NULL) {
       fp = ftp->ftable;                         /* else from ftable  */
       phs = FL(0.0);
       phsinc = (MYFLT)(ftp->flen/npts);
       for (n=npts; n--; phs += phsinc) {
-        iphs = (long)phs;
-        *ap++ = *(fp + iphs);
+        iphs = (int32)phs;
+        *ap++ = fp[iphs];
       }
     }
     *ap = *(MYFLT *)auxp;                       /* last = copy of 1st */
@@ -232,37 +253,37 @@ int plukset(CSOUND *csound, PLUCK *p)
     /* tuned pitch convt */
     p->sicps = (npts * FL(256.0) + FL(128.0)) * csound->onedsr;
     p->phs256 = 0;
-    p->method = (short)*p->imeth;
+    p->method = (int16)*p->imeth;
     p->param1 = *p->ipar1;
     p->param2 = *p->ipar2;
     switch(p->method) {
     case 1:     /* ignore any given parameters */
       break;
     case 2:     /* stretch factor: param1 >= 1 */
-      if (p->param1 < FL(1.0))
+      if (UNLIKELY(p->param1 < FL(1.0)))
         return csound->InitError(csound,
                                  Str("illegal stretch factor(param1) value"));
-      else p->thresh1 =  (short)(FL(32768.0) / p->param1);
+      else p->thresh1 =  (int16)(FL(32768.0) / p->param1);
       break;
     case 3: /* roughness factor: 0 <= param1 <= 1 */
-      if (p->param1 < FL(0.0) || p->param1 > FL(1.0))
+      if (UNLIKELY(p->param1 < FL(0.0) || p->param1 > FL(1.0)))
         return csound->InitError(csound,
                                  Str("illegal roughness factor(param1) value"));
       else
-        p->thresh1 = (short)(FL(32768.0) * p->param1);
+        p->thresh1 = (int16)(FL(32768.0) * p->param1);
       break;
     case 4: /* rough and stretch factor: 0 <= param1 <= 1, param2 >= 1 */
-      if (p->param1 < FL(0.0) || p->param1 > FL(1.0))
+      if (UNLIKELY(p->param1 < FL(0.0) || p->param1 > FL(1.0)))
         return csound->InitError(csound,
                                  Str("illegal roughness factor(param1) value"));
-      else p->thresh1 = (short)(FL(32768.0) * p->param1);
-      if (p->param2 < FL(1.0))
+      else p->thresh1 = (int16)(FL(32768.0) * p->param1);
+      if (UNLIKELY(p->param2 < FL(1.0)))
         return csound->InitError(csound,
                                  Str("illegal stretch factor(param2) value"));
-      else p->thresh2 = (short)(FL(32768.0) / p->param2);
+      else p->thresh2 = (int16)(FL(32768.0) / p->param2);
       break;
     case 5: /* weighting coeff's: param1 + param2 <= 1 */
-      if (p->param1 + p->param2 > 1)
+      if (UNLIKELY(p->param1 + p->param2 > 1))
         return csound->InitError(csound, Str("coefficients too large "
                                              "(param1 + param2)"));
       break;
@@ -278,35 +299,36 @@ int plukset(CSOUND *csound, PLUCK *p)
 int pluck(CSOUND *csound, PLUCK *p)
 {
     MYFLT       *ar, *fp;
-    long        phs256, phsinc, ltwopi, offset;
-    int nsmps;
+    int32       phs256, phsinc, ltwopi, offset;
+    uint32_t koffset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     MYFLT       frac, diff;
 
-    if (p->auxch.auxp==NULL) { /* RWD FIX */
-      return csound->PerfError(csound, Str("pluck: not initialised"));
-    }
+    if (UNLIKELY(p->auxch.auxp==NULL)) goto err1; /* RWD FIX */
     ar = p->ar;
-    phsinc = (long)(*p->kcps * p->sicps);
+    phsinc = (int32)(*p->kcps * p->sicps);
     phs256 = p->phs256;
     ltwopi = p->npts << 8;
-    if (phsinc > ltwopi) {
-      return csound->PerfError(csound,
-                               Str("pluck: kcps more than sample rate"));
+    if (UNLIKELY(phsinc > ltwopi)) goto err2;
+    if (UNLIKELY(koffset)) memset(ar, '\0', koffset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&ar[nsmps], '\0', early*sizeof(MYFLT));
     }
-    nsmps = csound->ksmps;
-    do {
+    for (n=koffset; n<nsmps; n++) {
       offset = phs256 >> 8;
-      fp = (MYFLT *)p->auxch.auxp + offset;     /* lookup position  */
-      diff = *(fp+1) - *fp;
+      fp = (MYFLT *)p->auxch.auxp + offset;     /* lookup position   */
+      diff = fp[1] - fp[0];
       frac = (MYFLT)(phs256 & 255) / FL(256.0); /*  w. interpolation */
-      *ar++ =   (*fp + diff*frac) * *p->kamp;   /*  gives output val */
+      ar[n] =   (fp[0] + diff*frac) * *p->kamp; /*  gives output val */
       if ((phs256 += phsinc) >= ltwopi) {
         int nn;
         MYFLT   newval, preval;
         phs256 -= ltwopi;               /* at phase wrap,    */
         fp=(MYFLT *)p->auxch.auxp;
-        preval = *fp;                   /*   copy last pnt   */
-        *fp = *(fp + p->npts);          /*     to first,     */
+        preval = fp[0];                 /*   copy last pnt   */
+        fp[0] = fp[p->npts];            /*     to first,     */
         fp++;                           /*   apply smoothing */
         nn = p->npts;                   /*     up to npts+1  */
         switch(p->method) {
@@ -365,29 +387,34 @@ int pluck(CSOUND *csound, PLUCK *p)
           break;
         }
       }
-    } while (--nsmps);
+    }
     p->phs256 = phs256;
     return OK;
+ err1:
+    return csound->PerfError(csound, p->h.insdshead, Str("pluck: not initialised"));
+ err2:
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("pluck: kcps more than sample rate"));
 }
 
 #define RNDMUL  15625
 #define MASK16  0xFFFF
 #define MASK15  0x7FFF
 
-/* quick generate a random short between -32768 and 32767 */
+/* quick generate a random int16 between -32768 and 32767 */
 
-static short rand16(CSOUND *csound)
+static int16 rand16(CSOUND *csound)
 {
     csound->ugens4_rand_16 = (csound->ugens4_rand_16 * RNDMUL + 1) & MASK16;
-    return (short) ((int16_t) csound->ugens4_rand_16);
+    return (int16) ((int16_t) csound->ugens4_rand_16);
 }
 
-/* quick generate a random short between 0 and 32767 */
+/* quick generate a random int16 between 0 and 32767 */
 
-static short rand15(CSOUND *csound)
+static int16 rand15(CSOUND *csound)
 {
     csound->ugens4_rand_15 = (csound->ugens4_rand_15 * RNDMUL + 1) & MASK15;
-    return (short) csound->ugens4_rand_15;
+    return (int16) csound->ugens4_rand_15;
 }
 
 /*=========================================================================
@@ -411,17 +438,17 @@ static short rand15(CSOUND *csound)
  */
 
 #define BIPOLAR 0x7FFFFFFF      /* Constant to make bipolar */
-#define RIA 16807               /* multiplier */
+#define RIA (16807)             /* multiplier */
 #define RIM 0x7FFFFFFFL         /* 2**31 - 1 */
 
 #define dv2_31          (FL(4.656612873077392578125e-10))
 
-long randint31(long seed31)
+int32 randint31(int32 seed31)
 {
-    unsigned long rilo, rihi;
+    uint32 rilo, rihi;
 
-    rilo = RIA * (long)(seed31 & 0xFFFF);
-    rihi = RIA * (long)((unsigned long)seed31 >> 16);
+    rilo = RIA * (int32)(seed31 & 0xFFFF);
+    rihi = RIA * (int32)((uint32)seed31 >> 16);
     rilo += (rihi & 0x7FFF) << 16;
     if (rilo > RIM) {
       rilo &= RIM;
@@ -432,48 +459,51 @@ long randint31(long seed31)
       rilo &= RIM;
       ++rilo;
     }
-    return (long)rilo;
+    return (int32)rilo;
 }
 
 int rndset(CSOUND *csound, RAND *p)
 {
     p->new = (*p->sel!=FL(0.0));
     if (*p->iseed >= FL(0.0)) {
-      if (*p->iseed > FL(1.0)) {    /* As manual suggest sseed in range [0,1] */
-        unsigned long seed;         /* I reinterpret >1 as a time seed */
+      if (*p->iseed > FL(1.0)) {    /* As manual suggest seed in range [0,1] */
+        uint32 seed;         /* I reinterpret >1 as a time seed */
         seed = csound->GetRandomSeedFromTime();
-        csound->Message(csound, Str("Seeding from current time %lu\n"), seed);
+        csound->Warning(csound, Str("Seeding from current time %lu\n"), seed);
         if (!p->new) {
-          p->rand = (long) (seed & 0xFFFFUL);
+          p->rand = (int32) (seed & 0xFFFFUL);
         }
         else {
-          p->rand = (long) (seed % 0x7FFFFFFEUL) + 1L;
+          p->rand = (int32) (seed % 0x7FFFFFFEUL) + 1L;
         }
       }
       else {
         if (p->new) {
-          p->rand = (long) (*p->iseed * FL(2147483648.0));
+          MYFLT seed = *p->iseed;
+          if (seed==FL(0.0)) seed = FL(0.5);
+          p->rand = (int32) (seed * FL(2147483648.0));
           p->rand = randint31(p->rand);
           p->rand = randint31(p->rand);
         }
         else
-          p->rand = ((short)(*p->iseed * FL(32768.0)))&0xffff;
+          p->rand = ((int16)(*p->iseed * FL(32768.0)))&0xffff;
       }
     }
-    p->ampcod = (XINARG1) ? 1 : 0;      /* (not used by krand) */
+    p->ampcod = IS_ASIG_ARG(p->xamp) ? 1 : 0;      /* (not used by krand) */
+
     return OK;
 }
 
 int krand(CSOUND *csound, RAND *p)
 {
     if (p->new) {
-      long r = randint31(p->rand);      /* result is a 31-bit value */
+      int32 r = randint31(p->rand);      /* result is a 31-bit value */
       p->rand = r;
       *p->ar = *p->base +
-        dv2_31 * (MYFLT)((long)((unsigned)r<<1)-BIPOLAR) * *p->xamp;
+        dv2_31 * (MYFLT)((int32)((unsigned)r<<1)-BIPOLAR) * *p->xamp;
     }
     else {
-      short rand = (short)p->rand;
+      int16 rand = (int16)p->rand;
       rand *= RNDMUL;
       rand += 1;
       /* IV - Jul 11 2002 */
@@ -486,29 +516,37 @@ int krand(CSOUND *csound, RAND *p)
 int arand(CSOUND *csound, RAND *p)
 {
     MYFLT       *ar;
-    short       rndmul = RNDMUL, n = csound->ksmps;
+    int16       rndmul = RNDMUL;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     MYFLT       ampscl;
     MYFLT       base = *p->base;
 
     ar = p->ar;
+    if (UNLIKELY(offset)) memset(ar, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&ar[nsmps], '\0', early*sizeof(MYFLT));
+    }
     if (!p->new) {
-      short     rand = p->rand;
+      int16     rand = p->rand;
       if (!(p->ampcod)) {
         ampscl = *p->xamp * DV32768;    /* IV - Jul 11 2002 */
-        do {
+        for (n=offset;n<nsmps;n++) {
           rand *= rndmul;
           rand += 1;
-          *ar++ = base + (MYFLT)rand * ampscl;          /* IV - Jul 11 2002 */
-        } while (--n);
+          ar[n] = base + (MYFLT)rand * ampscl;          /* IV - Jul 11 2002 */
+        }
       }
       else {
         MYFLT *xamp = p->xamp;
-        do {
+        for (n=offset;n<nsmps;n++) {
           rand *= rndmul;
           rand += 1;
           /* IV - Jul 11 2002 */
-          *ar++ = base + (MYFLT)rand * *xamp++ * DV32768;
-        } while (--n);
+          ar[n] = base + (MYFLT)rand * xamp[n] * DV32768;
+        }
       }
       p->rand = rand;   /* save current rand */
     }
@@ -516,18 +554,18 @@ int arand(CSOUND *csound, RAND *p)
       int       rand = p->rand;
       if (!(p->ampcod)) {
         ampscl = *p->xamp * dv2_31;
-        do {
+        for (n=offset;n<nsmps;n++) {
           rand = randint31(rand);
-          *ar++ = base + (MYFLT)((long)((unsigned)rand<<1)-BIPOLAR) * ampscl;
-        } while (--n);
+          ar[n] = base + (MYFLT)((int32)((unsigned)rand<<1)-BIPOLAR) * ampscl;
+        }
       }
       else {
-          MYFLT *xamp = p->xamp;
-          do {
-            rand = randint31(rand);
-            *ar++ = base +
-              dv2_31 * (MYFLT)((long)((unsigned)rand<<1)-BIPOLAR) * *xamp++;
-          } while (--n);
+        MYFLT *xamp = p->xamp;
+        for (n=offset;n<nsmps;n++) {
+          rand = randint31(rand);
+          ar[n] = base +
+            dv2_31 * (MYFLT)((int32)((unsigned)rand<<1)-BIPOLAR) * xamp[n];
+        }
       }
       p->rand = rand;   /* save current rand */
     }
@@ -540,34 +578,34 @@ int rhset(CSOUND *csound, RANDH *p)
     p->new = (*p->sel!=FL(0.0));
     if (*p->iseed >= FL(0.0)) {                       /* new seed:            */
       if (*p->iseed > FL(1.0)) {    /* As manual suggest sseed in range [0,1] */
-        unsigned long seed;         /* I reinterpret >1 as a time seed */
+        uint32 seed;         /* I reinterpret >1 as a time seed */
         seed = csound->GetRandomSeedFromTime();
-        csound->Message(csound, Str("Seeding from current time %lu\n"), seed);
+        csound->Warning(csound, Str("Seeding from current time %lu\n"), seed);
         if (!p->new) {
-          p->rand = (long) (seed & 0xFFFFUL);
-          p->num1 = (MYFLT) ((short) p->rand) * DV32768;
+          p->rand = (int32) (seed & 0xFFFFUL);
+          p->num1 = (MYFLT) ((int16) p->rand) * DV32768;
         }
         else {
-          p->rand = (long) (seed % 0x7FFFFFFEUL) + 1L;
-          p->num1 = (MYFLT) ((long) ((unsigned)p->rand<<1) - BIPOLAR) * dv2_31;
+          p->rand = (int32) (seed % 0x7FFFFFFEUL) + 1L;
+          p->num1 = (MYFLT) ((int32) ((unsigned)p->rand<<1) - BIPOLAR) * dv2_31;
         }
       }
       else if (!p->new) {
-        p->rand = 0xffff&(short)(*p->iseed * 32768L);   /* init rand integ    */
+        p->rand = 0xffff&(int16)(*p->iseed * 32768L);   /* init rand integ    */
         p->num1 = *p->iseed;                            /*    store fnum      */
       }
       else {
         MYFLT ss = *p->iseed;
-        if (ss>FL(1.0)) p->rand = (long) ss;
-        else p->rand = (long) (*p->iseed * FL(2147483648.0));
+        if (ss>FL(1.0)) p->rand = (int32) ss;
+        else p->rand = (int32) (*p->iseed * FL(2147483648.0));
         p->rand = randint31(p->rand);
         p->rand = randint31(p->rand);
-        p->num1 = (MYFLT) ((long) ((unsigned)p->rand<<1) - BIPOLAR) * dv2_31;
+        p->num1 = (MYFLT) ((int32) ((unsigned)p->rand<<1) - BIPOLAR) * dv2_31;
       }
       p->phs = 0;                               /*      & phs           */
     }
-    p->ampcod = (XINARG1) ? 1 : 0;      /* (not used by krandh) */
-    p->cpscod = (XINARG2) ? 1 : 0;
+    p->ampcod = IS_ASIG_ARG(p->xamp) ? 1 : 0;      /* (not used by krandh) */
+    p->cpscod = IS_ASIG_ARG(p->xcps) ? 1 : 0;
     return OK;
 }
 
@@ -575,20 +613,21 @@ int krandh(CSOUND *csound, RANDH *p)
 {
     /* IV - Jul 11 2002 */
     *p->ar = *p->base + p->num1 * *p->xamp;     /* rslt = num * amp     */
-    p->phs += (long)(*p->xcps * csound->kicvt); /* phs += inc           */
+    p->phs += (long)(*p->xcps * CS_KICVT); /* phs += inc           */
+
     if (p->phs >= MAXLEN) {                     /* when phs overflows,  */
       p->phs &= PHMASK;                         /*      mod the phs     */
       if (!p->new) {
-        short rand = (short)p->rand;
+        int16 rand = (int16)p->rand;
         rand *= RNDMUL;                         /*      & recalc number */
         rand += 1;
         p->num1 = (MYFLT)rand * DV32768;        /* IV - Jul 11 2002 */
         p->rand = rand;
       }
       else {
-        long r = randint31(p->rand);            /*      & recalc number */
+        int32 r = randint31(p->rand);            /*      & recalc number */
         p->rand = r;
-        p->num1 = (MYFLT)((long)((unsigned)r<<1)-BIPOLAR) * dv2_31;
+        p->num1 = (MYFLT)((int32)((unsigned)r<<1)-BIPOLAR) * dv2_31;
       }
     }
     return OK;
@@ -596,18 +635,25 @@ int krandh(CSOUND *csound, RANDH *p)
 
 int randh(CSOUND *csound, RANDH *p)
 {
-    long        phs = p->phs, inc;
-    int n = csound->ksmps;
+    long      phs = p->phs, inc;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     MYFLT       *ar, *ampp, *cpsp;
     MYFLT       base = *p->base;
 
     cpsp = p->xcps;
     ampp = p->xamp;
     ar = p->ar;
+    if (UNLIKELY(offset)) memset(ar, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&ar[nsmps], '\0', early*sizeof(MYFLT));
+    }
     inc = (long)(*cpsp++ * csound->sicvt);
-    do {
+    for (n=offset;n<nsmps;n++) {
       /* IV - Jul 11 2002 */
-      *ar++ = base + p->num1 * *ampp;   /* rslt = num * amp */
+      ar[n] = base + p->num1 * *ampp;   /* rslt = num * amp */
       if (p->ampcod)
         ampp++;
       phs += inc;                               /* phs += inc       */
@@ -616,19 +662,19 @@ int randh(CSOUND *csound, RANDH *p)
       if (phs >= MAXLEN) {                      /* when phs o'flows, */
         phs &= PHMASK;
         if (!p->new) {
-          short rand = p->rand;
+          int16 rand = p->rand;
           rand *= RNDMUL;               /*   calc new number */
           rand += 1;
           p->num1 = (MYFLT)rand * DV32768;      /* IV - Jul 11 2002 */
           p->rand = rand;
         }
         else {
-          long r = randint31(p->rand);          /*   calc new number */
+          int32 r = randint31(p->rand);          /*   calc new number */
           p->rand = r;
-          p->num1 = (MYFLT)((long)((unsigned)r<<1)-BIPOLAR) * dv2_31;
+          p->num1 = (MYFLT)((int32)((unsigned)r<<1)-BIPOLAR) * dv2_31;
         }
       }
-    } while (--n);
+    }
     p->phs = phs;
     return OK;
 }
@@ -636,14 +682,14 @@ int randh(CSOUND *csound, RANDH *p)
 int riset(CSOUND *csound, RANDI *p)
 {
     p->new = (*p->sel!=FL(0.0));
-    if (*p->iseed >= FL(0.0)) {                       /* new seed:            */
-      if (*p->iseed > FL(1.0)) {    /* As manual suggest sseed in range [0,1] */
-        unsigned long seed;         /* I reinterpret >1 as a time seed */
+    if (*p->iseed >= FL(0.0)) {                    /* new seed:            */
+      if (*p->iseed > FL(1.0)) { /* As manual suggest sseed in range [0,1] */
+        uint32 seed;             /* I reinterpret >1 as a time seed */
         seed = csound->GetRandomSeedFromTime();
-        csound->Message(csound, Str("Seeding from current time %lu\n"), seed);
+        csound->Warning(csound, Str("Seeding from current time %lu\n"), seed);
         if (!p->new) {
-          short rand = (short)seed;
-/*           short ss = rand; */
+          int16 rand = (int16)seed;
+/*           int16 ss = rand; */
           /* IV - Jul 11 2002 */
           p->num1 = (MYFLT)(rand) * DV32768; /* store num1,2 */
           rand *= RNDMUL;         /*      recalc random   */
@@ -654,7 +700,7 @@ int riset(CSOUND *csound, RANDI *p)
 /*                  ss,ss,p->rand, p->rand, p->num1, p->num2); */
         }
         else {
-          p->rand = randint31((long) (seed % 0x7FFFFFFEUL) + 1L);
+          p->rand = randint31((int32) (seed % 0x7FFFFFFEUL) + 1L);
           p->rand = randint31(p->rand);
           p->num1 = (MYFLT)(p->rand<<1) * dv2_31; /* store num1,2 */
           p->rand = randint31(p->rand);
@@ -662,7 +708,7 @@ int riset(CSOUND *csound, RANDI *p)
         }
       }
       else if (!p->new) {
-        short rand = (short)(*p->iseed * FL(32768.0)); /* init rand integ */
+        int16 rand = (int16)(*p->iseed * FL(32768.0)); /* init rand integ */
         rand *= RNDMUL;                 /*      to 2nd value    */
         rand += 1;
         p->num1 = *p->iseed;                    /*      store num1,2    */
@@ -671,8 +717,8 @@ int riset(CSOUND *csound, RANDI *p)
       }
       else {
         MYFLT ss = *p->iseed;
-        if (ss>FL(1.0)) p->rand = (long) ss;
-        else p->rand = (long) (*p->iseed * FL(2147483648.0));
+        if (ss>FL(1.0)) p->rand = (int32) ss;
+        else p->rand = (int32) (*p->iseed * FL(2147483648.0));
         p->rand = randint31(p->rand);
         p->rand = randint31(p->rand);
         p->num1 = (MYFLT)(p->rand<1) * dv2_31; /* store num1,2 */
@@ -682,8 +728,8 @@ int riset(CSOUND *csound, RANDI *p)
       p->phs = 0;                               /*      & clear phs     */
       p->dfdmax = (p->num2 - p->num1) / FMAXLEN;  /* & diff     */
     }
-    p->ampcod = (XINARG1) ? 1 : 0;      /* (not used by krandi) */
-    p->cpscod = (XINARG2) ? 1 : 0;
+    p->ampcod = IS_ASIG_ARG(p->xamp) ? 1 : 0;      /* (not used by krandi) */
+    p->cpscod = IS_ASIG_ARG(p->xcps) ? 1 : 0;
     return OK;
 }
 
@@ -691,11 +737,11 @@ int krandi(CSOUND *csound, RANDI *p)
 {                                       /* rslt = (num1 + diff*phs) * amp */
     /* IV - Jul 11 2002 */
     *p->ar = *p->base + (p->num1 + (MYFLT)p->phs * p->dfdmax) * *p->xamp;
-    p->phs += (long)(*p->xcps * csound->kicvt); /* phs += inc           */
+    p->phs += (long)(*p->xcps * CS_KICVT); /* phs += inc           */
     if (p->phs >= MAXLEN) {                     /* when phs overflows,  */
       p->phs &= PHMASK;                         /*      mod the phs     */
       if (!p->new) {
-        short rand = p->rand;
+        int16 rand = p->rand;
         rand *= RNDMUL;                         /*      recalc random   */
         rand += 1;
         p->num1 = p->num2;                      /*      & new num vals  */
@@ -703,10 +749,10 @@ int krandi(CSOUND *csound, RANDI *p)
         p->rand = rand;
       }
       else {
-        long r = randint31(p->rand);    /*      recalc random   */
+        int32 r = randint31(p->rand);    /*      recalc random   */
         p->rand = r;
         p->num1 = p->num2;              /*      & new num vals  */
-        p->num2 = (MYFLT)((long)((unsigned)r<<1)-BIPOLAR) * dv2_31;
+        p->num2 = (MYFLT)((int32)((unsigned)r<<1)-BIPOLAR) * dv2_31;
       }
       p->dfdmax = (p->num2 - p->num1) / FMAXLEN;
     }
@@ -715,18 +761,25 @@ int krandi(CSOUND *csound, RANDI *p)
 
 int randi(CSOUND *csound, RANDI *p)
 {
-    long        phs = p->phs, inc;
-    int         n = csound->ksmps;
+    long       phs = p->phs, inc;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     MYFLT       *ar, *ampp, *cpsp;
     MYFLT       base = *p->base;
 
     cpsp = p->xcps;
     ampp = p->xamp;
     ar = p->ar;
+    if (UNLIKELY(offset)) memset(ar, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&ar[nsmps], '\0', early*sizeof(MYFLT));
+    }
     inc = (long)(*cpsp++ * csound->sicvt);
-    do {
+    for (n=offset;n<nsmps;n++) {
       /* IV - Jul 11 2002 */
-      *ar++ = base + (p->num1 + (MYFLT)phs * p->dfdmax) * *ampp;
+      ar[n] = base + (p->num1 + (MYFLT)phs * p->dfdmax) * *ampp;
       if (p->ampcod)
         ampp++;
       phs += inc;                               /* phs += inc       */
@@ -735,7 +788,7 @@ int randi(CSOUND *csound, RANDI *p)
       if (phs >= MAXLEN) {                      /* when phs o'flows, */
         phs &= PHMASK;
         if (!p->new) {
-          short rand = p->rand;
+          int16 rand = p->rand;
           rand *= RNDMUL;                       /*   calc new numbers*/
           rand += 1;
           p->num1 = p->num2;
@@ -743,15 +796,14 @@ int randi(CSOUND *csound, RANDI *p)
           p->rand = rand;
         }
         else {
-          long r = randint31(p->rand);          /*   calc new numbers*/
+          int32 r = randint31(p->rand);          /*   calc new numbers*/
           p->rand = r;
           p->num1 = p->num2;
-          p->num2 = (MYFLT)((long)((unsigned)r<<1)-BIPOLAR) * dv2_31;
+          p->num2 = (MYFLT)((int32)((unsigned)r<<1)-BIPOLAR) * dv2_31;
         }
         p->dfdmax = (p->num2 - p->num1) / FMAXLEN;
       }
-    } while (--n);
+    }
     p->phs = phs;
     return OK;
 }
-
