@@ -1025,7 +1025,8 @@ LADSPAPluginSearch(CSOUND *csound,
       int len = strlen(pcLADSPAPath)+strlen(pcDSSIPath)+2;
       char *tmp = (char*)malloc(len);
       snprintf(tmp, len, "%s:%s", pcLADSPAPath, pcDSSIPath);
-      pcLADSPAPath = pcStart = (const char*)tmp;
+      pcLADSPAPath = tmp;
+      pcStart = (const char*)tmp;
     }
     // Search the list
     while (*pcStart != '\0') {
