@@ -21,7 +21,9 @@
     02111-1307 USA
 */
 
-#include "csdl.h"
+// #include "csdl.h"
+#include "csoundCore.h"
+#include "interlocks.h"
 #include "soundio.h"
 
 typedef struct SNDLOAD_OPCODE_ {
@@ -33,15 +35,20 @@ typedef struct SNDLOAD_OPCODE_ {
     MYFLT   *iLoopMode1, *iLoopStart1, *iLoopEnd1;
 } SNDLOAD_OPCODE;
 
-static int sndload_opcode_init(CSOUND *csound, SNDLOAD_OPCODE *p)
+static int sndload_opcode_init_(CSOUND *csound, SNDLOAD_OPCODE *p, int isstring)
 {
     char        *fname;
     SNDMEMFILE  *sf;
     SF_INFO     sfinfo;
     int         sampleFormat, loopMode;
 
-    fname = csound->strarg2name(csound, (char*) NULL, p->Sfname, "soundin.",
-                                (int) csound->GetInputArgSMask(p));
+    if (isstring) fname = ((STRINGDAT *)p->Sfname)->data;
+    else {
+      if(csound->ISSTRCOD(*p->Sfname))
+        fname = csound->Strdup(csound, get_arg_string(csound, *p->Sfname));
+      else
+        fname = csound->strarg2name(csound, (char*) NULL, p->Sfname, "soundin.", 0);
+    }
     memset(&sfinfo, 0, sizeof(SF_INFO));
     sampleFormat = (int) MYFLT2LRND(*(p->iFormat));
     sfinfo.format = (int) TYPE2SF(TYP_RAW);
@@ -67,13 +74,13 @@ static int sndload_opcode_init(CSOUND *csound, SNDLOAD_OPCODE *p)
       tmp = (int) MYFLT2LRND(*(p->iChannels));
       sfinfo.channels = (tmp > 0 ? tmp : 1);
       tmp = (int) MYFLT2LRND(*(p->iSampleRate));
-      sfinfo.samplerate = (tmp > 0 ? tmp : (int) MYFLT2LRND(csound->esr));
+      sfinfo.samplerate = (tmp > 0 ? tmp : (int) MYFLT2LRND(CS_ESR));
     }
-    sf = csound->LoadSoundFile(csound, fname, &sfinfo);
-    if (sf == NULL) {
-      csound->InitError(csound, Str("could not load '%s'"), fname);
+    sf = csound->LoadSoundFile(csound, fname,  &sfinfo);
+    if (UNLIKELY(sf == NULL)) {
+      int xx = csound->InitError(csound, Str("could not load '%s'"), fname);
       csound->Free(csound, fname);
-      return NOTOK;
+      return xx;
     }
     csound->Free(csound, fname);
     if (*(p->iBaseFreq) > FL(0.0))
@@ -84,7 +91,7 @@ static int sndload_opcode_init(CSOUND *csound, SNDLOAD_OPCODE *p)
       sf->startOffs = (double) *(p->iStartOffset);
     loopMode = (int) MYFLT2LRND(*(p->iLoopMode1));
     if (loopMode >= 0) {
-      if (loopMode > 3)
+      if (UNLIKELY(loopMode > 3))
         return csound->InitError(csound, Str("invalid loop mode: %d"),
                                          loopMode);
       sf->loopMode = loopMode + 1;
@@ -93,7 +100,7 @@ static int sndload_opcode_init(CSOUND *csound, SNDLOAD_OPCODE *p)
     }
     if (sf->loopMode < 2 || sf->loopStart == sf->loopEnd) {
       sf->loopStart = 0.0;
-      sf->loopEnd = (double) ((long) sf->nFrames);
+      sf->loopEnd = (double) ((int32) sf->nFrames);
     }
     else if (sf->loopStart > sf->loopEnd) {
       double  tmp = sf->loopStart;
@@ -104,6 +111,17 @@ static int sndload_opcode_init(CSOUND *csound, SNDLOAD_OPCODE *p)
     return OK;
 }
 
+static int sndload_opcode_init(CSOUND *csound, SNDLOAD_OPCODE *p)
+{
+    return sndload_opcode_init_(csound,p,0);
+}
+
+static int sndload_opcode_init_S(CSOUND *csound, SNDLOAD_OPCODE *p)
+{
+    return sndload_opcode_init_(csound,p,1); /* JPff: surely wrong as 0 */
+}
+
+
  /* ------------------------------------------------------------------------ */
 
 #define LOSCILX_MAXOUTS         (16)
@@ -122,7 +140,7 @@ typedef struct LOSCILX_OPCODE_ {
     int_least64_t   curLoopStart, curLoopEnd;
     MYFLT   prvKcps, frqScale, ampScale, warpFact, winFact;
     void    *dataPtr;
-    long    nFrames;
+    int32   nFrames;
     int     nChannels;
     int     winSize;
     int     enableWarp;         /* non-zero when downsampling */
@@ -141,7 +159,7 @@ static inline int_least64_t loscilx_convert_phase(double phs)
 #endif
 }
 
-static inline long loscilx_phase_int(int_least64_t phs)
+static inline int32 loscilx_phase_int(int_least64_t phs)
 {
     int32_t retval = (int32_t) ((uint32_t) ((uint_least64_t) phs >> 32));
 #ifndef __i386__
@@ -166,27 +184,30 @@ static int loscilx_opcode_init(CSOUND *csound, LOSCILX_OPCODE *p)
 
     p->dataPtr = NULL;
     nChannels = csound->GetOutputArgCnt(p);
-    if (nChannels < 1 || nChannels > LOSCILX_MAXOUTS)
-      csound->Die(csound, Str("loscilx: invalid number of output arguments"));
+    if (UNLIKELY(nChannels < 1 || nChannels > LOSCILX_MAXOUTS))
+      return csound->InitError(csound,
+                               Str("loscilx: invalid number of output arguments"));
     p->nChannels = nChannels;
-    if (csound->GetInputArgSMask(p) != 0UL) {
+    if (csound->ISSTRCOD(*p->ifn)) {
       SNDMEMFILE  *sf;
 
       p->usingFtable = 0;
-      sf = csound->LoadSoundFile(csound, (char*) p->ifn, (SF_INFO*) NULL);
-      if (sf == NULL)
+      sf = csound->LoadSoundFile(csound,
+                                 (char*) get_arg_string(csound, *p->ifn),
+                                 (SF_INFO *) NULL);
+      if (UNLIKELY(sf == NULL))
         return csound->InitError(csound, Str("could not load '%s'"),
                                          (char*) p->ifn);
       if (sf->loopMode < 2 || sf->loopStart == sf->loopEnd) {
         sf->loopStart = 0.0;
-        sf->loopEnd = (double) ((long) sf->nFrames);
+        sf->loopEnd = (double) ((int32) sf->nFrames);
       }
       else if (sf->loopStart > sf->loopEnd) {
         double  tmp = sf->loopStart;
         sf->loopStart = sf->loopEnd;
         sf->loopEnd = tmp;
       }
-      if (sf->nChannels != nChannels)
+      if (UNLIKELY(sf->nChannels != nChannels))
         return csound->InitError(csound, Str("number of output arguments "
                                              "inconsistent with number of "
                                              "sound file channels"));
@@ -201,12 +222,12 @@ static int loscilx_opcode_init(CSOUND *csound, LOSCILX_OPCODE *p)
       }
       if (*(p->ibas) > FL(0.0)) {
         frqScale = sf->sampleRate
-                   / ((double) csound->esr * (double) *(p->ibas));
+                   / ((double) CS_ESR * (double) *(p->ibas));
       }
       else
-        frqScale = sf->sampleRate / ((double) csound->esr * sf->baseFreq);
+        frqScale = sf->sampleRate / ((double) CS_ESR * sf->baseFreq);
       p->ampScale = (MYFLT) sf->scaleFac * csound->e0dbfs;
-      p->nFrames = (long) sf->nFrames;
+      p->nFrames = (int32) sf->nFrames;
     }
     else {
       FUNC  *ftp;
@@ -215,7 +236,7 @@ static int loscilx_opcode_init(CSOUND *csound, LOSCILX_OPCODE *p)
       ftp = csound->FTnp2Find(csound, p->ifn);
       if (ftp == NULL)
         return NOTOK;
-      if ((int) ftp->nchanls != nChannels)
+      if (UNLIKELY((int) ftp->nchanls != nChannels))
         return csound->InitError(csound, Str("number of output arguments "
                                              "inconsistent with number of "
                                              "sound file channels"));
@@ -236,7 +257,7 @@ static int loscilx_opcode_init(CSOUND *csound, LOSCILX_OPCODE *p)
       if (*(p->ibas) > FL(0.0)) {
         if (ftp->gen01args.sample_rate > FL(0.0))
           frqScale = (double) ftp->gen01args.sample_rate
-                     / ((double) csound->esr * (double) *(p->ibas));
+                     / ((double) CS_ESR * (double) *(p->ibas));
         else
           frqScale = 1.0 / (double) *(p->ibas);
       }
@@ -244,7 +265,7 @@ static int loscilx_opcode_init(CSOUND *csound, LOSCILX_OPCODE *p)
         frqScale = (double) ftp->cpscvt * (1.0 / (double) LOFACT);
       }
       else if (ftp->gen01args.sample_rate > FL(0.0))
-        frqScale = (double) ftp->gen01args.sample_rate / (double) csound->esr;
+        frqScale = (double) ftp->gen01args.sample_rate / (double) CS_ESR;
       p->ampScale = FL(1.0);
       p->nFrames = ftp->flenfrms + 1L;
     }
@@ -254,7 +275,7 @@ static int loscilx_opcode_init(CSOUND *csound, LOSCILX_OPCODE *p)
     p->curLoopDir = 1;
     loopMode = (int) MYFLT2LRND(*(p->imod1));
     if (loopMode >= 0) {
-      if (loopMode > 3)
+      if (UNLIKELY(loopMode > 3))
         return csound->InitError(csound, Str("invalid loop mode: %d"),
                                          loopMode);
       p->curLoopMode = loopMode;
@@ -284,12 +305,12 @@ static int loscilx_opcode_init(CSOUND *csound, LOSCILX_OPCODE *p)
       if (p->winSize > 4) {
         /* constant for window calculation */
         p->winFact =
-            (MYFLT) ((1.0 - pow((double) p->winSize * 0.85172, -0.89624))
-                     / ((double) ((p->winSize * p->winSize) >> 2)));
+          (FL(1.0) - POWER(p->winSize * FL(0.85172), -FL(0.89624)))
+          / ((p->winSize * p->winSize) >> 2);
       }
     }
     p->enableWarp = 0;
-    if (csound->GetInputArgAMask(p))
+    if (IS_ASIG_ARG(p->xamp))
       p->arateXamp = ~((int) 0);        /* used as a bit mask */
     else
       p->arateXamp = 0;
@@ -342,15 +363,17 @@ static inline void init_sine_gen(double a, double f, double p, double c,
 
 static int loscilx_opcode_perf(CSOUND *csound, LOSCILX_OPCODE *p)
 {
-    int     i, j;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t i, nsmps = CS_KSMPS;
+    int     j;
     double  frac_d, pidwarp_d = 0.0, c = 0.0;
     MYFLT   frac, ampScale, winFact = p->winFact;
-    long    ndx;
+    int32   ndx;
     int     winSmps;
     float   winBuf[LOSCILX_MAX_INTERP_SIZE];
 
-    if (p->dataPtr == NULL)
-      return csound->PerfError(csound, Str("loscilx: not initialised"));
+    if (UNLIKELY(p->dataPtr == NULL)) goto err1;
 
     if (*(p->kcps) != p->prvKcps) {
       double  f;
@@ -392,9 +415,12 @@ static int loscilx_opcode_perf(CSOUND *csound, LOSCILX_OPCODE *p)
       winFact = (MYFLT) (((double) p->winFact - tmp1) * tmp2 + tmp1);
     }
     ampScale = *(p->xamp) * p->ampScale;
-
-    i = 0;
-    do {
+    if (UNLIKELY(offset)) memset(p->ar, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&p->ar[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (i = offset; i<nsmps; i++) {
 
       frac_d = loscilx_phase_frac(p->curPos);
       frac = (MYFLT) frac_d;
@@ -433,7 +459,7 @@ static int loscilx_opcode_perf(CSOUND *csound, LOSCILX_OPCODE *p)
           MYFLT   a0, a1;
           int     wsized2 = winSmps >> 1;
 
-          ndx += (long) (1 - wsized2);
+          ndx += (int32) (1 - wsized2);
           d = (double) (1 - wsized2) - frac_d;
           j = 0;
           if (p->enableWarp) {              /* ...with window warp enabled */
@@ -478,7 +504,7 @@ static int loscilx_opcode_perf(CSOUND *csound, LOSCILX_OPCODE *p)
           else {                            /* ...with window warp disabled */
             /* avoid division by zero */
             if (frac_d < 0.00001 || frac_d > 0.99999) {
-              ndx += (long) (wsized2 - (frac_d < 0.5 ? 1 : 0));
+              ndx += (int32) (wsized2 - (frac_d < 0.5 ? 1 : 0));
               winSmps = 1;
               winBuf[0] = 1.0f;
             }
@@ -507,7 +533,7 @@ static int loscilx_opcode_perf(CSOUND *csound, LOSCILX_OPCODE *p)
         MYFLT   ar = FL(0.0);
         do {
           ndx++;
-          if ((unsigned long) ndx >= (unsigned long) p->nFrames) {
+          if ((uint32) ndx >= (uint32) p->nFrames) {
             if (!p->loopingWholeFile)
               continue;
             if (ndx < 0L) {
@@ -538,7 +564,7 @@ static int loscilx_opcode_perf(CSOUND *csound, LOSCILX_OPCODE *p)
         MYFLT   ar1 = FL(0.0), ar2 = FL(0.0);
         do {
           ndx++;
-          if ((unsigned long) ndx >= (unsigned long) p->nFrames) {
+          if ((uint32) ndx >= (uint32) p->nFrames) {
             if (!p->loopingWholeFile)
               continue;
             if (ndx < 0L) {
@@ -577,7 +603,7 @@ static int loscilx_opcode_perf(CSOUND *csound, LOSCILX_OPCODE *p)
         } while (++k < p->nChannels);
         do {
           ndx++;
-          if ((unsigned long) ndx >= (unsigned long) p->nFrames) {
+          if ((uint32) ndx >= (uint32) p->nFrames) {
             if (!p->loopingWholeFile)
               continue;
             if (ndx < 0L) {
@@ -593,7 +619,7 @@ static int loscilx_opcode_perf(CSOUND *csound, LOSCILX_OPCODE *p)
           }
 #ifdef USE_DOUBLE
           if (p->usingFtable) {
-            MYFLT *fp = &(((MYFLT*) p->dataPtr)[ndx * (long) p->nChannels]);
+            MYFLT *fp = &(((MYFLT*) p->dataPtr)[ndx * (int32) p->nChannels]);
 
             k = 0;
             do {
@@ -603,7 +629,7 @@ static int loscilx_opcode_perf(CSOUND *csound, LOSCILX_OPCODE *p)
           else
 #endif
           {
-            float *fp = &(((float*) p->dataPtr)[ndx * (long) p->nChannels]);
+            float *fp = &(((float*) p->dataPtr)[ndx * (int32) p->nChannels]);
 
             k = 0;
             do {
@@ -659,19 +685,23 @@ static int loscilx_opcode_perf(CSOUND *csound, LOSCILX_OPCODE *p)
       else
         p->curPos += p->curPosInc;
 
-    } while (++i < csound->ksmps);
+    }
 
     return OK;
+ err1:
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("loscilx: not initialised"));
 }
 
  /* ------------------------------------------------------------------------ */
 
-static OENTRY localops[] = {
-  { "sndload",  sizeof(SNDLOAD_OPCODE), 1,  "",                 "Tooooojjoo",
+static OENTRY loscilx_localops[] = {
+  { "sndload",  sizeof(SNDLOAD_OPCODE), _QQ, 1,  "",                 "iooooojjoo",
     (SUBR) sndload_opcode_init, (SUBR) NULL, (SUBR) NULL                      },
-  { "loscilx",  sizeof(LOSCILX_OPCODE), 5,  "mmmmmmmmmmmmmmmm", "xkToojjoo",
+ { "sndload.S",  sizeof(SNDLOAD_OPCODE), _QQ, 1,  "",                 "Sooooojjoo",
+    (SUBR) sndload_opcode_init_S, (SUBR) NULL, (SUBR) NULL                      },
+  { "loscilx",  sizeof(LOSCILX_OPCODE), TR, 5,  "mmmmmmmmmmmmmmmm", "xkioojjoo",
     (SUBR) loscilx_opcode_init, (SUBR) NULL, (SUBR) loscilx_opcode_perf       }
 };
 
-LINKAGE
-
+LINKAGE_BUILTIN(loscilx_localops)
