@@ -28,25 +28,27 @@
 /*  wavetable, an envelope, and four reso- */
 /*  nances (Non-Sweeping BiQuad Filters).  */
 /*******************************************/
-#include "csdl.h"
+// #include "csdl.h"
+#include "csoundCore.h"
 #include "modal4.h"
 #include "marimba.h"
 #include "vibraphn.h"
 #include <math.h>
-
+#include "interlocks.h"
 static int make_Modal4(CSOUND *csound,
                        Modal4 *m, MYFLT *ifn, MYFLT vgain, MYFLT vrate)
 {
     FUNC        *ftp;
 
-    if ((ftp = csound->FTFind(csound,ifn)) != NULL)
+    if (LIKELY((ftp = csound->FTnp2Find(csound,ifn)) != NULL))
       m->vibr = ftp;
     else {                                              /* Expect sine wave */
-      return csound->PerfError(csound, Str("No table for Modal4 case"));
+     csound->ErrorMsg(csound, Str("No table for Modal4 case"));
+     return NOTOK;
     }
     make_Envelope(&m->envelope);
-        /*  We don't make the excitation wave here yet,   */
-        /*  because we don't know what it's going to be.  */
+        /*  We do not make the excitation wave here yet,   */
+        /*  because we do not know what it's going to be.  */
     make_BiQuad(&m->filters[0]);
     make_BiQuad(&m->filters[1]);
     make_BiQuad(&m->filters[2]);
@@ -81,15 +83,6 @@ static int make_Modal4(CSOUND *csound,
     return OK;
 }
 
-/* void Modal4_clear(Modal4 *m) */
-/* {     */
-/*     OnePole_clear(&m->onepole); */
-/*     BiQuad_clear(&m->filters[0]); */
-/*     BiQuad_clear(&m->filters[1]); */
-/*     BiQuad_clear(&m->filters[2]); */
-/*     BiQuad_clear(&m->filters[3]); */
-/* } */
-
 void Modal4_setFreq(CSOUND *csound, Modal4 *m, MYFLT frequency)
 {
     m->baseFreq = frequency;
@@ -103,12 +96,12 @@ void Modal4_setRatioAndReson(CSOUND *csound,
                              Modal4 *m, int whichOne, MYFLT ratio,MYFLT reson)
 {
     MYFLT temp;
-    if (ratio* m->baseFreq < csound->esr * FL(0.5)) {
+    if (ratio* m->baseFreq < CS_ESR * FL(0.5)) {
       m->ratios[whichOne] = ratio;
     }
     else {
       temp = ratio;
-      while (temp* m->baseFreq > FL(0.5)*csound->esr) temp *= FL(0.5);
+      while (temp* m->baseFreq > FL(0.5)*CS_ESR) temp *= FL(0.5);
       m->ratios[whichOne] = temp;
     }
     m->resons[whichOne] = reson;
@@ -128,7 +121,7 @@ static void Modal4_strike(CSOUND *csound, Modal4 *m, MYFLT amplitude)
     OnePole_setPole(&m->onepole, FL(1.0) - amplitude);
     Envelope_tick(&m->envelope);
     m->w_time = FL(0.0);
-    m->w_lastOutput = FL(0.0);
+    //m->w_lastOutput = FL(0.0);
     m->w_allDone = 0;
                                 /*     wave->reset(); */
     for (i=0;i<4;i++)   {
@@ -156,8 +149,8 @@ static void Modal4_damp(CSOUND *csound, Modal4 *m, MYFLT amplitude)
 static MYFLT Modal4_tick(Modal4 *m)
 {
     MYFLT temp,temp2;
-    long itemp;
-    MYFLT temp_time, alpha;
+    int32 itemp;
+    MYFLT temp_time, alpha, lastOutput;
     int length = (int)m->wave->flen;
 
     m->w_time += m->w_rate;                  /*  Update current time          */
@@ -180,15 +173,15 @@ static MYFLT Modal4_tick(Modal4 *m)
     }
 #endif
 
-    itemp = (long) temp_time;                /* Integer part of time address  */
-    alpha = temp_time - (MYFLT)itemp;     /*  fractional part of time address */
-    m->w_lastOutput = m->wave->ftable[itemp]; /*  Do linear interpolation     */
-    m->w_lastOutput = m->w_lastOutput +      /*  same as alpha*data[temp+1]   */
+    itemp = (int32) temp_time;               /* Integer part of time address  */
+    alpha = temp_time - (MYFLT)itemp;      /* fractional part of time address */
+    lastOutput = m->wave->ftable[itemp];     /*  Do linear interpolation      */
+    lastOutput = lastOutput +                /*  same as alpha*data[temp+1]   */
         (alpha * (m->wave->ftable[itemp+1] -
-                m->w_lastOutput));            /*  + (1-alpha)data[temp]       */
+                  lastOutput));              /*  + (1-alpha)data[temp]        */
 
     temp   = m->masterGain *
-      OnePole_tick(&m->onepole, m->w_lastOutput * Envelope_tick(&m->envelope));
+      OnePole_tick(&m->onepole, lastOutput * Envelope_tick(&m->envelope));
     temp2  = BiQuad_tick(&m->filters[0], temp);
     temp2 += BiQuad_tick(&m->filters[1], temp);
     temp2 += BiQuad_tick(&m->filters[2], temp);
@@ -216,15 +209,15 @@ static MYFLT Modal4_tick(Modal4 *m)
       }
 #endif
 
-      itemp = (long) temp_time;    /*  Integer part of time address    */
+      itemp = (int32) temp_time;    /*  Integer part of time address    */
                                    /*  fractional part of time address */
       alpha = temp_time - (MYFLT)itemp;
-      m->v_lastOutput = m->vibr->ftable[itemp]; /* Do linear interpolation */
+      lastOutput = m->vibr->ftable[itemp]; /* Do linear interpolation */
       /*  same as alpha*data[itemp+1] + (1-alpha)data[temp] */
-      m->v_lastOutput = m->v_lastOutput +
-        (alpha * (m->vibr->ftable[itemp+1] - m->v_lastOutput));
+      lastOutput = /*m->v)*/lastOutput +
+        (alpha * (m->vibr->ftable[itemp+1] - lastOutput));
       /* End of vibrato tick */
-      temp = FL(1.0) + (m->v_lastOutput * m->vibrGain);   /*  Calculate AM      */
+      temp = FL(1.0) + (lastOutput * m->vibrGain);   /*  Calculate AM      */
       temp2 = temp * temp2;                          /* and apply to master out */
     }
 
@@ -248,14 +241,15 @@ int marimbaset(CSOUND *csound, MARIMBA *p)
     int         itemp;
     FUNC        *ftp;
 
-    if ((ftp = csound->FTFind(csound, p->ifn)) != NULL)
+    if (LIKELY((ftp = csound->FTnp2Find(csound, p->ifn)) != NULL))
       p->m4.wave = ftp;
     else {                                    /* Expect an impulslything */
-      return csound->PerfError(csound, Str("No table for Marimba strike"));
+      return csound->InitError(csound, Str("No table for Marimba strike"));
     }
 
-    if (make_Modal4(csound,
-                    m, p->ivfn, *p->vibAmt, *p->vibFreq)==NOTOK) return NOTOK;
+    if (UNLIKELY(make_Modal4(csound,
+                             m, p->ivfn, *p->vibAmt, *p->vibFreq)==NOTOK))
+      return NOTOK;
     p->m4.w_phaseOffset = FL(0.0);
 /*     p->m4.w_rate = 0.5; */
     Modal4_setRatioAndReson(csound,m,0, FL(1.00), FL(0.9996)); /* Set all 132.0 */
@@ -275,12 +269,12 @@ int marimbaset(CSOUND *csound, MARIMBA *p)
     p->m4.masterGain = (FL(0.1) + (FL(1.8) * p->stickHardness));
                                 /* Set Strike position */
     temp2 = p->strikePosition * PI_F;
-    temp = (MYFLT)sin((double)temp2);
+    temp = SIN(temp2);
     BiQuad_setGain(p->m4.filters[0], FL(0.12)*temp); /* 1st mode function of pos.*/
-    temp = (MYFLT)sin(0.05 + (3.9 * (double)temp2));
+    temp = SIN(FL(0.05) + (FL(3.9) * temp2));
     BiQuad_setGain(p->m4.filters[1],
                    -FL(0.03)*temp); /* 2nd mode function of pos.*/
-    temp = (MYFLT)sin(-0.05 + (11.0 * (double)temp2));
+    temp = SIN(-FL(0.05) + (FL(11.0) * temp2));
     BiQuad_setGain(p->m4.filters[2], FL(0.11)*temp); /* 3rd mode function of pos.*/
                                 /* Strike */
     {
@@ -303,13 +297,13 @@ int marimbaset(CSOUND *csound, MARIMBA *p)
     Modal4_setFreq(csound, m, *p->frequency);
     p->first = 1;
     {
-      int relestim = (int) (csound->ekr * *p->dettack);
+      int relestim = (int) (CS_EKR * *p->dettack);
       /* 0.1 second decay extention */
       if (relestim > p->h.insdshead->xtratim)
         p->h.insdshead->xtratim = relestim;
     }
-    p->kloop = (int) ((long) (p->h.insdshead->offtim * csound->ekr)
-                      - (long) (csound->ekr * *p->dettack));
+    p->kloop = (int) ((int32) (p->h.insdshead->offtim * CS_EKR)
+                      - (int32) (CS_EKR * *p->dettack));
     return OK;
 }
 
@@ -317,7 +311,9 @@ int marimba(CSOUND *csound, MARIMBA *p)
 {
     Modal4      *m = &(p->m4);
     MYFLT       *ar = p->ar;
-    int         n,nsmps = csound->ksmps;
+    uint32_t    offset = p->h.insdshead->ksmps_offset;
+    uint32_t    early  = p->h.insdshead->ksmps_no_end;
+    uint32_t    n, nsmps = CS_KSMPS;
     MYFLT       amp = (*p->amplitude) * AMP_RSCALE; /* Normalise */
 
     if (p->kloop>0 && p->h.insdshead->relesing) p->kloop=1;
@@ -326,17 +322,22 @@ int marimba(CSOUND *csound, MARIMBA *p)
     }
     p->m4.v_rate = *p->vibFreq; /* 6.0; */
     p->m4.vibrGain = *p->vibAmt; /* 0.05; */
-    if (p->first) {
+    if (UNLIKELY(p->first)) {
       Modal4_strike(csound, m, *p->amplitude * AMP_RSCALE);
       Modal4_setFreq(csound, m, *p->frequency);
       p->first = 0;
     }
-    for (n=0;n<nsmps;n++) {
+    if (UNLIKELY(offset)) memset(ar, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&ar[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n=offset;n<nsmps;n++) {
       MYFLT     lastOutput;
       if (p->multiStrike>0)
         if (p->m4.w_allDone) {
           p->m4.w_time = FL(0.0);
-          p->m4.w_lastOutput = FL(0.0);
+          //p->m4.w_lastOutput = FL(0.0);
           p->m4.w_allDone = 0;
           p->multiStrike -= 1;
         }
@@ -362,14 +363,14 @@ int vibraphnset(CSOUND *csound, VIBRAPHN *p)
     MYFLT       temp;
     FUNC        *ftp;
 
-    if ((ftp = csound->FTFind(csound, p->ifn)) != NULL)
+    if (LIKELY((ftp = csound->FTnp2Find(csound, p->ifn)) != NULL))
       p->m4.wave = ftp;         /* Expect an impulslything */
     else {
-      return csound->PerfError(csound, Str("No table for Vibraphone strike"));
+      return csound->InitError(csound, Str("No table for Vibraphone strike"));
     }
 
-    if (make_Modal4(csound,
-                    m, p->ivfn, *p->vibAmt, *p->vibFreq)==NOTOK) return NOTOK;
+    if (UNLIKELY(make_Modal4(csound, m, p->ivfn, *p->vibAmt, *p->vibFreq)==NOTOK))
+      return NOTOK;
 
     p->m4.w_phaseOffset = FL(0.0);
 /*     p->m4.w_rate = 13.33; */
@@ -388,10 +389,10 @@ int vibraphnset(CSOUND *csound, VIBRAPHN *p)
     p->m4.masterGain = FL(0.2) + (*p->hardness * FL(1.6));
                                 /* Set Strike position */
     temp = (p->strikePosition = *p->spos) * PI_F;
-    BiQuad_setGain(p->m4.filters[0], FL(0.025) * (MYFLT)sin((double)temp));
+    BiQuad_setGain(p->m4.filters[0], FL(0.025) * SIN(temp));
     BiQuad_setGain(p->m4.filters[1], FL(0.015) *
-                   (MYFLT)sin(0.1 + (2.01 *(double) temp)));
-    BiQuad_setGain(p->m4.filters[2], FL(0.015) * (MYFLT)sin(3.95 * (double)temp));
+                   SIN(FL(0.1) + (FL(2.01) * temp)));
+    BiQuad_setGain(p->m4.filters[2], FL(0.015) * SIN(FL(3.95) * temp));
                                 /* Strike */
     Modal4_strike(csound, m, *p->amplitude * AMP_RSCALE);
     Modal4_setFreq(csound, m, *p->frequency);
@@ -403,21 +404,28 @@ int vibraphn(CSOUND *csound, VIBRAPHN *p)
 {
     Modal4      *m = &(p->m4);
     MYFLT       *ar = p->ar;
-    int         n,nsmps = csound->ksmps;
+    uint32_t    offset = p->h.insdshead->ksmps_offset;
+    uint32_t    early  = p->h.insdshead->ksmps_no_end;
+    uint32_t    n, nsmps = CS_KSMPS;
     MYFLT       amp = (*p->amplitude)*AMP_RSCALE; /* Normalise */
 
     if (p->kloop>0 && p->h.insdshead->relesing) p->kloop=1;
     if ((--p->kloop) == 0) {
       Modal4_damp(csound, m, FL(1.0) - (amp * FL(0.03)));
     }
-    if (p->first) {
+    if (UNLIKELY(p->first)) {
       Modal4_strike(csound, m, *p->amplitude * AMP_RSCALE);
       Modal4_setFreq(csound, m, *p->frequency);
       p->first = 0;
     }
     p->m4.v_rate = *p->vibFreq;
     p->m4.vibrGain =*p->vibAmt;
-    for (n=0;n<nsmps;n++) {
+    if (UNLIKELY(offset)) memset(ar, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&ar[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n=offset;n<nsmps;n++) {
       MYFLT     lastOutput = Modal4_tick(m);
       ar[n] = lastOutput*FL(8.0)*AMP_SCALE;/* Times 8 as seems too quiet */
     }
@@ -444,13 +452,13 @@ int agogobelset(CSOUND *csound, VIBRAPHN *p)
     MYFLT       temp;
 
     /* Expect an impulslything */
-    if ((ftp = csound->FTFind(csound, p->ifn)) != NULL) p->m4.wave = ftp;
+    if (LIKELY((ftp = csound->FTnp2Find(csound, p->ifn)) != NULL)) p->m4.wave = ftp;
     else {
-      return csound->PerfError(csound, Str("No table for Agogobell strike"));
+      return csound->InitError(csound, Str("No table for Agogobell strike"));
     }
 
-    if (make_Modal4(csound,
-                    m, p->ivfn, *p->vibAmt, *p->vibFreq)==NOTOK) return NOTOK;
+    if (UNLIKELY(make_Modal4(csound, m, p->ivfn, *p->vibAmt, *p->vibFreq)==NOTOK))
+      return NOTOK;
 
     p->m4.w_phaseOffset = FL(0.0);
 /*     p->m4.w_rate = 7.0; */
@@ -469,9 +477,9 @@ int agogobelset(CSOUND *csound, VIBRAPHN *p)
     p->m4.masterGain = FL(1.0);
                                 /* Set Strike position */
     temp = (p->strikePosition = *p->spos) * PI_F;
-    BiQuad_setGain(p->m4.filters[0], FL(0.08) * (MYFLT)sin(0.7 * temp));
-    BiQuad_setGain(p->m4.filters[1], FL(0.07) * (MYFLT)sin(0.1 + (5.0 * temp)));
-    BiQuad_setGain(p->m4.filters[2], FL(0.04) * (MYFLT)sin(0.2 + (7.0 * temp)));
+    BiQuad_setGain(p->m4.filters[0], FL(0.08) * SIN(FL(0.7) * temp));
+    BiQuad_setGain(p->m4.filters[1], FL(0.07) * SIN(FL(0.1) + (FL(5.0) * temp)));
+    BiQuad_setGain(p->m4.filters[2], FL(0.04) * SIN(FL(0.2) + (FL(7.0) * temp)));
                                 /* Strike */
     Modal4_strike(csound, m, *p->amplitude*AMP_RSCALE);
     Modal4_setFreq(csound, m, *p->frequency);
@@ -482,16 +490,23 @@ int agogobel(CSOUND *csound, VIBRAPHN *p)
 {
     Modal4      *m = &(p->m4);
     MYFLT       *ar = p->ar;
-    int         n,nsmps = csound->ksmps;
+    uint32_t    offset = p->h.insdshead->ksmps_offset;
+    uint32_t    early  = p->h.insdshead->ksmps_no_end;
+    uint32_t    n, nsmps = CS_KSMPS;
 
     p->m4.v_rate = *p->vibFreq;
     p->m4.vibrGain =*p->vibAmt;
-    if (p->first) {
+    if (UNLIKELY(p->first)) {
       Modal4_strike(csound, m, *p->amplitude * AMP_RSCALE);
       Modal4_setFreq(csound, m, *p->frequency);
       p->first = 0;
     }
-    for (n=0;n<nsmps;n++) {
+    if (UNLIKELY(offset)) memset(ar, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&ar[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n=offset;n<nsmps;n++) {
       MYFLT     lastOutput = Modal4_tick(m);
       ar[n] = lastOutput*AMP_SCALE;
     }
@@ -500,14 +515,14 @@ int agogobel(CSOUND *csound, VIBRAPHN *p)
 
 #define S       sizeof
 
-static OENTRY localops[] = {
-{ "marimba", S(MARIMBA), 5, "a", "kkiiikkiijj",
+static OENTRY modal4_localops[] = {
+{ "marimba", S(MARIMBA), TR, 5, "a", "kkiiikkiijj",
                                  (SUBR)marimbaset, NULL, (SUBR)marimba},
-{ "vibes", S(VIBRAPHN),  5, "a", "kkiiikkii",
+{ "vibes", S(VIBRAPHN),  TR, 5, "a", "kkiiikkii",
                                  (SUBR)vibraphnset,NULL,(SUBR)vibraphn},
-{ "gogobel",S(VIBRAPHN), 5, "a", "kkiiikki",
+{ "gogobel",S(VIBRAPHN), TR, 5, "a", "kkiiikki",
                                  (SUBR)agogobelset,NULL, (SUBR)agogobel},
 };
 
-LINKAGE
+LINKAGE_BUILTIN(modal4_localops)
 
