@@ -63,41 +63,41 @@ PLUS a number of track processing opcodes.
 
 */
 
-#include "csdl.h"
+#include "pvs_ops.h"
 #include "pstream.h"
 
 typedef struct _psyn {
     OPDS    h;
     MYFLT   *out;
     PVSDAT  *fin;
-    MYFLT   *scal, *pitch, *maxtracks, *ftb;
+    MYFLT   *scal, *pitch, *maxtracks, *ftb, *thresh;
     int     tracks, pos, numbins, hopsize;
     FUNC    *func;
     AUXCH   sum, amps, freqs, phases, trackID;
-    MYFLT   factor, facsqr;
+    double   factor, facsqr, min;
 } _PSYN;
 
 typedef struct _psyn2 {
     OPDS    h;
     MYFLT   *out;
     PVSDAT  *fin;
-    MYFLT   *scal, *maxtracks, *ftb;
+    MYFLT   *scal, *maxtracks, *ftb, *thresh;
     int     tracks, pos, numbins, hopsize;
     FUNC    *func;
     AUXCH   sum, amps, freqs, phases, trackID;
-    MYFLT   factor, facsqr;
+    double   factor, facsqr, min;
 } _PSYN2;
 
 static int psynth_init(CSOUND *csound, _PSYN *p)
 {
     int     numbins = p->fin->N / 2 + 1;
 
-    if (p->fin->format != PVS_TRACKS) {
+    if (UNLIKELY(p->fin->format != PVS_TRACKS)) {
       return csound->InitError(csound,
                                Str("psynth: first input not in TRACKS format\n"));
     }
     p->func = csound->FTnp2Find(p->h.insdshead->csound, p->ftb);
-    if (p->func == NULL) {
+    if (UNLIKELY(p->func == NULL)) {
       return csound->InitError(csound, Str("psynth: function table not found\n"));
     }
 
@@ -107,56 +107,71 @@ static int psynth_init(CSOUND *csound, _PSYN *p)
     p->numbins = numbins;
     p->factor = p->hopsize * csound->onedsr;
     p->facsqr = p->factor * p->factor;
+    if(*p->thresh == -1) p->min = 0.00002*csound->Get0dBFS(csound);
+    else p->min = *p->thresh*csound->Get0dBFS(csound);
 
     if (p->amps.auxp == NULL ||
-        (unsigned) p->amps.size < sizeof(MYFLT) * numbins)
-      csound->AuxAlloc(csound, sizeof(MYFLT) * numbins, &p->amps);
+        (unsigned) p->amps.size < sizeof(double) * numbins)
+      csound->AuxAlloc(csound, sizeof(double) * numbins, &p->amps);
+    else
+      memset(p->amps.auxp, 0, sizeof(double) * numbins );
     if (p->freqs.auxp == NULL ||
-        (unsigned) p->freqs.size < sizeof(MYFLT) * numbins)
-      csound->AuxAlloc(csound, sizeof(MYFLT) * numbins, &p->freqs);
+        (unsigned) p->freqs.size < sizeof(double) * numbins)
+      csound->AuxAlloc(csound, sizeof(double) * numbins, &p->freqs);
+    else
+      memset(p->freqs.auxp, 0, sizeof(double) * numbins );
     if (p->phases.auxp == NULL ||
-        (unsigned) p->phases.size < sizeof(MYFLT) * numbins)
-      csound->AuxAlloc(csound, sizeof(MYFLT) * numbins, &p->phases);
+        (unsigned) p->phases.size < sizeof(double) * numbins)
+      csound->AuxAlloc(csound, sizeof(double) * numbins, &p->phases);
+    else
+      memset(p->phases.auxp, 0, sizeof(double) * numbins );
     if (p->sum.auxp == NULL ||
-        (unsigned) p->sum.size < sizeof(MYFLT) * p->hopsize)
-      csound->AuxAlloc(csound, sizeof(MYFLT) * p->hopsize, &p->sum);
+        (unsigned) p->sum.size < sizeof(double) * p->hopsize)
+      csound->AuxAlloc(csound, sizeof(double) * p->hopsize, &p->sum);
+    else
+      memset(p->sum.auxp, 0, sizeof(double) * p->hopsize );
     if (p->trackID.auxp == NULL ||
         (unsigned) p->trackID.size < sizeof(int) * numbins)
       csound->AuxAlloc(csound, sizeof(int) * numbins, &p->trackID);
-
-    memset(p->amps.auxp, 0, sizeof(MYFLT) * numbins );
-     memset(p->freqs.auxp, 0, sizeof(MYFLT) * numbins );
- memset(p->phases.auxp, 0, sizeof(MYFLT) * numbins  );
- memset(p->sum.auxp, 0, sizeof(MYFLT) * p->hopsize );
- memset(p->trackID.auxp, 0, sizeof(int) * numbins );
+    else
+      memset(p->trackID.auxp, 0, sizeof(int) * numbins );
 
     return OK;
 }
 
 static int psynth_process(CSOUND *csound, _PSYN *p)
 {
-    MYFLT   ampnext, amp, freq, freqnext, phase, ratio;
-    MYFLT   a, f, frac, incra, incrph, factor;
+    double  ampnext, amp, freq, freqnext, phase, ratio;
+    double  a, f, frac, incra, incrph, factor;
     MYFLT   scale = *p->scal, pitch = *p->pitch;
     int     ndx, size = p->func->flen;
-    int     i, j, k, n, m, id;
+    int     i, j, k, m, id;
     int     notcontin = 0;
     int     contin = 0;
     int     tracks = p->tracks, maxtracks = (int) *p->maxtracks;
     MYFLT   *tab = p->func->ftable, *out = p->out;
     float   *fin = (float *) p->fin->frame.auxp;
-    int     ksmps = csound->ksmps, pos = p->pos;
-    MYFLT   *amps = (MYFLT *) p->amps.auxp, *freqs = (MYFLT *) p->freqs.auxp;
-    MYFLT   *phases = (MYFLT *) p->phases.auxp, *outsum = (MYFLT *) p->sum.auxp;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
+    int      pos = p->pos;
+    double   *amps = (double *) p->amps.auxp, *freqs = (double *) p->freqs.auxp;
+    double   *phases = (double *) p->phases.auxp;
+    MYFLT    *outsum = (MYFLT *) p->sum.auxp;
     int     *trackID = (int *) p->trackID.auxp;
     int     hopsize = p->hopsize;
-
+    double  min = p->min;
     ratio = size * csound->onedsr;
     factor = p->factor;
 
     maxtracks = p->numbins > maxtracks ? maxtracks : p->numbins;
 
-    for (n = 0; n < ksmps; n++) {
+    if (UNLIKELY(offset)) memset(out, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&out[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n = offset; n < nsmps; n++) {
       out[n] = outsum[pos];
       pos++;
       if (pos == hopsize) {
@@ -165,8 +180,8 @@ static int psynth_process(CSOUND *csound, _PSYN *p)
         i = j = k = 0;
         while (i < maxtracks * 4) {
 
-          ampnext = (MYFLT) fin[i] * scale;
-          freqnext = (MYFLT) fin[i + 1] * pitch;
+          ampnext = (double) fin[i] * scale;
+          freqnext = (double) fin[i + 1] * pitch;
           if ((id = (int) fin[i + 3]) != -1) {
             j = k + notcontin;
 
@@ -185,7 +200,7 @@ static int psynth_process(CSOUND *csound, _PSYN *p)
                 freqnext = freq = freqs[j];
                 phase = phases[j];
                 amp = amps[j];
-                ampnext = 0.0f;
+                ampnext = FL(0.0);
               }
             }
             else {
@@ -193,11 +208,11 @@ static int psynth_process(CSOUND *csound, _PSYN *p)
               contin = 1;
               freq = freqnext;
               phase = -freq * factor;
-              amp = 0.0f;
+              amp = FL(0.0);
 
             }
+            if(amp > min){
             /* interpolation & track synthesis loop */
-
             a = amp;
             f = freq;
             incra = (ampnext - amp) / hopsize;
@@ -215,8 +230,10 @@ static int psynth_process(CSOUND *csound, _PSYN *p)
               a += incra;
               f += incrph;
             }
+            }
             /* keep amp, freq, and phase values for next time */
             if (contin) {
+
               amps[k] = ampnext;
               freqs[k] = freqnext;
               phases[k] = phase;
@@ -232,11 +249,9 @@ static int psynth_process(CSOUND *csound, _PSYN *p)
         }
         pos = 0;
         p->tracks = k;
-        /*csound->Message(csound, "synthesis %d \n",k); */
       }
     }
     p->pos = pos;
-
     return OK;
 }
 
@@ -244,12 +259,12 @@ static int psynth2_init(CSOUND *csound, _PSYN2 *p)
 {
     int     numbins = p->fin->N / 2 + 1;
 
-    if (p->fin->format != PVS_TRACKS) {
+    if (UNLIKELY(p->fin->format != PVS_TRACKS)) {
       return csound->InitError(csound,
                                Str("psynth: first input not in TRACKS format\n"));
     }
     p->func = csound->FTnp2Find(p->h.insdshead->csound, p->ftb);
-    if (p->func == NULL) {
+    if (UNLIKELY(p->func == NULL)) {
       return csound->InitError(csound, Str("psynth: function table not found\n"));
     }
 
@@ -259,71 +274,88 @@ static int psynth2_init(CSOUND *csound, _PSYN2 *p)
     p->numbins = numbins;
     p->factor = p->hopsize * csound->onedsr;
     p->facsqr = p->factor * p->factor;
+    if(*p->thresh == -1) p->min = 0.00002*csound->Get0dBFS(csound);
+    else p->min = *p->thresh*csound->Get0dBFS(csound);
 
-      if (p->amps.auxp == NULL ||
-    (unsigned) p->amps.size < sizeof(MYFLT) * numbins)
-      csound->AuxAlloc(csound, sizeof(MYFLT) * numbins, &p->amps);
-      if (p->freqs.auxp == NULL ||
-      (unsigned) p->freqs.size < sizeof(MYFLT) * numbins)
-      csound->AuxAlloc(csound, sizeof(MYFLT) * numbins, &p->freqs);
-      if (p->phases.auxp == NULL ||
-       (unsigned) p->phases.size < sizeof(MYFLT) * numbins)
-      csound->AuxAlloc(csound, sizeof(MYFLT) * numbins, &p->phases);
-      if (p->sum.auxp == NULL ||
-       (unsigned) p->sum.size < sizeof(MYFLT) * p->hopsize)
-      csound->AuxAlloc(csound, sizeof(MYFLT) * p->hopsize, &p->sum);
-      if (p->trackID.auxp == NULL ||
-       (unsigned) p->trackID.size < sizeof(int) * numbins)
+    if (p->amps.auxp == NULL ||
+        (unsigned) p->amps.size < sizeof(double) * numbins)
+      csound->AuxAlloc(csound, sizeof(double) * numbins, &p->amps);
+    else
+      memset(p->amps.auxp, 0, sizeof(double) * numbins );
+    if (p->freqs.auxp == NULL ||
+        (unsigned) p->freqs.size < sizeof(double) * numbins)
+      csound->AuxAlloc(csound, sizeof(double) * numbins, &p->freqs);
+    else
+      memset(p->freqs.auxp, 0, sizeof(double) * numbins );
+    if (p->phases.auxp == NULL ||
+        (unsigned) p->phases.size < sizeof(double) * numbins)
+      csound->AuxAlloc(csound, sizeof(double) * numbins, &p->phases);
+    else
+      memset(p->phases.auxp, 0, sizeof(double) * numbins  );
+    if (p->sum.auxp == NULL ||
+        (unsigned) p->sum.size < sizeof(double) * p->hopsize)
+      csound->AuxAlloc(csound, sizeof(double) * p->hopsize, &p->sum);
+    else
+      memset(p->sum.auxp, 0, sizeof(double) * p->hopsize );
+    if (p->trackID.auxp == NULL ||
+        (unsigned) p->trackID.size < sizeof(int) * numbins)
       csound->AuxAlloc(csound, sizeof(int) * numbins, &p->trackID);
-
-    memset(p->amps.auxp, 0, sizeof(MYFLT) * numbins );
-     memset(p->freqs.auxp, 0, sizeof(MYFLT) * numbins );
- memset(p->phases.auxp, 0, sizeof(MYFLT) * numbins  );
- memset(p->sum.auxp, 0, sizeof(MYFLT) * p->hopsize );
- memset(p->trackID.auxp, 0, sizeof(int) * numbins );
+    else
+      memset(p->trackID.auxp, 0, sizeof(int) * numbins );
 
     return OK;
 }
 
 static int psynth2_process(CSOUND *csound, _PSYN2 *p)
 {
-    MYFLT   ampnext, amp, freq, freqnext, phase, phasenext, ratio;
+    double   ampnext, amp, freq, freqnext, phase, phasenext;
     double  a2, a3, cph;
-    MYFLT   phasediff, facsqr, ph;
-    MYFLT   a, frac, incra, incrph, factor, lotwopi, cnt;
+    double   phasediff, facsqr, ph;
+    double   a, frac, incra, incrph, factor, lotwopi, cnt;
     MYFLT   scale = *p->scal;
     int     ndx, size = p->func->flen;
-    int     i, j, k, n, m, id;
+    int     i=0, j, k, m, id;
     int     notcontin = 0;
     int     contin = 0;
     int     tracks = p->tracks, maxtracks = (int) *p->maxtracks;
     MYFLT   *tab = p->func->ftable, *out = p->out;
     float   *fin = (float *) p->fin->frame.auxp;
-    int     ksmps = csound->ksmps, pos = p->pos;
-    MYFLT   *amps = (MYFLT *) p->amps.auxp, *freqs = (MYFLT *) p->freqs.auxp;
-    MYFLT   *phases = (MYFLT *) p->phases.auxp, *outsum = (MYFLT *) p->sum.auxp;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
+    int      pos = p->pos;
+    double   *amps = (double *) p->amps.auxp, *freqs = (double *) p->freqs.auxp;
+    double   *phases = (double *) p->phases.auxp;
+    MYFLT   *outsum = (MYFLT *) p->sum.auxp;
     int     *trackID = (int *) p->trackID.auxp;
     int     hopsize = p->hopsize;
+    double  min = p->min;
 
     incrph = csound->onedsr;
-    lotwopi = (MYFLT)(size) / TWOPI_F;
-    ratio = size * csound->onedsr;
+    lotwopi = (double)(size) / TWOPI_F;
     factor = p->factor;
     facsqr = p->facsqr;
     maxtracks = p->numbins > maxtracks ? maxtracks : p->numbins;
 
-    for (n = 0; n < ksmps; n++) {
+    if (UNLIKELY(offset)) memset(out, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&out[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n = offset; n < nsmps; n++) {
       out[n] = outsum[pos];
       pos++;
       if (pos == hopsize) {
+
         memset(outsum, 0, sizeof(MYFLT) * hopsize);
         /* for each track */
         i = j = k = 0;
         while (i < maxtracks * 4) {
-          ampnext = (MYFLT) fin[i] * scale;
-          freqnext = (MYFLT) fin[i + 1] * TWOPI_F;
-          phasenext = (MYFLT) fin[i + 2];
+          ampnext = (double) fin[i] * scale;
+          freqnext = (double) fin[i + 1] * TWOPI_F;
+          phasenext = (double) fin[i + 2];
           if ((id = (int) fin[i + 3]) != -1) {
+
             j = k + notcontin;
 
             if (k < tracks - notcontin) {
@@ -341,24 +373,26 @@ static int psynth2_process(CSOUND *csound, _PSYN2 *p)
                 phase = phases[j];
                 phasenext = phase + freq * factor;
                 amp = amps[j];
-                ampnext = 0.0f;
+                ampnext = FL(0.0);
               }
             }
             else {
               /* new track */
               contin = 1;
+              goto cont;
+#if 0
               freq = freqnext;
               phase = phasenext - freq * factor;
-              amp = 0.0f;
+              amp = FL(0.0);
+#endif
             }
+            if(amp > min){
             /* phasediff */
             phasediff = phasenext - phase;
-
             while (phasediff >= PI_F)
               phasediff -= TWOPI_F;
             while (phasediff < -PI_F)
               phasediff += TWOPI_F;
-
             /* update phasediff to match the freq */
             cph = ((freq + freqnext) * factor * 0.5 - phasediff) / TWOPI;
             phasediff += TWOPI_F * (int) (cph + 0.5);
@@ -385,7 +419,9 @@ static int psynth2_process(CSOUND *csound, _PSYN2 *p)
               cnt += incrph;
               ph = phase + cnt * (freq + cnt * (a2 + a3 * cnt));
             }
+           }
             /* keep amp, freq, and phase values for next time */
+          cont:
             if (contin) {
               amps[k] = ampnext;
               freqs[k] = freqnext;
@@ -399,10 +435,12 @@ static int psynth2_process(CSOUND *csound, _PSYN2 *p)
           }
           else
             break;
+
         }
         pos = 0;
         p->tracks = k;
       }
+
     }
     p->pos = pos;
 
@@ -411,32 +449,41 @@ static int psynth2_process(CSOUND *csound, _PSYN2 *p)
 
 static int psynth3_process(CSOUND *csound, _PSYN *p)
 {
-    MYFLT   ampnext, amp, freq, freqnext, phase, phasenext, ratio;
-    double  a2, a3, cph;
-    MYFLT   phasediff, facsqr, ph;
-    MYFLT   a, frac, incra, incrph, factor, lotwopi, cnt;
+    double   ampnext, amp, freq, freqnext, phase, phasenext;
+    double  a2, a3, cph=0.0;
+    double   phasediff, facsqr, ph;
+    double   a, frac, incra, incrph, factor, lotwopi, cnt;
     MYFLT   scale = *p->scal, pitch = *p->pitch;
     int     ndx, size = p->func->flen;
-    int     i, j, k, n, m, id;
+    int     i, j, k, m, id;
     int     notcontin = 0;
     int     contin = 0;
     int     tracks = p->tracks, maxtracks = (int) *p->maxtracks;
     MYFLT   *tab = p->func->ftable, *out = p->out;
     float   *fin = (float *) p->fin->frame.auxp;
-    int     ksmps = csound->ksmps, pos = p->pos;
-    MYFLT   *amps = (MYFLT *) p->amps.auxp, *freqs = (MYFLT *) p->freqs.auxp;
-    MYFLT   *phases = (MYFLT *) p->phases.auxp, *outsum = (MYFLT *) p->sum.auxp;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
+    int      pos = p->pos;
+    double   *amps = (double *) p->amps.auxp, *freqs = (double *) p->freqs.auxp;
+    double   *phases = (double *) p->phases.auxp;
+    MYFLT    *outsum = (MYFLT *) p->sum.auxp;
     int     *trackID = (int *) p->trackID.auxp;
     int     hopsize = p->hopsize;
+    double  min = p->min;
 
     incrph = csound->onedsr;
-    lotwopi = (MYFLT) (size) / TWOPI_F;
-    ratio = size * csound->onedsr;
+    lotwopi = (double) (size) / TWOPI_F;
     factor = p->factor;
     facsqr = p->facsqr;
     maxtracks = p->numbins > maxtracks ? maxtracks : p->numbins;
 
-    for (n = 0; n < ksmps; n++) {
+    if (UNLIKELY(offset)) memset(out, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&out[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n = offset; n < nsmps; n++) {
       out[n] = outsum[pos];
       pos++;
       if (pos == hopsize) {
@@ -444,9 +491,9 @@ static int psynth3_process(CSOUND *csound, _PSYN *p)
         /* for each track */
         i = j = k = 0;
         while (i < maxtracks * 4) {
-          ampnext = (MYFLT) fin[i] * scale;
-          freqnext = (MYFLT) fin[i + 1] * TWOPI_F * pitch;
-          phasenext = (MYFLT) fin[i + 2];
+          ampnext = (double) fin[i] * scale;
+          freqnext = (double) fin[i + 1] * TWOPI_F * pitch;
+          phasenext = (double) fin[i + 2];
           if ((id = (int) fin[i + 3]) != -1) {
             j = k + notcontin;
 
@@ -465,7 +512,7 @@ static int psynth3_process(CSOUND *csound, _PSYN *p)
                 phase = phases[j];
                 phasenext = phase + freq * factor;
                 amp = amps[j];
-                ampnext = 0.0f;
+                ampnext = FL(0.0);
               }
             }
             else {
@@ -473,11 +520,11 @@ static int psynth3_process(CSOUND *csound, _PSYN *p)
               contin = 1;
               freq = freqnext;
               phase = phasenext - freq * factor;
-              amp = 0.0f;
+              amp = FL(0.0);
             }
+            if(amp > min){
             /* phasediff */
             phasediff = phasenext - phase;
-
             while (phasediff >= PI_F)
               phasediff -= TWOPI_F;
             while (phasediff < -PI_F)
@@ -509,6 +556,7 @@ static int psynth3_process(CSOUND *csound, _PSYN *p)
               cnt += incrph;
               ph = phase + cnt * (freq + cnt * (a2 + a3 * cnt));
             }
+            }
             /* keep amp, freq, and phase values for next time */
             if (contin) {
               amps[k] = ampnext;
@@ -554,7 +602,7 @@ static int trans_init(CSOUND *csound, _PTRANS *p)
 {
     int     numbins;
 
-    if (p->fin->format != PVS_TRACKS) {
+    if (UNLIKELY(p->fin->format != PVS_TRACKS)) {
       return csound->InitError(csound, Str("Input not in TRACKS format\n"));
     }
 
@@ -578,7 +626,7 @@ static int trscale_process(CSOUND *csound, _PTRANS *p)
 {
     MYFLT   scale = *p->kpar;
     MYFLT   gain = (p->kgain != NULL ? *p->kgain : FL(1.0));
-    MYFLT   nyq = csound->esr * FL(0.5);
+    MYFLT   nyq = CS_ESR * FL(0.5);
     float   *framein = (float *) p->fin->frame.auxp;
     float   *frameout = (float *) p->fout->frame.auxp;
     int     i = 0, id = (int) framein[3], end = p->numbins * 4;
@@ -607,7 +655,7 @@ static int trshift_process(CSOUND *csound, _PTRANS *p)
 {
     MYFLT   shift = *p->kpar;
     MYFLT   gain = (p->kgain != NULL ? *p->kgain : FL(1.0));
-    MYFLT   nyq = csound->esr * FL(0.5);
+    MYFLT   nyq = CS_ESR * FL(0.5);
     float   *framein = (float *) p->fin->frame.auxp;
     float   *frameout = (float *) p->fout->frame.auxp;
     int     i = 0, id = (int) framein[3], end = p->numbins * 4;
@@ -647,7 +695,7 @@ static int trlowest_init(CSOUND *csound, _PLOW *p)
 {
     int     numbins;
 
-    if (p->fin->format != PVS_TRACKS) {
+    if (UNLIKELY(p->fin->format != PVS_TRACKS)) {
       return csound->InitError(csound, Str("Input not in TRACKS format\n"));
     }
 
@@ -670,7 +718,7 @@ static int trlowest_init(CSOUND *csound, _PLOW *p)
 static int trlowest_process(CSOUND *csound, _PLOW *p)
 {
     MYFLT   scale = *p->kpar;
-    MYFLT   nyq = csound->esr * FL(0.5);
+    MYFLT   nyq = CS_ESR * FL(0.5);
     float   lowest = (float) nyq, outamp = 0.0f, outph = 0.0f, outid = -1.0f;
     float   *framein = (float *) p->fin->frame.auxp;
     float   *frameout = (float *) p->fout->frame.auxp;
@@ -752,7 +800,7 @@ static int trsplit_init(CSOUND *csound, _PSPLIT *p)
 {
     int     numbins;
 
-    if (p->fsig3->format != PVS_TRACKS) {
+    if (UNLIKELY(p->fsig3->format != PVS_TRACKS)) {
       return csound->InitError(csound, Str("trsplit: input not "
                                            "in TRACKS format\n"));
     }
@@ -847,12 +895,12 @@ static int trmix_init(CSOUND *csound, _PSMIX *p)
 {
     int     numbins;
 
-    if (p->fsig2->format != PVS_TRACKS) {
+    if (UNLIKELY(p->fsig2->format != PVS_TRACKS)) {
       return csound->InitError(csound,
                                Str("trmix: first input not in TRACKS format\n"));
     }
 
-    if (p->fsig3->format != PVS_TRACKS) {
+    if (UNLIKELY(p->fsig3->format != PVS_TRACKS)) {
       return csound->InitError(csound,
                                Str("trmix: second input not in TRACKS format\n"));
     }
@@ -878,7 +926,7 @@ static int trmix_process(CSOUND *csound, _PSMIX *p)
     float   *framein2 = (float *) p->fsig3->frame.auxp;
     float   *frameout = (float *) p->fsig1->frame.auxp;
     float   *framein1 = (float *) p->fsig2->frame.auxp;
-    int     i = 0, j = 0, k = 0, id = (int) framein1[3], end = p->numbins * 4;
+    int     i = 0, j = 0, id = (int) framein1[3], end = p->numbins * 4;
 
     if (p->lastframe < p->fsig2->framecount) {
 
@@ -890,7 +938,6 @@ static int trmix_process(CSOUND *csound, _PSMIX *p)
         i += 4;
         id = (int) framein1[i + 3];
       }
-      k = i;
       id = (int) framein2[3];
       while (id != -1 && i < end && j < end) {
         frameout[i] = framein2[j];
@@ -905,7 +952,6 @@ static int trmix_process(CSOUND *csound, _PSMIX *p)
         frameout[i + 3] = -1.0f;
       p->fsig1->framecount = p->lastframe = p->fsig2->framecount;
 
-/*csound->Message(csound, "mix %d : %d\n", k/4, j/4);*/
     }
 
     return OK;
@@ -927,12 +973,12 @@ static int trfil_init(CSOUND *csound, _PSFIL *p)
 {
     int     numbins;
 
-    if (p->fin->format != PVS_TRACKS) {
+    if (UNLIKELY(p->fin->format != PVS_TRACKS)) {
       return csound->InitError(csound,
                                Str("trfil: input not in TRACKS format\n"));
     }
     p->tab = csound->FTnp2Find(csound, p->ifn);
-    if (p->tab == NULL) {
+    if (UNLIKELY(p->tab == NULL)) {
       return csound->InitError(csound,
                                Str("trfil: could not find function table\n"));
     }
@@ -956,7 +1002,7 @@ static int trfil_init(CSOUND *csound, _PSFIL *p)
 static int trfil_process(CSOUND *csound, _PSFIL *p)
 {
     MYFLT   amnt = *p->kpar, gain = FL(1.0);
-    MYFLT   nyq = csound->esr * FL(0.5);
+    MYFLT   nyq = CS_ESR * FL(0.5);
     MYFLT   *fil = p->tab->ftable;
     float   *framein = (float *) p->fin->frame.auxp;
     float   *frameout = (float *) p->fout->frame.auxp;
@@ -974,8 +1020,8 @@ static int trfil_process(CSOUND *csound, _PSFIL *p)
         fr = framein[i + 1];
         if (fr > nyq)
           fr = nyq;
-        if (fr < 0)
-          fr = -fr;
+        //if (fr < 0)
+        fr = FABS(fr);
         pos = fr * len / nyq;
         posi = (int) pos;
         frac = pos - posi;
@@ -1011,12 +1057,12 @@ static int trcross_init(CSOUND *csound, _PSCROSS *p)
 {
     int     numbins;
 
-    if (p->fsig2->format != PVS_TRACKS) {
+    if (UNLIKELY(p->fsig2->format != PVS_TRACKS)) {
       return csound->InitError(csound,
                                Str("trmix: first input not in TRACKS format\n"));
     }
 
-    if (p->fsig3->format != PVS_TRACKS) {
+    if (UNLIKELY(p->fsig3->format != PVS_TRACKS)) {
       return csound->InitError(csound,
                                Str("trmix: second input not in TRACKS format\n"));
     }
@@ -1119,7 +1165,7 @@ static int binit_init(CSOUND *csound, _PSBIN *p)
 {
     int     numbins, N;
 
-    if (p->fsig2->format != PVS_TRACKS) {
+    if (UNLIKELY(p->fsig2->format != PVS_TRACKS)) {
       return csound->InitError(csound,
                                Str("binit: first input not in TRACKS format\n"));
     }
@@ -1148,8 +1194,8 @@ static int binit_process(CSOUND *csound, _PSBIN *p)
     float   *framein = (float *) p->fsig2->frame.auxp;
     int     i = 0, n = 0, id = (int) framein[3], end = p->numbins * 4;
     int     maxi = -1;
-    MYFLT   bw = csound->esr / (MYFLT)N, boundup, boundown;
-    MYFLT   nyq = csound->esr * FL(0.5), centre;
+    MYFLT   bw = CS_ESR / (MYFLT)N, boundup, boundown;
+    MYFLT   nyq = CS_ESR * FL(0.5), centre;
 
     if (p->lastframe < p->fsig2->framecount) {
 
@@ -1188,48 +1234,47 @@ static int binit_process(CSOUND *csound, _PSBIN *p)
     return OK;
 }
 
-static OENTRY localops[] = 
+static OENTRY localops[] =
   {
-    {"tradsyn", sizeof(_PSYN), 5, "a", "fkkki", (SUBR) psynth_init, NULL,
+    {"tradsyn", sizeof(_PSYN),0,  5, "a", "fkkkij", (SUBR) psynth_init, NULL,
      (SUBR) psynth_process}
     ,
-    {"sinsyn", sizeof(_PSYN2), 5, "a", "fkki", (SUBR) psynth2_init, NULL,
+    {"sinsyn", sizeof(_PSYN2), TR, 5, "a", "fkkij", (SUBR) psynth2_init, NULL,
      (SUBR) psynth2_process}
     ,
-    {"resyn", sizeof(_PSYN), 5, "a", "fkkki", (SUBR) psynth_init, NULL,
+    {"resyn", sizeof(_PSYN), TR, 5, "a", "fkkkij", (SUBR) psynth_init, NULL,
      (SUBR) psynth3_process}
     ,
-    {"trscale", sizeof(_PTRANS), 3, "f", "fz", (SUBR) trans_init,
+    {"trscale", sizeof(_PTRANS),0,  3, "f", "fz", (SUBR) trans_init,
      (SUBR) trscale_process}
     ,
-    {"trshift", sizeof(_PTRANS), 3, "f", "fz", (SUBR) trans_init,
+    {"trshift", sizeof(_PTRANS),0,  3, "f", "fz", (SUBR) trans_init,
      (SUBR) trshift_process}
     ,
-    {"trsplit", sizeof(_PSPLIT), 3, "ff", "fz", (SUBR) trsplit_init,
+    {"trsplit", sizeof(_PSPLIT),0,  3, "ff", "fz", (SUBR) trsplit_init,
      (SUBR) trsplit_process}
     ,
-    {"trmix", sizeof(_PSMIX), 3, "f", "ff", (SUBR) trmix_init,
+    {"trmix", sizeof(_PSMIX),0,  3, "f", "ff", (SUBR) trmix_init,
      (SUBR) trmix_process}
     ,
-    {"trlowest", sizeof(_PLOW), 3, "fkk", "fk", (SUBR) trlowest_init,
+    {"trlowest", sizeof(_PLOW),0,  3, "fkk", "fk", (SUBR) trlowest_init,
      (SUBR) trlowest_process}
     ,
-    {"trhighest", sizeof(_PLOW), 3, "fkk", "fk", (SUBR) trlowest_init,
+    {"trhighest", sizeof(_PLOW),0,  3, "fkk", "fk", (SUBR) trlowest_init,
      (SUBR) trhighest_process}
     ,
-    {"trfilter", sizeof(_PSFIL), 3, "f", "fki", (SUBR) trfil_init,
+    {"trfilter", sizeof(_PSFIL),0,  3, "f", "fki", (SUBR) trfil_init,
      (SUBR) trfil_process}
     ,
-    {"trcross", sizeof(_PSCROSS), 3, "f", "ffkz", (SUBR) trcross_init,
+    {"trcross", sizeof(_PSCROSS),0,  3, "f", "ffkz", (SUBR) trcross_init,
      (SUBR) trcross_process}
     ,
-    {"binit", sizeof(_PSBIN), 3, "f", "fi", (SUBR) binit_init,
+    {"binit", sizeof(_PSBIN),0,  3, "f", "fi", (SUBR) binit_init,
      (SUBR) binit_process}
   };
 
 int psynth_init_(CSOUND *csound)
 {
   return csound->AppendOpcodes(csound, &(localops[0]),
-			       (int) (sizeof(localops) / sizeof(OENTRY)));
+                               (int) (sizeof(localops) / sizeof(OENTRY)));
 }
-
