@@ -21,52 +21,63 @@
     02111-1307 USA
 */
 
-#include "csdl.h"   /*                                      UGENS9.C        */
+#include "stdopcod.h"   /*                                      UGENS9.C        */
 #include <math.h>
 #include "convolve.h"
 #include "ugens9.h"
 #include "soundio.h"
-#include "oload.h"
 
-static int cvset(CSOUND *csound, CONVOLVE *p)
+static int cvset_(CSOUND *csound, CONVOLVE *p, int stringname)
 {
     char     cvfilnam[MAXNAME];
     MEMFIL   *mfp;
-    MYFLT *fltp;
+    MYFLT    *fltp;
     CVSTRUCT *cvh;
-    int i, siz;
-    long     Hlenpadded = 1, obufsiz, Hlen;
-    int      nchanls;
+    int       siz;
+    int32     Hlenpadded = 1, obufsiz, Hlen;
+    uint32_t  nchanls;
+    uint32_t  nsmps = CS_KSMPS;
 
-    if (csound->oparms->odebug)
+    if (UNLIKELY(csound->oparms->odebug))
       csound->Message(csound, CONVOLVE_VERSION_STRING);
 
-    csound->strarg2name(csound, cvfilnam, p->ifilno, "convolve.", p->XSTRCODE);
+    if (stringname==0){
+      if (csound->ISSTRCOD(*p->ifilno))
+        strncpy(cvfilnam,get_arg_string(csound, *p->ifilno), MAXNAME-1);
+      else csound->strarg2name(csound, cvfilnam,p->ifilno, "convolve.",0);
+    }
+    else strncpy(cvfilnam, ((STRINGDAT *)p->ifilno)->data, MAXNAME-1);
+
     if ((mfp = p->mfp) == NULL || strcmp(mfp->filename, cvfilnam) != 0) {
       /* if file not already readin */
-      if ((mfp = csound->ldmemfile2(csound, cvfilnam, CSFTYPE_CVANAL)) == NULL) {
-        return csound->InitError(csound, Str("CONVOLVE cannot load %s"), cvfilnam);
+      if (UNLIKELY((mfp = csound->ldmemfile2withCB(csound, cvfilnam,
+                                                   CSFTYPE_CVANAL,NULL))
+                   == NULL)) {
+        return csound->InitError(csound,
+                                 Str("CONVOLVE cannot load %s"), cvfilnam);
       }
     }
     cvh = (CVSTRUCT *)mfp->beginp;
-    if (cvh->magic != CVMAGIC) {
-      return csound->InitError(csound, Str("%s not a CONVOLVE file (magic %ld)"),
+    if (UNLIKELY(cvh->magic != CVMAGIC)) {
+      return csound->InitError(csound,
+                               Str("%s not a CONVOLVE file (magic %ld)"),
                                cvfilnam, cvh->magic);
     }
 
     nchanls = (cvh->channel == ALLCHNLS ? cvh->src_chnls : 1);
 
     if (*p->channel == FL(0.0)) {
-      if (p->OUTOCOUNT == nchanls)
+      if (LIKELY(p->OUTOCOUNT == nchanls))
         p->nchanls = nchanls;
       else {
-        return csound->InitError(csound, Str("CONVOLVE: output channels not equal "
-                                             "to number of channels in source"));
+        return csound->InitError(csound,
+                                 Str("CONVOLVE: output channels not equal "
+                                     "to number of channels in source"));
       }
     }
     else {
       if (*p->channel <= nchanls) {
-        if (p->OUTOCOUNT != 1) {
+        if (UNLIKELY(p->OUTOCOUNT != 1)) {
           return csound->InitError(csound,
                                    Str("CONVOLVE: output channels not equal "
                                         "to number of channels in source"));
@@ -88,46 +99,59 @@ static int cvset(CSOUND *csound, CONVOLVE *p)
     if ((p->nchanls == 1) && (*p->channel > 0))
       p->H += (Hlenpadded + 2) * (int)(*p->channel - 1);
 
-    if (cvh->samplingRate != csound->esr) {
+    if (UNLIKELY(cvh->samplingRate != CS_ESR)) {
       /* & chk the data */
       csound->Warning(csound, Str("%s's srate = %8.0f, orch's srate = %8.0f"),
-                              cvfilnam, cvh->samplingRate, csound->esr);
+                              cvfilnam, cvh->samplingRate, CS_ESR);
     }
-    if (cvh->dataFormat != CVMYFLT) {
+    if (UNLIKELY(cvh->dataFormat != CVMYFLT)) {
       return csound->InitError(csound,
-                               Str("unsupported CONVOLVE data format %ld in %s"),
+                               Str("unsupported CONVOLVE data "
+                                   "format %ld in %s"),
                                cvh->dataFormat, cvfilnam);
     }
 
     /* Determine size of circular output buffer */
-    if (Hlen >= csound->ksmps)
-      obufsiz = (long) ceil((double) Hlen / (double) csound->ksmps)
-                * csound->ksmps;
+    if (Hlen >= (int32)nsmps)
+      obufsiz = (int32) CEIL((MYFLT) Hlen / nsmps) * nsmps;
     else
-      obufsiz = (long) ceil((double) csound->ksmps / (double) Hlen) * Hlen;
+      obufsiz = (int32) CEIL(CS_KSMPS / (MYFLT) Hlen) * Hlen;
 
     siz = ((Hlenpadded + 2) + p->nchanls * ((Hlen - 1) + obufsiz)
               + (p->nchanls > 1 ? (Hlenpadded + 2) : 0));
     if (p->auxch.auxp == NULL || p->auxch.size < siz*sizeof(MYFLT)) {
       /* if no buffers yet, alloc now */
-      csound->AuxAlloc(csound, (long) siz*sizeof(MYFLT), &p->auxch);
+      csound->AuxAlloc(csound, (size_t) siz*sizeof(MYFLT), &p->auxch);
       fltp = (MYFLT *) p->auxch.auxp;
       p->fftbuf = fltp;   fltp += (Hlenpadded + 2); /* and insert addresses */
       p->olap = fltp;     fltp += p->nchanls*(Hlen - 1);
       p->outbuf = fltp;   fltp += p->nchanls*obufsiz;
       p->X  = fltp;
     }
-    fltp = (MYFLT *) p->auxch.auxp;
-    for(i=0; i < siz; i++) fltp[i] = FL(0.0);
-
+    else {
+      fltp = (MYFLT *) p->auxch.auxp;
+      memset(fltp, 0, sizeof(MYFLT)*siz);
+    /* for(i=0; i < siz; i++) fltp[i] = FL(0.0); */
+    }
     p->obufsiz = obufsiz;
     p->outcnt = obufsiz;
     p->incount = 0;
     p->obufend = p->outbuf + obufsiz - 1;
     p->outhead = p->outail = p->outbuf;
+    p->fwdsetup = csound->RealFFT2Setup(csound, Hlenpadded, FFT_FWD);
+    p->invsetup = csound->RealFFT2Setup(csound, Hlenpadded, FFT_INV);
     return OK;
 }
 
+static int cvset(CSOUND *csound, CONVOLVE *p){
+  return cvset_(csound,p,0);
+
+}
+
+static int cvset_S(CSOUND *csound, CONVOLVE *p){
+  return cvset_(csound,p,1);
+
+}
 /* Write from a circular buffer into a linear output buffer without
    clearing data
    UPDATES SOURCE & DESTINATION POINTERS TO REFLECT NEW POSITIONS */
@@ -136,11 +160,11 @@ static void writeFromCircBuf(
     MYFLT   **dst,              /* Circular source and linear destination */
     MYFLT   *sceStart,
     MYFLT   *sceEnd,            /* Address of start & end of source buffer */
-    long    numToDo)            /* How many points to write (<= circBufSize) */
+    int32    numToDo)            /* How many points to write (<= circBufSize) */
 {
     MYFLT   *srcindex = *sce;
     MYFLT   *dstindex = *dst;
-    long    breakPoint;     /* how many points to add before having to wrap */
+    int32    breakPoint;     /* how many points to add before having to wrap */
 
     breakPoint = sceEnd - srcindex + 1;
     if (numToDo >= breakPoint) { /*  we will do 2 in 1st loop, rest in 2nd. */
@@ -160,23 +184,26 @@ static void writeFromCircBuf(
 
 static int convolve(CSOUND *csound, CONVOLVE *p)
 {
-    int    nsmpso=csound->ksmps,nsmpsi=csound->ksmps,nsmpso_sav,outcnt_sav;
+    int    nsmpso=CS_KSMPS,nsmpsi=CS_KSMPS,outcnt_sav;
     int    nchm1 = p->nchanls - 1,chn;
-    long   i,j;
+    int32  i,j;
     MYFLT  *ar[4];
     MYFLT  *ai = p->ain;
     MYFLT  *fftbufind;
-    long   outcnt = p->outcnt;
-    long   incount=p->incount;
-    long   Hlen = p->Hlen;
-    long   Hlenm1 = Hlen - 1;
-    long   obufsiz = p->obufsiz;
+    int32  outcnt = p->outcnt;
+    int32  incount=p->incount;
+    int32  Hlen = p->Hlen;
+    int32  Hlenm1 = Hlen - 1;
+    int32  obufsiz = p->obufsiz;
     MYFLT  *outhead = NULL;
     MYFLT  *outail = p->outail;
     MYFLT  *olap;
     MYFLT  *X;
-    long   Hlenpadded = p->Hlenpadded;
+    int32  Hlenpadded = p->Hlenpadded;
     MYFLT  scaleFac;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t nn,nsmpso_sav;
 
     scaleFac = csound->GetInverseRealFFTScale(csound, (int) Hlenpadded);
     ar[0] = p->ar1;
@@ -184,15 +211,13 @@ static int convolve(CSOUND *csound, CONVOLVE *p)
     ar[2] = p->ar3;
     ar[3] = p->ar4;
 
-    if (p->auxch.auxp==NULL) {
-      return csound->PerfError(csound, Str("convolve: not initialised"));
-    }
+    if (p->auxch.auxp==NULL) goto err1;
   /* First dump as much pre-existing audio in output buffer as possible */
     if (outcnt > 0) {
-      if (outcnt <= csound->ksmps)
+      if (outcnt <= (int)CS_KSMPS)
         i = outcnt;
       else
-        i = csound->ksmps;
+        i = CS_KSMPS;
       nsmpso -= i; outcnt -= i;
       for (chn = nchm1;chn >= 0;chn--) {
         outhead = p->outhead + chn*obufsiz;
@@ -211,14 +236,19 @@ static int convolve(CSOUND *csound, CONVOLVE *p)
         i = Hlen - incount;
       nsmpsi -= i;
       incount += i;
-      while (i--)
-        *fftbufind++ = scaleFac * *ai++;
+      nsmpso_sav = CS_KSMPS-early;
+      for (nn=0; i>0; nn++, i--) {
+        if (nn<offset|| nn>(uint32_t)nsmpso_sav)
+          *fftbufind++ = FL(0.0);
+        else
+          *fftbufind++ = scaleFac * ai[nn];
+      }
       if (incount == Hlen) {
         /* We have enough audio for a convolution. */
         incount = 0;
         /* FFT the input (to create X) */
         /*csound->Message(csound, "CONVOLVE: ABOUT TO FFT\n"); */
-        csound->RealFFT(csound, p->fftbuf, (int) Hlenpadded);
+        csound->RealFFT2(csound, p->fwdsetup, p->fftbuf);
         p->fftbuf[Hlenpadded] = p->fftbuf[1];
         p->fftbuf[1] = p->fftbuf[Hlenpadded + 1L] = FL(0.0);
         /* save the result if multi-channel */
@@ -261,7 +291,7 @@ static int convolve(CSOUND *csound, CONVOLVE *p)
 
           p->fftbuf[1] = p->fftbuf[Hlenpadded];
           p->fftbuf[Hlenpadded] = p->fftbuf[Hlenpadded + 1L] = FL(0.0);
-          csound->InverseRealFFT(csound, p->fftbuf, (int) Hlenpadded);
+          csound->RealFFT2(csound, p->invsetup, p->fftbuf);
 
           /* Take the first Hlen output samples and output them to
              either the real audio output buffer or the local circular
@@ -332,6 +362,9 @@ static int convolve(CSOUND *csound, CONVOLVE *p)
     p->outhead = outhead;
     p->outail = outail;
     return OK;
+ err1:
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("convolve: not initialised"));
 }
 
 /* partitioned (low latency) overlap-save convolution.
@@ -343,13 +376,13 @@ static int convolve(CSOUND *csound, CONVOLVE *p)
    allow this opcode to accept .con files.
    -ma++ april 2004 */
 
-static int pconvset(CSOUND *csound, PCONVOLVE *p)
+static int pconvset_(CSOUND *csound, PCONVOLVE *p, int stringname)
 {
     int     channel = (*(p->channel) <= 0 ? ALLCHNLS : (int) *(p->channel));
     SNDFILE *infd;
     SOUNDIN IRfile;
     MYFLT   *inbuf, *fp1,*fp2;
-    long    i, j, read_in, part;
+    int32    i, j, read_in, part;
     MYFLT   *IRblock;
     MYFLT   ainput_dur, scaleFac;
     MYFLT   partitionSize;
@@ -358,45 +391,52 @@ static int pconvset(CSOUND *csound, PCONVOLVE *p)
     memset(&IRfile, 0, sizeof(SOUNDIN));
     /* open impulse response soundfile [code derived from SAsndgetset()] */
     IRfile.skiptime = FL(0.0);
-    csound->strarg2name(csound, IRfile.sfname, p->ifilno, "soundin.",
-                                p->XSTRCODE);
+
+     if (stringname==0){
+      if (csound->ISSTRCOD(*p->ifilno))
+        strncpy(IRfile.sfname,get_arg_string(csound, *p->ifilno), 511);
+      else csound->strarg2name(csound, IRfile.sfname, p->ifilno, "soundin.",0);
+    }
+    else strncpy(IRfile.sfname, ((STRINGDAT *)p->ifilno)->data, 511);
+
     IRfile.sr = 0;
-    if (channel < 1 || ((channel > 4) && (channel != ALLCHNLS))) {
-      return csound->InitError(csound, "channel request %d illegal", channel);
+    if (UNLIKELY(channel < 1 || ((channel > 4) && (channel != ALLCHNLS)))) {
+      return csound->InitError(csound, Str("channel request %d illegal"), channel);
     }
     IRfile.channel = channel;
     IRfile.analonly = 1;
-    if ((infd = csound->sndgetset(csound, &IRfile)) == NULL) {
-      return csound->InitError(csound, "pconvolve: error while impulse file");
+    if (UNLIKELY((infd = csound->sndgetset(csound, &IRfile)) == NULL)) {
+      return csound->InitError(csound, Str("pconvolve: error while impulse file"));
     }
 
-    if (IRfile.framesrem < 0) {
+    if (UNLIKELY(IRfile.framesrem < 0)) {
       csound->Warning(csound, Str("undetermined file length, "
                                   "will attempt requested duration"));
       ainput_dur = FL(0.0);     /* This is probably wrong -- JPff */
     }
     else {
-        IRfile.getframes = IRfile.framesrem;
-        ainput_dur = (MYFLT) IRfile.getframes / IRfile.sr;
+      IRfile.getframes = IRfile.framesrem;
+      if (UNLIKELY(IRfile.sr==0)) return csound->InitError(csound, Str("SR zero"));
+      ainput_dur = (MYFLT) IRfile.getframes / IRfile.sr;
       }
 
-    csound->Message(csound, Str("analyzing %ld sample frames (%3.1f secs)\n"),
+    csound->Warning(csound, Str("analyzing %ld sample frames (%3.1f secs)\n"),
                             (long) IRfile.getframes, ainput_dur);
 
     p->nchanls = (channel != ALLCHNLS ? 1 : IRfile.nchanls);
-    if (p->nchanls != p->OUTOCOUNT) {
-      return csound->InitError(csound, "PCONVOLVE: number of output channels "
-                                       "not equal to input channels");
+    if (UNLIKELY(p->nchanls != (int)p->OUTOCOUNT)) {
+      return csound->InitError(csound, Str("PCONVOLVE: number of output channels "
+                                           "not equal to input channels"));
     }
 
-    if (IRfile.sr != csound->esr) {
+    if (UNLIKELY(IRfile.sr != CS_ESR)) {
       /* ## RWD suggests performing sr conversion here! */
-      csound->Warning(csound, "IR srate != orch's srate");
+      csound->Warning(csound, Str("IR srate != orch's srate"));
     }
 
     /* make sure the partition size is nonzero and a power of 2  */
     if (*p->partitionSize <= 0)
-      partitionSize = csound->oparms->outbufsamps / csound->nchnls;
+      partitionSize = csound->oparms->outbufsamps / csound->GetNchnls(csound);
     else
       partitionSize = *p->partitionSize;
 
@@ -407,7 +447,7 @@ static int pconvset(CSOUND *csound, PCONVOLVE *p)
     p->Hlenpadded = 2*p->Hlen;
 
     /* determine the number of partitions */
-    p->numPartitions = ceil((MYFLT)(IRfile.getframes) / (MYFLT)p->Hlen);
+    p->numPartitions = CEIL((MYFLT)(IRfile.getframes) / (MYFLT)p->Hlen);
 
     /* set up FFT tables */
     inbuf = (MYFLT *) csound->Malloc(csound,
@@ -415,14 +455,16 @@ static int pconvset(CSOUND *csound, PCONVOLVE *p)
     csound->AuxAlloc(csound, p->numPartitions * (p->Hlenpadded + 2) *
              sizeof(MYFLT) * p->nchanls, &p->H);
     IRblock = (MYFLT *)p->H.auxp;
-
+    p->fwdsetup = csound->RealFFT2Setup(csound,p->Hlenpadded, FFT_FWD);
+    p->invsetup = csound->RealFFT2Setup(csound,p->Hlenpadded, FFT_INV);
     /* form each partition and take its FFT */
     for (part = 0; part < p->numPartitions; part++) {
       /* get the block of input samples and normalize -- soundin code
          handles finding the right channel */
-      if ((read_in = csound->getsndin(csound, infd, inbuf,
-                                      p->Hlen*p->nchanls, &IRfile)) <= 0)
-        csound->Die(csound, "PCONVOLVE: less sound than expected!");
+      if (UNLIKELY((read_in = csound->getsndin(csound, infd, inbuf,
+                                               p->Hlen*p->nchanls, &IRfile)) <= 0))
+        return csound->InitError(csound,
+                                 Str("PCONVOLVE: less sound than expected!"));
 
       /* take FFT of each channel */
       scaleFac = csound->dbfs_to_float
@@ -434,7 +476,8 @@ static int pconvset(CSOUND *csound, PCONVOLVE *p)
           *fp2++ = *fp1 * scaleFac;
           fp1 += p->nchanls;
         }
-        csound->RealFFT(csound, IRblock, (int) p->Hlenpadded);
+
+        csound->RealFFT2(csound, p->fwdsetup, IRblock);
         IRblock[p->Hlenpadded] = IRblock[1];
         IRblock[1] = IRblock[p->Hlenpadded + 1L] = FL(0.0);
         IRblock += (p->Hlenpadded + 2);
@@ -459,7 +502,7 @@ static int pconvset(CSOUND *csound, PCONVOLVE *p)
 
     /* allocate circular output sample buffer */
     p->outBufSiz = sizeof(MYFLT) * p->nchanls *
-      (p->Hlen >= csound->ksmps ? p->Hlenpadded : 2*csound->ksmps);
+      (p->Hlen >= (int)CS_KSMPS ? p->Hlenpadded : 2*(int)CS_KSMPS);
     csound->AuxAlloc(csound, p->outBufSiz, &p->output);
     p->outRead = (MYFLT *)p->output.auxp;
 
@@ -467,8 +510,8 @@ static int pconvset(CSOUND *csound, PCONVOLVE *p)
        empty ksmps pass after a few initial generated buffers.  There is
        probably an equation to figure this out to reduce the delay, but
        I can't seem to figure it out */
-    if (p->Hlen > csound->ksmps) {
-      p->outCount = p->Hlen + csound->ksmps;
+    if (p->Hlen > (int)CS_KSMPS) {
+      p->outCount = p->Hlen + CS_KSMPS;
       p->outWrite = p->outRead + (p->nchanls * p->outCount);
     }
     else {
@@ -478,19 +521,29 @@ static int pconvset(CSOUND *csound, PCONVOLVE *p)
     return OK;
 }
 
+static int pconvset(CSOUND *csound, PCONVOLVE *p){
+  return pconvset_(csound,p,0);
+}
+
+static int pconvset_S(CSOUND *csound, PCONVOLVE *p){
+  return pconvset_(csound,p,1);
+}
+
 static int pconvolve(CSOUND *csound, PCONVOLVE *p)
 {
-    int    nsmpsi = csound->ksmps;
+    uint32_t nn, nsmps = CS_KSMPS;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = nsmps - p->h.insdshead->ksmps_no_end;
     MYFLT  *ai = p->ain;
     MYFLT  *buf;
     MYFLT  *input = (MYFLT*) p->savedInput.auxp, *workWrite = p->workWrite;
     MYFLT  *a1 = p->ar1, *a2 = p->ar2, *a3 = p->ar3, *a4 = p->ar4;
-    long   i, j, count = p->inCount;
-    long   hlenpaddedplus2 = p->Hlenpadded+2;
+    int32  i, j, count = p->inCount;
+    int32  hlenpaddedplus2 = p->Hlenpadded+2;
 
-    while (nsmpsi-- > 0) {
+    for (nn=0; nn<nsmps; nn++) {
       /* Read input audio and place into buffer. */
-      input[count++] = *workWrite++ = *ai++;
+      input[count++] = *workWrite++ = (nn<offset||nn>early? FL(0.0) : ai[nn]);
 
       /* We have enough audio for a convolution. */
       if (count == p->Hlen) {
@@ -502,7 +555,7 @@ static int pconvolve(CSOUND *csound, PCONVOLVE *p)
         /* FFT the input (to create X) */
         *workWrite = FL(0.0); /* zero out nyquist bin from last fft result
                            - maybe is ignored for input(?) but just in case.. */
-        csound->RealFFT(csound, workBuf, (int) p->Hlenpadded);
+        csound->RealFFT2(csound, p->fwdsetup, workBuf);
         workBuf[p->Hlenpadded] = workBuf[1];
         workBuf[1] = workBuf[p->Hlenpadded + 1L] = FL(0.0);
 
@@ -527,7 +580,7 @@ static int pconvolve(CSOUND *csound, PCONVOLVE *p)
           bufp = buf + i * hlenpaddedplus2;
           bufp[1] = bufp[p->Hlenpadded];
           bufp[p->Hlenpadded] = bufp[p->Hlenpadded + 1L] = FL(0.0);
-          csound->InverseRealFFT(csound, bufp, (int) p->Hlenpadded);
+          csound->RealFFT2(csound, p->invsetup, bufp);
         }
         /* We only take only the last Hlen output samples so we first zero out
            the first half for next time, then we copy the rest to output buffer
@@ -561,10 +614,10 @@ static int pconvolve(CSOUND *csound, PCONVOLVE *p)
 
     /* copy to output if we have enough samples [we always should
        except the first Hlen samples] */
-    if (p->outCount >= csound->ksmps) {
-      int n;
-      p->outCount -= csound->ksmps;
-      for (n=0; n < csound->ksmps; n++) {
+    if (p->outCount >= (int)CS_KSMPS) {
+      unsigned int n;
+      p->outCount -= CS_KSMPS;
+      for (n=0; n < CS_KSMPS; n++) {
         switch (p->nchanls) {
         case 1:
           *a1++ = *p->outRead++;
@@ -597,11 +650,17 @@ static int pconvolve(CSOUND *csound, PCONVOLVE *p)
 }
 
 static OENTRY localops[] = {
-    { "convolve", sizeof(CONVOLVE),   5, "mmmm", "aTo",
+    { "convolve", sizeof(CONVOLVE),   0, 5, "mmmm", "aSo",
+            (SUBR) cvset_S,     (SUBR) NULL,    (SUBR) convolve   },
+    { "convle",   sizeof(CONVOLVE),   0, 5, "mmmm", "aSo",
+            (SUBR) cvset_S,     (SUBR) NULL,    (SUBR) convolve   },
+    { "pconvolve",sizeof(PCONVOLVE),  0, 5, "mmmm", "aSoo",
+      (SUBR) pconvset_S,  (SUBR) NULL,    (SUBR) pconvolve  },
+     { "convolve.i", sizeof(CONVOLVE),   0, 5, "mmmm", "aio",
             (SUBR) cvset,     (SUBR) NULL,    (SUBR) convolve   },
-    { "convle",   sizeof(CONVOLVE),   5, "mmmm", "aTo",
+    { "convle.i",   sizeof(CONVOLVE),   0, 5, "mmmm", "aio",
             (SUBR) cvset,     (SUBR) NULL,    (SUBR) convolve   },
-    { "pconvolve",sizeof(PCONVOLVE),  5, "mmmm", "aToo",
+    { "pconvolve.i",sizeof(PCONVOLVE),  0, 5, "mmmm", "aioo",
             (SUBR) pconvset,  (SUBR) NULL,    (SUBR) pconvolve  }
 };
 
@@ -610,4 +669,3 @@ int ugens9_init_(CSOUND *csound)
     return csound->AppendOpcodes(csound, &(localops[0]),
                                  (int) (sizeof(localops) / sizeof(OENTRY)));
 }
-
