@@ -23,7 +23,7 @@
 
 /* ugsc.c -- Opcodes from Sean Costello <costello@seanet.com> */
 
-#include "csdl.h"
+#include "stdopcod.h"
 #include "ugsc.h"
 
 /* svfilter.c
@@ -40,49 +40,67 @@
 
 static int svfset(CSOUND *csound, SVF *p)
 {
-    /* set initial delay states to 0 */
-    p->ynm1 = p->ynm2 = FL(0.0);
+    if (*p->iskip) {
+      /* set initial delay states to 0 */
+      p->ynm1 = p->ynm2 = FL(0.0);
+    }
     return OK;
 }
 
 static int svf(CSOUND *csound, SVF *p)
 {
-    MYFLT f1, q1, scale;
+    MYFLT f1 = FL(0.0), q1 = FL(1.0), scale = FL(1.0),
+          lfco = -FL(1.0), lq = -FL(1.0);
     MYFLT *low, *high, *band, *in, ynm1, ynm2;
     MYFLT low2, high2, band2;
-    MYFLT kfco = *p->kfco, kq = *p->kq;
-    int nsmps = csound->ksmps;
+    MYFLT *kfco = p->kfco, *kq = p->kq;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
 
-    /* calculate frequency and Q coefficients */
-    f1 = FL(2.0) * (MYFLT)sin((double)(kfco * csound->pidsr));
-    if (kq<FL(0.000001)) kq = FL(1.0); /* Protect against division by zero */
-    q1 = FL(1.0) / kq;
-
-    in = p->in;
-    low = p->low;
+    in   = p->in;
+    low  = p->low;
     band = p->band;
     high = p->high;
     ynm1 = p->ynm1;
     ynm2 = p->ynm2;
 
-    /* if there is a non-zero value for iscl, set scale to be
-     * equal to the Q coefficient.
-     */
-    if (*p->iscl)
-      scale = q1;
-    else
-      scale = FL(1.0);
 
     /* equations derived from Hal Chamberlin, "Musical Applications
      * of Microprocessors.
      */
-    do {
-      *low++ = low2 = ynm2 + f1 * ynm1;
-      *high++ = high2 = scale * *in++ - low2 - q1 * ynm1;
-      *band++ = band2 = f1 * high2 + ynm1;
-      ynm1 = band2;
-      ynm2 = low2;
-    } while (--nsmps);
+    if (UNLIKELY(offset)) {
+      memset(low,  '\0', offset*sizeof(MYFLT));
+      memset(high, '\0', offset*sizeof(MYFLT));
+      memset(band, '\0', offset*sizeof(MYFLT));
+    }
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&low[nsmps],  '\0', early*sizeof(MYFLT));
+      memset(&high[nsmps], '\0', early*sizeof(MYFLT));
+      memset(&band[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n=offset; n<nsmps; n++) {
+      MYFLT fco = IS_ASIG_ARG(p->kfco) ? kfco[n] : *kfco;
+      MYFLT q = IS_ASIG_ARG(p->kq) ? kq[n] : *kq;
+      if (fco != lfco || q != lq) {
+        lfco = fco; lq = q;
+        /* calculate frequency and Q coefficients */
+        f1 = FL(2.0) * (MYFLT)sin((double)(fco * csound->pidsr));
+        /* Protect against division by zero */
+        if (UNLIKELY(q<FL(0.000001))) q = FL(1.0);
+        q1 = FL(1.0) / q;
+        /* if there is a non-zero value for iscl, set scale to be
+         * equal to the Q coefficient.
+         */
+        if (*p->iscl) scale = q1;
+      }
+      low[n]  = low2 = ynm2 + f1 * ynm1;
+      high[n] = high2 = scale * in[n] - low2 - q1 * ynm1;
+      band[n] = band2 = f1 * high2 + ynm1;
+      ynm1    = band2;
+      ynm2    = low2;
+    }
     p->ynm1 = ynm1;
     p->ynm2 = ynm2;
     return OK;
@@ -106,39 +124,48 @@ static int hilbertset(CSOUND *csound, HILBERT *p)
     /* pole values taken from Bernie Hutchins, "Musical Engineer's Handbook" */
     double poles[12] = {0.3609, 2.7412, 11.1573, 44.7581, 179.6242, 798.4578,
                         1.2524, 5.5671, 22.3423, 89.6271, 364.7914, 2770.1114};
-    double polefreq[12], rc[12], alpha[12], beta[12];
+    double polefreq, rc, alpha, beta;
     /* calculate coefficients for allpass filters, based on sampling rate */
     for (j=0; j<12; j++) {
-      /*      p->coef[j] = (1 - (15 * PI * pole[j]) / csound->esr) /
-              (1 + (15 * PI * pole[j]) / csound->esr); */
-      polefreq[j] = poles[j] * 15.0;
-      rc[j] = 1.0 / (2.0 * PI * polefreq[j]);
-      alpha[j] = 1.0 / rc[j];
-      beta[j] = (1.0 - (alpha[j] * 0.5 * (double)csound->onedsr)) /
-                (1.0 + (alpha[j] * 0.5 * (double)csound->onedsr));
+      /*      p->coef[j] = (1 - (15 * PI * pole[j]) / CS_ESR) /
+              (1 + (15 * PI * pole[j]) / CS_ESR); */
+      polefreq = poles[j] * 15.0;
+      rc = 1.0 / (2.0 * PI * polefreq);
+      alpha = 1.0 / rc;
+      alpha = alpha * 0.5 * (double)csound->onedsr;
+      beta = (1.0 - alpha) / (1.0 + alpha);
       p->xnm1[j] = p->ynm1[j] = FL(0.0);
-      p->coef[j] = -(MYFLT)beta[j];
+      p->coef[j] = -(MYFLT)beta;
     }
     return OK;
 }
 
 static int hilbert(CSOUND *csound, HILBERT *p)
 {
-    MYFLT xn1 = FL(0.0), yn1 = FL(0.0), xn2 = FL(0.0), yn2 = FL(0.0);
+    MYFLT xn1, yn1, xn2, yn2;
     MYFLT *out1, *out2, *in;
-    MYFLT *xnm1, *ynm1, *coef;
-    int nsmps = csound->ksmps;
+    MYFLT *coef;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     int j;
 
-    xnm1 = p->xnm1;
-    ynm1 = p->ynm1;
     coef = p->coef;
     out1 = p->out1;
     out2 = p->out2;
     in = p->in;
 
-    do {
-      xn1 = *in;
+    if (UNLIKELY(offset)) {
+      memset(out1, '\0', offset*sizeof(MYFLT));
+      memset(out2, '\0', offset*sizeof(MYFLT));
+    }
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&out1[nsmps], '\0', early*sizeof(MYFLT));
+      memset(&out2[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n=offset; n<nsmps; n++) {
+      xn1 = in[n];
       /* 6th order allpass filter for sine output. Structure is
        * 6 first-order allpass sections in series. Coefficients
        * taken from arrays calculated at i-time.
@@ -149,7 +176,7 @@ static int hilbert(CSOUND *csound, HILBERT *p)
         p->ynm1[j] = yn1;
         xn1 = yn1;
       }
-      xn2 = *in++;
+      xn2 = in[n];
       /* 6th order allpass filter for cosine output. Structure is
        * 6 first-order allpass sections in series. Coefficients
        * taken from arrays calculated at i-time.
@@ -160,9 +187,9 @@ static int hilbert(CSOUND *csound, HILBERT *p)
         p->ynm1[j] = yn2;
         xn2 = yn2;
       }
-      *out1++ = yn2;
-      *out2++ = yn1;
-    } while (--nsmps);
+      out1[n] = yn2;
+      out2[n] = yn1;
+    }
     return OK;
 }
 
@@ -182,15 +209,12 @@ static int resonzset(CSOUND *csound, RESONZ *p)
     /* error message code derived from code for reson in ugens5.c */
     int scaletype;
     p->scaletype = scaletype = (int)*p->iscl;
-    if (scaletype && scaletype != 1 && scaletype != 2) {
+    if (UNLIKELY(UNLIKELY(scaletype && scaletype != 1 && scaletype != 2))) {
       return csound->InitError(csound, Str("illegal reson iscl value, %f"),
-                                       *p->iscl);
+                               (float)*p->iscl);
     }
     if (!(*p->istor))
       p->xnm1 = p->xnm2 = p->ynm1 = p->ynm2 = 0.0;
-
-/*     p->aratemod = (XINARG2) ? 1 : 0; */
-
     return OK;
 }
 
@@ -208,23 +232,15 @@ static int resonr(CSOUND *csound, RESONZ *p)
      *
      */
 
-    double r, scale; /* radius & scaling factor */
-    double c1, c2;   /* filter coefficients */
+    double r = 0.0, scale = 1.0; /* radius & scaling factor */
+    double c1=0.0, c2=0.0;   /* filter coefficients */
     MYFLT *out, *in;
     double xn, yn, xnm1, xnm2, ynm1, ynm2;
-    MYFLT kcf = *p->kcf, kbw = *p->kbw;
-    int n, nsmps = csound->ksmps;
-
-    r = exp((double)(kbw * csound->mpidsr));
-    c1 = 2.0 * r * cos((double)(kcf * csound->tpidsr));
-    c2 = r * r;
-
-    /* calculation of scaling coefficients */
-    if (p->scaletype == 1)
-      scale = 1.0 - r;
-    else if (p->scaletype == 2)
-      scale = sqrt(1.0 - r);
-    else scale = 1.0;
+    MYFLT *kcf = p->kcf, *kbw = p->kbw;
+    MYFLT lcf = -FL(1.0), lbw = -FL(1.0);
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
 
     out = p->out;
     in = p->in;
@@ -233,7 +249,24 @@ static int resonr(CSOUND *csound, RESONZ *p)
     ynm1 = p->ynm1;
     ynm2 = p->ynm2;
 
-    for (n=0; n<nsmps; n++) {
+    if (UNLIKELY(offset)) memset(out, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&out[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n=offset; n<nsmps; n++) {
+      MYFLT cf = IS_ASIG_ARG(p->kcf) ? kcf[n] : *kcf;
+      MYFLT bw = IS_ASIG_ARG(p->kbw) ? kbw[n] : *kbw;
+      if (cf != lcf || bw != lbw) {
+        lcf = cf; lbw = bw;
+        r = exp((double)(bw * csound->mpidsr));
+        c1 = 2.0 * r * cos((double)(cf * csound->tpidsr));
+        c2 = r * r;
+        if (p->scaletype == 1)
+          scale = 1.0 - r;
+        else if (p->scaletype == 2)
+          scale = sqrt(1.0 - r);
+      }
       xn = (double)in[n];
       out[n] = (MYFLT)(yn = scale * (xn - r * xnm2) + c1 * ynm1 - c2 * ynm2);
       xnm2 = xnm1;
@@ -261,26 +294,20 @@ static int resonz(CSOUND *csound, RESONZ *p)
      *
      */
 
-    double r, scale; /* radius & scaling factor */
-    double c1, c2;   /* filter coefficients */
+    double r = 0.0, scale = 1.0; /* radius & scaling factor */
+    double c1=0.0, c2=0.0;   /* filter coefficients */
     MYFLT *out, *in;
     double xn, yn, xnm1, xnm2, ynm1, ynm2;
-    MYFLT kcf = *p->kcf, kbw = *p->kbw;
-    int n, nsmps = csound->ksmps;
-
-    r = exp(-(double)(kbw * csound->pidsr));
-    c1 = 2.0 * r * cos((double)(csound->tpidsr*kcf));
-    c2 = r * r;
+    MYFLT *kcf = p->kcf, *kbw = p->kbw;
+    MYFLT lcf = -FL(1.0), lbw = -FL(1.0);
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
 
     /* Normalizing factors derived from equations in Ken Steiglitz,
      * "A Note on Constant-Gain Digital Resonators," Computer
      * Music Journal, vol. 18, no. 4, pp. 8-10, Winter 1982.
      */
-    if (p->scaletype == 1)
-      scale = (1.0 - c2) * 0.5;
-    else if (p->scaletype == 2)
-      scale = sqrt((1.0 - c2) * 0.5);
-    else scale = 1.0;
 
     out  = p->out;
     in   = p->in;
@@ -289,7 +316,24 @@ static int resonz(CSOUND *csound, RESONZ *p)
     ynm1 = p->ynm1;
     ynm2 = p->ynm2;
 
-    for (n=0; n<nsmps; n++) {
+    if (UNLIKELY(offset)) memset(out, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&out[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n=offset; n<nsmps; n++) {
+      MYFLT cf = IS_ASIG_ARG(p->kcf) ? kcf[n] : *kcf;
+      MYFLT bw = IS_ASIG_ARG(p->kbw) ? kbw[n] : *kbw;
+      if (cf != lcf || bw != lbw) {
+        lcf = cf; lbw = bw;
+        r = exp(-(double)(bw * csound->pidsr));
+        c1 = 2.0 * r * cos((double)(csound->tpidsr*cf));
+        c2 = r * r;
+        if (p->scaletype == 1)
+          scale = (1.0 - c2) * 0.5;
+        else if (p->scaletype == 2)
+          scale = sqrt((1.0 - c2) * 0.5);
+      }
       xn = (double)in[n];
       out[n] = (MYFLT)(yn = scale * (xn - xnm2) + c1 * ynm1 - c2 * ynm2);
       xnm2 = xnm1;
@@ -308,21 +352,22 @@ static int resonz(CSOUND *csound, RESONZ *p)
 static int phaser1set(CSOUND *csound, PHASER1 *p)
 {
     int   loop = (int) MYFLT2LONG(*p->iorder);
-    long  nBytes = (long) loop * (long) sizeof(MYFLT);
+    int32  nBytes = (int32) loop * (int32) sizeof(MYFLT);
 
-    if (*p->istor == FL(0.0) || p->auxx.auxp == NULL || p->auxx.size<nBytes ||
-                                p->auxy.auxp == NULL || p->auxy.size<nBytes) {
+    if (*p->istor == FL(0.0) || p->auxx.auxp == NULL ||
+        (int)p->auxx.size<nBytes || p->auxy.auxp == NULL ||
+        (int)p->auxy.size<nBytes) {
       csound->AuxAlloc(csound, nBytes, &p->auxx);
       csound->AuxAlloc(csound, nBytes, &p->auxy);
       p->xnm1 = (MYFLT *) p->auxx.auxp;
       p->ynm1 = (MYFLT *) p->auxy.auxp;
     }
-    else if ((long) p->auxx.size < nBytes || (long) p->auxy.size < nBytes) {
+    else if ((int32) p->auxx.size < nBytes || (int32) p->auxy.size < nBytes) {
       /* Existing arrays too small so copy */
       void    *tmp1, *tmp2;
       size_t  oldSize1 = (size_t) p->auxx.size;
       size_t  oldSize2 = (size_t) p->auxy.size;
-      tmp1 = csound->Malloc(csound, oldSize1 + oldSize2);
+      tmp1 = malloc(oldSize1 + oldSize2);
       tmp2 = (char*) tmp1 + (int) oldSize1;
       memcpy(tmp1, p->auxx.auxp, oldSize1);
       memcpy(tmp2, p->auxy.auxp, oldSize2);
@@ -330,7 +375,7 @@ static int phaser1set(CSOUND *csound, PHASER1 *p)
       csound->AuxAlloc(csound, nBytes, &p->auxy);
       memcpy(p->auxx.auxp, tmp1, oldSize1);
       memcpy(p->auxy.auxp, tmp2, oldSize2);
-      csound->Free(csound, tmp1);
+      free(tmp1);
       p->xnm1 = (MYFLT *) p->auxx.auxp;
       p->ynm1 = (MYFLT *) p->auxy.auxp;
     }
@@ -342,20 +387,19 @@ static int phaser1(CSOUND *csound, PHASER1 *p)
 {
     MYFLT xn = FL(0.0), yn = FL(0.0);
     MYFLT *out, *in;
-    MYFLT *xnm1, *ynm1, feedback;
-    MYFLT coef = *p->kcoef, fbgain = *p->fbgain;
+    MYFLT feedback;
+    MYFLT coef = FABS(*p->kcoef), fbgain = *p->fbgain;
     MYFLT beta, wp;
-    int nsmps = csound->ksmps;
-    int i, j;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t i, nsmps = CS_KSMPS;
+    int j;
 
-    xnm1 = p->xnm1;
-    ynm1 = p->ynm1;
     feedback = p->feedback;
     out = p->out;
     in = p->in;
 
-    if (coef <= FL(0.0))
-      coef = -coef;     /* frequency will "fold over" if <= 0 Hz */
+    //if (coef<=FL(0.0)) coef = -coef; /* frequency will "fold over" if <= 0 Hz */
     /* next two lines implement bilinear z-transform, to convert
      * frequency value into a useable coefficient for the
      * allpass filters.
@@ -363,7 +407,12 @@ static int phaser1(CSOUND *csound, PHASER1 *p)
     wp = csound->pidsr * coef;
     beta = (FL(1.0) - wp)/(FL(1.0) + wp);
 
-    for (i=0; i<nsmps; i++) {
+    if (UNLIKELY(offset)) memset(out, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&out[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (i=offset; i<nsmps; i++) {
       xn = in[i] + feedback * fbgain;
       for (j=0; j < p->loop; j++) {
         /* Difference equation for 1st order
@@ -383,22 +432,23 @@ static int phaser1(CSOUND *csound, PHASER1 *p)
 
 static int phaser2set(CSOUND *csound, PHASER2 *p)
 {
-    int modetype, j;
+    int modetype;
     int loop;
 
     p->modetype = modetype = (int)*p->mode;
-    if (modetype && modetype != 1 && modetype != 2) {
+    if (UNLIKELY(UNLIKELY(modetype && modetype != 1 && modetype != 2))) {
       return csound->InitError(csound,
                                Str("Phaser mode must be either 1 or 2"));
     }
 
     loop = p->loop = (int) MYFLT2LONG(*p->order);
-    csound->AuxAlloc(csound, (long)loop*sizeof(MYFLT), &p->aux1);
-    csound->AuxAlloc(csound, (long)loop*sizeof(MYFLT), &p->aux2);
+    csound->AuxAlloc(csound, (size_t)loop*sizeof(MYFLT), &p->aux1);
+    csound->AuxAlloc(csound, (size_t)loop*sizeof(MYFLT), &p->aux2);
     p->nm1 = (MYFLT *) p->aux1.auxp;
     p->nm2 = (MYFLT *) p->aux2.auxp;
-    for (j=0; j< loop; j++)
-      p->nm1[j] = p->nm2[j] = FL(0.0);
+    /* *** This is unnecessary as AuxAlloc zeros *** */
+    /* for (j=0; j< loop; j++) */
+    /*   p->nm1[j] = p->nm2[j] = FL(0.0); */
     return OK;
 }
 
@@ -411,7 +461,9 @@ static int phaser2(CSOUND *csound, PHASER2 *p)
     MYFLT b, a, r, freq;
     MYFLT temp;
     MYFLT *nm1, *nm2, feedback;
-    int nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     int j;
 
     nm1 = p->nm1;
@@ -430,8 +482,14 @@ static int phaser2(CSOUND *csound, PHASER2 *p)
     if (ksep <= FL(0.0))
       ksep = -ksep;
 
-    do {
-      xn = *in++ + feedback * fbgain;
+    if (UNLIKELY(offset)) memset(out, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&out[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n=offset; n<nsmps; n++) {
+      MYFLT kk = FL(1.0);
+      xn = in[n] + feedback * fbgain;
       /* The following code is used to determine
        * how the frequencies of the notches are calculated.
        * If imode=1, the notches will be in a harmonic
@@ -441,8 +499,11 @@ static int phaser2(CSOUND *csound, PHASER2 *p)
       for (j=0; j < p->loop; j++) {
         if (p->modetype == 1)
           freq = kbf + (kbf * ksep * j);
-        else
-          freq = kbf * csound->intpow(ksep,(long)j);
+        else {
+          freq = kbf * kk;
+          kk *= ksep;
+          //freq = kbf * csound->intpow(ksep,(int32)j);
+        }
         /* Note similarities of following equations to
          * equations in resonr/resonz. The 2nd-order
          * allpass filter used here is similar to the
@@ -451,22 +512,22 @@ static int phaser2(CSOUND *csound, PHASER2 *p)
          * notch, while the pole radius determines the q of
          * the notch.
          */
-        r = (MYFLT)exp(-(double)(freq * csound->pidsr / kq));
-        b = -FL(2.0) * r * (MYFLT)cos((double)(freq * csound->tpidsr));
+        r = EXP(-(freq * csound->pidsr / kq));
+        b = -FL(2.0) * r * COS(freq * csound->tpidsr);
         a = r * r;
 
         /* Difference equations for implementing canonical
          * 2nd order section. (Direct Form II)
          */
-        temp = xn - b * p->nm1[j] - a * p->nm2[j];
-        yn = a * temp + b * p->nm1[j] + nm2[j];
-        p->nm2[j] = p->nm1[j];
-        p->nm1[j] = temp;
+        temp = xn - b * nm1[j] - a * nm2[j];
+        yn = a * temp + b * nm1[j] + nm2[j];
+        nm2[j] = nm1[j];
+        nm1[j] = temp;
         xn = yn;
       }
-      *out++ = yn;
+      out[n] = yn;
       feedback = yn;
-    } while (--nsmps);
+    }
     p->feedback = feedback;
     return OK;
 }
@@ -487,11 +548,12 @@ static int lp2(CSOUND *csound, LP2 *p)
     MYFLT *out, *in;
     double yn, ynm1, ynm2;
     MYFLT kfco = *p->kfco, kres = *p->kres;
-    int nsmps = csound->ksmps;
-    int n;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
 
     temp = (double)(csound->mpidsr * kfco / kres);
-      /* (-PI_F * kfco / (kres * csound->esr)); */
+      /* (-PI_F * kfco / (kres * CS_ESR)); */
     a = 2.0 * cos((double) (kfco * csound->tpidsr)) * exp(temp);
     b = exp(temp+temp);
     c = 1.0 - a + b;
@@ -501,7 +563,147 @@ static int lp2(CSOUND *csound, LP2 *p)
     ynm1 = p->ynm1;
     ynm2 = p->ynm2;
 
-    for (n=0; n<nsmps; n++) {
+    if (UNLIKELY(offset)) memset(out, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&out[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n=offset; n<nsmps; n++) {
+      out[n] = (MYFLT)(yn = a * ynm1 - b * ynm2 + c * (double)in[n]);
+      ynm2 = ynm1;
+      ynm1 = yn;
+    }
+    p->ynm1 = ynm1;
+    p->ynm2 = ynm2;
+    return OK;
+}
+
+static int lp2aa(CSOUND *csound, LP2 *p)
+{
+    double a, b, c, temp;
+    MYFLT *out, *in;
+    double yn, ynm1, ynm2;
+    MYFLT *fcop = p->kfco, *resp = p->kres;
+    MYFLT fco = fcop[0], res = resp[0];
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
+
+    temp = (double)(csound->mpidsr * fco / res);
+      /* (-PI_F * kfco / (kres * CS_ESR)); */
+    a = 2.0 * cos((double) (fco * csound->tpidsr)) * exp(temp);
+    b = exp(temp+temp);
+    c = 1.0 - a + b;
+
+    out  = p->out;
+    in   = p->in;
+    ynm1 = p->ynm1;
+    ynm2 = p->ynm2;
+
+    if (UNLIKELY(offset)) memset(out, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&out[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n=offset; n<nsmps; n++) {
+      if (res!=resp[n] || fco!=fcop[n]) {
+        res=resp[n]; fco=fcop[n];
+        temp = (double)(csound->mpidsr * fco / res);
+        /* (-PI_F * kfco / (kres * CS_ESR)); */
+        a = 2.0 * cos((double) (fco * csound->tpidsr)) * exp(temp);
+        b = exp(temp+temp);
+        c = 1.0 - a + b;
+      }
+      out[n] = (MYFLT)(yn = a * ynm1 - b * ynm2 + c * (double)in[n]);
+      ynm2 = ynm1;
+      ynm1 = yn;
+    }
+    p->ynm1 = ynm1;
+    p->ynm2 = ynm2;
+    return OK;
+}
+
+static int lp2ka(CSOUND *csound, LP2 *p)
+{
+    double a, b, c, temp;
+    MYFLT *out, *in;
+    double yn, ynm1, ynm2;
+    MYFLT *resp = p->kres;
+    MYFLT fco = *p->kfco, res = resp[0];
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
+
+    temp = (double)(csound->mpidsr * fco / res);
+      /* (-PI_F * kfco / (kres * CS_ESR)); */
+    a = 2.0 * cos((double) (fco * csound->tpidsr)) * exp(temp);
+    b = exp(temp+temp);
+    c = 1.0 - a + b;
+
+    out  = p->out;
+    in   = p->in;
+    ynm1 = p->ynm1;
+    ynm2 = p->ynm2;
+
+    if (UNLIKELY(offset)) memset(out, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&out[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n=offset; n<nsmps; n++) {
+      if (res!=resp[n]) {
+        res=resp[n];
+        temp = (double)(csound->mpidsr * fco / res);
+        /* (-PI_F * kfco / (kres * CS_ESR)); */
+        a = 2.0 * cos((double) (fco * csound->tpidsr)) * exp(temp);
+        b = exp(temp+temp);
+        c = 1.0 - a + b;
+      }
+      out[n] = (MYFLT)(yn = a * ynm1 - b * ynm2 + c * (double)in[n]);
+      ynm2 = ynm1;
+      ynm1 = yn;
+    }
+    p->ynm1 = ynm1;
+    p->ynm2 = ynm2;
+    return OK;
+}
+
+static int lp2ak(CSOUND *csound, LP2 *p)
+{
+    double a, b, c, temp;
+    MYFLT *out, *in;
+    double yn, ynm1, ynm2;
+    MYFLT *fcop = p->kfco;
+    MYFLT fco = fcop[0], res = *p->kres;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
+
+    temp = (double)(csound->mpidsr * fco / res);
+      /* (-PI_F * kfco / (kres * CS_ESR)); */
+    a = 2.0 * cos((double) (fco * csound->tpidsr)) * exp(temp);
+    b = exp(temp+temp);
+    c = 1.0 - a + b;
+
+    out  = p->out;
+    in   = p->in;
+    ynm1 = p->ynm1;
+    ynm2 = p->ynm2;
+
+    if (UNLIKELY(offset)) memset(out, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&out[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n=offset; n<nsmps; n++) {
+      if (fco!=fcop[n]) {
+        fco=fcop[n];
+        temp = (double)(csound->mpidsr * fco / res);
+        /* (-PI_F * kfco / (kres * CS_ESR)); */
+        a = 2.0 * cos((double) (fco * csound->tpidsr)) * exp(temp);
+        b = exp(temp+temp);
+        c = 1.0 - a + b;
+      }
       out[n] = (MYFLT)(yn = a * ynm1 - b * ynm2 + c * (double)in[n]);
       ynm2 = ynm1;
       ynm1 = yn;
@@ -514,13 +716,16 @@ static int lp2(CSOUND *csound, LP2 *p)
 #define S(x)    sizeof(x)
 
 static OENTRY localops[] = {
-{ "svfilter", S(SVF),    5, "aaa", "akko", (SUBR)svfset, NULL, (SUBR)svf     },
-{ "hilbert", S(HILBERT), 5, "aa",  "a",    (SUBR)hilbertset, NULL, (SUBR)hilbert },
-{ "resonr", S(RESONZ),   5, "a",   "akkoo", (SUBR)resonzset, NULL, (SUBR)resonr  },
-{ "resonz", S(RESONZ),   5, "a",   "akkoo", (SUBR)resonzset, NULL, (SUBR)resonz  },
-{ "lowpass2", S(LP2),    5, "a",   "akko",  (SUBR)lp2_set, NULL, (SUBR)lp2       },
-{ "phaser2", S(PHASER2), 5, "a", "akkkkkk",(SUBR)phaser2set, NULL, (SUBR)phaser2 },
-{ "phaser1", S(PHASER1), 5, "a",   "akkko", (SUBR)phaser1set, NULL, (SUBR)phaser1 }
+{ "svfilter", S(SVF),    0, 5, "aaa", "axxoo", (SUBR)svfset, NULL, (SUBR)svf    },
+{ "hilbert", S(HILBERT), 0,5, "aa", "a", (SUBR)hilbertset, NULL, (SUBR)hilbert },
+{ "resonr", S(RESONZ),   0,5, "a", "axxoo", (SUBR)resonzset, NULL, (SUBR)resonr},
+{ "resonz", S(RESONZ),   0,5, "a", "axxoo", (SUBR)resonzset, NULL, (SUBR)resonz},
+{ "lowpass2.kk", S(LP2), 0,5, "a", "akko",  (SUBR)lp2_set, NULL, (SUBR)lp2     },
+{ "lowpass2.aa", S(LP2), 0,5, "a", "aaao",  (SUBR)lp2_set, NULL, (SUBR)lp2aa   },
+{ "lowpass2.ak", S(LP2), 0,5, "a", "aakao", (SUBR)lp2_set, NULL, (SUBR)lp2ak   },
+{ "lowpass2.ka", S(LP2), 0,5, "a", "akao",  (SUBR)lp2_set, NULL, (SUBR)lp2ka   },
+{ "phaser2", S(PHASER2), 0,5, "a", "akkkkkk",(SUBR)phaser2set,NULL,(SUBR)phaser2},
+{ "phaser1", S(PHASER1), 0,5, "a", "akkko", (SUBR)phaser1set, NULL,(SUBR)phaser1}
 };
 
 int ugsc_init_(CSOUND *csound)
