@@ -23,6 +23,7 @@
 
 #include "csoundCore.h"     /*                      CSCORFNS.C      */
 #include "cscore.h"
+#include "corfile.h"
 
 #define TYP_FREE   0
 #define TYP_EVENT  1
@@ -76,7 +77,7 @@ void cscoreRESET(CSOUND *csound)
       SPACE *n;
       do {
         n = p->nxtspace;
-        mfree(csound, p);
+        csound->Free(csound, p);
         p = n;
       }
       while (p != NULL);
@@ -97,7 +98,7 @@ static SPACE *morespace(CSOUND *csound)
     prvspace = &spaceanchor;
     while ((space = prvspace->nxtspace) != NULL)
       prvspace = space;
-    space = (SPACE *) mmalloc(csound, (long) MAXALLOC);
+    space = (SPACE *) csound->Malloc(csound, (long) MAXALLOC);
     prvspace->nxtspace = space;
     space->nxtspace = NULL;
     space->h.prvblk = NULL;
@@ -162,7 +163,7 @@ static void csfree(CSHDR *bp)
     }
 }
 
-/* creat an array of event pointer slots */
+/* create an array of event pointer slots */
 
 PUBLIC EVLIST * cscoreListCreate(CSOUND *csound, int nslots)
 {
@@ -171,6 +172,10 @@ PUBLIC EVLIST * cscoreListCreate(CSOUND *csound, int nslots)
     int   needsiz = sizeof(EVLIST) + nslots * sizeof(EVENT *);
     int   minfreesiz = needsiz + sizeof(CSHDR);
 
+    if (minfreesiz > MAXALLOC) {
+      csound->Message(csound, Str("Not enough memory\n"));
+      exit(1);
+    }
     if (nxtfree != NULL && nxtfree->size >= minfreesiz)
       newblk = nxtfree;
     else newblk = getfree(csound, minfreesiz);
@@ -198,6 +203,10 @@ PUBLIC EVENT * cscoreCreateEvent(CSOUND *csound, int pcnt)
     int   needsiz = sizeof(EVENT) + pcnt * sizeof(MYFLT);
     int   minfreesiz = needsiz + sizeof(CSHDR);
 
+    if (minfreesiz > MAXALLOC) {
+      csound->Message(csound, Str("Not enough memory\n"));
+      exit(1);
+    }
     if (nxtfree != NULL && nxtfree->size >= minfreesiz)
       newblk = nxtfree;
     else newblk = getfree(csound, minfreesiz);
@@ -261,9 +270,9 @@ PUBLIC EVENT * cscoreDefineEvent(CSOUND *csound, char *s)
     p = &evtmp->p[1];
     q = &evtmp->p[PMAX];
 #ifdef USE_DOUBLE
-    while (sscanf(s,"%lf",p++) > 0)         /* read pfields */
+    while (CS_SSCANF(s,"%lf",p++) > 0)         /* read pfields */
 #else
-    while (sscanf(s,"%f",p++) > 0)          /* read pfields */
+    while (CS_SSCANF(s,"%f",p++) > 0)          /* read pfields */
 #endif
     {
       while ((*s >= '0' && *s <= '9') || *s == '.' || *s == '-')
@@ -290,7 +299,7 @@ PUBLIC EVENT * cscoreGetEvent(CSOUND *csound)
 {
     EVENT *e;
 
-    if (csound->scfp != NULL && !atEOF && nxtevt->op != '\0')
+    if (!atEOF && nxtevt->op != '\0')
       e = cscoreCopyEvent(csound, nxtevt);
     else e = NULL;
     if (!(rdscor(csound, nxtevtblk))) {
@@ -313,7 +322,7 @@ PUBLIC void cscorePutEvent(CSOUND *csound, EVENT *e)
     q = &e->p[1];
     if ((pcnt = e->pcnt)) {
       if (pcnt--)       fprintf(csound->oscfp," %g",*q++);
-      else goto termin;
+      //else goto termin; /* cannot happen */
       if (pcnt--) {
         if (warpout)    fprintf(csound->oscfp," %g", e->p2orig);
                         fprintf(csound->oscfp," %g",*q++);
@@ -389,6 +398,8 @@ PUBLIC EVLIST * cscoreListGetSection(CSOUND *csound)
 
     a = cscoreListCreate(csound, NSLOTS);
     p = &a->e[1];
+    if (csound->scstr == NULL || csound->scstr->body[0] == '\0')
+      return a;
     while ((e = cscoreGetEvent(csound)) != NULL) {
       if (e->op == 's' || e->op == 'e')
         break;
@@ -614,6 +625,7 @@ PUBLIC EVLIST * cscoreListExtractTime(CSOUND *csound,
       case 'f':
         if (e->p[2] < to) {
           *q++ = e = cscoreCopyEvent(csound,e);
+          b->nevents++;
           if (e->p[2] <= from)
             e->p[2] = FL(0.0);
           else e->p[2] -= from;
@@ -623,21 +635,24 @@ PUBLIC EVLIST * cscoreListExtractTime(CSOUND *csound,
         if (e->p[2] < from) {
           if (e->p[2] + e->p[3] > from) {
             *q++ = e = cscoreCopyEvent(csound,e);
+            b->nevents++;
             e->p[3] -= from - e->p[2];
             e->p[2] = FL(0.0);
             if (e->p[3] > maxp3)
               e->p[3] = maxp3;
           }
         }
-        else if (e->p[2] < to) {
+        else if ((e->p[2] >= from) && (e->p[2] < to)) {
           *q++ = e = cscoreCopyEvent(csound, e);
+          b->nevents++;
           if (e->p[2] + e->p[3] > to)
             e->p[3] = to - e->p[2];
-            e->p[2] -= from;
+          e->p[2] -= from;
         }
         break;
       default:
         *q++ = cscoreCopyEvent(csound,e);
+        b->nevents++;
         break;
       }
     c = cscoreListCopy(csound,b);
@@ -751,7 +766,7 @@ static void savinfdata(         /* store input file data */
     int    n;
 
     if ((infp = infiles) == NULL) {
-      infp = infiles = (INFILE *) mcalloc(csound, MAXOPEN * sizeof(INFILE));
+      infp = infiles = (INFILE *) csound->Calloc(csound, MAXOPEN * sizeof(INFILE));
       goto save;
     }
     for (n = MAXOPEN; n--; infp++)
@@ -792,7 +807,9 @@ static void makecurrent(CSOUND *csound, FILE *fp)
           atEOF = infp->atEOF;
           csound->warped = infp->warped;
           if (nxtevt->op == '\0')
-            if (!(rdscor(csound, nxtevtblk))) {
+            if (csound->scstr == NULL ||
+                csound->scstr->body[0] == '\0' ||
+                !(rdscor(csound, nxtevtblk))) {
               nxtevt->op = '\0';
               atEOF = 1;
             }
@@ -805,15 +822,16 @@ static void makecurrent(CSOUND *csound, FILE *fp)
 
 /* verify initial scfp, init other data */
 /* record & make all this current       */
-
 PUBLIC int csoundInitializeCscore(CSOUND *csound, FILE* insco, FILE* outsco)
 {
-    EVENT   *next;
+    EVENT  *next;
 
-    if (insco == NULL) {
-      csound->ErrorMsg(csound,
-                       Str("csoundInitializeCscore: no input score given."));
-      return CSOUND_INITIALIZATION;
+    if (insco != NULL) {
+      CORFIL *inf = corfile_create_w();
+      int c;
+      while ((c=getc(insco))!=EOF) corfile_putc(c, inf);
+      corfile_rewind(inf);
+      csound->scstr = inf;
     }
     if (outsco == NULL) {
       csound->ErrorMsg(csound,
@@ -825,13 +843,16 @@ PUBLIC int csoundInitializeCscore(CSOUND *csound, FILE* insco, FILE* outsco)
 
     next = cscoreCreateEvent(csound, PMAX); /* creat EVENT blk receiving buf */
     next->op = '\0';
+
     savinfdata(csound, csound->scfp,
-               next, FL(0.0), 1, 0, 0);    /* curuntil 0, wasend, non-warp, not eof */
+               next, FL(0.0), 1, 0, 0);/* curuntil 0, wasend, non-warp, not eof */
     makecurrent(csound, csound->scfp);  /* make all this current         */
 
     return CSOUND_SUCCESS;
 }
 
+
+
 /* open new cscore input file, init data */
 /* & save;  no rdscor until made current */
 
@@ -849,7 +870,7 @@ PUBLIC FILE *cscoreFileOpen(CSOUND *csound, char *name)
       exit(0);     /* FIXME: should not call exit */
     }
     csoundNotifyFileOpened(csound, pathname, CSFTYPE_SCORE, 0, 0);
-    mfree(csound, pathname);
+    csound->Free(csound, pathname);
     /* alloc a receiving evtblk */
     next = cscoreCreateEvent(csound,PMAX);  /* FIXME: need next->op = '\0' ?? */
     /* save all, wasend, non-warped, not eof */
@@ -870,7 +891,7 @@ PUBLIC void cscoreFileClose(CSOUND *csound, FILE *fp)
       for (n = MAXOPEN; n--; infp++)
         if (infp->iscfp == fp) {
           infp->iscfp = NULL;
-          mfree(csound, (char *)infp->next);
+          csound->Free(csound, (char *)infp->next);
           fclose(fp);
           if (csound->scfp == fp) csound->scfp = NULL;
           return;
@@ -892,6 +913,17 @@ PUBLIC FILE *cscoreFileGetCurrent(CSOUND *csound)
 
 PUBLIC void cscoreFileSetCurrent(CSOUND *csound, FILE *fp)
 {
+    if (fp != NULL) {
+      CORFIL *inf = corfile_create_w();
+      int c;
+      fseek(fp, 0, SEEK_SET);
+      while ((c=getc(fp))!=EOF) corfile_putc(c, inf);
+      corfile_rewind(inf);
+      corfile_rm(&csound->scstr);
+      csound->scstr = inf;
+      nxtevt->op = '\0';
+      atEOF = 0;
+    }
     if (csound->scfp != NULL)
       savinfdata(csound,
                  csound->scfp, nxtevt, curuntil, wasend, csound->warped, atEOF);
