@@ -39,7 +39,7 @@
 
 extern int UDPServerStart(CSOUND *csound, int port);
 extern  void    dieu(CSOUND *, char *, ...);
-extern  int     argdecode(CSOUND *, int, char **);
+extern  int     argdecode(CSOUND *, int, const char **);
 extern  int     init_pvsys(CSOUND *);
 //extern  char    *get_sconame(CSOUND *);
 extern  void    print_benchmark_info(CSOUND *, const char *);
@@ -104,7 +104,7 @@ static void checkOptions(CSOUND *csound)
 }
 
 
-PUBLIC int csoundCompileArgs(CSOUND *csound, int argc, char **argv)
+PUBLIC int csoundCompileArgs(CSOUND *csound, int argc, const char **argv)
 {
     OPARMS  *O = csound->oparms;
     char    *s;
@@ -171,7 +171,6 @@ PUBLIC int csoundCompileArgs(CSOUND *csound, int argc, char **argv)
 
       if (csound->orchname != NULL) {
       csound->csdname = csound->orchname; /* save original CSD name */
-#ifndef OLD
       {
         CORFIL *cf = copy_to_corefile(csound, csound->csdname, NULL, 0);
         if (cf == NULL) {
@@ -183,12 +182,6 @@ PUBLIC int csoundCompileArgs(CSOUND *csound, int argc, char **argv)
         }
         /* cf is deleted in read_unified_file4 */
       }
-#else
-      if (!read_unified_file(csound, &(csound->orchname),
-                                       &(csound->scorename))) {
-        csound->Die(csound, Str("Reading CSD failed ... stopping"));
-      }
-#endif
       csdFound = 1;
       }
     }
@@ -211,7 +204,11 @@ PUBLIC int csoundCompileArgs(CSOUND *csound, int argc, char **argv)
 
     if (csound->scorename == NULL && csound->scorestr==NULL) {
       /* No scorename yet */
-      csound->scorestr = corfile_create_r("f0 800000000000.0\n");
+      csound->Message(csound, "scoreless operation\n");
+      // csound->scorestr = corfile_create_r("f0 800000000000.0 \n");
+      // VL 21-09-2016: it looks like #exit is needed for the
+      // new score parser to work.
+      csound->scorestr = corfile_create_r("\n#exit\n");
       corfile_flush(csound->scorestr);
       if (O->RTevents)
         csound->Message(csound, Str("realtime performance using dummy "
@@ -282,6 +279,7 @@ PUBLIC int csoundCompileArgs(CSOUND *csound, int argc, char **argv)
     if (!csoundYield(csound))
       return -1;
     /* IV - Oct 31 2002: now we can read and sort the score */
+
     if (csound->scorename != NULL &&
         (n = strlen(csound->scorename)) > 4 &&  /* if score ?.srt or ?.xtr */
         (!strcmp(csound->scorename + (n - 4), ".srt") ||
@@ -299,6 +297,7 @@ PUBLIC int csoundCompileArgs(CSOUND *csound, int argc, char **argv)
           csoundDie(csound, Str("cannot open scorefile %s"), csound->scorename);
       }
       csound->Message(csound, Str("sorting score ...\n"));
+      //printf("score:\n%s", corfile_current(csound->scorestr));
       scsortstr(csound, csound->scorestr);
       if (csound->keep_tmp) {
         FILE *ff = fopen("score.srt", "w");
@@ -508,7 +507,7 @@ PUBLIC int csoundStart(CSOUND *csound) // DEBUG
     return musmon(csound);
 }
 
-PUBLIC int csoundCompile(CSOUND *csound, int argc, char **argv){
+PUBLIC int csoundCompile(CSOUND *csound, int argc, const char **argv){
 
     int result = csoundCompileArgs(csound,argc,argv);
 
@@ -516,8 +515,7 @@ PUBLIC int csoundCompile(CSOUND *csound, int argc, char **argv){
     else return result;
 }
 
-PUBLIC int csoundCompileCsd(CSOUND *csound, char *str) {
-#ifndef OLD
+PUBLIC int csoundCompileCsd(CSOUND *csound, const char *str) {
     CORFIL *tt = copy_to_corefile(csound, str, NULL, 0);
     if(tt != NULL){
     int res = csoundCompileCsdText(csound, tt->body);
@@ -525,33 +523,10 @@ PUBLIC int csoundCompileCsd(CSOUND *csound, char *str) {
     return res;
     }
     return CSOUND_ERROR;
-#else
-    if ((csound->engineStatus & CS_STATE_COMP) == 0) {
-      char *argv[2] = { "csound", (char *) str };
-      int argc = 2;
-      return csoundCompile(csound, argc, argv);
-    }
-    else {
-      int res = read_unified_file2(csound, (char *) str);
-      if (res) {
-        res = csoundCompileOrc(csound, NULL);
-        if (res == CSOUND_SUCCESS){
-          csoundLockMutex(csound->API_lock);
-          char *sc = scsortstr(csound, csound->scorestr);
-          csoundInputMessageInternal(csound, (const char *) sc);
-          free(sc);
-          csoundUnlockMutex(csound->API_lock);
-          return CSOUND_SUCCESS;
-        }
-      }
-      return res;
-    }
-#endif
 }
 
 PUBLIC int csoundCompileCsdText(CSOUND *csound, const char *csd_text)
 {
-#ifndef OLD
     //csound->oparms->odebug = 1; /* *** SWITCH ON EXTRA DEBUGGING *** */
     int res = read_unified_file4(csound, corfile_create_r(csd_text));
     if (res) {
@@ -568,16 +543,4 @@ PUBLIC int csoundCompileCsdText(CSOUND *csound, const char *csd_text)
       }
     }
     return res;
-#else
-    FILE *temporary_file;
-    char temporary_filename[L_tmpnam];
-    int result = CSOUND_SUCCESS;
-    tmpnam(temporary_filename);
-    temporary_file = fopen(temporary_filename, "w+");
-    fwrite(csd_text, sizeof(char), strlen(csd_text), temporary_file);
-    fclose(temporary_file);
-    result = csoundCompileCsd(csound, temporary_filename);
-    remove(temporary_filename);
-    return result;
-#endif
 }
