@@ -35,13 +35,13 @@ static std::string about = "CSOUND AND CSOUND VST\n"
   "Version "
   CS_PACKAGE_VERSION
   " beta "
-  __DATE__
+  CS_PACKAGE_DATE
   "\n"
 #else
   "Version "
   CS_PACKAGE_VERSION
   " "
-  __DATE__
+  CS_PACKAGE_DATE
   "\n"
 #endif
   "\n"
@@ -159,6 +159,7 @@ CsoundVstFltk::CsoundVstFltk(AudioEffect *audioEffect) :
   windowHandle (0),
   csoundVstUi (0),
   useCount (0),
+  updateFlag (0),
   mainTabs (0),
   commandInput (0),
   runtimeMessagesBrowser (0),
@@ -170,8 +171,7 @@ CsoundVstFltk::CsoundVstFltk(AudioEffect *audioEffect) :
   aboutTextBuffer (0),
   aboutTextDisplay (0),
   orchestraGroup (0),
-  scoreGroup (0),
-  updateFlag (0)
+  scoreGroup (0)
 {
   this->csoundVstUi = make_window(this);
   this->mainTabs = ::mainTabs;
@@ -179,18 +179,18 @@ CsoundVstFltk::CsoundVstFltk(AudioEffect *audioEffect) :
   this->runtimeMessagesBrowser = ::runtimeMessagesBrowser;
   this->orchestraTextEdit = ::orchestraTextEdit;
   this->orchestraTextBuffer = new Fl_Text_Buffer();
-  this->orchestraTextEdit->buffer(this->orchestraTextBuffer);
   this->scoreTextEdit = ::scoreTextEdit;
   this->scoreTextBuffer = new Fl_Text_Buffer();
-  this->scoreTextEdit->buffer(this->scoreTextBuffer);
   this->settingsEditSoundfileInput = ::settingsEditSoundfileInput;
   this->settingsVstPluginModeEffect = ::settingsVstPluginModeEffect;
   this->settingsVstPluginModeInstrument = ::settingsVstPluginModeInstrument;
   this->aboutTextBuffer = new Fl_Text_Buffer();
   this->aboutTextDisplay = ::aboutTextDisplay;
-  this->aboutTextDisplay->buffer(this->aboutTextBuffer);
   this->orchestraGroup = ::orchestraGroup;
   this->scoreGroup = ::scoreGroup;
+  this->orchestraTextEdit->buffer(this->orchestraTextBuffer);
+  this->scoreTextEdit->buffer(this->scoreTextBuffer);
+  this->aboutTextDisplay->buffer(this->aboutTextBuffer);
   csoundVST->setEditor(this);
 }
 
