@@ -405,6 +405,7 @@ extern "C" {
     MYFLT   e0dbfs_override;   /* overriding 0dbfs */
     int     daemon;  /* daemon mode */
     int     ksmps_override; /* ksmps override */
+    int     FFT_library;    /* fft_lib */
   } CSOUND_PARAMS;
 
     /**
@@ -659,7 +660,7 @@ extern "C" {
      *  Read arguments, parse and compile an orchestra, read, process and
      *  load a score.
     */
-    PUBLIC int csoundCompileArgs(CSOUND *, int argc, char **argv);
+    PUBLIC int csoundCompileArgs(CSOUND *, int argc, const char **argv);
 
     /**
      * Prepares Csound for performance after compilation
@@ -686,7 +687,7 @@ extern "C" {
      *  Calls csoundStart() internally.
      *  Can only be called again after reset (see csoundReset())
      */
-    PUBLIC int csoundCompile(CSOUND *, int argc, char **argv);
+    PUBLIC int csoundCompile(CSOUND *, int argc, const char **argv);
 
     /**
      * Compiles a Csound input file (CSD, .csd file)
@@ -706,7 +707,7 @@ extern "C" {
      *
      */
 
-    PUBLIC int csoundCompileCsd(CSOUND *csound, char *str);
+    PUBLIC int csoundCompileCsd(CSOUND *csound, const char *str);
 
     /**
      * Compiles a Csound input file contained in a string of text,
@@ -845,7 +846,7 @@ extern "C" {
      * Set a single csound option (flag). Returns CSOUND_SUCCESS on success.
      * NB: blank spaces are not allowed
      */
-    PUBLIC int csoundSetOption(CSOUND *csound, char *option);
+    PUBLIC int csoundSetOption(CSOUND *csound, const char *option);
 
     /**
      *  Configure Csound with a given set of parameters defined in
@@ -899,33 +900,43 @@ extern "C" {
     *   For RT audio, use device_id from CS_AUDIODEVICE for a given audio device.
     *
     */
-  PUBLIC void csoundSetOutput(CSOUND *csound, char *name,
-                              char *type, char *format);
+  PUBLIC void csoundSetOutput(CSOUND *csound, const char *name,
+                              const char *type, const char *format);
+
+  /**
+    *  Get output type and format.
+    *  type should have space for at least 5 chars excluding termination,
+    *  and format should have space for at least 7 chars.
+    *  On return, these will hold the current values for
+    *  these parameters.
+    */
+  PUBLIC void csoundGetOutputFormat(CSOUND *csound,char *type,
+                                    char *format);
 
     /**
      *  Set input source
      */
-  PUBLIC void csoundSetInput(CSOUND *csound, char *name);
+  PUBLIC void csoundSetInput(CSOUND *csound, const char *name);
 
    /**
     *  Set MIDI input device name/number
     */
-  PUBLIC void csoundSetMIDIInput(CSOUND *csound, char *name);
+  PUBLIC void csoundSetMIDIInput(CSOUND *csound, const char *name);
 
    /**
     *  Set MIDI file input name
     */
-  PUBLIC void csoundSetMIDIFileInput(CSOUND *csound, char *name);
+  PUBLIC void csoundSetMIDIFileInput(CSOUND *csound, const char *name);
 
    /**
     *  Set MIDI output device name/number
     */
-  PUBLIC void csoundSetMIDIOutput(CSOUND *csound, char *name);
+  PUBLIC void csoundSetMIDIOutput(CSOUND *csound, const char *name);
 
    /**
     *  Set MIDI file utput name
     */
-  PUBLIC void csoundSetMIDIFileOutput(CSOUND *csound, char *name);
+  PUBLIC void csoundSetMIDIFileOutput(CSOUND *csound, const char *name);
 
 #if !defined(SWIG)
     /**
@@ -952,7 +963,7 @@ extern "C" {
      /**
      *  Sets the current RT audio module
      */
-    PUBLIC void csoundSetRTAudioModule(CSOUND *csound, char *module);
+    PUBLIC void csoundSetRTAudioModule(CSOUND *csound, const char *module);
 
      /**
       * retrieves a module name and type ("audio" or "midi") given a
@@ -1127,7 +1138,7 @@ extern "C" {
     /**
      *  Sets the current MIDI IO module
      */
-    PUBLIC void csoundSetMIDIModule(CSOUND *csound, char *module);
+    PUBLIC void csoundSetMIDIModule(CSOUND *csound, const char *module);
 
      /**
       * call this function with state 1 if the host is implementing
@@ -1730,6 +1741,19 @@ extern "C" {
      */
     PUBLIC int csoundGetTableArgs(CSOUND *csound, MYFLT **argsPtr, int tableNum);
 
+    /**
+     * Checks if a given GEN number num is a named GEN
+     * if so, it returns the string length (excluding terminating NULL char)
+     * Otherwise it returns 0.
+     */
+    PUBLIC int csoundIsNamedGEN(CSOUND *csound, int num);
+
+    /**
+     * Gets the GEN name from a number num, if this is a named GEN
+     * The final parameter is the max len of the string (excluding termination)
+     */
+    PUBLIC void csoundGetNamedGEN(CSOUND *csound, int num, char *name, int len);
+
     /** @}*/
     /** @defgroup TABLEDISPLAY Function table display
      *
