@@ -80,7 +80,7 @@
 
 #define ERR(x)                          \
 {                                       \
-    csound->Message(csound, Str(x));    \
+    csound->Message(csound, x);         \
     return -1;                          \
 }
 
@@ -97,7 +97,7 @@
 static  int     dnoise_usage(CSOUND *, int);
 static  void    hamming(MYFLT *, int, int);
 
-static int writebuffer(CSOUND *, SNDFILE *, MYFLT *, int, int *);
+static int writebuffer(CSOUND *, SNDFILE *, MYFLT *, int, int *, OPARMS *);
 
 static void fast(CSOUND *csound, MYFLT *b, int N)
 {
@@ -139,7 +139,8 @@ static void fsst(CSOUND *csound, MYFLT *b, int N)
 
 static int dnoise(CSOUND *csound, int argc, char **argv)
 {
-    OPARMS  *O = csound->oparms;
+     OPARMS  O;
+     csound->GetOParms(csound, &O);
 
     MYFLT   beg = -FL(1.0), end = -FL(1.0);
     long    Beg = 0, End = 99999999;
@@ -203,11 +204,11 @@ static int dnoise(CSOUND *csound, int argc, char **argv)
 
     MYFLT
         Ninv,       /* 1. / N */
-        RoverTwoPi, /* R/D divided by 2*Pi */
-        TwoPioverR, /* 2*Pi divided by R/I */
+      //RoverTwoPi, /* R/D divided by 2*Pi */
+      //TwoPioverR, /* 2*Pi divided by R/I */
         sum,        /* scale factor for renormalizing windows */
-        rIn,        /* decimated sampling rate */
-        rOut,       /* pre-interpolated sampling rate */
+      //rIn,        /* decimated sampling rate */
+      //rOut,       /* pre-interpolated sampling rate */
         invR,       /* 1. / srate */
         time,       /* nI / srate */
         gain,       /* gain of noise gate */
@@ -220,8 +221,6 @@ static int dnoise(CSOUND *csound, int argc, char **argv)
         R = -FL(1.0);  /* input sampling rate */
 
     int    i,j,k,   /* index variables */
-        Dd,         /* number of new inputs to read (Dd <= D) */
-        Ii,         /* number of new outputs to write (Ii <= I) */
         ibs,        /* current starting location in input buffer */
         ibc,        /* current location in input buffer */
         obs,        /* current starting location in output buffer */
@@ -243,7 +242,6 @@ static int dnoise(CSOUND *csound, int argc, char **argv)
         first = 0;  /* first-time-thru flag */
 
     SOUNDIN     *p, *pn;
-    void        *dummy;
     char        *infile = NULL, *nfile = NULL;
     SNDFILE     *inf = NULL, *outfd = NULL;
     char        c, *s;
@@ -252,18 +250,18 @@ static int dnoise(CSOUND *csound, int argc, char **argv)
     MYFLT       beg_ntime = FL(0.0), input_ndur = FL(0.0), srn = FL(0.0);
     const char  *envoutyp = NULL;
     unsigned int  outbufsiz = 0U;
-    char        outformch = 's';
     int         nrecs = 0;
 
-    csound->e0dbfs = csound->dbfs_to_float = FL(1.0);
+    /* audio is now normalised after call to getsndin  */
+    /* csound->e0dbfs = csound->dbfs_to_float = FL(1.0); */
 
     if ((envoutyp = csound->GetEnv(csound, "SFOUTYP")) != NULL) {
       if (strcmp(envoutyp, "AIFF") == 0)
-        O->filetyp = TYP_AIFF;
+        O.filetyp = TYP_AIFF;
       else if (strcmp(envoutyp, "WAV") == 0)
-        O->filetyp = TYP_WAV;
+        O.filetyp = TYP_WAV;
       else if (strcmp(envoutyp, "IRCAM") == 0)
-        O->filetyp = TYP_IRCAM;
+        O.filetyp = TYP_IRCAM;
       else {
         csound->Message(csound, Str("%s not a recognised SFOUTYP env setting"),
                                 envoutyp);
@@ -279,9 +277,9 @@ static int dnoise(CSOUND *csound, int argc, char **argv)
             switch (c) {
             case 'o':
               FIND("no outfilename");
-              O->outfilename = s;                 /* soundout name */
+              O.outfilename = s;                 /* soundout name */
               for ( ; *s != '\0'; s++) ;
-              if (strcmp(O->outfilename, "stdin") == 0) {
+              if (strcmp(O.outfilename, "stdin") == 0) {
                 csound->Message(csound, Str("-o cannot be stdin\n"));
                 return -1;
               }
@@ -292,72 +290,65 @@ static int dnoise(CSOUND *csound, int argc, char **argv)
               for ( ; *s != '\0'; s++) ;
               break;
             case 'A':
-              if (O->filetyp == TYP_WAV)
+              if (O.filetyp == TYP_WAV)
                 csound->Warning(csound,
                                 Str("-A overriding local default WAV out"));
-              O->filetyp = TYP_AIFF;    /* AIFF output request*/
+              O.filetyp = TYP_AIFF;    /* AIFF output request*/
               break;
             case 'J':
-              if (O->filetyp == TYP_AIFF || O->filetyp == TYP_WAV)
+              if (O.filetyp == TYP_AIFF || O.filetyp == TYP_WAV)
                 csound->Warning(csound, Str("-J overriding local default "
                                             "AIFF/WAV out"));
-              O->filetyp = TYP_IRCAM;   /* IRCAM output request */
+              O.filetyp = TYP_IRCAM;   /* IRCAM output request */
               break;
             case 'W':
-              if (O->filetyp == TYP_AIFF)
+              if (O.filetyp == TYP_AIFF)
                 csound->Warning(csound,
                                 Str("-W overriding local default AIFF out"));
-              O->filetyp = TYP_WAV;      /* WAV output request */
+              O.filetyp = TYP_WAV;      /* WAV output request */
               break;
             case 'h':
-              O->filetyp = TYP_RAW;
-              O->sfheader = 0;           /* skip sfheader  */
+              O.filetyp = TYP_RAW;
+              O.sfheader = 0;           /* skip sfheader  */
               break;
             case 'c':
-              outformch = c;
-              O->outformat = AE_CHAR;     /* 8-bit char soundfile */
+              O.outformat = AE_CHAR;     /* 8-bit char soundfile */
               break;
             case '8':
-              outformch = c;
-              O->outformat = AE_UNCH;     /* 8-bit unsigned char file */
+              O.outformat = AE_UNCH;     /* 8-bit unsigned char file */
               break;
             case 'a':
-              outformch = c;
-              O->outformat = AE_ALAW;     /* a-law soundfile */
+              O.outformat = AE_ALAW;     /* a-law soundfile */
               break;
             case 'u':
-              outformch = c;
-              O->outformat = AE_ULAW;     /* mu-law soundfile */
+              O.outformat = AE_ULAW;     /* mu-law soundfile */
               break;
             case 's':
-              outformch = c;
-              O->outformat = AE_SHORT;    /* short_int soundfile */
+              O.outformat = AE_SHORT;    /* short_int soundfile */
               break;
             case 'l':
-              outformch = c;
-              O->outformat = AE_LONG;     /* long_int soundfile */
+              O.outformat = AE_LONG;     /* long_int soundfile */
               break;
             case 'f':
-              outformch = c;
-              O->outformat = AE_FLOAT;    /* float soundfile */
+              O.outformat = AE_FLOAT;    /* float soundfile */
               break;
             case 'R':
-              O->rewrt_hdr = 1;
+              O.rewrt_hdr = 1;
               break;
             case 'H':
               if (isdigit(*s)) {
                 int n;
-                sscanf(s, "%d%n", &O->heartbeat, &n);
+                sscanf(s, "%d%n", &O.heartbeat, &n);
                 s += n;
               }
-              else O->heartbeat = 1;
+              else O.heartbeat = 1;
               break;
             case 't':
-              FIND("no t arg");
+              FIND(Str("no t argument"));
 #if defined(USE_DOUBLE)
-              sscanf(s,"%lf",&th);
+              csound->sscanf(s,"%lf",&th);
 #else
-              sscanf(s,"%f",&th);
+              csound->sscanf(s,"%f",&th);
 #endif
               while (*++s);
               break;
@@ -369,86 +360,86 @@ static int dnoise(CSOUND *csound, int argc, char **argv)
             case 'm':
               FIND("no m arg");
 #if defined(USE_DOUBLE)
-              sscanf(s,"%lf",&g0);
+              csound->sscanf(s,"%lf",&g0);
 #else
-              sscanf(s,"%f",&g0);
+              csound->sscanf(s,"%f",&g0);
 #endif
               while (*++s);
               break;
             case 'n':
-              FIND("no n arg");
+              FIND(Str("no n argument"));
               sscanf(s,"%d", &m);
               while (*++s);
               break;
             case 'b':
-              FIND("no b arg");
+              FIND(Str("no b argument"));
 #if defined(USE_DOUBLE)
-              sscanf(s,"%lf",&beg);
+              csound->sscanf(s,"%lf",&beg);
 #else
-              sscanf(s,"%f",&beg);
+              csound->sscanf(s,"%f",&beg);
 #endif
               while (*++s);
               break;
-            case 'B': FIND("no B arg");
+            case 'B': FIND(Str("no B argument"));
               sscanf(s,"%ld", &Beg);
               while (*++s);
               break;
             case 'e': FIND("no e arg");
 #if defined(USE_DOUBLE)
-              sscanf(s,"%lf",&end);
+              csound->sscanf(s,"%lf",&end);
 #else
-              sscanf(s,"%f",&end);
+              csound->sscanf(s,"%f",&end);
 #endif
               while (*++s);
               break;
-            case 'E': FIND("no E arg");
+            case 'E': FIND(Str("no E argument"));
               sscanf(s,"%ld", &End);
               while (*++s);
               break;
-            case 'N': FIND("no N arg");
+            case 'N': FIND(Str("no N argument"));
               sscanf(s,"%d", &N);
               while (*++s);
               break;
-            case 'M': FIND("no M arg");
+            case 'M': FIND(Str("no M argument"));
               sscanf(s,"%d", &M);
               while (*++s);
               break;
-            case 'L': FIND("no L arg");
+            case 'L': FIND(Str("no L argument"));
               sscanf(s,"%d", &L);
               while (*++s);
               break;
-            case 'w': FIND("no w arg");
+            case 'w': FIND(Str("no w argument"));
               sscanf(s,"%d", &W);
               while (*++s);
               break;
-            case 'D': FIND("no D arg");
+            case 'D': FIND(Str("no D argument"));
               sscanf(s,"%d", &D);
               while (*++s);
               break;
             case 'V':
               Verbose = 1; break;
             default:
-              csound->Message(csound, "Looking at %c\n", c);
+              csound->Message(csound, Str("Looking at %c\n"), c);
               return dnoise_usage(csound, -1);  /* this exits with error */
             }
           }
         }
         else if (infile==NULL) {
           infile = --s;
-          csound->Message(csound, "Infile set to %s\n", infile);
+          csound->Message(csound, Str("Infile set to %s\n"), infile);
         }
         else {
-          csound->Message(csound, "End with %s\n", s);
+          csound->Message(csound, Str("End with %s\n"), s);
           return dnoise_usage(csound, -1);
         }
       }
     }
     if (infile == NULL) {
-      csound->Message(csound, "dnoise: no input file\n");
+      csound->Message(csound, Str("dnoise: no input file\n"));
       return dnoise_usage(csound, -1);
     }
     if (nfile == NULL) {
-      csound->Message(csound, "Must have an example noise file (-i name)\n");
+      csound->Message(csound, Str("Must have an example noise file (-i name)\n"));
       return -1;
     }
     if ((inf = csound->SAsndgetset(csound, infile, &p, &beg_time,
@@ -456,48 +447,49 @@ static int dnoise(CSOUND *csound, int argc, char **argv)
       csound->Message(csound, Str("error while opening %s"), infile);
       return -1;
     }
-    if (O->outformat == 0) O->outformat = p->format;
-    O->sfsampsize = csound->sfsampsize(FORMAT2SF(O->outformat));
-    if (O->filetyp == TYP_RAW) {
-      O->sfheader = 0;
-      O->rewrt_hdr = 0;
+    if (O.outformat == 0) O.outformat = p->format;
+    O.sfsampsize = csound->sfsampsize(FORMAT2SF(O.outformat));
+    if (O.filetyp == TYP_RAW) {
+      O.sfheader = 0;
+      O.rewrt_hdr = 0;
     }
     else
-      O->sfheader = 1;
-    if (O->outfilename == NULL)
-      O->outfilename = "test";
+      O.sfheader = 1;
+    if (O.outfilename == NULL)
+      O.outfilename = "test";
     {
       SF_INFO sfinfo;
       char    *name;
       memset(&sfinfo, 0, sizeof(SF_INFO));
       sfinfo.samplerate = (int) p->sr;
       sfinfo.channels = (int) p->nchanls;
-      sfinfo.format = TYPE2SF(O->filetyp) | FORMAT2SF(O->outformat);
-      if (strcmp(O->outfilename, "stdout") != 0) {
-        name = csound->FindOutputFile(csound, O->outfilename, "SFDIR");
+      sfinfo.format = TYPE2SF(O.filetyp) | FORMAT2SF(O.outformat);
+      if (strcmp(O.outfilename, "stdout") != 0) {
+        name = csound->FindOutputFile(csound, O.outfilename, "SFDIR");
         if (name == NULL) {
-          csound->Message(csound, Str("cannot open %s.\n"), O->outfilename);
+          csound->Message(csound, Str("cannot open %s.\n"), O.outfilename);
           return -1;
         }
         outfd = sf_open(name, SFM_WRITE, &sfinfo);
         if (outfd != NULL)
-          csound->NotifyFileOpened(csound, name, 
-                      csound->type2csfiletype(O->filetyp, O->outformat), 1, 0);
+          csound->NotifyFileOpened(csound, name,
+                      csound->type2csfiletype(O.filetyp, O.outformat), 1, 0);
         csound->Free(csound, name);
       }
       else
         outfd = sf_open_fd(1, SFM_WRITE, &sfinfo, 1);
       if (outfd == NULL) {
-        csound->Message(csound, Str("cannot open %s."), O->outfilename);
+        csound->Message(csound, Str("cannot open %s."), O.outfilename);
         return -1;
       }
       /* register file to be closed by csoundReset() */
-      dummy = csound->CreateFileHandle(csound, &outfd, CSFILE_SND_W,
-                                               O->outfilename);
+      (void)csound->CreateFileHandle(csound, &outfd, CSFILE_SND_W,
+                                     O.outfilename);
       sf_command(outfd, SFC_SET_CLIPPING, NULL, SF_TRUE);
     }
-    csound->esr = (MYFLT) p->sr;
-    csound->nchnls = Chans = p->nchanls;
+
+    csound->SetUtilSr(csound, (MYFLT)p->sr);
+    csound->SetUtilNchnls(csound, Chans = p->nchanls);
 
     /* read header info */
     if (R < FL(0.0))
@@ -507,7 +499,7 @@ static int dnoise(CSOUND *csound, int argc, char **argv)
     p->nchanls = Chans;
 
     if (Chans > 2) {
-      csound->Message(csound, "dnoise: input MUST be mono or stereo\n");
+      csound->Message(csound, Str("dnoise: input MUST be mono or stereo\n"));
       return -1;
     }
 
@@ -515,12 +507,12 @@ static int dnoise(CSOUND *csound, int argc, char **argv)
 
     if ((fp = csound->SAsndgetset(csound, nfile, &pn, &beg_ntime,
                                   &input_ndur, &srn, channel)) == NULL) {
-      csound->Message(csound, "dnoise: cannot open noise reference file\n");
+      csound->Message(csound, Str("dnoise: cannot open noise reference file\n"));
       return -1;
     }
 
     if (sr != srn) {
-      csound->Message(csound, "Incompatible sample rates\n");
+      csound->Message(csound, Str("Incompatible sample rates\n"));
       return -1;
     }
     /* calculate begin and end times in NOISE file */
@@ -539,8 +531,8 @@ static int dnoise(CSOUND *csound, int argc, char **argv)
         break;
     if (i != N)
       csound->Message(csound,
-                       "dnoise: warning - N not a valid power of two; "
-              "revised N = %d\n",i);
+                      Str("dnoise: warning - N not a valid power of two; "
+                          "revised N = %d\n"),i);
     N = i;
     N2 = N / 2;
     Np2 = N + 2;
@@ -549,7 +541,7 @@ static int dnoise(CSOUND *csound, int argc, char **argv)
     if (W != -1) {
       if (M != 0)
         csound->Message(csound,
-                        "dnoise: warning - don't specify both M and W\n");
+                        Str("dnoise: warning - do not specify both M and W\n"));
       else if (W == 0)
         M = 4*N;
       else if (W == 1)
@@ -559,7 +551,7 @@ static int dnoise(CSOUND *csound, int argc, char **argv)
       else if (W == 3)
         M = N2;
       else
-        csound->Message(csound, "dnoise: warning - invalid W ignored\n");
+        csound->Message(csound, Str("dnoise: warning - invalid W ignored\n"));
     }
 
     if (M == 0)
@@ -572,9 +564,10 @@ static int dnoise(CSOUND *csound, int argc, char **argv)
     if ((L%2) == 0)
       Leven = 1;
 
-    if (M < 7)
-      csound->Message(csound, "dnoise: warning - M is too small\n");
-
+    if (M < 7) {
+      csound->Message(csound, Str("dnoise: warning - M is too small\n"));
+      exit(~1);
+    }
     if (D == 0)
       D = M / 8;
 
@@ -583,13 +576,13 @@ static int dnoise(CSOUND *csound, int argc, char **argv)
     lj = (long) M + 3 * (long) D;
     lj *= (long) Chans;
     if (lj > 32767) {
-      csound->Message(csound, "dnoise: M too large\n");
+      csound->Message(csound, Str("dnoise: M too large\n"));
       return -1;
     }
     lj = (long) L + 3 * (long) I;
     lj *= (long) Chans;
     if (lj > 32767) {
-      csound->Message(csound, "dnoise: L too large\n");
+      csound->Message(csound, Str("dnoise: L too large\n"));
       return -1;
     }
 
@@ -600,9 +593,9 @@ static int dnoise(CSOUND *csound, int argc, char **argv)
     outbuf = csound->Malloc(csound, (size_t) outbufsiz); /* & alloc bufspace */
 #endif
     csound->Message(csound, Str("writing %u-byte blks of %s to %s"),
-                    outbufsiz, csound->getstrformat(O->outformat),
-                    O->outfilename);
-    csound->Message(csound, " (%s)\n", csound->type2string(O->filetyp));
+                    outbufsiz, csound->getstrformat(O.outformat),
+                    O.outfilename);
+    csound->Message(csound, " (%s)\n", csound->type2string(O.filetyp));
 /*  spoutran = spoutsf; */
 
     minv = FL(1.0) / (MYFLT)m;
@@ -624,7 +617,7 @@ static int dnoise(CSOUND *csound, int argc, char **argv)
 
     if ((aWin = (MYFLT*) csound->Calloc(csound,
                                         (M+Meven) * sizeof(MYFLT))) == NULL) {
-      ERR("dnoise: insufficient memory\n");
+      ERR(Str("dnoise: insufficient memory\n"));
     }
 
     aLen = M/2;
@@ -663,7 +656,7 @@ static int dnoise(CSOUND *csound, int argc, char **argv)
 
     if ((sWin = (MYFLT*) csound->Calloc(csound,
                                         (L+Leven) * sizeof(MYFLT))) == NULL) {
-      ERR("dnoise: insufficient memory\n");
+      ERR(Str("dnoise: insufficient memory\n"));
     }
 
     sLen = L/2;
@@ -718,7 +711,7 @@ static int dnoise(CSOUND *csound, int argc, char **argv)
     }
     if ((ibuf2 = (MYFLT *) csound->Calloc(csound,
                                           ibuflen * sizeof(MYFLT))) == NULL) {
-      ERR("dnoise: insufficient memory\n");
+      ERR(Str("dnoise: insufficient memory\n"));
     }
 
     /* set up output buffer:  nextOut always points to the next word
@@ -729,18 +722,18 @@ static int dnoise(CSOUND *csound, int argc, char **argv)
 
     if ((obuf1 = (MYFLT*) csound->Calloc(csound,
                                          obuflen * sizeof(MYFLT))) == NULL) {
-      ERR("dnoise: insufficient memory\n");
+      ERR(Str("dnoise: insufficient memory\n"));
     }
     if ((obuf2 = (MYFLT*) csound->Calloc(csound,
                                          obuflen * sizeof(MYFLT))) == NULL) {
-      ERR("dnoise: insufficient memory\n");
+      ERR(Str("dnoise: insufficient memory\n"));
     }
 
     /* set up analysis buffer for (N/2 + 1) channels: The input is real,
         so the other channels are redundant. */
 
     if ((fbuf = (MYFLT*) csound->Calloc(csound, Np2 * sizeof(MYFLT))) == NULL) {
-      ERR("dnoise: insufficient memory\n");
+      ERR(Str("dnoise: insufficient memory\n"));
     }
 
 /* noise reduction: calculate noise reference by taking as many
@@ -750,24 +743,24 @@ static int dnoise(CSOUND *csound, int argc, char **argv)
 
     if ((nref = (MYFLT*) csound->Calloc(csound,
                                         (N2 + 1) * sizeof(MYFLT))) == NULL) {
-      ERR("dnoise: insufficient memory\n");
+      ERR(Str("dnoise: insufficient memory\n"));
     }
 
     if ((mbuf = (MYFLT*) csound->Calloc(csound,
                                         (m * Np2) * sizeof(MYFLT))) == NULL) {
-      ERR("dnoise: insufficient memory\n");
+      ERR(Str("dnoise: insufficient memory\n"));
     }
     if ((nbuf = (MYFLT*) csound->Calloc(csound,
                                         (m * Np2) * sizeof(MYFLT))) == NULL) {
-      ERR("dnoise: insufficient memory\n");
+      ERR(Str("dnoise: insufficient memory\n"));
     }
     if ((rsum = (MYFLT*) csound->Calloc(csound,
                                         (N2 + 1) * sizeof(MYFLT))) == NULL) {
-      ERR("dnoise: insufficient memory\n");
+      ERR(Str("dnoise: insufficient memory\n"));
     }
     if ((ssum = (MYFLT*) csound->Calloc(csound,
                                         (N2 + 1) * sizeof(MYFLT))) == NULL) {
-      ERR("dnoise: insufficient memory\n");
+      ERR(Str("dnoise: insufficient memory\n"));
     }
 
     /* skip over nMin samples */
@@ -775,8 +768,10 @@ static int dnoise(CSOUND *csound, int argc, char **argv)
       if (!csound->CheckEvents(csound))
         csound->LongJmp(csound, 1);
       nread = csound->getsndin(csound, fp, ibuf1, ibuflen, pn);
+      for(i=0; i < nread; i++)
+        ibuf1[i] *= 1.0/csound->Get0dBFS(csound);
       if (nread < ibuflen) {
-        ERR("dnoise: begin time is greater than EOF of noise file!");
+        ERR(Str("dnoise: begin time is greater than EOF of noise file!"));
       }
       nMin -= (long) ibuflen;
     }
@@ -784,8 +779,10 @@ static int dnoise(CSOUND *csound, int argc, char **argv)
       csound->LongJmp(csound, 1);
     i = (int) nMin;
     nread = csound->getsndin(csound, fp, ibuf1, i, pn);
+    for(i=0; i < nread; i++)
+        ibuf1[i] *= 1.0/csound->Get0dBFS(csound);
     if (nread < i) {
-      ERR("dnoise: begin time is greater than EOF of noise file!");
+      ERR(Str("dnoise: begin time is greater than EOF of noise file!"));
     }
     k = 0;
     lj = Beg;  /* single channel only */
@@ -794,6 +791,8 @@ static int dnoise(CSOUND *csound, int argc, char **argv)
         csound->LongJmp(csound, 1);
       lj += (long) N;
       nread = csound->getsndin(csound, fp, fbuf, N, pn);
+      for(i=0; i < nread; i++)
+        fbuf[i] *= 1.0/csound->Get0dBFS(csound);
       if (nread < N)
         break;
 
@@ -817,7 +816,7 @@ static int dnoise(CSOUND *csound, int argc, char **argv)
       k++;
     }
     if (k == 0) {
-      ERR("dnoise: not enough samples of noise reference\n");
+      ERR(Str("dnoise: not enough samples of noise reference\n"));
     }
     fac = th * th / k;
     f = nref;
@@ -838,20 +837,18 @@ static int dnoise(CSOUND *csound, int argc, char **argv)
     nread = csound->getsndin(csound, inf, ibuf2, ibuflen, p);
 /*     nread = read(inf, ibuf2, ibuflen*sizeof(MYFLT)); */
 /*     nread /= sizeof(MYFLT); */
+    for(i=0; i < nread; i++)
+        ibuf2[i] *= 1.0/csound->Get0dBFS(csound);
     lnread = nread;
     f = ibuf2 + nread;
     for (i = nread; i < ibuflen; i++, f++)
       *f = FL(0.0);
 
-    rIn = ((MYFLT) R / D);
-    rOut = ((MYFLT) R / I);
+    //rIn = ((MYFLT) R / D);
+    //rOut = ((MYFLT) R / I);
     invR = FL(1.0) / R;
-    RoverTwoPi = rIn / TWOPI_F;
-    TwoPioverR = TWOPI_F / rOut;
     nI = -(aLen / D) * D;    /* input time (in samples) */
     nO = nI;                 /* output time (in samples) */
-    Dd = aLen + nI + 1;      /* number of new inputs to read */
-    Ii = 0;                  /* number of new outputs to write */
     ibs = ibuflen + Chans * (nI - aLen - 1);    /* starting position in ib1 */
     ib1 = ibuf1;        /* filled with zeros to start */
     ib2 = ibuf2;        /* first buffer of speech */
@@ -890,6 +887,8 @@ static int dnoise(CSOUND *csound, int argc, char **argv)
           ibs -= ibuflen;
           /* fill ib2 */
           nread = csound->getsndin(csound, inf, ib2, ibuflen, p);
+          for(i=0; i < nread; i++)
+               ibuf2[i] *= 1.0/csound->Get0dBFS(csound);
           lnread += nread;
           f = ib2 + nread;
           for (i = nread; i < ibuflen; i++, f++)
@@ -905,11 +904,11 @@ static int dnoise(CSOUND *csound, int argc, char **argv)
           }
           else {
             if ((oCnt + obuflen) < nMaxOut) {
-              oCnt += writebuffer(csound, outfd, ob1, obuflen, &nrecs);
+              oCnt += writebuffer(csound, outfd, ob1, obuflen, &nrecs, &O);
             }
             else {
               i = (int) (nMaxOut - oCnt);
-              oCnt += writebuffer(csound, outfd, ob1, i, &nrecs);
+              oCnt += writebuffer(csound, outfd, ob1, i, &nrecs, &O);
             }
           }
           /* zero ob1 */
@@ -1115,7 +1114,7 @@ static int dnoise(CSOUND *csound, int argc, char **argv)
         if (nImodR > (long) R) {
           nImodR -= (long) R;
           csound->Message(csound,
-                           "%5.1f seconds of input complete\n",(time+D*invR));
+                          Str("%5.1f seconds of input complete\n"),(time+D*invR));
         }
       }
 
@@ -1124,17 +1123,17 @@ static int dnoise(CSOUND *csound, int argc, char **argv)
     nMaxOut = (long) (nMax * Chans);
     i = (int) (nMaxOut - oCnt);
     if (i > obuflen) {
-      writebuffer(csound, outfd, ob1, obuflen, &nrecs);
+      writebuffer(csound, outfd, ob1, obuflen, &nrecs, &O);
       i -= obuflen;
       ob1 = ob2;
     }
     if (i > 0)
-      writebuffer(csound, outfd, ob1, i, &nrecs);
+      writebuffer(csound, outfd, ob1, i, &nrecs, &O);
 
 /*  csound->rewriteheader(outfd); */
     csound->Message(csound, "\n\n");
     if (Verbose) {
-      csound->Message(csound, "processing complete\n");
+      csound->Message(csound, Str("processing complete\n"));
       csound->Message(csound, "N = %d\n", N);
       csound->Message(csound, "M = %d\n", M);
       csound->Message(csound, "L = %d\n", L);
@@ -1144,28 +1143,28 @@ static int dnoise(CSOUND *csound, int argc, char **argv)
 }
 
 static const char *usage_txt[] = {
-    "usage: dnoise [flags] input_file",
+  Str_noop("usage: dnoise [flags] input_file"),
     "",
-    "flags:",
-    "i = noise reference soundfile",
-    "o = output file",
-    "N = # of bandpass filters (1024)",
-    "w = filter overlap factor: {0,1,(2),3} DON'T USE -w AND -M",
-    "M = analysis window length (N-1 unless -w is specified)",
-    "L = synthesis window length (M)",
-    "D = decimation factor (M/8)",
-    "b = begin time in noise reference soundfile (0)",
-    "B = starting sample in noise reference soundfile (0)",
-    "e = end time in noise reference soundfile (end)",
-    "E = final sample in noise reference soundfile (end)",
-    "t = threshold above noise reference in dB (30)",
-    "S = sharpness of noise-gate turnoff (1) (1 to 5)",
-    "n = number of FFT frames to average over (5)",
-    "m = minimum gain of noise-gate when off in dB (-40)",
-    "V : verbose - print status info",
-    "A : AIFF format output",
-    "W : WAV format output",
-    "J : IRCAM format output",
+  Str_noop("flags:"),
+  Str_noop("i = noise reference soundfile"),
+  Str_noop("o = output file"),
+  Str_noop("N = # of bandpass filters (1024)"),
+  Str_noop("w = filter overlap factor: {0,1,(2),3} DO NOT USE -w AND -M"),
+  Str_noop("M = analysis window length (N-1 unless -w is specified)"),
+  Str_noop("L = synthesis window length (M)"),
+  Str_noop("D = decimation factor (M/8)"),
+  Str_noop("b = begin time in noise reference soundfile (0)"),
+  Str_noop("B = starting sample in noise reference soundfile (0)"),
+  Str_noop("e = end time in noise reference soundfile (end)"),
+  Str_noop("E = final sample in noise reference soundfile (end)"),
+  Str_noop("t = threshold above noise reference in dB (30)"),
+  Str_noop("S = sharpness of noise-gate turnoff (1) (1 to 5)"),
+  Str_noop("n = number of FFT frames to average over (5)"),
+  Str_noop("m = minimum gain of noise-gate when off in dB (-40)"),
+  Str_noop("V : verbose - print status info"),
+  Str_noop("A : AIFF format output"),
+  Str_noop("W : WAV format output"),
+  Str_noop("J : IRCAM format output"),
     NULL
 };
 
@@ -1182,26 +1181,26 @@ static int dnoise_usage(CSOUND *csound, int exitcode)
 /* report soundfile write(osfd) error      */
 /*    called after chk of write() bytecnt  */
 
-static void sndwrterr(CSOUND *csound, SNDFILE *outfd, int nret, int nput)
+static void sndwrterr(CSOUND *csound, int nret, int nput)
 {
     csound->Message(csound, Str("soundfile write returned sample count of %d, "
                                 "not %d\n"), nret, nput);
     csound->Message(csound, Str("(disk may be full...\n"
                                 " closing the file ...)\n"));
     /* FIXME: should clean up */
-    csound->Die(csound, Str("\t... closed\n"));
+    //csound->Die(csound, Str("\t... closed\n"));
 }
 
 static int writebuffer(CSOUND *csound, SNDFILE *outfd,
-                       MYFLT *outbuf, int nsmps, int *nrecs)
+                       MYFLT *outbuf, int nsmps, int *nrecs, OPARMS *O)
 {
-    OPARMS  *O = csound->oparms;
     int     n;
 
     if (outfd == NULL) return 0;
     n = sf_write_MYFLT(outfd, outbuf, nsmps);
     if (n < nsmps) {
-      sndwrterr(csound, outfd, n, nsmps);
+      sf_close(outfd);
+      sndwrterr(csound, n, nsmps);
       return -1;
     }
     if (O->rewrt_hdr)
@@ -1252,9 +1251,9 @@ int dnoise_init_(CSOUND *csound)
 {
     int retval = csound->AddUtility(csound, "dnoise", dnoise);
     if (!retval) {
-      retval = csound->SetUtilityDescription(csound, "dnoise",
-                                             "Removes noise from a sound file");
+      retval =
+        csound->SetUtilityDescription(csound, "dnoise",
+                                      Str("Removes noise from a sound file"));
     }
     return retval;
 }
-
