@@ -6,19 +6,20 @@
 
     This file is part of Csound.
 
-    Csound is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
+    The Csound Library is free software; you can redistribute it
+    and/or modify it under the terms of the GNU Lesser General Public
+    License as published by the Free Software Foundation; either
+    version 2.1 of the License, or (at your option) any later version.
 
     Csound is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
+    GNU Lesser General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with Csound; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    You should have received a copy of the GNU Lesser General Public
+    License along with Csound; if not, write to the Free Software
+    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
+    02111-1307 USA
 */
 
 /*******************************************************\
@@ -39,28 +40,28 @@
                             csound->Die(csound, MSG);
 
 static const char *usage_txt[] = {
-    "Usage:\tscale [-flags] soundfile",
-    "Legal flags are:",
-    "-o fnam\tsound output filename",
-    "-A\tcreate an AIFF format output soundfile",
-    "-W\tcreate a WAV format output soundfile",
-    "-h\tno header on output soundfile",
-    "-c\t8-bit signed_char sound samples",
-    "-a\talaw sound samples",
-    "-u\tulaw sound samples",
-    "-s\tshort_int sound samples",
-    "-l\tlong_int sound samples",
-    "-f\tfloat sound samples",
-    "-F fpnum\tamount to scale amplitude",
-    "-F file \tfile of scaling information (alternative)",
-    "-M fpnum\tScale file to given maximum",
-    "-P fpnum\tscale file to given percentage of full",
-    "-R\tcontinually rewrite header while writing soundfile (WAV/AIFF)",
-    "-H#\tprint a heartbeat style 1, 2 or 3 at each soundfile write",
-    "-N\tnotify (ring the bell) when score or miditrack is done",
-    "-- fnam\tlog output to file",
-    "flag defaults: scale -s -otest -F 0.0",
-    "If scale is 0.0 then reports maximum possible scaling",
+  Str_noop("Usage:\tscale [-flags] soundfile"),
+  Str_noop("Legal flags are:"),
+  Str_noop("-o fnam\tsound output filename"),
+  Str_noop("-A\tcreate an AIFF format output soundfile"),
+  Str_noop("-W\tcreate a WAV format output soundfile"),
+  Str_noop("-h\tno header on output soundfile"),
+  Str_noop("-c\t8-bit signed_char sound samples"),
+  Str_noop("-a\talaw sound samples"),
+  Str_noop("-u\tulaw sound samples"),
+  Str_noop("-s\tshort_int sound samples"),
+  Str_noop("-l\tlong_int sound samples"),
+  Str_noop("-f\tfloat sound samples"),
+  Str_noop("-F fpnum\tamount to scale amplitude"),
+  Str_noop("-F file \tfile of scaling information (alternative)"),
+  Str_noop("-M fpnum\tScale file to given maximum"),
+  Str_noop("-P fpnum\tscale file to given percentage of full"),
+  Str_noop("-R\tcontinually rewrite header while writing soundfile (WAV/AIFF)"),
+  Str_noop("-H#\tprint a heartbeat style 1, 2 or 3 at each soundfile write"),
+  Str_noop("-N\tnotify (ring the bell) when score or miditrack is done"),
+  Str_noop("-- fnam\tlog output to file"),
+  Str_noop("flag defaults: scale -s -otest -F 0.0"),
+  Str_noop("If scale is 0.0 then reports maximum possible scaling"),
     NULL
 };
 
@@ -72,7 +73,7 @@ static void usage(CSOUND *csound, char *mesg)
     csound->Die(csound, "\n%s", mesg);
 }
 
-static char set_output_format(CSOUND *csound, OPARMS *p, char c, char outformch)
+static char set_output_format(OPARMS *p, char c, char outformch)
 {
     switch (c) {
       case 'a': p->outformat = AE_ALAW;   /* a-law soundfile */
@@ -123,8 +124,8 @@ typedef struct {
 
 static void  InitScaleTable(CSOUND *,SCALE *, double, char *);
 static SNDFILE *SCsndgetset(CSOUND *, SCALE *, char *);
-static void  ScaleSound(CSOUND *, SCALE *, SNDFILE *, SNDFILE *);
-static float FindAndReportMax(CSOUND *, SCALE *, SNDFILE *);
+static void  ScaleSound(CSOUND *, SCALE *, SNDFILE *, SNDFILE *, OPARMS *);
+static float FindAndReportMax(CSOUND *, SCALE *, SNDFILE *, OPARMS *);
 
 static int scale(CSOUND *csound, int argc, char **argv)
 {
@@ -137,7 +138,7 @@ static int scale(CSOUND *csound, int argc, char **argv)
     char        outformch = 's', c, *s;
     const char  *envoutyp;
     SF_INFO     sfinfo;
-    OPARMS      *O = csound->oparms;
+    OPARMS      O;
     SCALE       sc;
     unsigned    outbufsiz;
 
@@ -147,15 +148,16 @@ static int scale(CSOUND *csound, int argc, char **argv)
     sc.scale_table = stattab;
     sc.end_table = &sc.scale_table;
 
-    O->filetyp = O->outformat = 0;
+    O.filetyp = O.outformat = 0;
+    O.ringbell = O.heartbeat = 0;
     /* Check arguments */
     if ((envoutyp = csound->GetEnv(csound, "SFOUTYP")) != NULL) {
       if (strcmp(envoutyp, "AIFF") == 0)
-        O->filetyp = TYP_AIFF;
+        O.filetyp = TYP_AIFF;
       else if (strcmp(envoutyp, "WAV") == 0)
-        O->filetyp = TYP_WAV;
+        O.filetyp = TYP_WAV;
       else if (strcmp(envoutyp, "IRCAM") == 0)
-        O->filetyp = TYP_IRCAM;
+        O.filetyp = TYP_IRCAM;
       else {
         csound->Die(csound, Str("%s not a recognized SFOUTYP env setting"),
                             envoutyp);
@@ -170,24 +172,24 @@ static int scale(CSOUND *csound, int argc, char **argv)
           switch(c) {
           case 'o':
             FIND(Str("no outfilename"))
-            O->outfilename = s;         /* soundout name */
+            O.outfilename = s;         /* soundout name */
             for ( ; *s != '\0'; s++) ;
-            if (strcmp(O->outfilename, "stdin") == 0)
-              csound->Die(csound, "-o cannot be stdin");
-#if defined(mac_classic) || defined(WIN32)
-            if (strcmp(O->outfilename, "stdout") == 0) {
+            if (strcmp(O.outfilename, "stdin") == 0)
+              csound->Die(csound, Str("-o cannot be stdin"));
+#if defined(WIN32)
+            if (strcmp(O.outfilename, "stdout") == 0) {
               csound->Die(csound, Str("stdout audio not supported"));
             }
 #endif
             break;
           case 'A':
-            O->filetyp = TYP_AIFF;      /* AIFF output request  */
+            O.filetyp = TYP_AIFF;      /* AIFF output request  */
             break;
           case 'J':
-            O->filetyp = TYP_IRCAM;     /* IRCAM output request */
+            O.filetyp = TYP_IRCAM;     /* IRCAM output request */
             break;
           case 'W':
-            O->filetyp = TYP_WAV;       /* WAV output request  */
+            O.filetyp = TYP_WAV;       /* WAV output request  */
             break;
           case 'F':
             FIND(Str("no scale factor"));
@@ -204,11 +206,11 @@ static int scale(CSOUND *csound, int argc, char **argv)
             break;
           case 'P':       /* Percentage */
             FIND(Str("No maximum"));
-            maximum = atof(s) * 0.01 * csound->e0dbfs;
+            maximum = atof(s) * 0.01 * csound->Get0dBFS(csound);
             while (*++s);
             break;
           case 'h':
-            O->filetyp = TYP_RAW;       /* skip sfheader  */
+            O.filetyp = TYP_RAW;       /* skip sfheader  */
             break;
           case 'c':
           case 'a':
@@ -218,26 +220,26 @@ static int scale(CSOUND *csound, int argc, char **argv)
           case '3':
           case 'l':
           case 'f':
-            outformch = set_output_format(csound, O, c, outformch);
+            outformch = set_output_format(&O, c, outformch);
             break;
           case 'R':
-            O->rewrt_hdr = 1;
+            O.rewrt_hdr = 1;
             break;
           case 'H':
             if (isdigit(*s)) {
               int n;
-              sscanf(s, "%d%n", &O->heartbeat, &n);
+              sscanf(s, "%d%n", &O.heartbeat, &n);
               s += n;
             }
-            else O->heartbeat = 1;
+            else O.heartbeat = 1;
             break;
           case 'N':
-            O->ringbell = 1;             /* notify on completion */
+            O.ringbell = 1;             /* notify on completion */
             break;
           default:
             {
               char  err_msg[64];
-              sprintf(err_msg, Str("unknown flag -%c"), c);
+              snprintf(err_msg, 64, Str("unknown flag -%c"), c);
               usage(csound, err_msg);
             }
           }
@@ -249,33 +251,35 @@ static int scale(CSOUND *csound, int argc, char **argv)
 
  retry:
     /* Read sound file */
+    if (inputfile == NULL) return -1;
     if (!(infile = SCsndgetset(csound, &sc, inputfile))) {
       csound->Message(csound, Str("%s: error while opening %s"),
                               argv[0], inputfile);
       return -1;
     }
     if (factor != 0.0 || factorfile != NULL) {          /* perform scaling */
-      if (!O->filetyp)
-        O->filetyp = sc.p->filetyp;
-      if (!O->outformat)
-        O->outformat = sc.p->format;
-      O->sfheader = (O->filetyp == TYP_RAW ? 0 : 1);
-      O->sfsampsize = csound->sfsampsize(FORMAT2SF(O->outformat));
-      if (!O->sfheader)
-        O->rewrt_hdr = 0;
-      if (O->outfilename == NULL)
-        O->outfilename = "test";
-      csound->esr = sc.p->sr;
-      csound->nchnls = sc.p->nchanls;
+      if (!O.filetyp)
+        O.filetyp = sc.p->filetyp;
+      if (!O.outformat)
+        O.outformat = sc.p->format;
+      O.sfheader = (O.filetyp == TYP_RAW ? 0 : 1);
+      O.sfsampsize = csound->sfsampsize(FORMAT2SF(O.outformat));
+      if (!O.sfheader)
+        O.rewrt_hdr = 0;
+      if (O.outfilename == NULL)
+        O.outfilename = "test";
+      csound->SetUtilSr(csound, (MYFLT)sc.p->sr);
+      csound->SetUtilNchnls(csound, sc.p->nchanls);
+
       memset(&sfinfo, 0, sizeof(SF_INFO));
       sfinfo.frames = -1;
-      sfinfo.samplerate = (int) MYFLT2LRND(csound->esr);
-      sfinfo.channels = csound->nchnls;
-      sfinfo.format = TYPE2SF(O->filetyp) | FORMAT2SF(O->outformat);
+      sfinfo.samplerate = (int) MYFLT2LRND( sc.p->sr);
+      sfinfo.channels = sc.p->nchanls;
+      sfinfo.format = TYPE2SF(O.filetyp) | FORMAT2SF(O.outformat);
       /* open file for write */
       fd = NULL;
-      if (strcmp(O->outfilename, "stdout") == 0 ||
-          strcmp(O->outfilename, "-") == 0) {
+      if (strcmp(O.outfilename, "stdout") == 0 ||
+          strcmp(O.outfilename, "-") == 0) {
         outfile = sf_open_fd(1, SFM_WRITE, &sfinfo, 0);
         if (outfile != NULL) {
           if ((fd = csound->CreateFileHandle(csound, &outfile,
@@ -287,28 +291,28 @@ static int scale(CSOUND *csound, int argc, char **argv)
       }
       else
         fd = csound->FileOpen2(csound, &outfile, CSFILE_SND_W,
-                       O->outfilename, &sfinfo, "SFDIR",
-                       csound->type2csfiletype(O->filetyp, O->outformat), 0);
+                       O.outfilename, &sfinfo, "SFDIR",
+                       csound->type2csfiletype(O.filetyp, O.outformat), 0);
       if (fd == NULL)
         csound->Die(csound, Str("Failed to open output file %s"),
-                            O->outfilename);
-      outbufsiz = 1024 * O->sfsampsize;    /* calc outbuf size  */
+                            O.outfilename);
+      outbufsiz = 1024 * O.sfsampsize;    /* calc outbuf size  */
       csound->Message(csound, Str("writing %d-byte blks of %s to %s %s\n"),
                               (int) outbufsiz,
-                              csound->getstrformat(O->outformat),
-                              O->outfilename,
-                              csound->type2string(O->filetyp));
+                              csound->getstrformat(O.outformat),
+                              O.outfilename,
+                              csound->type2string(O.filetyp));
       InitScaleTable(csound, &sc, factor, factorfile);
-      ScaleSound(csound, &sc, infile, outfile);
+      ScaleSound(csound, &sc, infile, outfile, &O);
     }
     else if (maximum != 0.0) {
-      float mm = FindAndReportMax(csound, &sc, infile);
+      float mm = FindAndReportMax(csound, &sc, infile, &O);
       factor = maximum / mm;
       goto retry;
     }
     else
-      FindAndReportMax(csound, &sc, infile);
-    if (O->ringbell)
+      FindAndReportMax(csound, &sc, infile, &O);
+    if (O.ringbell)
       csound->MessageS(csound, CSOUNDMSG_REALTIME, "%c", '\007');
     return 0;
 }
@@ -389,17 +393,17 @@ SCsndgetset(CSOUND *csound, SCALE *thissc, char *inputfile)
     double  dur;
     SOUNDIN *p;
 
-    csound->esr = FL(0.0);      /* set esr 0. with no orchestra   */
+    csound->SetUtilSr(csound, FL(0.0));         /* set esr 0. with no orchestra */
     thissc->p = p = (SOUNDIN *) csound->Calloc(csound, sizeof(SOUNDIN));
     p->channel = ALLCHNLS;
     p->skiptime = FL(0.0);
     p->analonly = 1;
-    strcpy(p->sfname, inputfile);
+    strncpy(p->sfname, inputfile, MAXSNDNAME-1);p->sfname[MAXSNDNAME-1]='\0';
     if ((infile = csound->sndgetset(csound, p)) == 0) /*open sndfil, do skptim*/
       return(0);
     p->getframes = p->framesrem;
     dur = (double) p->getframes / p->sr;
-    csound->Message(csound, "scaling %ld sample frames (%3.1f secs)\n",
+    csound->Message(csound, Str("scaling %ld sample frames (%3.1f secs)\n"),
                             (long) p->getframes, dur);
     return(infile);
 }
@@ -407,7 +411,8 @@ SCsndgetset(CSOUND *csound, SCALE *thissc, char *inputfile)
 #define BUFFER_LEN (1024)
 
 static void
-ScaleSound(CSOUND *csound, SCALE *thissc, SNDFILE *infile, SNDFILE *outfd)
+ScaleSound(CSOUND *csound, SCALE *thissc, SNDFILE *infile,
+           SNDFILE *outfd, OPARMS *oparms)
 {
     MYFLT buffer[BUFFER_LEN];
     long  read_in;
@@ -434,27 +439,29 @@ ScaleSound(CSOUND *csound, SCALE *thissc, SNDFILE *infile, SNDFILE *outfd)
           max = buffer[i], mxpos = i + bufferLenSamples * block, maxtimes = 1;
         if (buffer[i] < min)
           min = buffer[i], minpos = i + bufferLenSamples * block, mintimes = 1;
-        buffer[i] *= csound->dbfs_to_float;
+        buffer[i] *= (1.0/csound->Get0dBFS(csound));
       }
       sf_write_MYFLT(outfd, buffer, read_in);
       block++;
-      if (csound->oparms->heartbeat) {
+      if (oparms->heartbeat) {
         csound->MessageS(csound, CSOUNDMSG_REALTIME, "%c\b", "|/-\\"[block&3]);
       }
     }
-    csound->Message(csound, "Max val %.3f at index %ld (time %.4f, chan %d) "
-                            "%d times\n", max, (long) mxpos / (long) chans,
+    csound->Message(csound, Str("Max val %.3f at index %ld (time %.4f, chan %d) "
+                                "%d times\n"), max, (long) mxpos / (long) chans,
                             tpersample * (double) mxpos / (double) chans,
                             ((int) mxpos % chans) + 1, (int) maxtimes);
-    csound->Message(csound, "Min val %.3f at index %ld (time %.4f, chan %d) "
-                            "%d times\n", min, (long) minpos / (long) chans,
+    csound->Message(csound, Str("Min val %.3f at index %ld (time %.4f, chan %d) "
+                                "%d times\n"), min, (long) minpos / (long) chans,
                             tpersample * (double) minpos / (double) chans,
                             ((int) minpos % chans) + 1, (int) mintimes);
-    csound->Message(csound, "Max scale factor = %.3f\n",
-                            (double) csound->e0dbfs / (max > -min ? max:-min));
+    csound->Message(csound, Str("Max scale factor = %.3f\n"),
+                            (double) csound->Get0dBFS(csound) / (max > -min ?
+                                                                 max:-min));
 }
 
-static float FindAndReportMax(CSOUND *csound, SCALE *thissc, SNDFILE *infile)
+static float FindAndReportMax(CSOUND *csound, SCALE *thissc,
+                              SNDFILE *infile, OPARMS *oparms)
 {
     MYFLT   buffer[BUFFER_LEN];
     long    read_in;
@@ -462,7 +469,7 @@ static float FindAndReportMax(CSOUND *csound, SCALE *thissc, SNDFILE *infile)
     double  max, min;
     long    mxpos, minpos;
     int     maxtimes, mintimes;
-    int     i, j, chans = thissc->p->nchanls;
+    int     i, chans = thissc->p->nchanls;
     int     block = 0;
     int     bufferLenFrames = (int) BUFFER_LEN / chans;
     int     bufferLenSamples = bufferLenFrames * chans;
@@ -473,7 +480,7 @@ static float FindAndReportMax(CSOUND *csound, SCALE *thissc, SNDFILE *infile)
     while ((read_in = csound->getsndin(csound, infile, buffer,
                                        bufferLenSamples, thissc->p)) > 0) {
       for (i = 0; i < read_in; i++) {
-        j = (i / chans) + (bufferLenFrames * block);
+        //j = (i / chans) + (bufferLenFrames * block);
         if (buffer[i] >= max) ++maxtimes;
         if (buffer[i] <= min) ++mintimes;
         if (buffer[i] > max)
@@ -482,20 +489,21 @@ static float FindAndReportMax(CSOUND *csound, SCALE *thissc, SNDFILE *infile)
           min = buffer[i], minpos = i + bufferLenSamples * block, mintimes = 1;
       }
       block++;
-      if (csound->oparms->heartbeat) {
+      if (oparms->heartbeat) {
         csound->MessageS(csound, CSOUNDMSG_REALTIME, "%c\b", "|/-\\"[block&3]);
       }
     }
-    csound->Message(csound, "Max val %.3f at index %ld (time %.4f, chan %d) "
-                            "%d times\n", max, (long) mxpos / (long) chans,
+    csound->Message(csound, Str("Max val %.3f at index %ld (time %.4f, chan %d) "
+                                "%d times\n"), max, (long) mxpos / (long) chans,
                             tpersample * (double) mxpos / (double) chans,
                             ((int) mxpos % chans) + 1, (int) maxtimes);
-    csound->Message(csound, "Min val %.3f at index %ld (time %.4f, chan %d) "
-                            "%d times\n", min, (long) minpos / (long) chans,
+    csound->Message(csound, Str("Min val %.3f at index %ld (time %.4f, chan %d) "
+                                "%d times\n"), min, (long) minpos / (long) chans,
                             tpersample * (double) minpos / (double) chans,
                             ((int) minpos % chans) + 1, (int) mintimes);
-    csound->Message(csound, "Max scale factor = %.3f\n",
-                            (double) csound->e0dbfs / (max > -min ? max:-min));
+    csound->Message(csound, Str("Max scale factor = %.3f\n"),
+                            (double) csound->Get0dBFS(csound)/ (max > -min ?
+                                                                max:-min));
     return (float) (max > -min ? max : -min);
 }
 
@@ -506,7 +514,7 @@ int scale_init_(CSOUND *csound)
     int retval = csound->AddUtility(csound, "scale", scale);
     if (retval)
       return retval;
-    return csound->SetUtilityDescription(csound, "scale",
-                                         "Reports and/or adjusts maximum gain");
+    return
+      csound->SetUtilityDescription(csound, "scale",
+                                    Str("Reports and/or adjusts maximum gain"));
 }
-
