@@ -5,19 +5,20 @@
 
     This file is part of Csound.
 
-    Csound is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
+    The Csound Library is free software; you can redistribute it
+    and/or modify it under the terms of the GNU Lesser General Public
+    License as published by the Free Software Foundation; either
+    version 2.1 of the License, or (at your option) any later version.
 
     Csound is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
+    GNU Lesser General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with Csound; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    You should have received a copy of the GNU Lesser General Public
+    License along with Csound; if not, write to the Free Software
+    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
+    02111-1307 USA
 */
 
 /*
@@ -60,17 +61,18 @@
       }                                                             \
 }
 
-static  void    kaiser(int, float *, int, int, double);
+//static  void    kaiser(int, float *, int, int, double);
 static  void    usage(CSOUND *);
 
+#if 0
 static int writebuffer(CSOUND *csound, MYFLT *out_buf, int *block,
-                                        SNDFILE *outfd, int length)
+                       SNDFILE *outfd, int length, OPARMS *oparms)
 {
     sf_write_MYFLT(outfd, out_buf, length);
     (*block)++;
-    if (csound->oparms->rewrt_hdr)
-       csound->rewriteheader(outfd);
-    switch (csound->oparms->heartbeat) {
+    if (oparms->rewrt_hdr)
+      csound->rewriteheader((struct SNDFILE *)outfd);
+    switch (oparms->heartbeat) {
       case 1:
         csound->MessageS(csound, CSOUNDMSG_REALTIME, "%c\010",
                                                      "|/-\\"[*block & 3]);
@@ -91,41 +93,43 @@ static int writebuffer(CSOUND *csound, MYFLT *out_buf, int *block,
     return length;
 }
 
-static char set_output_format(CSOUND *csound, char c, char outformch)
+static char set_output_format(CSOUND *csound, char c, char outformch, OPARMS *oparms)
 {
-    if (csound->oparms->outformat) {
+    if (oparms->outformat) {
       csound->Warning(csound, Str("Sound format -%c has been overruled by -%c"),
                               outformch, c);
     }
     switch (c) {
     case 'a':
-      csound->oparms->outformat = AE_ALAW;  /* a-law soundfile */
+      oparms->outformat = AE_ALAW;  /* a-law soundfile */
       break;
     case 'c':
-      csound->oparms->outformat = AE_CHAR;  /* signed 8-bit soundfile */
+      oparms->outformat = AE_CHAR;  /* signed 8-bit soundfile */
       break;
     case '8':
-      csound->oparms->outformat = AE_UNCH;  /* unsigned 8-bit soundfile */
-      break;
+      oparms->outformat = AE_UNCH;  /* unsigned 8-bit soundfile */
+     break;
     case 'f':
-      csound->oparms->outformat = AE_FLOAT; /* float soundfile */
+      oparms->outformat = AE_FLOAT; /* float soundfile */
       break;
     case 's':
-      csound->oparms->outformat = AE_SHORT; /* short_int soundfile*/
+      oparms->outformat = AE_SHORT; /* short_int soundfile*/
       break;
     case 'l':
-      csound->oparms->outformat = AE_LONG;  /* long_int soundfile */
+      oparms->outformat = AE_LONG;  /* long_int soundfile */
       break;
     case 'u':
-      csound->oparms->outformat = AE_ULAW;  /* mu-law soundfile */
+      oparms->outformat = AE_ULAW;  /* mu-law soundfile */
       break;
     case '3':
-      csound->oparms->outformat = AE_24INT; /* 24bit packed soundfile*/
+      oparms->outformat = AE_24INT; /* 24bit packed soundfile*/
       break;
     case 'e':
-      csound->oparms->outformat = AE_FLOAT; /* float sndfile (for rescaling) */
+      oparms->outformat = AE_FLOAT; /* float sndfile (for rescaling) */
       break;
     default:
+      oparms->outformat = 0;
+      csound->ErrorMsg(csound, Str("srconv: unknown outout format '%c'\n"), c);
       return outformch; /* do nothing */
     };
     return c;
@@ -144,7 +148,7 @@ static int srconv(CSOUND *csound, int argc, char **argv)
       *output,    /* pointer to start of output buffer */
       *nextIn,    /* pointer to next empty word in input */
       *nextOut,   /* pointer to next empty word in output */
-      *fxval = 0, /* pointer to start of time-array for time-vary function */
+      *fxval = 0, /* pointer to startb of time-array for time-vary function */
       *fyval = 0, /* pointer to start of P-scale-array for time-vary func */
       *i0,        /* pointer */
       *i1;        /* pointer */
@@ -198,13 +202,12 @@ static int srconv(CSOUND *csound, int argc, char **argv)
       tvflg = 0,                /* flag for time-varying time-scaling */
       tvnxt = 0,                /* counter for stepping thru time-var func */
       tvlen,                    /* length of time-varying function */
-      Chans = 0,                /* number of channels */
+      Chans = 1,                /* number of channels */
       chan,                     /* current channel */
       Q = 2;                    /* quality factor */
 
     FILE        *tvfp = NULL;   /* time-vary function file */
     SOUNDIN     *p;
-    void        *dummy;
     int         channel = ALLCHNLS;
     MYFLT       beg_time = FL(0.0), input_dur = FL(0.0), sr = FL(0.0);
     char        *infile = NULL, *bfile = NULL;
@@ -214,22 +217,23 @@ static int srconv(CSOUND *csound, int argc, char **argv)
     char        outformch = 's';
     unsigned    outbufsiz = 0U;
     SNDFILE     *outfd = NULL;
-    OPARMS      *O = csound->oparms;
+    OPARMS      O;
     int         block = 0;
     char        err_msg[256];
 
-    csound->e0dbfs = csound->dbfs_to_float = FL(1.0);
+    O.outformat = AE_SHORT;
+    /* csound->e0dbfs = csound->dbfs_to_float = FL(1.0);*/
 
     if ((envoutyp = csound->GetEnv(csound, "SFOUTYP")) != NULL) {
       if (strcmp(envoutyp, "AIFF") == 0)
-        O->filetyp = TYP_AIFF;
+        O.filetyp = TYP_AIFF;
       else if (strcmp(envoutyp, "WAV") == 0)
-        O->filetyp = TYP_WAV;
+        O.filetyp = TYP_WAV;
       else if (strcmp(envoutyp, "IRCAM") == 0)
-        O->filetyp = TYP_IRCAM;
+        O.filetyp = TYP_IRCAM;
       else {
-        sprintf(err_msg, Str("%s not a recognized SFOUTYP env setting"),
-                         envoutyp);
+        snprintf(err_msg, 256, Str("%s not a recognized SFOUTYP env setting"),
+                envoutyp);
         dieu(csound, err_msg);
         return -1;
       }
@@ -245,30 +249,30 @@ static int srconv(CSOUND *csound, int argc, char **argv)
           switch (c) {
           case 'o':
             FIND(Str("no outfilename"))
-            O->outfilename = s;         /* soundout name */
+            O.outfilename = s;         /* soundout name */
             for ( ; *s != '\0'; s++) ;
-            if (strcmp(O->outfilename, "stdin") == 0) {
+            if (strcmp(O.outfilename, "stdin") == 0) {
               csound->ErrorMsg(csound, Str("-o cannot be stdin"));
               return -1;
             }
-#if defined mac_classic || defined WIN32
-            if (strcmp(O->outfilename, "stdout") == 0) {
+#if defined(WIN32)
+            if (strcmp(O.outfilename, "stdout") == 0) {
               csound->ErrorMsg(csound, Str("stdout audio not supported"));
               return -1;
             }
 #endif
             break;
           case 'A':
-            O->filetyp = TYP_AIFF;      /* AIFF output request*/
+            O.filetyp = TYP_AIFF;      /* AIFF output request*/
             break;
           case 'J':
-            O->filetyp = TYP_IRCAM;     /* IRCAM output request */
+            O.filetyp = TYP_IRCAM;     /* IRCAM output request */
             break;
           case 'W':
-            O->filetyp = TYP_WAV;       /* WAV output request */
+            O.filetyp = TYP_WAV;       /* WAV output request */
             break;
           case 'h':
-            O->filetyp = TYP_RAW;       /* skip sfheader  */
+            O.filetyp = TYP_RAW;       /* skip sfheader  */
             break;
           case 'c':
           case '8':
@@ -278,53 +282,53 @@ static int srconv(CSOUND *csound, int argc, char **argv)
           case 'l':
           case '3':
           case 'f':
-            outformch = set_output_format(csound, c, outformch);
+            outformch = set_output_format(csound, c, outformch, &O);
             break;
           case 'R':
-            O->rewrt_hdr = 1;
+            O.rewrt_hdr = 1;
             break;
           case 'H':
             if (isdigit(*s)) {
               int n;
-              sscanf(s, "%d%n", &O->heartbeat, &n);
+              sscanf(s, "%d%n", &O.heartbeat, &n);
               s += n;
             }
-            else O->heartbeat = 1;
+            else O.heartbeat = 1;
             break;
           case 'N':
-            O->ringbell = 1;        /* notify on completion */
+            O.ringbell = 1;        /* notify on completion */
             break;
           case 'Q':
-            FIND("No Q argument")
+            FIND(Str("No Q argument"))
             sscanf(s,"%d", &Q);
             while (*++s);
             break;
           case 'P':
-            FIND("No P argument")
+            FIND(Str("No P argument"))
 #if defined(USE_DOUBLE)
-            sscanf(s,"%lf", &P);
+            csound->sscanf(s,"%lf", &P);
 #else
-            sscanf(s,"%f", &P);
+            csound->sscanf(s,"%f", &P);
 #endif
             while (*++s);
             break;
           case 'r':
-            FIND("No r argument")
+            FIND(Str("No r argument"))
 #if defined(USE_DOUBLE)
-            sscanf(s,"%lf", &Rout);
+            csound->sscanf(s,"%lf", &Rout);
 #else
-            sscanf(s,"%f", &Rout);
+            csound->sscanf(s,"%f", &Rout);
 #endif
             while (*++s);
             break;
           case 'i':
-            FIND("No break file")
+            FIND(Str("No break file"))
             tvflg = 1;
             bfile = s;
-            while ((*s++)); s--;
+            while ((*s++)) {}; s--;
             break;
           default:
-            csound->Message(csound, "Looking at %c\n", c);
+            csound->Message(csound, Str("Looking at %c\n"), c);
             usage(csound);    /* this exits with error */
             return -1;
           }
@@ -332,16 +336,16 @@ static int srconv(CSOUND *csound, int argc, char **argv)
       }
       else if (infile == NULL) {
         infile = --s;
-        csound->Message(csound, "Infile set to %s\n", infile);
+        csound->Message(csound, Str("Infile set to %s\n"), infile);
       }
       else {
-        csound->Message(csound, "End with %s\n", s);
+        csound->Message(csound, Str("End with %s\n"), s);
         usage(csound);
         return -1;
       }
     }
     if (infile == NULL) {
-      csound->Message(csound, "No input given\n");
+      csound->Message(csound, Str("No input given\n"));
       usage(csound);
       return -1;
     }
@@ -358,7 +362,7 @@ static int srconv(CSOUND *csound, int argc, char **argv)
       Chans = 1;
 
     if ((P != FL(0.0)) && (Rout != FL(0.0))) {
-      sprintf(err_msg, Str("srconv: cannot specify both -r and -P"));
+      strncpy(err_msg, Str("srconv: cannot specify both -r and -P"), 256);
       goto err_rtn_msg;
     }
     if (P != FL(0.0))
@@ -369,26 +373,33 @@ static int srconv(CSOUND *csound, int argc, char **argv)
     if (tvflg) {
       P = FL(0.0);        /* will be reset to max in time-vary function */
       if ((tvfp = fopen(bfile, "r")) == NULL) {
-        sprintf(err_msg, Str("srconv: cannot open time-vary function file"));
+        strncpy(err_msg,
+                Str("srconv: cannot open time-vary function file"), 256);
         goto err_rtn_msg;
       }
       /* register file to be closed by csoundReset() */
-      dummy = csound->CreateFileHandle(csound, &tvfp, CSFILE_STD, bfile);
-      fscanf(tvfp, "%d", &tvlen);
+      (void) csound->CreateFileHandle(csound, &tvfp, CSFILE_STD, bfile);
+      if (UNLIKELY(fscanf(tvfp, "%d", &tvlen) != 1))
+        csound->Message(csound, Str("Read failure\n"));
+      if(tvlen <= 0) {
+            strncpy(err_msg, Str("srconv: tvlen <= 0 "), 256);
+            goto err_rtn_msg;
+       }
       fxval = (MYFLT*) csound->Malloc(csound, tvlen * sizeof(MYFLT));
       fyval = (MYFLT*) csound->Malloc(csound, tvlen * sizeof(MYFLT));
       i0 = fxval;
       i1 = fyval;
       for (i = 0; i < tvlen; i++, i0++, i1++) {
 #ifdef USE_DOUBLE
-        if ((fscanf(tvfp, "%lf %lf", i0, i1)) == EOF) {
+        if ((fscanf(tvfp, "%lf %lf", i0, i1)) != 2)
 #else
-        if ((fscanf(tvfp, "%f %f", i0, i1)) == EOF) {
+        if ((fscanf(tvfp, "%f %f", i0, i1)) != 2)
 #endif
-          sprintf(err_msg, Str("srconv: too few x-y pairs "
-                               "in time-vary function file"));
-          goto err_rtn_msg;
-        }
+          {
+            strncpy(err_msg, Str("srconv: too few x-y pairs "
+                                 "in time-vary function file"), 256);
+            goto err_rtn_msg;
+          }
         if (*i1 > P)
           P = *i1;
       }
@@ -399,52 +410,55 @@ static int srconv(CSOUND *csound, int argc, char **argv)
       tvy1 = fyval[1];
       tvdx = tvx1 - tvx0;
       if (tvx0 != FL(0.0)) {
-        sprintf(err_msg, Str("srconv: first x value "
-                             "in time-vary function must be 0"));
+        strncpy(err_msg, Str("srconv: first x value "
+                             "in time-vary function must be 0"), 256);
         goto err_rtn_msg;
       }
       if (tvy0 <= FL(0.0)) {
-        sprintf(err_msg, Str("srconv: invalid initial y value "
-                             "in time-vary function"));
+        strncpy(err_msg, Str("srconv: invalid initial y value "
+                             "in time-vary function"),256);
         goto err_rtn_msg;
       }
       if (tvdx <= FL(0.0)) {
-        sprintf(err_msg, Str("srconv: invalid x values in time-vary function"));
+        strncpy(err_msg,
+                       Str("srconv: invalid x values in time-vary function"),
+                       256);
         goto err_rtn_msg;
       }
       tvdy = tvy1 - tvy0;
       tvslope = tvdy / tvdx;
       tvnxt = 1;
     }
-
-    if (P != FL(0.0)) {         /* This is not right *********  */
-      csound->esr = Rin;
+    /* This is not right *********  */
+    if (P != FL(0.0)) {
+      csound->SetUtilSr(csound,Rin);
     }
     if (P == FL(0.0)) {
-      csound->esr = Rout;
+      csound->SetUtilSr(csound,Rout);
     }
-    if (O->outformat == 0)
-      O->outformat = p->format;
-    O->sfsampsize = csound->sfsampsize(FORMAT2SF(O->outformat));
-    if (O->filetyp == TYP_RAW) {
-      O->sfheader = 0;
-      O->rewrt_hdr = 0;
+
+    if (O.outformat == 0)
+      O.outformat = AE_SHORT;//p->format;
+    O.sfsampsize = csound->sfsampsize(FORMAT2SF(O.outformat));
+    if (O.filetyp == TYP_RAW) {
+      O.sfheader = 0;
+      O.rewrt_hdr = 0;
     }
     else
-      O->sfheader = 1;
+      O.sfheader = 1;
 #ifdef NeXT
-    if (O->outfilename == NULL && !O->filetyp)
-      O->outfilename = "test.snd";
-    else if (O->outfilename == NULL)
-      O->outfilename = "test";
+    if (O.outfilename == NULL && !O.filetyp)
+      O.outfilename = "test.snd";
+    else if (O.outfilename == NULL)
+      O.outfilename = "test";
 #else
-    if (O->outfilename == NULL) {
-      if (O->filetyp == TYP_WAV)
-        O->outfilename = "test.wav";
-      else if (O->filetyp == TYP_AIFF)
-        O->outfilename = "test.aif";
+    if (O.outfilename == NULL) {
+      if (O.filetyp == TYP_WAV)
+        O.outfilename = "test.wav";
+      else if (O.filetyp == TYP_AIFF)
+        O.outfilename = "test.aif";
       else
-        O->outfilename = "test";
+        O.outfilename = "test";
     }
 #endif
     {
@@ -453,38 +467,45 @@ static int srconv(CSOUND *csound, int argc, char **argv)
       memset(&sfinfo, 0, sizeof(SF_INFO));
       sfinfo.samplerate = (int) ((double) Rout + 0.5);
       sfinfo.channels = (int) p->nchanls;
-      sfinfo.format = TYPE2SF(O->filetyp) | FORMAT2SF(O->outformat);
-      if (strcmp(O->outfilename, "stdout") != 0) {
-        name = csound->FindOutputFile(csound, O->outfilename, "SFDIR");
+      //printf("filetyp=%x outformat=%x\n", O.filetyp, O.outformat);
+      sfinfo.format = TYPE2SF(O.filetyp) | FORMAT2SF(O.outformat);
+      if (strcmp(O.outfilename, "stdout") != 0) {
+        name = csound->FindOutputFile(csound, O.outfilename, "SFDIR");
         if (name == NULL) {
-          sprintf(err_msg, Str("cannot open %s."), O->outfilename);
+          snprintf(err_msg, 256, Str("cannot open %s."), O.outfilename);
           goto err_rtn_msg;
         }
         outfd = sf_open(name, SFM_WRITE, &sfinfo);
         if (outfd != NULL)
-          csound->NotifyFileOpened(csound, name, 
-                      csound->type2csfiletype(O->filetyp, O->outformat), 1, 0);
+          csound->NotifyFileOpened(csound, name,
+                                   csound->type2csfiletype(O.filetyp,
+                                                           O.outformat),
+                                   1, 0);
+        else {
+          snprintf(err_msg, 256, Str("libsndfile error: %s\n"), sf_strerror(NULL));
+          goto err_rtn_msg;
+        }
         csound->Free(csound, name);
       }
       else
         outfd = sf_open_fd(1, SFM_WRITE, &sfinfo, 1);
       if (outfd == NULL) {
-        sprintf(err_msg, Str("cannot open %s."), O->outfilename);
+        snprintf(err_msg, 256, Str("cannot open %s."), O.outfilename);
         goto err_rtn_msg;
       }
       /* register file to be closed by csoundReset() */
-      dummy = csound->CreateFileHandle(csound, &outfd, CSFILE_SND_W,
-                                               O->outfilename);
+      (void) csound->CreateFileHandle(csound, &outfd, CSFILE_SND_W,
+                                      O.outfilename);
       sf_command(outfd, SFC_SET_CLIPPING, NULL, SF_TRUE);
     }
-    csound->esr = (MYFLT) p->sr;
-    csound->nchnls = Chans = p->nchanls;
+    csound->SetUtilSr(csound, (MYFLT)p->sr);
+    csound->SetUtilNchnls(csound, Chans = p->nchanls);
 
-    outbufsiz = OBUF * O->sfsampsize;                   /* calc outbuf size */
+    outbufsiz = OBUF * O.sfsampsize;                   /* calc outbuf size */
     csound->Message(csound, Str("writing %d-byte blks of %s to %s"),
-                            outbufsiz, csound->getstrformat(O->outformat),
-                            O->outfilename);
-    csound->Message(csound, " (%s)\n", csound->type2string(O->filetyp));
+                    outbufsiz, csound->getstrformat(O.outformat),
+                    O.outfilename);
+    csound->Message(csound, " (%s)\n", csound->type2string(O.filetyp));
 
  /* this program performs arbitrary sample-rate conversion
     with high fidelity.  the method is to step through the
@@ -498,7 +519,7 @@ static int srconv(CSOUND *csound, int argc, char **argv)
 
  /* calculate increment: if decimating, then window is impulse response of low-
     pass filter with cutoff frequency at half of Rout; if interpolating,
-    then window is impulse response of lowpass filter with cutoff frequency
+    then window is ipulse response of lowpass filter with cutoff frequency
     at half of Rin. */
 
     fdel = ((MYFLT) (L * Rin) / Rout);
@@ -565,6 +586,8 @@ static int srconv(CSOUND *csound, int argc, char **argv)
  /* initialization: */
 
     nread = csound->getsndin(csound, inf, input, IBUF2, p);
+    for(i=0; i < nread; i++)
+       input[i] *= 1.0/csound->Get0dBFS(csound);
     nMax = (long)(input_dur * p->sr);
     nextIn = input + nread;
     for (i = nread; i < IBUF2; i++)
@@ -602,7 +625,7 @@ static int srconv(CSOUND *csound, int argc, char **argv)
           nextOut++;
           if (nextOut >= (output + OBUF)) {
             nextOut = output;
-            writebuffer(csound, output, &block, outfd, OBUF);
+            writebuffer(csound, output, &block, outfd, OBUF, &O);
           }
         }
 
@@ -620,6 +643,8 @@ static int srconv(CSOUND *csound, int argc, char **argv)
             if (nextIn >= (input + IBUF))
               nextIn = input;
             nread = csound->getsndin(csound, inf, nextIn, IBUF2, p);
+            for(i=0; i < nread; i++)
+               input[i] *= 1.0/csound->Get0dBFS(csound);
             nextIn += nread;
             if (nread < IBUF2)
               nMax = n + wLen + (nread / Chans) + 1;
@@ -660,7 +685,7 @@ static int srconv(CSOUND *csound, int argc, char **argv)
           nextOut++;
           if (nextOut >= (output + OBUF)) {
             nextOut = output;
-            writebuffer(csound, output, &block, outfd, OBUF);
+            writebuffer(csound, output, &block, outfd, OBUF, &O);
           }
         }
 
@@ -678,6 +703,8 @@ static int srconv(CSOUND *csound, int argc, char **argv)
             if (nextIn >= (input + IBUF))
               nextIn = input;
             nread = csound->getsndin(csound, inf, nextIn, IBUF2, p);
+            for(i=0; i < nread; i++)
+               input[i] *= 1.0/csound->Get0dBFS(csound);
             nextIn += nread;
             if (nread < IBUF2)
               nMax = n + wLen + (nread / Chans) + 1;
@@ -701,8 +728,8 @@ static int srconv(CSOUND *csound, int argc, char **argv)
               tvy1 = fyval[tvnxt];
               tvdx = tvx1 - tvx0;
               if (tvdx <= FL(0.0)) {
-                sprintf(err_msg, Str("srconv: invalid x values "
-                                     "in time-vary function"));
+                strncpy(err_msg, Str("srconv: invalid x values "
+                                     "in time-vary function"), 256);
                 goto err_rtn_msg;
               }
               tvdy = tvy1 - tvy0;
@@ -716,41 +743,49 @@ static int srconv(CSOUND *csound, int argc, char **argv)
 
     }
     nread = nextOut - output;
-    writebuffer(csound, output, &block, outfd, nread);
+    writebuffer(csound, output, &block, outfd, nread, &O);
     csound->Message(csound, "\n\n");
-    if (O->ringbell)
+    if (O.ringbell)
       csound->MessageS(csound, CSOUNDMSG_REALTIME, "\a");
     return 0;
 
  err_rtn_msg:
-    csound->ErrorMsg(csound, "%s", err_msg);
+    csound->ErrorMsg(csound, err_msg);
     return -1;
 }
+#else
+static int srconv(CSOUND *csound, int argc, char **argv)
+{
+    csound->Message(csound, Str("Do not use srconv but the src_conv program\n"));
+    return execv("src_conv", argv);
+}
+#endif
 
 static const char *usage_txt[] = {
-    "usage: srconv [flags] infile\n\nflags:",
-    "-P num\tpitch transposition ratio (srate/r) [don't specify both P and r]",
-    "-Q num\tquality factor (1 to 8: default = 2)",
-    "-i filnam\tbreak file",
-    "-r num\toutput sample rate (must be specified)",
-    "-o fnam\tsound output filename\n",
-    "-A\tcreate an AIFF format output soundfile",
-    "-J\tcreate an IRCAM format output soundfile",
-    "-W\tcreate a WAV format output soundfile",
-    "-h\tno header on output soundfile",
-    "-c\t8-bit signed_char sound samples",
-    "-a\talaw sound samples",
-    "-8\t8-bit unsigned_char sound samples",
-    "-u\tulaw sound samples",
-    "-s\tshort_int sound samples",
-    "-l\tlong_int sound samples",
-    "-f\tfloat sound samples",
-    "-r N\torchestra srate override",
-    "-K\tDo not generate PEAK chunks",
-    "-R\tcontinually rewrite header while writing soundfile (WAV/AIFF)",
-    "-H#\tprint a heartbeat style 1, 2 or 3 at each soundfile write",
-    "-N\tnotify (ring the bell) when score or miditrack is done",
-    "-- fnam\tlog output to file",
+  Str_noop("usage: srconv [flags] infile\n\nflags:"),
+  Str_noop("-P num\tpitch transposition ratio (srate/r) [do not specify "
+           "both P and r]"),
+  Str_noop("-Q num\tquality factor (1 to 8: default = 2)"),
+  Str_noop("-i filnam\tbreak file"),
+  Str_noop("-r num\toutput sample rate (must be specified)"),
+  Str_noop("-o fnam\tsound output filename\n"),
+  Str_noop("-A\tcreate an AIFF format output soundfile"),
+  Str_noop("-J\tcreate an IRCAM format output soundfile"),
+  Str_noop("-W\tcreate a WAV format output soundfile"),
+  Str_noop("-h\tno header on output soundfile"),
+  Str_noop("-c\t8-bit signed_char sound samples"),
+  Str_noop("-a\talaw sound samples"),
+  Str_noop("-8\t8-bit unsigned_char sound samples"),
+  Str_noop("-u\tulaw sound samples"),
+  Str_noop("-s\tshort_int sound samples"),
+  Str_noop("-l\tlong_int sound samples"),
+  Str_noop("-f\tfloat sound samples"),
+  Str_noop("-r N\torchestra srate override"),
+  Str_noop("-K\tDo not generate PEAK chunks"),
+  Str_noop("-R\tcontinually rewrite header while writing soundfile (WAV/AIFF)"),
+  Str_noop("-H#\tprint a heartbeat style 1, 2 or 3 at each soundfile write"),
+  Str_noop("-N\tnotify (ring the bell) when score or miditrack is done"),
+  Str_noop("-- fnam\tlog output to file"),
     NULL
 };
 
@@ -782,6 +817,7 @@ static double ino(double x)
     return e;
 }
 
+#if 0
 static void kaiser(int nf, float *w, int n, int ieo, double beta)
 {
 
@@ -808,6 +844,7 @@ static void kaiser(int nf, float *w, int n, int ieo, double beta)
       w[i] = (float) (ino(beta * xi) / bes);
     }
 }
+#endif
 
 /* module interface */
 
@@ -816,8 +853,7 @@ int srconv_init_(CSOUND *csound)
     int retval = csound->AddUtility(csound, "srconv", srconv);
     if (!retval) {
       retval = csound->SetUtilityDescription(csound, "srconv",
-                                             "Sample rate conversion");
+                                             Str("Sample rate conversion"));
     }
     return retval;
 }
-
